@@ -50,12 +50,13 @@ archiveAsyncSpoolQueue(ArchiveMode archiveMode)
         FUNCTION_TEST_PARAM(STRING_ID, archiveMode);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN((archiveMode == archiveModeGet ? STORAGE_SPOOL_ARCHIVE_IN_STR : STORAGE_SPOOL_ARCHIVE_OUT_STR));
+    FUNCTION_TEST_RETURN_CONST(
+        STRING, archiveMode == archiveModeGet ? STORAGE_SPOOL_ARCHIVE_IN_STR : STORAGE_SPOOL_ARCHIVE_OUT_STR);
 }
 
 /**********************************************************************************************************************************/
 void
-archiveAsyncErrorClear(ArchiveMode archiveMode, const String *archiveFile)
+archiveAsyncErrorClear(const ArchiveMode archiveMode, const String *const archiveFile)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING_ID, archiveMode);
@@ -64,9 +65,13 @@ archiveAsyncErrorClear(ArchiveMode archiveMode, const String *archiveFile)
 
     ASSERT(archiveFile != NULL);
 
-    storageRemoveP(storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s" STATUS_EXT_ERROR, strZ(archiveFile)));
+    String *const errorFile = strNewFmt(STORAGE_SPOOL_ARCHIVE_OUT "/%s" STATUS_EXT_ERROR, strZ(archiveFile));
+
+    storageRemoveP(storageSpoolWrite(), errorFile);
     storageRemoveP(storageSpoolWrite(), STRDEF(STORAGE_SPOOL_ARCHIVE_OUT "/" STATUS_FILE_GLOBAL_ERROR));
 
+    strFree(errorFile);
+
     FUNCTION_LOG_RETURN_VOID();
 }
 
@@ -281,7 +286,7 @@ archiveAsyncExec(ArchiveMode archiveMode, const StringList *commandExec)
     // If the process does not exit immediately then something probably went wrong with the double fork.  It's possible that this
     // test will fail on very slow systems so it may need to be tuned.  The idea is to make sure that the waitpid() above is not
     // waiting on the async process.
-    ASSERT(timeMSec() - timeBegin < 10);
+    CHECK(AssertError, timeMSec() - timeBegin < 10, "the process does not exit immediately");
 #endif
 
     FUNCTION_LOG_RETURN_VOID();
@@ -484,7 +489,7 @@ walSegmentNext(const String *walSegment, size_t walSegmentSize, unsigned int pgV
     ASSERT(walSegment != NULL);
     ASSERT(strSize(walSegment) == 24);
     ASSERT(UINT32_MAX % walSegmentSize == walSegmentSize - 1);
-    ASSERT(pgVersion >= PG_VERSION_11 || walSegmentSize == 16 * 1024 * 1024);
+    ASSERT(pgVersion >= PG_VERSION_11 || walSegmentSize == PG_WAL_SEGMENT_SIZE_DEFAULT);
 
     // Extract WAL parts
     uint32_t timeline = 0;
