@@ -20,6 +20,7 @@ Archive Get Command
 #include "common/wait.h"
 #include "config/config.h"
 #include "config/exec.h"
+#include "config/load.h"
 #include "info/infoArchive.h"
 #include "postgres/interface.h"
 #include "protocol/helper.h"
@@ -31,7 +32,7 @@ Archive Get Command
 Constants for log messages that are used multiple times to keep them consistent
 ***********************************************************************************************************************************/
 #define FOUND_IN_ARCHIVE_MSG                                        "found %s in the archive"
-#define FOUND_IN_REPO_ARCHIVE_MSG                                   "found %s in the repo%u: %s archive"
+#define FOUND_IN_REPO_ARCHIVE_MSG                                   "found %s in the %s: %s archive"
 #define UNABLE_TO_FIND_IN_ARCHIVE_MSG                               "unable to find %s in the archive"
 #define UNABLE_TO_FIND_VALID_REPO_MSG                               "unable to find a valid repository"
 #define REPO_INVALID_OR_ERR_MSG                                     "some repositories were invalid or encountered errors"
@@ -61,8 +62,7 @@ typedef struct ArchiveGetCheckResult
 static void
 archiveGetErrorAdd(StringList *warnList, bool log, unsigned int repoIdx, const ErrorType *type, const String *message)
 {
-    const String *warn = strNewFmt(
-        "repo%u: [%s] %s", cfgOptionGroupIdxToKey(cfgOptGrpRepo, repoIdx), errorTypeName(type), strZ(message));
+    const String *warn = strNewFmt("%s: [%s] %s", cfgOptionGroupName(cfgOptGrpRepo, repoIdx), errorTypeName(type), strZ(message));
 
     if (!strLstExists(warnList, warn))
     {
@@ -298,7 +298,7 @@ archiveGetFind(
 
                         if (repoKey != repoKeyLast)
                         {
-                            strCatFmt(message, "\nrepo%u:", repoKey);
+                            strCatFmt(message, "\n%s:", cfgOptionGroupName(cfgOptGrpRepo, file->repoIdx));
                             repoKeyLast = repoKey;
                             first = true;
                         }
@@ -648,11 +648,15 @@ cmdArchiveGet(void)
                 // Check if the WAL segment is already in the queue
                 found = storageExistsP(storageSpool(), strNewFmt(STORAGE_SPOOL_ARCHIVE_IN "/%s", strZ(walSegment)));
 
+                // Determine whether a missing WAL segment will be retried. Retrying is safer, but not retrying lets PostgreSQL
+                // know that there are probably no more WAL segments in the archive which means it can switch to streaming.
+                const bool missingRetry = first && cfgOptionBool(cfgOptArchiveMissingRetry);
+
                 // Check for errors or missing files. For archive-get ok indicates that the process succeeded but there is no WAL
                 // file to download, or that there was a warning. Do not error on the first run so the async process can be spawned
                 // to correct any errors from a previous run. Do not warn on the first run if the segment was not found so the async
                 // process can be spawned to check for the file again.
-                if (archiveAsyncStatus(archiveModeGet, walSegment, !first, found || !first))
+                if (archiveAsyncStatus(archiveModeGet, walSegment, !first, found || !missingRetry))
                 {
                     storageRemoveP(
                         storageSpoolWrite(), strNewFmt(STORAGE_SPOOL_ARCHIVE_IN "/%s" STATUS_EXT_OK, strZ(walSegment)),
@@ -663,7 +667,7 @@ cmdArchiveGet(void)
                     // spawned by a prior archive-get execution, which means we should spawn the async process again to see if the
                     // file exists now. This also prevents spool files from a previous recovery interfering with the current
                     // recovery.
-                    if (!found && !first)
+                    if (!found && !missingRetry)
                     {
                         foundOk = true;
                         break;
@@ -712,7 +716,7 @@ cmdArchiveGet(void)
                 // If the WAL segment has not already been found then start the async process to get it.  There's no point in
                 // forking the async process off more than once so track that as well.  Use an archive lock to prevent forking if
                 // the async process was launched by another process.
-                if (!forked && (!found || !queueFull)  &&
+                if (!forked && (!found || !queueFull) &&
                     lockAcquire(
                         cfgOptionStr(cfgOptLockPath), cfgOptionStr(cfgOptStanza), cfgOptionStr(cfgOptExecId), cfgLockType(), 0,
                         false))
@@ -773,8 +777,11 @@ cmdArchiveGet(void)
                 if (!foundOk)
                 {
                     THROW_FMT(
-                        ArchiveTimeoutError, "unable to get WAL file '%s' from the archive asynchronously after %s second(s)",
-                        strZ(walSegment), strZ(strNewDbl((double)cfgOptionInt64(cfgOptArchiveTimeout) / MSEC_PER_SEC)));
+                        ArchiveTimeoutError,
+                        "unable to get WAL file '%s' from the archive asynchronously after %s second(s)\n"
+                        "HINT: check '%s' for errors.",
+                        strZ(walSegment), strZ(strNewDbl((double)cfgOptionInt64(cfgOptArchiveTimeout) / MSEC_PER_SEC)),
+                        strZ(cfgLoadLogFileName(cfgCmdRoleAsync)));
                 }
                 // Else report that the WAL segment could not be found
                 else
@@ -814,7 +821,7 @@ cmdArchiveGet(void)
                 ASSERT(file != NULL);
 
                 LOG_INFO_FMT(
-                    FOUND_IN_REPO_ARCHIVE_MSG, strZ(walSegment), cfgOptionGroupIdxToKey(cfgOptGrpRepo, file->repoIdx),
+                    FOUND_IN_REPO_ARCHIVE_MSG, strZ(walSegment), cfgOptionGroupName(cfgOptGrpRepo, file->repoIdx),
                     strZ(file->archiveId));
 
                 result = 0;
@@ -884,7 +891,7 @@ static ProtocolParallelJob *archiveGetAsyncCallback(void *data, unsigned int cli
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(PROTOCOL_PARALLEL_JOB, result);
 }
 
 void
@@ -907,7 +914,7 @@ cmdArchiveGetAsync(void)
                 "get %u WAL file(s) from archive: %s%s",
                 strLstSize(cfgCommandParam()), strZ(strLstGet(cfgCommandParam(), 0)),
                 strLstSize(cfgCommandParam()) == 1 ?
-                    "" : strZ(strNewFmt("...%s", strZ(strLstGet(cfgCommandParam(), strLstSize(cfgCommandParam()) - 1)))));
+                    "" : zNewFmt("...%s", strZ(strLstGet(cfgCommandParam(), strLstSize(cfgCommandParam()) - 1))));
 
             // Check for archive files
             ArchiveGetCheckResult checkResult = archiveGetCheck(cfgCommandParam());
@@ -980,7 +987,7 @@ cmdArchiveGetAsync(void)
 
                                 LOG_DETAIL_PID_FMT(
                                     processId, FOUND_IN_REPO_ARCHIVE_MSG, strZ(walSegment),
-                                    cfgOptionGroupIdxToKey(cfgOptGrpRepo, file->repoIdx), strZ(file->archiveId));
+                                    cfgOptionGroupName(cfgOptGrpRepo, file->repoIdx), strZ(file->archiveId));
 
                                 // Rename temp WAL segment to actual name. This is done after the ok file is written so the ok file
                                 // is guaranteed to exist before the foreground process finds the WAL segment.
@@ -1003,7 +1010,7 @@ cmdArchiveGetAsync(void)
                                     archiveModeGet, walSegment, protocolParallelJobErrorCode(job),
                                     strNewFmt(
                                         "%s%s", strZ(protocolParallelJobErrorMessage(job)),
-                                        strSize(warning) == 0 ? "" : strZ(strNewFmt("\n%s", strZ(warning)))));
+                                        strSize(warning) == 0 ? "" : zNewFmt("\n%s", strZ(warning))));
                             }
 
                             protocolParallelJobFree(job);
