@@ -31,7 +31,8 @@ typedef struct ArchivePushFileResult
 
 // Copy a file from the source to the archive
 ArchivePushFileResult archivePushFile(
-    const String *walSource, bool headerCheck, unsigned int pgVersion, uint64_t pgSystemId, const String *archiveFile,
-    CompressType compressType, int compressLevel, const List *const repoList, const StringList *const priorErrorList);
+    const String *walSource, bool headerCheck, bool modeCheck, unsigned int pgVersion, uint64_t pgSystemId,
+    const String *archiveFile, CompressType compressType, int compressLevel, const List *repoList,
+    const StringList *priorErrorList);
 
 #endif
