@@ -17,6 +17,7 @@ Archive Push Command
 #include "common/memContext.h"
 #include "common/wait.h"
 #include "config/config.h"
+#include "config/load.h"
 #include "config/exec.h"
 #include "info/infoArchive.h"
 #include "postgres/interface.h"
@@ -48,7 +49,7 @@ archivePushDropWarning(const String *walFile, uint64_t queueMax)
     FUNCTION_TEST_END();
 
     FUNCTION_TEST_RETURN(
-        strNewFmt("dropped WAL file '%s' because archive queue exceeded %s", strZ(walFile), strZ(strSizeFormat(queueMax))));
+        STRING, strNewFmt("dropped WAL file '%s' because archive queue exceeded %s", strZ(walFile), strZ(strSizeFormat(queueMax))));
 }
 
 /***********************************************************************************************************************************
@@ -66,17 +67,24 @@ archivePushDrop(const String *walPath, const StringList *const processList)
     uint64_t queueSize = 0;
     bool result = false;
 
-    for (unsigned int processIdx = 0; processIdx < strLstSize(processList); processIdx++)
+    MEM_CONTEXT_TEMP_RESET_BEGIN()
     {
-        queueSize += storageInfoP(
-            storagePg(), strNewFmt("%s/%s", strZ(walPath), strZ(strLstGet(processList, processIdx)))).size;
-
-        if (queueSize > queueMax)
+        for (unsigned int processIdx = 0; processIdx < strLstSize(processList); processIdx++)
         {
-            result = true;
-            break;
+            queueSize += storageInfoP(
+                storagePg(), strNewFmt("%s/%s", strZ(walPath), strZ(strLstGet(processList, processIdx)))).size;
+
+            if (queueSize > queueMax)
+            {
+                result = true;
+                break;
+            }
+
+            // Reset the memory context occasionally so we don't use too much memory or slow down processing
+            MEM_CONTEXT_TEMP_RESET(1000);
         }
     }
+    MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN(BOOL, result);
 }
@@ -108,9 +116,8 @@ archivePushReadyList(const String *walPath)
 
         for (unsigned int readyIdx = 0; readyIdx < strLstSize(readyListRaw); readyIdx++)
         {
-            strLstAdd(
-                result,
-                strSubN(strLstGet(readyListRaw, readyIdx), 0, strSize(strLstGet(readyListRaw, readyIdx)) - STATUS_EXT_READY_SIZE));
+            const String *const ready = strLstGet(readyListRaw, readyIdx);
+            strLstAddSub(result, ready, strSize(ready) - STATUS_EXT_READY_SIZE);
         }
 
         strLstMove(result, memContextPrior());
@@ -156,7 +163,7 @@ archivePushProcessList(const String *walPath)
             const String *statusFile = strLstGet(statusList, statusIdx);
 
             if (strEndsWithZ(statusFile, STATUS_EXT_OK))
-                strLstAdd(okList, strSubN(statusFile, 0, strSize(statusFile) - STATUS_EXT_OK_SIZE));
+                strLstAddSub(okList, statusFile, strSize(statusFile) - STATUS_EXT_OK_SIZE);
             else
             {
                 storageRemoveP(
@@ -250,10 +257,10 @@ archivePushCheck(bool pgPathSet)
                             ArchiveMismatchError,
                             "%s version %s, system-id %" PRIu64 " do not match %s stanza version %s, system-id %" PRIu64
                             "\nHINT: are you archiving to the correct stanza?",
-                            pgPathSet ? PG_NAME : strZ(strNewFmt("repo%u stanza", cfgOptionGroupIdxToKey(cfgOptGrpRepo, 0))),
+                            pgPathSet ? PG_NAME : zNewFmt("%s stanza", cfgOptionGroupName(cfgOptGrpRepo, 0)),
                             strZ(pgVersionToStr(result.pgVersion)), result.pgSystemId,
-                            strZ(strNewFmt("repo%u", cfgOptionGroupIdxToKey(cfgOptGrpRepo, repoIdx))),
-                            strZ(pgVersionToStr(archiveInfo.version)), archiveInfo.systemId);
+                            cfgOptionGroupName(cfgOptGrpRepo, repoIdx), strZ(pgVersionToStr(archiveInfo.version)),
+                            archiveInfo.systemId);
                     }
                 }
 
@@ -276,11 +283,9 @@ archivePushCheck(bool pgPathSet)
             }
             CATCH_ANY()
             {
-                strLstAdd(
-                    result.errorList,
-                    strNewFmt(
-                        "repo%u: [%s] %s", cfgOptionGroupIdxToKey(cfgOptGrpRepo, repoIdx), errorTypeName(errorType()),
-                        errorMessage()));
+                strLstAddFmt(
+                    result.errorList, "%s: [%s] %s", cfgOptionGroupName(cfgOptGrpRepo, repoIdx), errorTypeName(errorType()),
+                    errorMessage());
             }
             TRY_END();
         }
@@ -389,8 +394,10 @@ cmdArchivePush(void)
             if (!pushed)
             {
                 THROW_FMT(
-                    ArchiveTimeoutError, "unable to push WAL file '%s' to the archive asynchronously after %s second(s)",
-                    strZ(archiveFile), strZ(cfgOptionDisplay(cfgOptArchiveTimeout)));
+                    ArchiveTimeoutError,
+                    "unable to push WAL file '%s' to the archive asynchronously after %s second(s)\n"
+                    "HINT: check '%s' for errors.",
+                    strZ(archiveFile), strZ(cfgOptionDisplay(cfgOptArchiveTimeout)), strZ(cfgLoadLogFileName(cfgCmdRoleAsync)));
             }
 
             // Log success
@@ -412,9 +419,9 @@ cmdArchivePush(void)
 
                 // Push the file to the archive
                 ArchivePushFileResult fileResult = archivePushFile(
-                    walFile, cfgOptionBool(cfgOptArchiveHeaderCheck), archiveInfo.pgVersion, archiveInfo.pgSystemId, archiveFile,
-                    compressTypeEnum(cfgOptionStrId(cfgOptCompressType)), cfgOptionInt(cfgOptCompressLevel), archiveInfo.repoList,
-                    archiveInfo.errorList);
+                    walFile, cfgOptionBool(cfgOptArchiveHeaderCheck), cfgOptionBool(cfgOptArchiveModeCheck), archiveInfo.pgVersion,
+                    archiveInfo.pgSystemId, archiveFile, compressTypeEnum(cfgOptionStrId(cfgOptCompressType)),
+                    cfgOptionInt(cfgOptCompressLevel), archiveInfo.repoList, archiveInfo.errorList);
 
                 // If a warning was returned then log it
                 for (unsigned int warnIdx = 0; warnIdx < strLstSize(fileResult.warnList); warnIdx++)
@@ -469,6 +476,7 @@ archivePushAsyncCallback(void *data, unsigned int clientIdx)
 
             pckWriteStrP(param, strNewFmt("%s/%s", strZ(jobData->walPath), strZ(walFile)));
             pckWriteBoolP(param, cfgOptionBool(cfgOptArchiveHeaderCheck));
+            pckWriteBoolP(param, cfgOptionBool(cfgOptArchiveModeCheck));
             pckWriteU32P(param, jobData->archiveInfo.pgVersion);
             pckWriteU64P(param, jobData->archiveInfo.pgSystemId);
             pckWriteStrP(param, walFile);
@@ -502,7 +510,7 @@ archivePushAsyncCallback(void *data, unsigned int clientIdx)
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(PROTOCOL_PARALLEL_JOB, result);
 }
 
 void
@@ -545,7 +553,7 @@ cmdArchivePushAsync(void)
             LOG_INFO_FMT(
                 "push %u WAL file(s) to archive: %s%s", strLstSize(jobData.walFileList), strZ(strLstGet(jobData.walFileList, 0)),
                 strLstSize(jobData.walFileList) == 1 ?
-                    "" : strZ(strNewFmt("...%s", strZ(strLstGet(jobData.walFileList, strLstSize(jobData.walFileList) - 1)))));
+                    "" : zNewFmt("...%s", strZ(strLstGet(jobData.walFileList, strLstSize(jobData.walFileList) - 1))));
 
             // Drop files if queue max has been exceeded
             if (cfgOptionTest(cfgOptArchivePushQueueMax) && archivePushDrop(jobData.walPath, jobData.walFileList))
@@ -629,7 +637,7 @@ cmdArchivePushAsync(void)
             }
         }
         // On any global error write a single error file to cover all unprocessed files
-        CATCH_ANY()
+        CATCH_FATAL()
         {
             archiveAsyncStatusErrorWrite(archiveModePush, NULL, errorCode(), STR(errorMessage()));
             RETHROW();
