@@ -10,6 +10,7 @@ Info Command
 
 #include "command/archive/common.h"
 #include "command/info/info.h"
+#include "common/crypto/common.h"
 #include "common/debug.h"
 #include "common/io/fdWrite.h"
 #include "common/lock.h"
@@ -17,7 +18,6 @@ Info Command
 #include "common/memContext.h"
 #include "common/type/json.h"
 #include "config/config.h"
-#include "common/crypto/common.h"
 #include "info/info.h"
 #include "info/infoArchive.h"
 #include "info/infoBackup.h"
@@ -41,6 +41,7 @@ VARIANT_STRDEF_STATIC(BACKUP_KEY_DATABASE_REF_VAR,                  "database-re
 VARIANT_STRDEF_STATIC(BACKUP_KEY_INFO_VAR,                          "info");
 VARIANT_STRDEF_STATIC(BACKUP_KEY_LABEL_VAR,                         "label");
 VARIANT_STRDEF_STATIC(BACKUP_KEY_LINK_VAR,                          "link");
+VARIANT_STRDEF_STATIC(BACKUP_KEY_LSN_VAR,                           "lsn");
 VARIANT_STRDEF_STATIC(BACKUP_KEY_PRIOR_VAR,                         "prior");
 VARIANT_STRDEF_STATIC(BACKUP_KEY_REFERENCE_VAR,                     "reference");
 VARIANT_STRDEF_STATIC(BACKUP_KEY_TABLESPACE_VAR,                    "tablespace");
@@ -70,6 +71,7 @@ VARIANT_STRDEF_STATIC(STATUS_KEY_CODE_VAR,                          "code");
 VARIANT_STRDEF_STATIC(STATUS_KEY_LOCK_VAR,                          "lock");
 VARIANT_STRDEF_STATIC(STATUS_KEY_LOCK_BACKUP_VAR,                   "backup");
 VARIANT_STRDEF_STATIC(STATUS_KEY_LOCK_BACKUP_HELD_VAR,              "held");
+VARIANT_STRDEF_STATIC(STATUS_KEY_LOCK_BACKUP_PERCENT_COMPLETE_VAR,  "pct-cplt");
 VARIANT_STRDEF_STATIC(STATUS_KEY_MESSAGE_VAR,                       "message");
 
 #define INFO_STANZA_STATUS_OK                                       "ok"
@@ -128,6 +130,7 @@ typedef struct InfoStanzaRepo
     unsigned int currentPgVersion;                                  // Current postgres version for the stanza
     bool backupLockChecked;                                         // Has the check for a backup lock already been performed?
     bool backupLockHeld;                                            // Is backup lock held on the system where info command is run?
+    const Variant *percentComplete;                                 // Percentage of backup complete * 100 (when not NULL)
     InfoRepoData *repoList;                                         // List of configured repositories
 } InfoStanzaRepo;
 
@@ -174,15 +177,16 @@ infoStanzaErrorAdd(InfoRepoData *repoList, const ErrorType *type, const String *
 Set the overall error status code and message for the stanza to the code and message passed
 ***********************************************************************************************************************************/
 static void
-stanzaStatus(const int code, bool backupLockHeld, Variant *stanzaInfo)
+stanzaStatus(const int code, const InfoStanzaRepo *const stanzaData, Variant *stanzaInfo)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(INT, code);
-        FUNCTION_TEST_PARAM(BOOL, backupLockHeld);
+        FUNCTION_TEST_PARAM(INFO_STANZA_REPO, stanzaData);
         FUNCTION_TEST_PARAM(VARIANT, stanzaInfo);
     FUNCTION_TEST_END();
 
     ASSERT((code >= 0 && code <= 6) || code == 99);
+    ASSERT(stanzaData != NULL);
     ASSERT(stanzaInfo != NULL);
 
     KeyValue *statusKv = kvPutKv(varKv(stanzaInfo), STANZA_KEY_STATUS_VAR);
@@ -227,7 +231,10 @@ stanzaStatus(const int code, bool backupLockHeld, Variant *stanzaInfo)
     // Construct a specific lock part
     KeyValue *lockKv = kvPutKv(statusKv, STATUS_KEY_LOCK_VAR);
     KeyValue *backupLockKv = kvPutKv(lockKv, STATUS_KEY_LOCK_BACKUP_VAR);
-    kvPut(backupLockKv, STATUS_KEY_LOCK_BACKUP_HELD_VAR, VARBOOL(backupLockHeld));
+    kvPut(backupLockKv, STATUS_KEY_LOCK_BACKUP_HELD_VAR, VARBOOL(stanzaData->backupLockHeld));
+
+    if (stanzaData->percentComplete != NULL && cfgOptionStrId(cfgOptOutput) != CFGOPTVAL_OUTPUT_JSON)
+        kvPut(backupLockKv, STATUS_KEY_LOCK_BACKUP_PERCENT_COMPLETE_VAR, stanzaData->percentComplete);
 
     FUNCTION_TEST_RETURN_VOID();
 }
@@ -395,6 +402,10 @@ backupListAdd(
 
     Variant *backupInfo = varNewKv(kvNew());
 
+    // Flags used to decide what data to add
+    const bool outputJson = cfgOptionStrId(cfgOptOutput) == CFGOPTVAL_OUTPUT_JSON;
+    const bool backupLabelMatch = backupLabel != NULL && strEq(backupData->backupLabel, backupLabel);
+
     // main keys
     kvPut(varKv(backupInfo), BACKUP_KEY_LABEL_VAR, VARSTR(backupData->backupLabel));
     kvPut(varKv(backupInfo), BACKUP_KEY_TYPE_VAR, VARSTR(strIdToStr(backupData->backupType)));
@@ -450,8 +461,16 @@ backupListAdd(
     if (backupData->backupError != NULL)
         kvPut(varKv(backupInfo), BACKUP_KEY_ERROR_VAR, backupData->backupError);
 
+    // Add start/stop backup lsn info to json output or --set text
+    if ((outputJson || backupLabelMatch) && backupData->backupLsnStart != NULL && backupData->backupLsnStop != NULL)
+    {
+        KeyValue *const lsnInfo = kvPutKv(varKv(backupInfo), BACKUP_KEY_LSN_VAR);
+        kvPut(lsnInfo, KEY_START_VAR, VARSTR(backupData->backupLsnStart));
+        kvPut(lsnInfo, KEY_STOP_VAR, VARSTR(backupData->backupLsnStop));
+    }
+
     // If a backup label was specified and this is that label, then get the data from the loaded manifest
-    if (backupLabel != NULL && strEq(backupData->backupLabel, backupLabel))
+    if (backupLabelMatch)
     {
         // Get the list of databases in this backup
         VariantList *databaseSection = varLstNew();
@@ -461,7 +480,7 @@ backupListAdd(
             const ManifestDb *db = manifestDb(repoData->manifest, dbIdx);
 
             // Do not display template databases
-            if (db->id > db->lastSystemId)
+            if (!pgDbIsTemplate(db->name))
             {
                 Variant *database = varNewKv(kvNew());
 
@@ -520,10 +539,10 @@ backupListAdd(
 
         for (unsigned int fileIdx = 0; fileIdx < manifestFileTotal(repoData->manifest); fileIdx++)
         {
-            const ManifestFile *file = manifestFile(repoData->manifest, fileIdx);
+            const ManifestFile file = manifestFile(repoData->manifest, fileIdx);
 
-            if (file->checksumPageError)
-                varLstAdd(checksumPageErrorList, varNewStr(manifestPathPg(file->name)));
+            if (file.checksumPageError)
+                varLstAdd(checksumPageErrorList, varNewStr(manifestPathPg(file.name)));
         }
 
         if (!varLstEmpty(checksumPageErrorList))
@@ -552,8 +571,8 @@ For each current backup in the backup.info file of the stanza, set the data for
 ***********************************************************************************************************************************/
 static void
 backupList(
-    VariantList *backupSection, InfoStanzaRepo *stanzaData, const String *backupLabel, unsigned int repoIdxMin,
-    unsigned int repoIdxMax)
+    VariantList *const backupSection, InfoStanzaRepo *const stanzaData, const String *const backupLabel,
+    const unsigned int repoIdxMin, const unsigned int repoIdxMax)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(VARIANT_LIST, backupSection);           // The section to add the backup data to
@@ -610,7 +629,8 @@ backupList(
         repoData->backupIdx++;
 
         // Add the backup data to the backup section
-        backupListAdd(backupSection, &backupData, backupLabel, repoData);
+        if (!cfgOptionTest(cfgOptType) || cfgOptionStrId(cfgOptType) == backupData.backupType)
+            backupListAdd(backupSection, &backupData, backupLabel, repoData);
 
         backupTotalProcessed++;
     }
@@ -622,7 +642,7 @@ backupList(
 Set the stanza data for each stanza found in the repo
 ***********************************************************************************************************************************/
 static VariantList *
-stanzaInfoList(List *stanzaRepoList, const String *backupLabel, unsigned int repoIdxMin, unsigned int repoIdxMax)
+stanzaInfoList(List *stanzaRepoList, const String *const backupLabel, const unsigned int repoIdxMin, const unsigned int repoIdxMax)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(LIST, stanzaRepoList);
@@ -752,7 +772,7 @@ stanzaInfoList(List *stanzaRepoList, const String *backupLabel, unsigned int rep
         kvPut(varKv(stanzaInfo), STANZA_KEY_BACKUP_VAR, varNewVarLst(backupSection));
 
         // Set the overall stanza status
-        stanzaStatus(stanzaStatusCode, stanzaData->backupLockHeld, stanzaInfo);
+        stanzaStatus(stanzaStatusCode, stanzaData, stanzaInfo);
 
         // Set the overall cipher type
         if (stanzaCipherType != INFO_STANZA_STATUS_CODE_MIXED)
@@ -763,7 +783,7 @@ stanzaInfoList(List *stanzaRepoList, const String *backupLabel, unsigned int rep
         varLstAdd(result, stanzaInfo);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(VARIANT_LIST, result);
 }
 
 /***********************************************************************************************************************************
@@ -821,6 +841,16 @@ formatTextBackup(const DbGroup *dbGroup, String *resultStr)
         else
             strCatZ(resultStr, "n/a\n");
 
+        const KeyValue *const lsnInfo = varKv(kvGet(backupInfo, BACKUP_KEY_LSN_VAR));
+
+        if (lsnInfo != NULL)
+        {
+            strCatFmt(
+                resultStr, "            lsn start/stop: %s / %s\n",
+                strZ(varStr(kvGet(lsnInfo, KEY_START_VAR))),
+                strZ(varStr(kvGet(lsnInfo, KEY_STOP_VAR))));
+        }
+
         KeyValue *info = varKv(kvGet(backupInfo, BACKUP_KEY_INFO_VAR));
 
         strCatFmt(
@@ -1165,13 +1195,16 @@ infoUpdateStanza(
                 // If a backup lock check has not already been performed, then do so
                 if (!stanzaRepo->backupLockChecked)
                 {
-                    // Try to acquire a lock. If not possible, assume another backup or expire is already running.
-                    stanzaRepo->backupLockHeld = !lockAcquire(
-                        cfgOptionStr(cfgOptLockPath), stanzaRepo->name, cfgOptionStr(cfgOptExecId), lockTypeBackup, 0, false);
-
-                    // Immediately release the lock acquired
-                    lockRelease(!stanzaRepo->backupLockHeld);
+                    // If there is a valid backup lock for this stanza then backup/expire must be running
+                    stanzaRepo->backupLockHeld = lockRead(
+                        cfgOptionStr(cfgOptLockPath), stanzaRepo->name, lockTypeBackup).status == lockReadStatusValid;
                     stanzaRepo->backupLockChecked = true;
+
+                    if (stanzaRepo->backupLockHeld)
+                    {
+                        stanzaRepo->percentComplete =
+                            lockRead(cfgOptionStr(cfgOptLockPath), stanzaRepo->name, lockTypeBackup).data.percentComplete;
+                    }
                 }
             }
 
@@ -1407,7 +1440,7 @@ infoRender(void)
         if (cfgOptionStrId(cfgOptOutput) == CFGOPTVAL_OUTPUT_TEXT)
         {
             // Process any stanza directories
-            if  (!varLstEmpty(infoList))
+            if (!varLstEmpty(infoList))
             {
                 for (unsigned int stanzaIdx = 0; stanzaIdx < varLstSize(infoList); stanzaIdx++)
                 {
@@ -1416,7 +1449,7 @@ infoRender(void)
 
                     // Add a carriage return between stanzas
                     if (stanzaIdx > 0)
-                        strCatFmt(resultStr, "\n");
+                        strCatZ(resultStr, "\n");
 
                     // Stanza name and status
                     strCatFmt(resultStr, "stanza: %s\n    status: ", strZ(stanzaName));
@@ -1429,6 +1462,10 @@ infoRender(void)
                     KeyValue *lockKv = varKv(kvGet(stanzaStatus, STATUS_KEY_LOCK_VAR));
                     KeyValue *backupLockKv = varKv(kvGet(lockKv, STATUS_KEY_LOCK_BACKUP_VAR));
                     bool backupLockHeld = varBool(kvGet(backupLockKv, STATUS_KEY_LOCK_BACKUP_HELD_VAR));
+                    const Variant *const percentComplete = kvGet(backupLockKv, STATUS_KEY_LOCK_BACKUP_PERCENT_COMPLETE_VAR);
+                    const String *const percentCompleteStr = percentComplete != NULL ?
+                        strNewFmt(" - %u.%02u%% complete", varUInt(percentComplete) / 100, varUInt(percentComplete) % 100) :
+                        EMPTY_STR;
 
                     if (statusCode != INFO_STANZA_STATUS_CODE_OK)
                     {
@@ -1439,10 +1476,13 @@ infoRender(void)
                             // Stanza status
                             strCatFmt(
                                 resultStr, "%s%s\n",
-                                statusCode == INFO_STANZA_STATUS_CODE_MIXED ? INFO_STANZA_MIXED :
-                                    strZ(strNewFmt(INFO_STANZA_STATUS_ERROR " (%s)",
-                                    strZ(varStr(kvGet(stanzaStatus, STATUS_KEY_MESSAGE_VAR))))),
-                                backupLockHeld == true ? " (" INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP ")" : "");
+                                statusCode == INFO_STANZA_STATUS_CODE_MIXED ?
+                                    INFO_STANZA_MIXED :
+                                    zNewFmt(
+                                        INFO_STANZA_STATUS_ERROR " (%s)",
+                                        strZ(varStr(kvGet(stanzaStatus, STATUS_KEY_MESSAGE_VAR)))),
+                                backupLockHeld == true ?
+                                    zNewFmt(" (" INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP "%s)", strZ(percentCompleteStr)) : "");
 
                             // Output the status per repo
                             VariantList *repoSection = kvGetList(stanzaInfo, STANZA_KEY_REPO_VAR);
@@ -1470,7 +1510,7 @@ infoRender(void)
                                         strCatFmt(
                                             resultStr, "%s%s%s\n",
                                             multiRepo ? INFO_STANZA_STATUS_ERROR " (" INFO_STANZA_STATUS_MESSAGE_OTHER ")\n" : "",
-                                            formatSpacer, strZ(strLstJoin(repoError, strZ(strNewFmt("\n%s", formatSpacer)))));
+                                            formatSpacer, strZ(strLstJoin(repoError, zNewFmt("\n%s", formatSpacer))));
                                     }
                                     else
                                     {
@@ -1487,14 +1527,19 @@ infoRender(void)
                             strCatFmt(
                                 resultStr, "%s (%s%s\n", INFO_STANZA_STATUS_ERROR,
                                 strZ(varStr(kvGet(stanzaStatus, STATUS_KEY_MESSAGE_VAR))),
-                                backupLockHeld == true ? ", " INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP ")" : ")");
+                                backupLockHeld == true ?
+                                    zNewFmt(", " INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP "%s)", strZ(percentCompleteStr)) : ")");
                         }
                     }
                     else
                     {
                         // Change displayed status if backup lock is found
                         if (backupLockHeld)
-                            strCatFmt(resultStr, "%s (%s)\n", INFO_STANZA_STATUS_OK, INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP);
+                        {
+                            strCatFmt(
+                                resultStr, "%s (%s%s)\n", INFO_STANZA_STATUS_OK, INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP,
+                                strZ(percentCompleteStr));
+                        }
                         else
                             strCatFmt(resultStr, "%s\n", INFO_STANZA_STATUS_OK);
                     }
