@@ -24,18 +24,25 @@ cmdRepoCreate(void)
         switch (storageType(storageRepo()))
         {
             case STORAGE_AZURE_TYPE:
-            storageAzureRequestP(
-                (StorageAzure *)storageDriver(storageRepoWrite()), HTTP_VERB_PUT_STR,
-                .query = httpQueryAdd(httpQueryNewP(), AZURE_QUERY_RESTYPE_STR, AZURE_QUERY_VALUE_CONTAINER_STR));
+            {
+                storageAzureRequestP(
+                    (StorageAzure *)storageDriver(storageRepoWrite()), HTTP_VERB_PUT_STR,
+                    .query = httpQueryAdd(httpQueryNewP(), AZURE_QUERY_RESTYPE_STR, AZURE_QUERY_VALUE_CONTAINER_STR));
+
                 break;
+            }
 
             case STORAGE_GCS_TYPE:
-        {
-                const KeyValue *const kvContent = kvPut(kvNew(), GCS_JSON_NAME_VAR, VARSTR(cfgOptionStr(cfgOptRepoGcsBucket)));
-
-            storageGcsRequestP(
-                (StorageGcs *)storageDriver(storageRepoWrite()), HTTP_VERB_POST_STR, .noBucket = true,
-                .content = BUFSTR(jsonFromKv(kvContent)));
+            {
+                storageGcsRequestP(
+                    (StorageGcs *)storageDriver(storageRepoWrite()), HTTP_VERB_POST_STR, .noBucket = true,
+                    .content = BUFSTR(
+                        jsonWriteResult(
+                            jsonWriteObjectEnd(
+                                jsonWriteStr(
+                                    jsonWriteKeyZ(
+                                        jsonWriteObjectBegin(
+                                            jsonWriteNewP()), GCS_JSON_NAME), cfgOptionStr(cfgOptRepoGcsBucket))))));
 
                 break;
             }
