@@ -17,10 +17,14 @@ Verify File
 /**********************************************************************************************************************************/
 VerifyResult
 verifyFile(
-    const String *filePathName, const String *fileChecksum, uint64_t fileSize, const String *cipherPass)
+    const String *const filePathName, const uint64_t offset, const Variant *const limit, const CompressType compressType,
+    const String *const fileChecksum, const uint64_t fileSize, const String *const cipherPass)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING, filePathName);                   // Fully qualified file name
+        FUNCTION_LOG_PARAM(UINT64, offset);                         // Offset to read in file
+        FUNCTION_LOG_PARAM(VARIANT, limit);                         // Limit to read from file
+        FUNCTION_LOG_PARAM(ENUM, compressType);                     // Compression type
         FUNCTION_LOG_PARAM(STRING, fileChecksum);                   // Checksum for the file
         FUNCTION_LOG_PARAM(UINT64, fileSize);                       // Size of file
         FUNCTION_TEST_PARAM(STRING, cipherPass);                    // Password to access the repo file if encrypted
@@ -28,6 +32,7 @@ verifyFile(
 
     ASSERT(filePathName != NULL);
     ASSERT(fileChecksum != NULL);
+    ASSERT(limit == NULL || varType(limit) == varTypeUInt64);
 
     // Is the file valid?
     VerifyResult result = verifyOk;
@@ -35,7 +40,8 @@ verifyFile(
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Prepare the file for reading
-        IoRead *read = storageReadIo(storageNewReadP(storageRepo(), filePathName, .ignoreMissing = true));
+        IoRead *read = storageReadIo(
+            storageNewReadP(storageRepo(), filePathName, .ignoreMissing = true, .offset = offset, .limit = limit));
         IoFilterGroup *filterGroup = ioReadFilterGroup(read);
 
         // Add decryption filter
@@ -43,11 +49,11 @@ verifyFile(
             ioFilterGroupAdd(filterGroup, cipherBlockNew(cipherModeDecrypt, cipherTypeAes256Cbc, BUFSTR(cipherPass), NULL));
 
         // Add decompression filter
-        if (compressTypeFromName(filePathName) != compressTypeNone)
-            ioFilterGroupAdd(filterGroup, decompressFilter(compressTypeFromName(filePathName)));
+        if (compressType != compressTypeNone)
+            ioFilterGroupAdd(filterGroup, decompressFilter(compressType));
 
         // Add sha1 filter
-        ioFilterGroupAdd(filterGroup, cryptoHashNew(HASH_TYPE_SHA1_STR));
+        ioFilterGroupAdd(filterGroup, cryptoHashNew(hashTypeSha1));
 
         // Add size filter
         ioFilterGroupAdd(filterGroup, ioSizeNew());
