@@ -28,11 +28,22 @@ verifyFileProtocol(PackRead *const param, ProtocolServer *const server)
     {
         // Verify file
         const String *const filePathName = pckReadStrP(param);
+
+        uint64_t offset = 0;
+        const Variant *limit = NULL;
+
+        if (pckReadBoolP(param))
+        {
+            offset = pckReadU64P(param);
+            limit = varNewUInt64(pckReadU64P(param));
+        }
+
+        const CompressType compressType = (CompressType)pckReadU32P(param);
         const String *const fileChecksum = pckReadStrP(param);
         const uint64_t fileSize = pckReadU64P(param);
         const String *const cipherPass = pckReadStrP(param);
 
-        const VerifyResult result = verifyFile(filePathName, fileChecksum, fileSize, cipherPass);
+        const VerifyResult result = verifyFile(filePathName, offset, limit, compressType, fileChecksum, fileSize, cipherPass);
 
         // Return result
         protocolServerDataPut(server, pckWriteU32P(protocolPackNew(), result));
