@@ -135,7 +135,7 @@ bz2CompressDone(const THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->done);
+    FUNCTION_TEST_RETURN(BOOL, this->done);
 }
 
 /***********************************************************************************************************************************
@@ -152,7 +152,7 @@ bz2CompressInputSame(const THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->inputSame);
+    FUNCTION_TEST_RETURN(BOOL, this->inputSame);
 }
 
 /**********************************************************************************************************************************/
@@ -167,7 +167,7 @@ bz2CompressNew(int level)
 
     IoFilter *this = NULL;
 
-    OBJ_NEW_BEGIN(Bz2Compress)
+    OBJ_NEW_BEGIN(Bz2Compress, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
         Bz2Compress *driver = OBJ_NEW_ALLOC();
 
