@@ -11,8 +11,7 @@ Compress IO using the gz format.
 /***********************************************************************************************************************************
 Filter type constant
 ***********************************************************************************************************************************/
-#define GZ_COMPRESS_FILTER_TYPE                                     "gzCompress"
-    STRING_DECLARE(GZ_COMPRESS_FILTER_TYPE_STR);
+#define GZ_COMPRESS_FILTER_TYPE                                     STRID5("gz-cmp", 0x20d1ef470)
 
 /***********************************************************************************************************************************
 Constructors
