@@ -122,7 +122,7 @@ gzDecompressDone(const THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->done);
+    FUNCTION_TEST_RETURN(BOOL, this->done);
 }
 
 /***********************************************************************************************************************************
@@ -139,7 +139,7 @@ gzDecompressInputSame(const THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->inputSame);
+    FUNCTION_TEST_RETURN(BOOL, this->inputSame);
 }
 
 /**********************************************************************************************************************************/
@@ -150,7 +150,7 @@ gzDecompressNew(void)
 
     IoFilter *this = NULL;
 
-    OBJ_NEW_BEGIN(GzDecompress)
+    OBJ_NEW_BEGIN(GzDecompress, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
         // Allocate state and set context
         GzDecompress *driver = OBJ_NEW_ALLOC();
