@@ -101,9 +101,6 @@ static const struct CompressHelperLocal
     },
 };
 
-#define COMPRESS_LIST_SIZE                                                                                                         \
-    (sizeof(compressHelperLocal) / sizeof(struct CompressHelperLocal))
-
 /**********************************************************************************************************************************/
 CompressType
 compressTypeEnum(const StringId type)
@@ -116,16 +113,16 @@ compressTypeEnum(const StringId type)
 
     CompressType result = compressTypeNone;
 
-    for (; result < COMPRESS_LIST_SIZE; result++)
+    for (; result < LENGTH_OF(compressHelperLocal); result++)
     {
         if (type == compressHelperLocal[result].typeId)
             break;
     }
 
-    if (result == COMPRESS_LIST_SIZE)
+    if (result == LENGTH_OF(compressHelperLocal))
         THROW_FMT(AssertError, "invalid compression type '%s'", strZ(strIdToStr(type)));
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(ENUM, result);
 }
 
 /**********************************************************************************************************************************/
@@ -136,7 +133,7 @@ compressTypePresent(CompressType type)
         FUNCTION_TEST_PARAM(ENUM, type);
     FUNCTION_TEST_END();
 
-    ASSERT(type < COMPRESS_LIST_SIZE);
+    ASSERT(type < LENGTH_OF(compressHelperLocal));
 
     if (type != compressTypeNone && compressHelperLocal[type].compressNew == NULL)
         THROW_FMT(OptionInvalidValueError, PROJECT_NAME " not compiled with %s support", strZ(compressHelperLocal[type].type));
@@ -152,9 +149,9 @@ compressTypeStr(CompressType type)
         FUNCTION_TEST_PARAM(ENUM, type);
     FUNCTION_TEST_END();
 
-    ASSERT(type < COMPRESS_LIST_SIZE);
+    ASSERT(type < LENGTH_OF(compressHelperLocal));
 
-    FUNCTION_TEST_RETURN(compressHelperLocal[type].type);
+    FUNCTION_TEST_RETURN_CONST(STRING, compressHelperLocal[type].type);
 }
 
 /**********************************************************************************************************************************/
@@ -167,16 +164,16 @@ compressTypeFromName(const String *name)
 
     CompressType result = compressTypeNone + 1;
 
-    for (; result < COMPRESS_LIST_SIZE; result++)
+    for (; result < LENGTH_OF(compressHelperLocal); result++)
     {
         if (strEndsWith(name, compressHelperLocal[result].ext))
             break;
     }
 
-    if (result == COMPRESS_LIST_SIZE)
+    if (result == LENGTH_OF(compressHelperLocal))
         result = compressTypeNone;
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(ENUM, result);
 }
 
 /**********************************************************************************************************************************/
@@ -187,10 +184,10 @@ compressLevelDefault(CompressType type)
         FUNCTION_TEST_PARAM(ENUM, type);
     FUNCTION_TEST_END();
 
-    ASSERT(type < COMPRESS_LIST_SIZE);
+    ASSERT(type < LENGTH_OF(compressHelperLocal));
     compressTypePresent(type);
 
-    FUNCTION_TEST_RETURN(compressHelperLocal[type].levelDefault);
+    FUNCTION_TEST_RETURN(INT, compressHelperLocal[type].levelDefault);
 }
 
 /**********************************************************************************************************************************/
@@ -202,11 +199,11 @@ compressFilter(CompressType type, int level)
         FUNCTION_TEST_PARAM(INT, level);
     FUNCTION_TEST_END();
 
-    ASSERT(type < COMPRESS_LIST_SIZE);
+    ASSERT(type < LENGTH_OF(compressHelperLocal));
     ASSERT(type != compressTypeNone);
     compressTypePresent(type);
 
-    FUNCTION_TEST_RETURN(compressHelperLocal[type].compressNew(level));
+    FUNCTION_TEST_RETURN(IO_FILTER, compressHelperLocal[type].compressNew(level));
 }
 
 /**********************************************************************************************************************************/
@@ -224,7 +221,7 @@ compressFilterPack(const StringId filterType, const Pack *const filterParam)
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        for (CompressType compressIdx = compressTypeNone + 1; compressIdx < COMPRESS_LIST_SIZE; compressIdx++)
+        for (CompressType compressIdx = compressTypeNone + 1; compressIdx < LENGTH_OF(compressHelperLocal); compressIdx++)
         {
             const struct CompressHelperLocal *compress = &compressHelperLocal[compressIdx];
 
@@ -255,11 +252,11 @@ decompressFilter(CompressType type)
         FUNCTION_TEST_PARAM(ENUM, type);
     FUNCTION_TEST_END();
 
-    ASSERT(type < COMPRESS_LIST_SIZE);
+    ASSERT(type < LENGTH_OF(compressHelperLocal));
     ASSERT(type != compressTypeNone);
     compressTypePresent(type);
 
-    FUNCTION_TEST_RETURN(compressHelperLocal[type].decompressNew());
+    FUNCTION_TEST_RETURN(IO_FILTER, compressHelperLocal[type].decompressNew());
 }
 
 /**********************************************************************************************************************************/
@@ -270,9 +267,9 @@ compressExtStr(CompressType type)
         FUNCTION_TEST_PARAM(ENUM, type);
     FUNCTION_TEST_END();
 
-    ASSERT(type < COMPRESS_LIST_SIZE);
+    ASSERT(type < LENGTH_OF(compressHelperLocal));
 
-    FUNCTION_TEST_RETURN(compressHelperLocal[type].ext);
+    FUNCTION_TEST_RETURN_CONST(STRING, compressHelperLocal[type].ext);
 }
 
 /**********************************************************************************************************************************/
@@ -305,5 +302,5 @@ compressExtStrip(const String *file, CompressType type)
     if (!strEndsWith(file, compressExtStr(type)))
         THROW_FMT(FormatError, "'%s' must have '%s' extension", strZ(file), strZ(compressExtStr(type)));
 
-    FUNCTION_TEST_RETURN(strSubN(file, 0, strSize(file) - strSize(compressExtStr(type))));
+    FUNCTION_TEST_RETURN(STRING, strSubN(file, 0, strSize(file) - strSize(compressExtStr(type))));
 }
