@@ -21,7 +21,7 @@ lz4Error(LZ4F_errorCode_t error)
     if (LZ4F_isError(error))
         THROW_FMT(FormatError, "lz4 error: [%zd] %s", (ssize_t)error, LZ4F_getErrorName(error));
 
-    FUNCTION_TEST_RETURN(error);
+    FUNCTION_TEST_RETURN_TYPE(LZ4F_errorCode_t, error);
 }
 
 #endif // HAVE_LIBLZ4
