@@ -142,7 +142,7 @@ zstCompressDone(const THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->flushing && !this->inputSame);
+    FUNCTION_TEST_RETURN(BOOL, this->flushing && !this->inputSame);
 }
 
 /***********************************************************************************************************************************
@@ -159,7 +159,7 @@ zstCompressInputSame(const THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->inputSame);
+    FUNCTION_TEST_RETURN(BOOL, this->inputSame);
 }
 
 /**********************************************************************************************************************************/
@@ -174,7 +174,7 @@ zstCompressNew(int level)
 
     IoFilter *this = NULL;
 
-    OBJ_NEW_BEGIN(ZstCompress)
+    OBJ_NEW_BEGIN(ZstCompress, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
         ZstCompress *driver = OBJ_NEW_ALLOC();
 
