@@ -215,10 +215,10 @@ size_t typeToLog(const char *typeName, char *buffer, size_t bufferSize);
 /***********************************************************************************************************************************
 Macros to return function results (or void)
 ***********************************************************************************************************************************/
-#define FUNCTION_LOG_RETURN_BASE(typePre, typeMacroPrefix, typePost, result)                                                       \
+#define FUNCTION_LOG_RETURN_BASE(typePre, typeMacroPrefix, typePost, ...)                                                          \
     do                                                                                                                             \
     {                                                                                                                              \
-        typePre FUNCTION_LOG_##typeMacroPrefix##_TYPE typePost FUNCTION_LOG_RETURN_result = result;                                \
+        typePre FUNCTION_LOG_##typeMacroPrefix##_TYPE typePost FUNCTION_LOG_RETURN_result = __VA_ARGS__;                           \
                                                                                                                                    \
         STACK_TRACE_POP(false);                                                                                                    \
                                                                                                                                    \
@@ -234,25 +234,25 @@ Macros to return function results (or void)
     }                                                                                                                              \
     while (0)
 
-#define FUNCTION_LOG_RETURN(typeMacroPrefix, result)                                                                               \
-    FUNCTION_LOG_RETURN_BASE(, typeMacroPrefix, , result)
+#define FUNCTION_LOG_RETURN(typeMacroPrefix, ...)                                                                                  \
+    FUNCTION_LOG_RETURN_BASE(, typeMacroPrefix, , __VA_ARGS__)
 
-#define FUNCTION_LOG_RETURN_P(typeMacroPrefix, result)                                                                             \
-    FUNCTION_LOG_RETURN_BASE(, typeMacroPrefix, *, result)
+#define FUNCTION_LOG_RETURN_P(typeMacroPrefix, ...)                                                                                \
+    FUNCTION_LOG_RETURN_BASE(, typeMacroPrefix, *, __VA_ARGS__)
 
-#define FUNCTION_LOG_RETURN_PP(typeMacroPrefix, result)                                                                            \
-    FUNCTION_LOG_RETURN_BASE(, typeMacroPrefix, **, result)
+#define FUNCTION_LOG_RETURN_PP(typeMacroPrefix, ...)                                                                               \
+    FUNCTION_LOG_RETURN_BASE(, typeMacroPrefix, **, __VA_ARGS__)
 
-#define FUNCTION_LOG_RETURN_CONST(typeMacroPrefix, result)                                                                         \
-    FUNCTION_LOG_RETURN_BASE(const, typeMacroPrefix, , result)
+#define FUNCTION_LOG_RETURN_CONST(typeMacroPrefix, ...)                                                                            \
+    FUNCTION_LOG_RETURN_BASE(const, typeMacroPrefix, , __VA_ARGS__)
 
-#define FUNCTION_LOG_RETURN_CONST_P(typeMacroPrefix, result)                                                                       \
-    FUNCTION_LOG_RETURN_BASE(const, typeMacroPrefix, *, result)
+#define FUNCTION_LOG_RETURN_CONST_P(typeMacroPrefix, ...)                                                                          \
+    FUNCTION_LOG_RETURN_BASE(const, typeMacroPrefix, *, __VA_ARGS__)
 
-#define FUNCTION_LOG_RETURN_CONST_PP(typeMacroPrefix, result)                                                                      \
-    FUNCTION_LOG_RETURN_BASE(const, typeMacroPrefix, **, result)
+#define FUNCTION_LOG_RETURN_CONST_PP(typeMacroPrefix, ...)                                                                         \
+    FUNCTION_LOG_RETURN_BASE(const, typeMacroPrefix, **, __VA_ARGS__)
 
-#define FUNCTION_LOG_RETURN_STRUCT(result)                                                                                         \
+#define FUNCTION_LOG_RETURN_STRUCT(...)                                                                                            \
     do                                                                                                                             \
     {                                                                                                                              \
         STACK_TRACE_POP(false);                                                                                                    \
@@ -260,7 +260,7 @@ Macros to return function results (or void)
         IF_LOG_ANY(FUNCTION_LOG_LEVEL())                                                                                           \
             LOG_FMT(FUNCTION_LOG_LEVEL(), 0, "=> struct");                                                                         \
                                                                                                                                    \
-        return result;                                                                                                             \
+        return __VA_ARGS__;                                                                                                        \
     }                                                                                                                              \
     while (0)
 
@@ -287,6 +287,9 @@ Ignore DEBUG_TEST_TRACE_MACRO if DEBUG is not defined because the underlying fun
 #endif // DEBUG_TEST_TRACE
 #endif // DEBUG
 
+// Annotate functions that do not return so it is clear why there is no FUNCTION_TEST_RETURN*() macro
+#define FUNCTION_TEST_NO_RETURN()
+
 #ifdef DEBUG_TEST_TRACE_MACRO
     #define FUNCTION_TEST_BEGIN()                                                                                                  \
         /* Ensure that FUNCTION_LOG_BEGIN() and FUNCTION_TEST_BEGIN() are not both used in a single function by declaring the */   \
@@ -324,17 +327,49 @@ Ignore DEBUG_TEST_TRACE_MACRO if DEBUG is not defined because the underlying fun
         FUNCTION_TEST_BEGIN();                                                                                                     \
         FUNCTION_TEST_END();
 
-    #define FUNCTION_TEST_RETURN(result)                                                                                           \
+    #define FUNCTION_TEST_RETURN_TYPE_BASE(typePre, type, typePost, ...)                                                           \
         do                                                                                                                         \
         {                                                                                                                          \
             /* CHECK for presense of FUNCTION_TEST_BEGIN*() */                                                                     \
             (void)FUNCTION_TEST_BEGIN_exists;                                                                                      \
                                                                                                                                    \
+            typePre type typePost FUNCTION_TEST_result = __VA_ARGS__;                                                              \
+                                                                                                                                   \
             STACK_TRACE_POP(true);                                                                                                 \
-            return result;                                                                                                         \
+                                                                                                                                   \
+            return FUNCTION_TEST_result;                                                                                           \
         }                                                                                                                          \
         while (0)
 
+    #define FUNCTION_TEST_RETURN_TYPE_MACRO_BASE(typePre, typeMacroPrefix, typePost, ...)                                          \
+        FUNCTION_TEST_RETURN_TYPE_BASE(typePre, FUNCTION_LOG_##typeMacroPrefix##_TYPE, typePost, __VA_ARGS__)
+
+    #define FUNCTION_TEST_RETURN(typeMacroPrefix, ...)                                                                             \
+        FUNCTION_TEST_RETURN_TYPE_MACRO_BASE(, typeMacroPrefix, , __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_P(typeMacroPrefix, ...)                                                                           \
+        FUNCTION_TEST_RETURN_TYPE_MACRO_BASE(, typeMacroPrefix, *, __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_PP(typeMacroPrefix, ...)                                                                          \
+        FUNCTION_TEST_RETURN_TYPE_MACRO_BASE(, typeMacroPrefix, **, __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_CONST(typeMacroPrefix, ...)                                                                       \
+        FUNCTION_TEST_RETURN_TYPE_MACRO_BASE(const, typeMacroPrefix, , __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_CONST_P(typeMacroPrefix, ...)                                                                     \
+        FUNCTION_TEST_RETURN_TYPE_MACRO_BASE(const, typeMacroPrefix, *, __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_CONST_PP(typeMacroPrefix, ...)                                                                    \
+        FUNCTION_TEST_RETURN_TYPE_MACRO_BASE(const, typeMacroPrefix, **, __VA_ARGS__)
+
+    #define FUNCTION_TEST_RETURN_TYPE(type, ...)                                                                                   \
+        FUNCTION_TEST_RETURN_TYPE_BASE(, type, , __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_TYPE_P(type, ...)                                                                                 \
+        FUNCTION_TEST_RETURN_TYPE_BASE(, type, *, __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_TYPE_PP(type, ...)                                                                                \
+        FUNCTION_TEST_RETURN_TYPE_BASE(, type, **, __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_TYPE_CONST(type, ...)                                                                             \
+        FUNCTION_TEST_RETURN_TYPE_BASE(const, type, , __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_TYPE_CONST_P(type, ...)                                                                           \
+        FUNCTION_TEST_RETURN_TYPE_BASE(const, type, *, __VA_ARGS__)
+    #define FUNCTION_TEST_RETURN_TYPE_CONST_PP(type, ...)                                                                          \
+        FUNCTION_TEST_RETURN_TYPE_BASE(const, type, **, __VA_ARGS__)
+
     #define FUNCTION_TEST_RETURN_VOID()                                                                                            \
         do                                                                                                                         \
         {                                                                                                                          \
@@ -342,6 +377,7 @@ Ignore DEBUG_TEST_TRACE_MACRO if DEBUG is not defined because the underlying fun
             (void)FUNCTION_TEST_BEGIN_exists;                                                                                      \
                                                                                                                                    \
             STACK_TRACE_POP(true);                                                                                                 \
+            return;                                                                                                                \
         }                                                                                                                          \
         while (0)
 #else
@@ -351,9 +387,35 @@ Ignore DEBUG_TEST_TRACE_MACRO if DEBUG is not defined because the underlying fun
     #define FUNCTION_TEST_PARAM_PP(typeMacroPrefix, param)
     #define FUNCTION_TEST_END()
     #define FUNCTION_TEST_VOID()
-    #define FUNCTION_TEST_RETURN(result)                                                                                           \
-        return result
-    #define FUNCTION_TEST_RETURN_VOID()
+
+    #define FUNCTION_TEST_RETURN(typeMacroPrefix, ...)                                                                             \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_P(typeMacroPrefix, ...)                                                                           \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_PP(typeMacroPrefix, ...)                                                                          \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_CONST(typeMacroPrefix, ...)                                                                       \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_CONST_P(typeMacroPrefix, ...)                                                                     \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_CONST_PP(typeMacroPrefix, ...)                                                                    \
+        return __VA_ARGS__
+
+    #define FUNCTION_TEST_RETURN_TYPE(type, ...)                                                                                   \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_TYPE_P(type, ...)                                                                                 \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_TYPE_PP(type, ...)                                                                                \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_TYPE_CONST(type, ...)                                                                             \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_TYPE_CONST_P(type, ...)                                                                           \
+        return __VA_ARGS__
+    #define FUNCTION_TEST_RETURN_TYPE_CONST_PP(type, ...)                                                                          \
+        return __VA_ARGS__
+
+    #define FUNCTION_TEST_RETURN_VOID()                                                                                            \
+        return
 #endif // DEBUG_TEST_TRACE_MACRO
 
 #endif
