@@ -10,6 +10,7 @@ Error Handler
 #include <string.h>
 
 #include "common/error.h"
+#include "common/macro.h"
 #include "common/stackTrace.h"
 
 /***********************************************************************************************************************************
@@ -18,21 +19,22 @@ Represents an error type
 struct ErrorType
 {
     const int code;
+    const bool fatal;
     const char *name;
     const struct ErrorType *parentType;
 };
 
 // Macro for defining new error types
-#define ERROR_DEFINE(code, name, parentType)                                                                                       \
-    const ErrorType name = {code, #name, &parentType}
+#define ERROR_DEFINE(code, name, fatal, parentType)                                                                                \
+    const ErrorType name = {code, fatal, #name, &parentType}
 
 // Define test error
 #ifdef DEBUG
-    ERROR_DEFINE(1, TestError, RuntimeError);
+    ERROR_DEFINE(1, TestError, false, RuntimeError);
 #endif
 
 // Include error type definitions
-#include "common/error.auto.c"
+#include "common/error.auto.c.inc"
 
 /***********************************************************************************************************************************
 Maximum allowed number of nested try blocks
@@ -112,6 +114,13 @@ errorTypeCode(const ErrorType *errorType)
     return errorType->code;
 }
 
+/**********************************************************************************************************************************/
+bool
+errorTypeFatal(const ErrorType *const errorType)
+{
+    return errorType->fatal;
+}
+
 /**********************************************************************************************************************************/
 const ErrorType *
 errorTypeFromCode(int code)
@@ -191,6 +200,13 @@ errorCode(void)
     return errorTypeCode(errorType());
 }
 
+/**********************************************************************************************************************************/
+bool
+errorFatal(void)
+{
+    return errorTypeFatal(errorType());
+}
+
 /**********************************************************************************************************************************/
 const char *
 errorFileName(void)
@@ -284,7 +300,7 @@ errorInternalJump(void)
 
 /**********************************************************************************************************************************/
 bool
-errorInternalCatch(const ErrorType *const errorTypeCatch)
+errorInternalCatch(const ErrorType *const errorTypeCatch, const bool fatalCatch)
 {
     // If just entering error state clean up the stack
     if (errorInternalState() == errorStateTry)
@@ -295,7 +311,7 @@ errorInternalCatch(const ErrorType *const errorTypeCatch)
         errorContext.tryList[errorContext.tryTotal].state++;
     }
 
-    if (errorInternalState() == errorStateCatch && errorInstanceOf(errorTypeCatch))
+    if (errorInternalState() == errorStateCatch && errorInstanceOf(errorTypeCatch) && (fatalCatch || !errorFatal()))
     {
         errorContext.tryList[errorContext.tryTotal].uncaught = false;
         errorContext.tryList[errorContext.tryTotal].state++;
@@ -332,11 +348,8 @@ void
 errorInternalTryEnd(void)
 {
     // Any catch blocks have been processed and none of them called RETHROW() so clear the error
-    if (errorContext.tryList[errorContext.tryTotal].state == errorStateEnd &&
-        !errorContext.tryList[errorContext.tryTotal].uncaught)
-    {
+    if (errorInternalState() == errorStateEnd && !errorContext.tryList[errorContext.tryTotal].uncaught)
         errorContext.error = (Error){0};
-    }
 
     // Remove the try
     errorContext.tryTotal--;
