@@ -67,6 +67,9 @@ Functions to get information about a generic error type
 // Error type code
 int errorTypeCode(const ErrorType *errorType);
 
+// Is the error type fatal
+bool errorTypeFatal(const ErrorType *errorType);
+
 // Get error type using a code
 const ErrorType *errorTypeFromCode(int code);
 
@@ -88,6 +91,9 @@ const ErrorType *errorType(void);
 // Error code (pulled from error type)
 int errorCode(void);
 
+// Is the error fatal?
+bool errorFatal(void);
+
 // Error filename
 const char *errorFileName(void);
 
@@ -126,6 +132,9 @@ Begin a block where errors can be thrown
 #define TRY_BEGIN()                                                                                                                \
     do                                                                                                                             \
     {                                                                                                                              \
+        volatile bool TRY_finally = false;                                                                                         \
+        (void)TRY_finally;                                          /* Will be unused if there is no finally block */              \
+                                                                                                                                   \
         errorInternalTryBegin(__FILE__, __func__, __LINE__);                                                                       \
                                                                                                                                    \
         if (setjmp(*errorInternalJump()) == 0)                                                                                     \
@@ -136,15 +145,23 @@ Catch a specific error thrown in the try block
 ***********************************************************************************************************************************/
 #define CATCH(errorTypeCatch)                                                                                                      \
         }                                                                                                                          \
-        else if (errorInternalCatch(&errorTypeCatch))                                                                              \
+        else if (errorInternalCatch(&errorTypeCatch, true))                                                                        \
+        {
+
+/***********************************************************************************************************************************
+Catch any non-fatal error thrown in the try block
+***********************************************************************************************************************************/
+#define CATCH_ANY()                                                                                                                \
+        }                                                                                                                          \
+        else if (errorInternalCatch(&RuntimeError, false))                                                                         \
         {
 
 /***********************************************************************************************************************************
 Catch any error thrown in the try block
 ***********************************************************************************************************************************/
-#define CATCH_ANY()                                                                                                                \
+#define CATCH_FATAL()                                                                                                              \
         }                                                                                                                          \
-        else if (errorInternalCatch(&RuntimeError))                                                                                \
+        else if (errorInternalCatch(&RuntimeError, true))                                                                          \
         {
 
 /***********************************************************************************************************************************
@@ -152,7 +169,10 @@ Code to run whether the try block was successful or not
 ***********************************************************************************************************************************/
 #define FINALLY()                                                                                                                  \
         }                                                                                                                          \
-        {
+                                                                                                                                   \
+        if (!TRY_finally)                                                                                                          \
+        {                                                                                                                          \
+            TRY_finally = true;
 
 /***********************************************************************************************************************************
 End the try block
@@ -291,31 +311,30 @@ void errorInternalTryBegin(const char *fileName, const char *functionName, int f
 jmp_buf *errorInternalJump(void);
 
 // True when in catch state and the expected error matches
-bool errorInternalCatch(const ErrorType *errorTypeCatch);
+bool errorInternalCatch(const ErrorType *errorTypeCatch, bool fatalCatch);
 
 // Propagate the error up so it can be caught
-void errorInternalPropagate(void) __attribute__((__noreturn__));
+FN_NO_RETURN void errorInternalPropagate(void);
 
 // End the try block
 void errorInternalTryEnd(void);
 
 // Throw an error
-void errorInternalThrow(
+FN_NO_RETURN void errorInternalThrow(
     const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *message,
-    const char *stackTrace) __attribute__((__noreturn__));
-void errorInternalThrowFmt(
+    const char *stackTrace);
+FN_NO_RETURN void errorInternalThrowFmt(
     const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *format, ...)
-    __attribute__((format(printf, 5, 6))) __attribute__((__noreturn__));
+    __attribute__((format(printf, 5, 6)));
 
 // Throw a system error
-void errorInternalThrowSys(
-    int errNo, const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *message)
-    __attribute__((__noreturn__));
+FN_NO_RETURN void errorInternalThrowSys(
+    int errNo, const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *message);
 
 // Throw a formatted system error
-void errorInternalThrowSysFmt(
+FN_NO_RETURN void errorInternalThrowSysFmt(
     int errNo, const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *format, ...)
-    __attribute__((format(printf, 6, 7))) __attribute__((__noreturn__));
+    __attribute__((format(printf, 6, 7)));
 
 // Versions of the above for coverage testing which checks the error condition inside the function
 #ifdef DEBUG_COVERAGE
