@@ -14,14 +14,14 @@ bool fdReady(int fd, bool read, bool write, TimeMSec timeout);
 
 // Wait until the file descriptor is ready to read or timeout
 __attribute__((always_inline)) static inline bool
-fdReadyRead(int fd, TimeMSec timeout)
+fdReadyRead(const int fd, const TimeMSec timeout)
 {
     return fdReady(fd, true, false, timeout);
 }
 
 // Wait until the file descriptor is ready to write or timeout
 __attribute__((always_inline)) static inline bool
-fdReadyWrite(int fd, TimeMSec timeout)
+fdReadyWrite(const int fd, const TimeMSec timeout)
 {
     return fdReady(fd, false, true, timeout);
 }
