@@ -43,7 +43,7 @@ httpClientNew(IoClient *ioClient, TimeMSec timeout)
 
     HttpClient *this = NULL;
 
-    OBJ_NEW_BEGIN(HttpClient)
+    OBJ_NEW_BEGIN(HttpClient, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         this = OBJ_NEW_ALLOC();
 
