@@ -49,6 +49,10 @@ HTTP Constants
     STRING_DECLARE(HTTP_HEADER_CONTENT_TYPE_STR);
 #define HTTP_HEADER_CONTENT_TYPE_APP_FORM_URL                       "application/x-www-form-urlencoded"
     STRING_DECLARE(HTTP_HEADER_CONTENT_TYPE_APP_FORM_URL_STR);
+#define HTTP_HEADER_CONTENT_TYPE_JSON                               "application/json"
+    STRING_DECLARE(HTTP_HEADER_CONTENT_TYPE_JSON_STR);
+#define HTTP_HEADER_CONTENT_TYPE_XML                                "application/xml"
+    STRING_DECLARE(HTTP_HEADER_CONTENT_TYPE_XML_STR);
 #define HTTP_HEADER_CONTENT_RANGE_BYTES                             "bytes"
 #define HTTP_HEADER_DATE                                            "date"
     STRING_DECLARE(HTTP_HEADER_DATE_STR);
@@ -58,6 +62,9 @@ HTTP Constants
     STRING_DECLARE(HTTP_HEADER_HOST_STR);
 #define HTTP_HEADER_LAST_MODIFIED                                   "last-modified"
     STRING_DECLARE(HTTP_HEADER_LAST_MODIFIED_STR);
+#define HTTP_HEADER_RANGE                                           "range"
+    STRING_DECLARE(HTTP_HEADER_RANGE_STR);
+#define HTTP_HEADER_RANGE_BYTES                                     "bytes"
 
 /***********************************************************************************************************************************
 Constructors
@@ -121,7 +128,7 @@ Functions
 HttpResponse *httpRequestResponse(HttpRequest *this, bool contentCache);
 
 // Throw an error if the request failed
-void httpRequestError(const HttpRequest *this, HttpResponse *response) __attribute__((__noreturn__));
+FN_NO_RETURN void httpRequestError(const HttpRequest *this, HttpResponse *response);
 
 // Move to a new parent mem context
 __attribute__((always_inline)) static inline HttpRequest *
