@@ -62,7 +62,7 @@ httpUrlNewParse(const String *const url, HttpUrlNewParseParam param)
 
     HttpUrl *this = NULL;
 
-    OBJ_NEW_BEGIN(HttpUrl)
+    OBJ_NEW_BEGIN(HttpUrl, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         // Allocate state and set context
         this = OBJ_NEW_ALLOC();
@@ -191,7 +191,7 @@ httpUrlNewParse(const String *const url, HttpUrlNewParseParam param)
     }
     OBJ_NEW_END();
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(HTTP_URL, this);
 }
 
 /**********************************************************************************************************************************/
