@@ -35,7 +35,7 @@ ioReadNew(void *driver, IoReadInterface interface)
 
     IoRead *this = NULL;
 
-    OBJ_NEW_BEGIN(IoRead)
+    OBJ_NEW_BEGIN(IoRead, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         this = OBJ_NEW_ALLOC();
 
@@ -264,7 +264,7 @@ ioReadSmall(IoRead *this, Buffer *buffer)
     }
     while (!bufFull(buffer) && !ioReadEof(this));
 
-    FUNCTION_TEST_RETURN(outputRemains - bufRemains(buffer));
+    FUNCTION_TEST_RETURN(SIZE, outputRemains - bufRemains(buffer));
 }
 
 /***********************************************************************************************************************************
