@@ -25,6 +25,7 @@ typedef struct TlsSession
     SSL *session;                                                   // TLS session on the file descriptor
     TimeMSec timeout;                                               // Timeout for any i/o operation (read, write, etc.)
     bool shutdownOnClose;                                           // Shutdown the TLS connection when closing the session
+    bool ignoreUnexpectedEof;                                       // Ignore unexpected EOF when requested
 
     IoRead *read;                                                   // Read interface
     IoWrite *write;                                                 // Write interface
@@ -39,8 +40,7 @@ tlsSessionToLog(const THIS_VOID)
     THIS(const TlsSession);
 
     return strNewFmt(
-        "{ioSession: %s, timeout: %" PRIu64", shutdownOnClose: %s}",
-        objMemContextFreeing(this) ? NULL_Z : strZ(ioSessionToLog(this->ioSession)), this->timeout,
+        "{ioSession: %s, timeout: %" PRIu64", shutdownOnClose: %s}", strZ(ioSessionToLog(this->ioSession)), this->timeout,
         cvtBoolToConstZ(this->shutdownOnClose));
 }
 
@@ -126,20 +126,9 @@ tlsSessionResultProcess(TlsSession *this, int errorTls, long unsigned int errorT
 
     switch (errorTls)
     {
-        // The connection was closed
-        case SSL_ERROR_ZERO_RETURN:
-        {
-            if (!closeOk)
-                THROW(ProtocolError, "unexpected TLS eof");
-
-            this->shutdownOnClose = false;
-            tlsSessionClose(this);
-            break;
-        }
-
         // Try again after waiting for read ready
         case SSL_ERROR_WANT_READ:
-            ioReadReadyP(ioSessionIoRead(this->ioSession), .error = true);
+            ioReadReadyP(ioSessionIoReadP(this->ioSession), .error = true);
             result = 0;
             break;
 
@@ -149,19 +138,30 @@ tlsSessionResultProcess(TlsSession *this, int errorTls, long unsigned int errorT
             result = 0;
             break;
 
-        // A syscall failed (this usually indicates unexpected eof)
-        case SSL_ERROR_SYSCALL:
-            THROW_SYS_ERROR_CODE(errorSys, KernelError, "TLS syscall error");
-
-        // Any other error that we cannot handle
+        // Handle graceful termination by the server or unexpected EOF/error
         default:
         {
-            // Get detailed error message when available
-            const char *errorTlsDetailMessage = ERR_reason_error_string(errorTlsDetail);
+            // Close connection on graceful termination by the server or unexpected EOF/error when allowed
+            if (errorTls == SSL_ERROR_ZERO_RETURN || this->ignoreUnexpectedEof)
+            {
+                if (!closeOk)
+                    THROW(ProtocolError, "unexpected TLS eof");
 
-            THROW_FMT(
-                ServiceError, "TLS error [%d:%lu] %s", errorTls, errorTlsDetail,
-                errorTlsDetailMessage == NULL ? "no details available" : errorTlsDetailMessage);
+                this->shutdownOnClose = false;
+                tlsSessionClose(this);
+            }
+            // Else error
+            else
+            {
+                // Get detailed error message when available
+                const char *errorTlsDetailMessage = ERR_reason_error_string(errorTlsDetail);
+
+                THROW_FMT(
+                    ServiceError, "TLS error [%d:%lu] %s", errorTls, errorTlsDetail,
+                    errorTlsDetailMessage == NULL ? "no details available" : errorTlsDetailMessage);
+            }
+
+            break;
         }
     }
 
@@ -220,7 +220,7 @@ tlsSessionRead(THIS_VOID, Buffer *buffer, bool block)
     {
         // If no TLS data pending then check the io to reduce blocking
         if (!SSL_pending(this->session))
-            ioReadReadyP(ioSessionIoRead(this->ioSession), .error = true);
+            ioReadReadyP(ioSessionIoReadP(this->ioSession), .error = true);
 
         // Read and handle errors. The error queue must be cleared before this operation.
         ERR_clear_error();
@@ -292,17 +292,20 @@ tlsSessionEof(THIS_VOID)
 
 /**********************************************************************************************************************************/
 static IoRead *
-tlsSessionIoRead(THIS_VOID)
+tlsSessionIoRead(THIS_VOID, const bool ignoreUnexpectedEof)
 {
     THIS(TlsSession);
 
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(TLS_SESSION, this);
+        FUNCTION_TEST_PARAM(BOOL, ignoreUnexpectedEof);
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->read);
+    this->ignoreUnexpectedEof = ignoreUnexpectedEof;
+
+    FUNCTION_TEST_RETURN(IO_READ, this->read);
 }
 
 /**********************************************************************************************************************************/
@@ -317,7 +320,7 @@ tlsSessionIoWrite(THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->write);
+    FUNCTION_TEST_RETURN(IO_WRITE, this->write);
 }
 
 /**********************************************************************************************************************************/
@@ -332,7 +335,7 @@ tlsSessionRole(const THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(ioSessionRole(this->ioSession));
+    FUNCTION_TEST_RETURN(STRING_ID, ioSessionRole(this->ioSession));
 }
 
 /**********************************************************************************************************************************/
@@ -360,7 +363,7 @@ tlsSessionNew(SSL *session, IoSession *ioSession, TimeMSec timeout)
 
     IoSession *this = NULL;
 
-    OBJ_NEW_BEGIN(TlsSession)
+    OBJ_NEW_BEGIN(TlsSession, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
         TlsSession *driver = OBJ_NEW_ALLOC();
 
