@@ -74,7 +74,7 @@ regExpNew(const String *expression)
 
     RegExp *this = NULL;
 
-    OBJ_NEW_BEGIN(RegExp)
+    OBJ_NEW_BEGIN(RegExp, .childQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
         this = OBJ_NEW_ALLOC();
         *this = (RegExp){{0}};                                      // Extra braces are required for older gcc versions
@@ -90,7 +90,7 @@ regExpNew(const String *expression)
     }
     OBJ_NEW_END();
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(REGEXP, this);
 }
 
 /**********************************************************************************************************************************/
@@ -125,7 +125,7 @@ regExpMatch(RegExp *this, const String *string)
         this->matchSize = 0;
     }
 
-    FUNCTION_TEST_RETURN(result == 0);
+    FUNCTION_TEST_RETURN(BOOL, result == 0);
 }
 
 /***********************************************************************************************************************************
@@ -140,7 +140,7 @@ regExpMatchPtr(RegExp *this)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->matchPtr);
+    FUNCTION_TEST_RETURN_CONST(STRINGZ, this->matchPtr);
 }
 
 size_t
@@ -152,7 +152,7 @@ regExpMatchSize(RegExp *this)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->matchSize);
+    FUNCTION_TEST_RETURN(SIZE, this->matchSize);
 }
 
 String *
@@ -164,7 +164,7 @@ regExpMatchStr(RegExp *this)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->matchPtr == NULL ? NULL : strNewZN(regExpMatchPtr(this), regExpMatchSize(this)));
+    FUNCTION_TEST_RETURN(STRING, this->matchPtr == NULL ? NULL : strNewZN(regExpMatchPtr(this), regExpMatchSize(this)));
 }
 
 /**********************************************************************************************************************************/
@@ -179,20 +179,15 @@ regExpMatchOne(const String *expression, const String *string)
     ASSERT(expression != NULL);
     ASSERT(string != NULL);
 
-    bool result = false;
-    RegExp *regExp = regExpNew(expression);
+    bool result;
 
-    TRY_BEGIN()
+    MEM_CONTEXT_TEMP_BEGIN()
     {
-        result = regExpMatch(regExp, string);
+        result = regExpMatch(regExpNew(expression), string);
     }
-    FINALLY()
-    {
-        regExpFree(regExp);
-    }
-    TRY_END();
+    MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(BOOL, result);
 }
 
 /**********************************************************************************************************************************/
@@ -245,5 +240,5 @@ regExpPrefix(const String *expression)
         }
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
