@@ -6,13 +6,9 @@ Statistics Collector
 #include "common/debug.h"
 #include "common/memContext.h"
 #include "common/stat.h"
+#include "common/type/json.h"
 #include "common/type/list.h"
 
-/***********************************************************************************************************************************
-Stat output constants
-***********************************************************************************************************************************/
-VARIANT_STRDEF_EXTERN(STAT_VALUE_TOTAL_VAR,                         STAT_VALUE_TOTAL);
-
 /***********************************************************************************************************************************
 Cumulative statistics
 ***********************************************************************************************************************************/
@@ -25,7 +21,7 @@ typedef struct Stat
 /***********************************************************************************************************************************
 Local data
 ***********************************************************************************************************************************/
-struct
+static struct
 {
     MemContext *memContext;                                         // Mem context to store data in this struct
     List *stat;                                                     // Cumulative stats
@@ -41,7 +37,7 @@ statInit(void)
 
     MEM_CONTEXT_BEGIN(memContextTop())
     {
-        MEM_CONTEXT_NEW_BEGIN("StatLocalData")
+        MEM_CONTEXT_NEW_BEGIN(StatLocalData, .childQty = MEM_CONTEXT_QTY_MAX)
         {
             statLocalData.memContext = MEM_CONTEXT_NEW();
             statLocalData.stat = lstNewP(sizeof(Stat), .sortOrder = sortOrderAsc, .comparator = lstComparatorStr);
@@ -86,7 +82,7 @@ statGetOrCreate(const String *key)
         ASSERT(stat != NULL);
     }
 
-    FUNCTION_TEST_RETURN(stat);
+    FUNCTION_TEST_RETURN_TYPE_P(Stat, stat);
 }
 
 /**********************************************************************************************************************************/
@@ -102,26 +98,40 @@ statInc(const String *key)
 
     statGetOrCreate(key)->total++;
 
-    FUNCTION_TEST_RETURN();
+    FUNCTION_TEST_RETURN_VOID();
 }
 
 /**********************************************************************************************************************************/
-KeyValue *
-statToKv(void)
+String *
+statToJson(void)
 {
     FUNCTION_TEST_VOID();
 
     ASSERT(statLocalData.memContext != NULL);
 
-    KeyValue *result = kvNew();
+    String *result = NULL;
 
-    for (unsigned int statIdx = 0; statIdx < lstSize(statLocalData.stat); statIdx++)
+    if (!lstEmpty(statLocalData.stat))
     {
-        Stat *stat = lstGet(statLocalData.stat, statIdx);
+        result = strNew();
 
-        KeyValue *statKv = kvPutKv(result, VARSTR(stat->key));
-        kvPut(statKv, STAT_VALUE_TOTAL_VAR, VARUINT64(stat->total));
+        MEM_CONTEXT_TEMP_BEGIN()
+        {
+            JsonWrite *const json = jsonWriteObjectBegin(jsonWriteNewP(.json = result));
+
+            for (unsigned int statIdx = 0; statIdx < lstSize(statLocalData.stat); statIdx++)
+            {
+                const Stat *const stat = lstGet(statLocalData.stat, statIdx);
+
+                jsonWriteObjectBegin(jsonWriteKey(json, stat->key));
+                jsonWriteUInt64(jsonWriteKeyZ(json, "total"), stat->total);
+                jsonWriteObjectEnd(json);
+            }
+
+            jsonWriteObjectEnd(json);
+        }
+        MEM_CONTEXT_TEMP_END();
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
