@@ -12,16 +12,14 @@ Buffer Handler
 /***********************************************************************************************************************************
 Constant buffers that are generally useful
 ***********************************************************************************************************************************/
-BUFFER_STRDEF_EXTERN(BRACEL_BUF,                                    BRACEL_Z);
-BUFFER_STRDEF_EXTERN(BRACER_BUF,                                    BRACER_Z);
-BUFFER_STRDEF_EXTERN(BRACKETL_BUF,                                  BRACKETL_Z);
-BUFFER_STRDEF_EXTERN(BRACKETR_BUF,                                  BRACKETR_Z);
-BUFFER_STRDEF_EXTERN(COMMA_BUF,                                     COMMA_Z);
-BUFFER_STRDEF_EXTERN(CR_BUF,                                        CR_Z);
-BUFFER_STRDEF_EXTERN(DOT_BUF,                                       DOT_Z);
-BUFFER_STRDEF_EXTERN(EQ_BUF,                                        EQ_Z);
-BUFFER_STRDEF_EXTERN(LF_BUF,                                        LF_Z);
-BUFFER_STRDEF_EXTERN(QUOTED_BUF,                                    QUOTED_Z);
+BUFFER_STRDEF_EXTERN(BRACEL_BUF,                                    "{");
+BUFFER_STRDEF_EXTERN(BRACER_BUF,                                    "}");
+BUFFER_STRDEF_EXTERN(BRACKETL_BUF,                                  "[");
+BUFFER_STRDEF_EXTERN(BRACKETR_BUF,                                  "]");
+BUFFER_STRDEF_EXTERN(COMMA_BUF,                                     ",");
+BUFFER_STRDEF_EXTERN(EQ_BUF,                                        "=");
+BUFFER_STRDEF_EXTERN(LF_BUF,                                        "\n");
+BUFFER_STRDEF_EXTERN(QUOTED_BUF,                                    "\"");
 
 /***********************************************************************************************************************************
 Contains information about the buffer
@@ -41,7 +39,7 @@ bufNew(size_t size)
 
     Buffer *this = NULL;
 
-    OBJ_NEW_BEGIN(Buffer)
+    OBJ_NEW_BEGIN(Buffer, .allocQty = 1)
     {
         // Create object
         this = OBJ_NEW_ALLOC();
@@ -61,7 +59,7 @@ bufNew(size_t size)
     }
     OBJ_NEW_END();
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(BUFFER, this);
 }
 
 /**********************************************************************************************************************************/
@@ -80,7 +78,7 @@ bufNewC(const void *buffer, size_t size)
     memcpy(this->pub.buffer, buffer, bufSize(this));
     this->pub.used = bufSize(this);
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(BUFFER, this);
 }
 
 /**********************************************************************************************************************************/
@@ -97,7 +95,7 @@ bufNewDecode(EncodeType type, const String *string)
     decodeToBin(type, strZ(string), bufPtr(this));
     bufUsedSet(this, bufSize(this));
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(BUFFER, this);
 }
 
 /**********************************************************************************************************************************/
@@ -111,11 +109,14 @@ bufDup(const Buffer *buffer)
     ASSERT(buffer != NULL);
 
     // Create object and copy data
-    Buffer *this = bufNew(buffer->pub.used);
-    memcpy(this->pub.buffer, buffer->pub.buffer, bufSize(this));
+    Buffer *this = bufNew(bufUsed(buffer));
+
+    if (bufUsed(buffer) != 0)
+        memcpy(this->pub.buffer, buffer->pub.buffer, bufSize(this));
+
     this->pub.used = bufSize(this);
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(BUFFER, this);
 }
 
 /**********************************************************************************************************************************/
@@ -130,9 +131,9 @@ bufCat(Buffer *this, const Buffer *cat)
     ASSERT(this != NULL);
 
     if (cat != NULL)
-        bufCatC(this, cat->pub.buffer, 0, cat->pub.used);
+        bufCatC(this, cat->pub.buffer, 0, bufUsed(cat));
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(BUFFER, this);
 }
 
 /**********************************************************************************************************************************/
@@ -161,7 +162,7 @@ bufCatC(Buffer *this, const unsigned char *cat, size_t catOffset, size_t catSize
         this->pub.used += catSize;
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(BUFFER, this);
 }
 
 /**********************************************************************************************************************************/
@@ -179,13 +180,13 @@ bufCatSub(Buffer *this, const Buffer *cat, size_t catOffset, size_t catSize)
 
     if (cat != NULL)
     {
-        ASSERT(catOffset <= cat->pub.used);
-        ASSERT(catSize <= cat->pub.used - catOffset);
+        ASSERT(catOffset <= bufUsed(cat));
+        ASSERT(catSize <= bufUsed(cat) - catOffset);
 
         bufCatC(this, cat->pub.buffer, catOffset, catSize);
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(BUFFER, this);
 }
 
 /**********************************************************************************************************************************/
@@ -201,9 +202,9 @@ bufEq(const Buffer *this, const Buffer *compare)
     ASSERT(compare != NULL);
 
     if (bufUsed(this) == bufUsed(compare))
-        FUNCTION_TEST_RETURN(memcmp(bufPtrConst(this), bufPtrConst(compare), bufUsed(compare)) == 0);
+        FUNCTION_TEST_RETURN(BOOL, memcmp(bufPtrConst(this), bufPtrConst(compare), bufUsed(compare)) == 0);
 
-    FUNCTION_TEST_RETURN(false);
+    FUNCTION_TEST_RETURN(BOOL, false);
 }
 
 /**********************************************************************************************************************************/
@@ -221,7 +222,7 @@ bufHex(const Buffer *this)
     for (unsigned int bufferIdx = 0; bufferIdx < bufUsed(this); bufferIdx++)
         strCatFmt(result, "%02x", bufPtrConst(this)[bufferIdx]);
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 /**********************************************************************************************************************************/
@@ -244,11 +245,11 @@ bufResize(Buffer *this, size_t size)
             // When setting size down to 0 the buffer should always be allocated
             ASSERT(bufPtrConst(this) != NULL);
 
-            MEM_CONTEXT_BEGIN(objMemContext(this))
+            MEM_CONTEXT_OBJ_BEGIN(this)
             {
                 memFree(bufPtr(this));
             }
-            MEM_CONTEXT_END();
+            MEM_CONTEXT_OBJ_END();
 
             this->pub.buffer = NULL;
             this->pub.sizeAlloc = 0;
@@ -256,14 +257,14 @@ bufResize(Buffer *this, size_t size)
         // Else allocate or resize
         else
         {
-            MEM_CONTEXT_BEGIN(objMemContext(this))
+            MEM_CONTEXT_OBJ_BEGIN(this)
             {
                 if (bufPtrConst(this) == NULL)
                     this->pub.buffer = memNew(size);
                 else
                     this->pub.buffer = memResize(bufPtr(this), size);
             }
-            MEM_CONTEXT_END();
+            MEM_CONTEXT_OBJ_END();
 
             this->pub.sizeAlloc = size;
         }
@@ -277,7 +278,7 @@ bufResize(Buffer *this, size_t size)
             this->pub.size = bufSizeAlloc(this);
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(BUFFER, this);
 }
 
 /**********************************************************************************************************************************/
@@ -367,7 +368,7 @@ bufToLog(const Buffer *this)
 {
     String *result = strNewFmt(
         "{used: %zu, size: %zu%s", bufUsed(this), bufSize(this),
-        bufSizeLimit(this) ? strZ(strNewFmt(", sizeAlloc: %zu}", bufSizeAlloc(this))) : "}");
+        bufSizeLimit(this) ? zNewFmt(", sizeAlloc: %zu}", bufSizeAlloc(this)) : "}");
 
     return result;
 }
