@@ -201,8 +201,6 @@ BUFFER_DECLARE(BRACER_BUF);
 BUFFER_DECLARE(BRACKETL_BUF);
 BUFFER_DECLARE(BRACKETR_BUF);
 BUFFER_DECLARE(COMMA_BUF);
-BUFFER_DECLARE(CR_BUF);
-BUFFER_DECLARE(DOT_BUF);
 BUFFER_DECLARE(EQ_BUF);
 BUFFER_DECLARE(LF_BUF);
 BUFFER_DECLARE(QUOTED_BUF);
