@@ -4,7 +4,6 @@ List Handler
 #include "build.auto.h"
 
 #include <stdarg.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -36,7 +35,7 @@ lstNew(size_t itemSize, ListParam param)
 
     List *this = NULL;
 
-    OBJ_NEW_BEGIN(List)
+    OBJ_NEW_BEGIN(List, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
     {
         // Create object
         this = OBJ_NEW_ALLOC();
@@ -50,7 +49,7 @@ lstNew(size_t itemSize, ListParam param)
     }
     OBJ_NEW_END();
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(LIST, this);
 }
 
 /**********************************************************************************************************************************/
@@ -75,7 +74,7 @@ lstClear(List *this)
         this->listSizeMax = 0;
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(LIST, this);
 }
 
 /**********************************************************************************************************************************/
@@ -90,7 +89,7 @@ lstComparatorStr(const void *item1, const void *item2)
     ASSERT(item1 != NULL);
     ASSERT(item2 != NULL);
 
-    FUNCTION_TEST_RETURN(strCmp(*(String **)item1, *(String **)item2));
+    FUNCTION_TEST_RETURN(INT, strCmp(*(String **)item1, *(String **)item2));
 }
 
 /**********************************************************************************************************************************/
@@ -105,7 +104,7 @@ lstComparatorZ(const void *item1, const void *item2)
     ASSERT(item1 != NULL);
     ASSERT(item2 != NULL);
 
-    FUNCTION_TEST_RETURN(strcmp(*(char **)item1, *(char **)item2));
+    FUNCTION_TEST_RETURN(INT, strcmp(*(char **)item1, *(char **)item2));
 }
 
 /***********************************************************************************************************************************
@@ -135,7 +134,7 @@ lstGet(const List *this, unsigned int listIdx)
         THROW_FMT(AssertError, "cannot get index %u from list with %u value(s)", listIdx, lstSize(this));
 
     // Return pointer to list item
-    FUNCTION_TEST_RETURN(this->list + (listIdx * this->itemSize));
+    FUNCTION_TEST_RETURN_P(VOID, this->list + (listIdx * this->itemSize));
 }
 
 void *
@@ -152,7 +151,7 @@ lstGetLast(const List *this)
         THROW(AssertError, "cannot get last from list with no values");
 
     // Return pointer to list item
-    FUNCTION_TEST_RETURN(lstGet(this, lstSize(this) - 1));
+    FUNCTION_TEST_RETURN_P(VOID, lstGet(this, lstSize(this) - 1));
 }
 
 /**********************************************************************************************************************************/
@@ -168,24 +167,27 @@ lstFind(const List *this, const void *item)
     ASSERT(this->comparator != NULL);
     ASSERT(item != NULL);
 
-    if (this->sortOrder == sortOrderAsc)
-        FUNCTION_TEST_RETURN(bsearch(item, this->list, lstSize(this), this->itemSize, this->comparator));
-    else if (this->sortOrder == sortOrderDesc)
+    if (this->list != NULL)
     {
-        // Assign the list for the descending comparator to use
-        comparatorDescList = this;
+        if (this->sortOrder == sortOrderAsc)
+            FUNCTION_TEST_RETURN_P(VOID, bsearch(item, this->list, lstSize(this), this->itemSize, this->comparator));
+        else if (this->sortOrder == sortOrderDesc)
+        {
+            // Assign the list for the descending comparator to use
+            comparatorDescList = this;
 
-        FUNCTION_TEST_RETURN(bsearch(item, this->list, lstSize(this), this->itemSize, lstComparatorDesc));
+            FUNCTION_TEST_RETURN_P(VOID, bsearch(item, this->list, lstSize(this), this->itemSize, lstComparatorDesc));
+        }
+
+        // Fall back on an iterative search
+        for (unsigned int listIdx = 0; listIdx < lstSize(this); listIdx++)
+        {
+            if (this->comparator(item, lstGet(this, listIdx)) == 0)
+                FUNCTION_TEST_RETURN_P(VOID, lstGet(this, listIdx));
+        }
     }
 
-    // Fall back on an iterative search
-    for (unsigned int listIdx = 0; listIdx < lstSize(this); listIdx++)
-    {
-        if (this->comparator(item, lstGet(this, listIdx)) == 0)
-            FUNCTION_TEST_RETURN(lstGet(this, listIdx));
-    }
-
-    FUNCTION_TEST_RETURN(NULL);
+    FUNCTION_TEST_RETURN_P(VOID, NULL);
 }
 
 unsigned int
@@ -201,7 +203,7 @@ lstFindIdx(const List *this, const void *item)
 
     void *result = lstFind(this, item);
 
-    FUNCTION_TEST_RETURN(result == NULL ? LIST_NOT_FOUND : lstIdx(this, result));
+    FUNCTION_TEST_RETURN(UINT, result == NULL ? LIST_NOT_FOUND : lstIdx(this, result));
 }
 
 void *
@@ -218,7 +220,7 @@ lstFindDefault(const List *this, const void *item, void *itemDefault)
 
     void *result= lstFind(this, item);
 
-    FUNCTION_TEST_RETURN(result == NULL ? itemDefault : result);
+    FUNCTION_TEST_RETURN_P(VOID, result == NULL ? itemDefault : result);
 }
 
 /**********************************************************************************************************************************/
@@ -241,7 +243,7 @@ lstIdx(const List *this, const void *item)
     // Item pointers should always be in range
     ASSERT(result < lstSize(this));
 
-    FUNCTION_TEST_RETURN((unsigned int)result);
+    FUNCTION_TEST_RETURN(UINT, (unsigned int)result);
 }
 
 /**********************************************************************************************************************************/
@@ -300,7 +302,7 @@ lstInsert(List *this, unsigned int listIdx, const void *item)
     memcpy(itemPtr, item, this->itemSize);
     this->pub.listSize++;
 
-    FUNCTION_TEST_RETURN(itemPtr);
+    FUNCTION_TEST_RETURN_P(VOID, itemPtr);
 }
 
 /**********************************************************************************************************************************/
@@ -331,7 +333,7 @@ lstRemoveIdx(List *this, unsigned int listIdx)
             (lstSize(this) - listIdx) * this->itemSize);
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(LIST, this);
 }
 
 bool
@@ -350,10 +352,10 @@ lstRemove(List *this, const void *item)
     if (listIdx != LIST_NOT_FOUND)
     {
         lstRemoveIdx(this, listIdx);
-        FUNCTION_TEST_RETURN(true);
+        FUNCTION_TEST_RETURN(BOOL, true);
     }
 
-    FUNCTION_TEST_RETURN(false);
+    FUNCTION_TEST_RETURN(BOOL, false);
 }
 
 List *
@@ -368,7 +370,7 @@ lstRemoveLast(List *this)
     if (lstSize(this) == 0)
         THROW(AssertError, "cannot remove last from list with no values");
 
-    FUNCTION_TEST_RETURN(lstRemoveIdx(this, lstSize(this) - 1));
+    FUNCTION_TEST_RETURN(LIST, lstRemoveIdx(this, lstSize(this) - 1));
 }
 
 /**********************************************************************************************************************************/
@@ -383,28 +385,31 @@ lstSort(List *this, SortOrder sortOrder)
     ASSERT(this != NULL);
     ASSERT(this->comparator != NULL);
 
-    switch (sortOrder)
+    if (this->list != NULL)
     {
-        case sortOrderAsc:
-            qsort(this->list, lstSize(this), this->itemSize, this->comparator);
-            break;
-
-        case sortOrderDesc:
+        switch (sortOrder)
         {
-            // Assign the list that will be sorted for the comparator function to use
-            comparatorDescList = this;
+            case sortOrderAsc:
+                qsort(this->list, lstSize(this), this->itemSize, this->comparator);
+                break;
 
-            qsort(this->list, lstSize(this), this->itemSize, lstComparatorDesc);
-            break;
+            case sortOrderDesc:
+            {
+                // Assign the list that will be sorted for the comparator function to use
+                comparatorDescList = this;
+
+                qsort(this->list, lstSize(this), this->itemSize, lstComparatorDesc);
+                break;
+            }
+
+            case sortOrderNone:
+                break;
         }
-
-        case sortOrderNone:
-            break;
     }
 
     this->sortOrder = sortOrder;
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(LIST, this);
 }
 
 /**********************************************************************************************************************************/
@@ -421,7 +426,7 @@ lstComparatorSet(List *this, ListComparator *comparator)
     this->comparator = comparator;
     this->sortOrder = sortOrderNone;
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(LIST, this);
 }
 
 /**********************************************************************************************************************************/
