@@ -29,8 +29,11 @@ OBJ_NEW_BEGIN(MyObj)
 }
 OBJ_NEW_END();
 ***********************************************************************************************************************************/
-#define OBJ_NEW_BEGIN(type)                                                                                                        \
-    MEM_CONTEXT_NEW_BEGIN(STRINGIFY(type), .allocExtra = sizeof(type))
+#define OBJ_NEW_EXTRA_BEGIN(type, extra, ...)                                                                                      \
+    MEM_CONTEXT_NEW_BEGIN(type, .allocExtra = extra, __VA_ARGS__)
+
+#define OBJ_NEW_BEGIN(type, ...)                                                                                                   \
+    OBJ_NEW_EXTRA_BEGIN(type, sizeof(type), __VA_ARGS__)
 
 #define OBJ_NEW_ALLOC()                                                                                                            \
     memContextAllocExtra(memContextCurrent())
@@ -52,12 +55,6 @@ Create a local "this" variable of the correct type from a THIS_VOID parameter
 ***********************************************************************************************************************************/
 #define THIS(type)                                                  type *this = thisVoid
 
-/***********************************************************************************************************************************
-Get the mem context of this object
-***********************************************************************************************************************************/
-#define THIS_MEM_CONTEXT()                                                                                                         \
-    memContextFromAllocExtra(this)
-
 /***********************************************************************************************************************************
 Cast this private struct, e.g. List, to the associated public struct, e.g. ListPub. Note that the public struct must be the first
 member of the private struct. For example:
@@ -79,6 +76,12 @@ thisNotNull(const void *const this)
     return this;
 }
 
+/***********************************************************************************************************************************
+Switch to the object memory context and ensure that the prior memory context is restored after the block executes (even on error)
+***********************************************************************************************************************************/
+#define MEM_CONTEXT_OBJ_BEGIN(this)                         MEM_CONTEXT_BEGIN(objMemContext(this))
+#define MEM_CONTEXT_OBJ_END()                               MEM_CONTEXT_END()
+
 /***********************************************************************************************************************************
 Functions
 
@@ -97,13 +100,6 @@ objMemContext(void *const this)
     return memContextFromAllocExtra(this);
 }
 
-// Is the object mem context currently being freed?
-__attribute__((always_inline)) static inline bool
-objMemContextFreeing(const void *const this)
-{
-    return memContextFreeing(memContextConstFromAllocExtra(this));
-}
-
 // Move an object to a new context if this != NULL
 void *objMove(THIS_VOID, MemContext *parentNew);
 
