@@ -14,30 +14,23 @@ String Handler
 #include "common/memContext.h"
 #include "common/type/string.h"
 #include "common/type/stringList.h"
-#include "common/type/stringZ.h"
 
 /***********************************************************************************************************************************
 Constant strings that are generally useful
 ***********************************************************************************************************************************/
-STRING_EXTERN(BRACKETL_STR,                                         BRACKETL_Z);
-STRING_EXTERN(BRACKETR_STR,                                         BRACKETR_Z);
-STRING_EXTERN(COLON_STR,                                            COLON_Z);
-STRING_EXTERN(CR_STR,                                               CR_Z);
-STRING_EXTERN(CRLF_STR,                                             CRLF_Z);
-STRING_EXTERN(DASH_STR,                                             DASH_Z);
-STRING_EXTERN(DOT_STR,                                              DOT_Z);
-STRING_EXTERN(DOTDOT_STR,                                           DOTDOT_Z);
-STRING_EXTERN(EMPTY_STR,                                            EMPTY_Z);
-STRING_EXTERN(EQ_STR,                                               EQ_Z);
+STRING_EXTERN(CR_STR,                                               "\r");
+STRING_EXTERN(CRLF_STR,                                             "\r\n");
+STRING_EXTERN(DOT_STR,                                              ".");
+STRING_EXTERN(DOTDOT_STR,                                           "..");
+STRING_EXTERN(EMPTY_STR,                                            "");
 STRING_EXTERN(FALSE_STR,                                            FALSE_Z);
-STRING_EXTERN(FSLASH_STR,                                           FSLASH_Z);
-STRING_EXTERN(LF_STR,                                               LF_Z);
-STRING_EXTERN(N_STR,                                                N_Z);
+STRING_EXTERN(FSLASH_STR,                                           "/");
+STRING_EXTERN(LF_STR,                                               "\n");
+STRING_EXTERN(N_STR,                                                "n");
 STRING_EXTERN(NULL_STR,                                             NULL_Z);
-STRING_EXTERN(QUOTED_STR,                                           QUOTED_Z);
 STRING_EXTERN(TRUE_STR,                                             TRUE_Z);
-STRING_EXTERN(Y_STR,                                                Y_Z);
-STRING_EXTERN(ZERO_STR,                                             ZERO_Z);
+STRING_EXTERN(Y_STR,                                                "y");
+STRING_EXTERN(ZERO_STR,                                             "0");
 
 /***********************************************************************************************************************************
 Buffer macros
@@ -73,7 +66,6 @@ Object type
 struct String
 {
     StringPub pub;                                                  // Publicly accessible variables
-    MemContext *memContext;                                         // Required for dynamically allocated strings
 };
 
 /**********************************************************************************************************************************/
@@ -82,20 +74,24 @@ strNew(void)
 {
     FUNCTION_TEST_VOID();
 
-    // Create object
-    String *this = memNew(sizeof(String));
+    String *this = NULL;
 
-    *this = (String)
+    OBJ_NEW_BEGIN(String, .allocQty = 1)
     {
-        .pub =
-        {
-            // Set empty so nothing is allocated until needed
-            .buffer = STR_EMPTY_BUFFER,
-        },
-        .memContext = memContextCurrent(),
-    };
+        this = OBJ_NEW_ALLOC();
 
-    FUNCTION_TEST_RETURN(this);
+        *this = (String)
+        {
+            .pub =
+            {
+                // Set empty so nothing is allocated until needed
+                .buffer = STR_EMPTY_BUFFER,
+            },
+        };
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /***********************************************************************************************************************************
@@ -110,19 +106,47 @@ strNewFixed(const size_t size)
 
     CHECK_SIZE(size);
 
-    String *this = memNew(sizeof(String) + size + 1);
+    String *this = NULL;
 
-    *this = (String)
+    // If the string is larger than the extra allowed with a mem context then allocate the buffer separately
+    size_t allocExtra = sizeof(String) + size + 1;
+
+    if (allocExtra > MEM_CONTEXT_ALLOC_EXTRA_MAX)
     {
-        .pub =
+        OBJ_NEW_BEGIN(String, .allocQty = 1)
         {
-            .size = (unsigned int)size,
-            .buffer = STR_FIXED_BUFFER,
-        },
-        .memContext = memContextCurrent(),
-    };
+            this = OBJ_NEW_ALLOC();
 
-    FUNCTION_TEST_RETURN(this);
+            *this = (String)
+            {
+                .pub =
+                {
+                    .size = (unsigned int)size,
+                    .buffer = memNew(size + 1),
+                },
+            };
+        }
+        OBJ_NEW_END();
+
+        FUNCTION_TEST_RETURN(STRING, this);
+    }
+
+    OBJ_NEW_EXTRA_BEGIN(String, (uint16_t)(allocExtra))
+    {
+        this = OBJ_NEW_ALLOC();
+
+        *this = (String)
+        {
+            .pub =
+            {
+                .size = (unsigned int)size,
+                .buffer = STR_FIXED_BUFFER,
+            },
+        };
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -142,7 +166,7 @@ strNewZ(const char *const string)
     strncpy(this->pub.buffer, string, strSize(this));
     this->pub.buffer[strSize(this)] = '\0';
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -156,7 +180,7 @@ String *strNewDbl(double value)
 
     cvtDoubleToZ(value, working, sizeof(working));
 
-    FUNCTION_TEST_RETURN(strNewZ(working));
+    FUNCTION_TEST_RETURN(STRING, strNewZ(working));
 }
 
 /**********************************************************************************************************************************/
@@ -173,10 +197,12 @@ strNewBuf(const Buffer *buffer)
     String *this = strNewFixed(bufUsed(buffer));
 
     // Assign string
-    memcpy(this->pub.buffer, bufPtrConst(buffer), strSize(this));
+    if (strSize(this) != 0)
+        memcpy(this->pub.buffer, bufPtrConst(buffer), strSize(this));
+
     this->pub.buffer[strSize(this)] = 0;
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -196,7 +222,7 @@ strNewEncode(EncodeType type, const Buffer *buffer)
     // Encode buffer
     encodeToStr(type, bufPtrConst(buffer), bufUsed(buffer), this->pub.buffer);
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -220,7 +246,7 @@ strNewFmt(const char *format, ...)
     vsnprintf(this->pub.buffer, strSize(this) + 1, format, argumentList);
     va_end(argumentList);
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -241,7 +267,7 @@ strNewZN(const char *string, size_t size)
     strncpy(this->pub.buffer, string, strSize(this));
     this->pub.buffer[strSize(this)] = 0;
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -254,7 +280,7 @@ strBase(const String *this)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(strNewZ(strBaseZ(this)));
+    FUNCTION_TEST_RETURN(STRING, strNewZ(strBaseZ(this)));
 }
 
 const char *
@@ -271,7 +297,7 @@ strBaseZ(const String *this)
     while (end > this->pub.buffer && *(end - 1) != '/')
         end--;
 
-    FUNCTION_TEST_RETURN(end);
+    FUNCTION_TEST_RETURN_CONST(STRINGZ, end);
 }
 
 /**********************************************************************************************************************************/
@@ -286,7 +312,7 @@ strBeginsWith(const String *this, const String *beginsWith)
     ASSERT(this != NULL);
     ASSERT(beginsWith != NULL);
 
-    FUNCTION_TEST_RETURN(strBeginsWithZ(this, strZ(beginsWith)));
+    FUNCTION_TEST_RETURN(BOOL, strBeginsWithZ(this, strZ(beginsWith)));
 }
 
 bool
@@ -306,7 +332,7 @@ strBeginsWithZ(const String *this, const char *beginsWith)
     if (strSize(this) >= beginsWithSize)
         result = strncmp(strZ(this), beginsWith, beginsWithSize) == 0;
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(BOOL, result);
 }
 
 /***********************************************************************************************************************************
@@ -335,14 +361,14 @@ strResize(String *this, size_t requested)
         if (this->pub.extra < STRING_EXTRA_MIN)
             this->pub.extra = STRING_EXTRA_MIN;
 
-        MEM_CONTEXT_BEGIN(this->memContext)
+        MEM_CONTEXT_OBJ_BEGIN(this)
         {
             if (STR_IS_EMPTY_BUFFER())
                 this->pub.buffer = memNew(strSize(this) + this->pub.extra + 1);
             else
                 this->pub.buffer = memResize(this->pub.buffer, strSize(this) + this->pub.extra + 1);
         }
-        MEM_CONTEXT_END();
+        MEM_CONTEXT_OBJ_END();
     }
 
     FUNCTION_TEST_RETURN_VOID();
@@ -360,7 +386,7 @@ strCat(String *this, const String *cat)
     ASSERT(this != NULL);
     ASSERT(cat != NULL);
 
-    FUNCTION_TEST_RETURN(strCatZN(this, strZ(cat), strSize(cat)));
+    FUNCTION_TEST_RETURN(STRING, strCatZN(this, strZ(cat), strSize(cat)));
 }
 
 String *
@@ -388,7 +414,7 @@ strCatZ(String *this, const char *cat)
         this->pub.extra -= (unsigned int)sizeGrow;
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 String *
@@ -418,7 +444,7 @@ strCatZN(String *this, const char *cat, size_t size)
         this->pub.extra -= (unsigned int)size;
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -433,7 +459,7 @@ strCatBuf(String *const this, const Buffer *const buffer)
     ASSERT(this != NULL);
     ASSERT(buffer != NULL);
 
-    FUNCTION_TEST_RETURN(strCatZN(this, (char *)bufPtrConst(buffer), bufUsed(buffer)));
+    FUNCTION_TEST_RETURN(STRING, strCatZN(this, (char *)bufPtrConst(buffer), bufUsed(buffer)));
 }
 
 /**********************************************************************************************************************************/
@@ -456,7 +482,7 @@ strCatChr(String *this, char cat)
     this->pub.buffer[this->pub.size] = 0;
     this->pub.extra--;
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -487,7 +513,7 @@ strCatEncode(String *this, EncodeType type, const Buffer *buffer)
         this->pub.extra -= (unsigned int)encodeSize;
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -522,7 +548,7 @@ strCatFmt(String *this, const char *format, ...)
         this->pub.extra -= (unsigned int)sizeGrow;
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -535,16 +561,16 @@ strCmp(const String *this, const String *compare)
     FUNCTION_TEST_END();
 
     if (this != NULL && compare != NULL)
-        FUNCTION_TEST_RETURN(strcmp(strZ(this), strZ(compare)));
+        FUNCTION_TEST_RETURN(INT, strcmp(strZ(this), strZ(compare)));
     else if (this == NULL)
     {
         if (compare == NULL)
-            FUNCTION_TEST_RETURN(0);
+            FUNCTION_TEST_RETURN(INT, 0);
 
-        FUNCTION_TEST_RETURN(-1);
+        FUNCTION_TEST_RETURN(INT, -1);
     }
 
-    FUNCTION_TEST_RETURN(1);
+    FUNCTION_TEST_RETURN(INT, 1);
 }
 
 int
@@ -555,7 +581,7 @@ strCmpZ(const String *this, const char *compare)
         FUNCTION_TEST_PARAM(STRINGZ, compare);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(strCmp(this, compare == NULL ? NULL : STRDEF(compare)));
+    FUNCTION_TEST_RETURN(INT, strCmp(this, compare == NULL ? NULL : STR(compare)));
 }
 
 /**********************************************************************************************************************************/
@@ -571,7 +597,7 @@ strDup(const String *this)
     if (this != NULL)
         result = strNewZ(strZ(this));
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 /**********************************************************************************************************************************/
@@ -582,7 +608,7 @@ strEmpty(const String *this)
         FUNCTION_TEST_PARAM(STRING, this);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(strSize(this) == 0);
+    FUNCTION_TEST_RETURN(BOOL, strSize(this) == 0);
 }
 
 /**********************************************************************************************************************************/
@@ -597,7 +623,7 @@ strEndsWith(const String *this, const String *endsWith)
     ASSERT(this != NULL);
     ASSERT(endsWith != NULL);
 
-    FUNCTION_TEST_RETURN(strEndsWithZ(this, strZ(endsWith)));
+    FUNCTION_TEST_RETURN(BOOL, strEndsWithZ(this, strZ(endsWith)));
 }
 
 bool
@@ -617,7 +643,7 @@ strEndsWithZ(const String *this, const char *endsWith)
     if (strSize(this) >= endsWithSize)
         result = strcmp(strZ(this) + (strSize(this) - endsWithSize), endsWith) == 0;
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(BOOL, result);
 }
 
 /***********************************************************************************************************************************
@@ -642,7 +668,7 @@ strEq(const String *this, const String *compare)
     else
         result = this == NULL && compare == NULL;
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(BOOL, result);
 }
 
 bool
@@ -656,7 +682,7 @@ strEqZ(const String *this, const char *compare)
     ASSERT(this != NULL);
     ASSERT(compare != NULL);
 
-    FUNCTION_TEST_RETURN(strcmp(strZ(this), compare) == 0);
+    FUNCTION_TEST_RETURN(BOOL, strcmp(strZ(this), compare) == 0);
 }
 
 /**********************************************************************************************************************************/
@@ -672,7 +698,7 @@ strFirstUpper(String *this)
     if (strSize(this) > 0)
         this->pub.buffer[0] = (char)toupper(this->pub.buffer[0]);
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -688,7 +714,7 @@ strFirstLower(String *this)
     if (strSize(this) > 0)
         this->pub.buffer[0] = (char)tolower(this->pub.buffer[0]);
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -704,7 +730,7 @@ strUpper(String *this)
     for (size_t idx = 0; idx < strSize(this); idx++)
         this->pub.buffer[idx] = (char)toupper(this->pub.buffer[idx]);
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -720,7 +746,7 @@ strLower(String *this)
     for (size_t idx = 0; idx < strSize(this); idx++)
         this->pub.buffer[idx] = (char)tolower(this->pub.buffer[idx]);
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -739,6 +765,7 @@ strPath(const String *this)
         end--;
 
     FUNCTION_TEST_RETURN(
+        STRING,
         strNewZN(
             this->pub.buffer,
             end - this->pub.buffer <= 1 ? (size_t)(end - this->pub.buffer) : (size_t)(end - this->pub.buffer - 1)));
@@ -825,7 +852,7 @@ strPathAbsolute(const String *this, const String *base)
     if (strstr(strZ(result), "/..") != NULL || strstr(strZ(result), "//") != NULL)
         THROW_FMT(AssertError, "result path '%s' is not absolute", strZ(result));
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 /**********************************************************************************************************************************/
@@ -836,7 +863,7 @@ strZNull(const String *this)
         FUNCTION_TEST_PARAM(STRING, this);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(this == NULL ? NULL : strZ(this));
+    FUNCTION_TEST_RETURN_CONST(STRINGZ, this == NULL ? NULL : strZ(this));
 }
 
 /**********************************************************************************************************************************/
@@ -851,7 +878,7 @@ strQuote(const String *this, const String *quote)
     ASSERT(this != NULL);
     ASSERT(quote != NULL);
 
-    FUNCTION_TEST_RETURN(strQuoteZ(this, strZ(quote)));
+    FUNCTION_TEST_RETURN(STRING, strQuoteZ(this, strZ(quote)));
 }
 
 String *
@@ -865,7 +892,7 @@ strQuoteZ(const String *this, const char *quote)
     ASSERT(this != NULL);
     ASSERT(quote != NULL);
 
-    FUNCTION_TEST_RETURN(strNewFmt("%s%s%s", quote, strZ(this), quote));
+    FUNCTION_TEST_RETURN(STRING, strNewFmt("%s%s%s", quote, strZ(this), quote));
 }
 
 /**********************************************************************************************************************************/
@@ -886,7 +913,50 @@ strReplaceChr(String *this, char find, char replace)
             this->pub.buffer[stringIdx] = replace;
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
+}
+
+/**********************************************************************************************************************************/
+String *
+strReplace(String *const this, const String *const replace, const String *const with)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, this);
+        FUNCTION_TEST_PARAM(STRING, replace);
+        FUNCTION_TEST_PARAM(STRING, with);
+    FUNCTION_TEST_END();
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        // Does replace exist?
+        char *found = strstr(this->pub.buffer, strZ(replace));
+
+        while (found != NULL)
+        {
+            // Offset into string
+            const size_t offset = (size_t)(found - this->pub.buffer);
+
+            // Calculate new size and resize
+            const size_t size = strSize(this) - strSize(replace) + strSize(with);
+            strResize(this, size);
+
+            // Replace
+            memmove(
+                this->pub.buffer + offset + strSize(with), this->pub.buffer + offset + strSize(replace),
+                strSize(this) - offset - strSize(replace));
+            memcpy(this->pub.buffer + offset, strZ(with), strSize(with));
+
+            // Set new size and zero-terminate
+            this->pub.size = (unsigned int)size;
+            this->pub.buffer[size] = '\0';
+
+            // Does replace exist again?
+            found = strstr(this->pub.buffer + offset + strSize(with), strZ(replace));
+        }
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -901,7 +971,7 @@ strSub(const String *this, size_t start)
     ASSERT(this != NULL);
     ASSERT(start <= this->pub.size);
 
-    FUNCTION_TEST_RETURN(strSubN(this, start, strSize(this) - start));
+    FUNCTION_TEST_RETURN(STRING, strSubN(this, start, strSize(this) - start));
 }
 
 /**********************************************************************************************************************************/
@@ -918,7 +988,7 @@ strSubN(const String *this, size_t start, size_t size)
     ASSERT(start <= strSize(this));
     ASSERT(start + size <= strSize(this));
 
-    FUNCTION_TEST_RETURN(strNewZN(strZ(this) + start, size));
+    FUNCTION_TEST_RETURN(STRING, strNewZN(strZ(this) + start, size));
 }
 
 /**********************************************************************************************************************************/
@@ -961,7 +1031,7 @@ strTrim(String *this)
         }
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /**********************************************************************************************************************************/
@@ -985,12 +1055,12 @@ strChr(const String *this, char chr)
             result = (int)(ptr - this->pub.buffer);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(INT, result);
 }
 
 /**********************************************************************************************************************************/
 String *
-strTrunc(String *this, int idx)
+strTruncIdx(String *this, int idx)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(STRING, this);
@@ -1007,7 +1077,7 @@ strTrunc(String *this, int idx)
         this->pub.buffer[strSize(this)] = 0;
     }
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(STRING, this);
 }
 
 /***********************************************************************************************************************************
@@ -1039,7 +1109,6 @@ strSizeFormat(const uint64_t size)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(UINT64, size);
-
     FUNCTION_TEST_END();
 
     String *result = NULL;
@@ -1068,30 +1137,5 @@ strSizeFormat(const uint64_t size)
             result = strNewFmt("%" PRIu64 "GB", size / (1024 * 1024 * 1024));
     }
 
-    FUNCTION_TEST_RETURN(result);
-}
-
-/***********************************************************************************************************************************
-Free the string
-***********************************************************************************************************************************/
-void
-strFree(String *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(STRING, this);
-    FUNCTION_TEST_END();
-
-    if (this != NULL)
-    {
-        MEM_CONTEXT_BEGIN(this->memContext)
-        {
-            if (!STR_IS_EMPTY_BUFFER() && !STR_IS_FIXED_BUFFER())
-                memFree(this->pub.buffer);
-
-            memFree(this);
-        }
-        MEM_CONTEXT_END();
-    }
-
-    FUNCTION_TEST_RETURN_VOID();
+    FUNCTION_TEST_RETURN(STRING, result);
 }
