@@ -5,7 +5,7 @@ Strings are lightweight objects in that they do not have their own memory contex
 which they are instantiated. If a string is needed outside the current memory context, the memory context must be switched to the
 old context and then back. Below is a simplified example:
 
-    String *result = NULL;     <--- is created in the current memory context  (referred to as "old context" below)
+    String *result = NULL;     <--- is created in the current memory context (referred to as "old context" below)
     MEM_CONTEXT_TEMP_BEGIN()   <--- begins a new temporary context
     {
         String *resultStr = strNewZN("myNewStr"); <--- creates a string in the temporary memory context
@@ -38,6 +38,7 @@ typedef struct String String;
 #include "common/assert.h"
 #include "common/encode.h"
 #include "common/type/buffer.h"
+#include "common/type/stringZ.h"
 
 /***********************************************************************************************************************************
 Constructors
@@ -180,6 +181,9 @@ String *strPathAbsolute(const String *this, const String *base);
 String *strQuote(const String *this, const String *quote);
 String *strQuoteZ(const String *this, const char *quote);
 
+// Replace a substring with another string
+String *strReplace(String *this, const String *replace, const String *with);
+
 // Replace a character with another character
 String *strReplaceChr(String *this, char find, char replace);
 
@@ -196,12 +200,23 @@ String *strSubN(const String *this, size_t start, size_t size);
 String *strTrim(String *this);
 
 // Truncate the end of a string from the index provided to the current end (e.g. 123KB pass index of K returns 123)
-String *strTrunc(String *this, int idx);
+String *strTruncIdx(String *this, int idx);
+
+// Truncate the string to zero size
+__attribute__((always_inline)) static inline String *
+strTrunc(String *const this)
+{
+    return strTruncIdx(this, 0);
+}
 
 /***********************************************************************************************************************************
 Destructor
 ***********************************************************************************************************************************/
-void strFree(String *this);
+__attribute__((always_inline)) static inline void
+strFree(String *const this)
+{
+    objFree(this);
+}
 
 /***********************************************************************************************************************************
 Macros for constant strings
@@ -213,13 +228,16 @@ String * will result in a segfault due to modifying read-only memory.
 
 By convention all string constant identifiers are appended with _STR.
 ***********************************************************************************************************************************/
+#define STR_SIZE(bufferParam, sizeParam)                                                                                           \
+    ((const String *const)&(const StringPub){.buffer = (char *)(bufferParam), .size = (unsigned int)(sizeParam)})
+
 // Create a String constant inline from any zero-terminated string
-#define STR(bufferParam)                                                                                                           \
-    ((const String *)&(const StringPub){.buffer = (char *)(bufferParam), .size = (unsigned int)strlen(bufferParam)})
+#define STR(buffer)                                                                                                                \
+    STR_SIZE(buffer, strlen(buffer))
 
 // Create a String constant inline from a #define or inline string constant
-#define STRDEF(bufferParam)                                                                                                        \
-    ((const String *)&(const StringPub){.buffer = (char *)(bufferParam), .size = (unsigned int)sizeof(bufferParam) - 1})
+#define STRDEF(buffer)                                                                                                             \
+    STR_SIZE(buffer, sizeof(buffer) - 1)
 
 // Used to declare String constants that will be externed using STRING_DECLARE().  Must be used in a .c file.
 #define STRING_EXTERN(name, buffer)                                                                                                \
@@ -236,22 +254,16 @@ By convention all string constant identifiers are appended with _STR.
 /***********************************************************************************************************************************
 Constant strings that are generally useful
 ***********************************************************************************************************************************/
-STRING_DECLARE(BRACKETL_STR);
-STRING_DECLARE(BRACKETR_STR);
-STRING_DECLARE(COLON_STR);
 STRING_DECLARE(CR_STR);
 STRING_DECLARE(CRLF_STR);
-STRING_DECLARE(DASH_STR);
 STRING_DECLARE(DOT_STR);
 STRING_DECLARE(DOTDOT_STR);
 STRING_DECLARE(EMPTY_STR);
-STRING_DECLARE(EQ_STR);
 STRING_DECLARE(FALSE_STR);
 STRING_DECLARE(FSLASH_STR);
 STRING_DECLARE(LF_STR);
 STRING_DECLARE(N_STR);
 STRING_DECLARE(NULL_STR);
-STRING_DECLARE(QUOTED_STR);
 STRING_DECLARE(TRUE_STR);
 STRING_DECLARE(Y_STR);
 STRING_DECLARE(ZERO_STR);
