@@ -37,51 +37,43 @@ struct Variant
 typedef struct VariantBool
 {
     VariantBoolPub pub;                                             // Publicly accessible variables
-    MemContext *memContext;
 } VariantBool;
 
 typedef struct VariantInt
 {
     VariantIntPub pub;                                              // Publicly accessible variables
-    MemContext *memContext;
 } VariantInt;
 
 typedef struct VariantInt64
 {
     VariantInt64Pub pub;                                            // Publicly accessible variables
-    MemContext *memContext;
 } VariantInt64;
 
 typedef struct VariantKeyValue
 {
     VARIANT_COMMON
     KeyValue *data;                                                 // KeyValue data
-    MemContext *memContext;
 } VariantKeyValue;
 
 typedef struct VariantString
 {
     VariantStringPub pub;                                           // Publicly accessible variables
-    MemContext *memContext;
 } VariantString;
 
 typedef struct VariantUInt
 {
     VariantUIntPub pub;                                             // Publicly accessible variables
-    MemContext *memContext;
 } VariantUInt;
 
 typedef struct VariantUInt64
 {
     VariantUInt64Pub pub;                                           // Publicly accessible variables
-    MemContext *memContext;
 } VariantUInt64;
 
 typedef struct VariantVariantList
 {
     VARIANT_COMMON
     VariantList *data;                                              // VariantList data
-    MemContext *memContext;
 } VariantVariantList;
 
 /***********************************************************************************************************************************
@@ -127,16 +119,7 @@ varDup(const Variant *this)
 
             case varTypeKeyValue:
             {
-                VariantKeyValue *keyValue = memNew(sizeof(VariantKeyValue));
-
-                *keyValue = (VariantKeyValue)
-                {
-                    .memContext = memContextCurrent(),
-                    .type = varTypeKeyValue,
-                    .data = kvDup(varKv(this)),
-                };
-
-                result = (Variant *)keyValue;
+                result = varNewKv(kvDup(varKv(this)));
                 break;
             }
 
@@ -158,7 +141,7 @@ varDup(const Variant *this)
         }
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(VARIANT, result);
 }
 
 /**********************************************************************************************************************************/
@@ -213,7 +196,7 @@ varEq(const Variant *this1, const Variant *this2)
     else
         result = this1 == NULL && this2 == NULL;
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(BOOL, result);
 }
 
 /**********************************************************************************************************************************/
@@ -224,20 +207,24 @@ varNewBool(bool data)
         FUNCTION_TEST_PARAM(BOOL, data);
     FUNCTION_TEST_END();
 
-    // Allocate memory for the variant and set the type and data
-    VariantBool *this = memNew(sizeof(VariantBool));
+    VariantBool *this = NULL;
 
-    *this = (VariantBool)
+    OBJ_NEW_BEGIN(VariantBool)
     {
-        .pub =
-        {
-            .type = varTypeBool,
-            .data = data,
-        },
-        .memContext = memContextCurrent(),
-    };
+        this = OBJ_NEW_ALLOC();
 
-    FUNCTION_TEST_RETURN((Variant *)this);
+        *this = (VariantBool)
+        {
+            .pub =
+            {
+                .type = varTypeBool,
+                .data = data,
+            },
+        };
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(VARIANT, (Variant *)this);
 }
 
 /**********************************************************************************************************************************/
@@ -251,7 +238,7 @@ varBool(const Variant *this)
     ASSERT(this != NULL);
     ASSERT(varType(this) == varTypeBool);
 
-    FUNCTION_TEST_RETURN(((VariantBool *)this)->pub.data);
+    FUNCTION_TEST_RETURN(BOOL, ((VariantBool *)this)->pub.data);
 }
 
 bool
@@ -292,16 +279,16 @@ varBoolForce(const Variant *this)
             const char *string = strZ(varStr(this));
             unsigned int boolIdx;
 
-            for (boolIdx = 0; boolIdx < sizeof(boolString) / sizeof(char *); boolIdx++)
+            for (boolIdx = 0; boolIdx < LENGTH_OF(boolString); boolIdx++)
                 if (strcasecmp(string, boolString[boolIdx]) == 0)
                     break;
 
             // If string was not found then not a boolean
-            if (boolIdx == sizeof(boolString) / sizeof(char *))
+            if (boolIdx == LENGTH_OF(boolString))
                 THROW_FMT(FormatError, "unable to convert str '%s' to bool", string);
 
             // False if in first half of list, true if in second half
-            result = boolIdx / (sizeof(boolString) / sizeof(char *) / 2);
+            result = boolIdx / (LENGTH_OF(boolString) / 2);
 
             break;
         }
@@ -318,7 +305,7 @@ varBoolForce(const Variant *this)
             THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[varType(this)], variantTypeName[varTypeBool]);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(BOOL, result);
 }
 
 /**********************************************************************************************************************************/
@@ -329,20 +316,24 @@ varNewInt(int data)
         FUNCTION_TEST_PARAM(INT, data);
     FUNCTION_TEST_END();
 
-    // Allocate memory for the variant and set the type and data
-    VariantInt *this = memNew(sizeof(VariantInt));
+    VariantInt *this = NULL;
 
-    *this = (VariantInt)
+    OBJ_NEW_BEGIN(VariantInt)
     {
-        .pub =
-        {
-            .type = varTypeInt,
-            .data = data,
-        },
-        .memContext = memContextCurrent(),
-    };
+        this = OBJ_NEW_ALLOC();
 
-    FUNCTION_TEST_RETURN((Variant *)this);
+        *this = (VariantInt)
+        {
+            .pub =
+            {
+                .type = varTypeInt,
+                .data = data,
+            },
+        };
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(VARIANT, (Variant *)this);
 }
 
 /**********************************************************************************************************************************/
@@ -356,7 +347,7 @@ varInt(const Variant *this)
     ASSERT(this != NULL);
     ASSERT(varType(this) == varTypeInt);
 
-    FUNCTION_TEST_RETURN(((VariantInt *)this)->pub.data);
+    FUNCTION_TEST_RETURN(INT, ((VariantInt *)this)->pub.data);
 }
 
 int
@@ -430,7 +421,7 @@ varIntForce(const Variant *this)
             THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[varType(this)], variantTypeName[varTypeInt]);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(INT, result);
 }
 
 /**********************************************************************************************************************************/
@@ -441,20 +432,24 @@ varNewInt64(int64_t data)
         FUNCTION_TEST_PARAM(INT64, data);
     FUNCTION_TEST_END();
 
-    // Allocate memory for the variant and set the type and data
-    VariantInt64 *this = memNew(sizeof(VariantInt64));
+    VariantInt64 *this = NULL;
 
-    *this = (VariantInt64)
+    OBJ_NEW_BEGIN(VariantInt64)
     {
-        .pub =
-        {
-            .type = varTypeInt64,
-            .data = data,
-        },
-        .memContext = memContextCurrent(),
-    };
+        this = OBJ_NEW_ALLOC();
 
-    FUNCTION_TEST_RETURN((Variant *)this);
+        *this = (VariantInt64)
+        {
+            .pub =
+            {
+                .type = varTypeInt64,
+                .data = data,
+            },
+        };
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(VARIANT, (Variant *)this);
 }
 
 /**********************************************************************************************************************************/
@@ -468,7 +463,7 @@ varInt64(const Variant *this)
     ASSERT(this != NULL);
     ASSERT(varType(this) == varTypeInt64);
 
-    FUNCTION_TEST_RETURN(((VariantInt64 *)this)->pub.data);
+    FUNCTION_TEST_RETURN(INT64, ((VariantInt64 *)this)->pub.data);
 }
 
 int64_t
@@ -526,7 +521,7 @@ varInt64Force(const Variant *this)
             THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[varType(this)], variantTypeName[varTypeInt64]);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(INT64, result);
 }
 
 /**********************************************************************************************************************************/
@@ -537,20 +532,24 @@ varNewUInt(unsigned int data)
         FUNCTION_TEST_PARAM(UINT, data);
     FUNCTION_TEST_END();
 
-    // Allocate memory for the variant and set the type and data
-    VariantUInt *this = memNew(sizeof(VariantUInt));
+    VariantUInt *this = NULL;
 
-    *this = (VariantUInt)
+    OBJ_NEW_BEGIN(VariantUInt)
     {
-        .pub =
-        {
-            .type = varTypeUInt,
-            .data = data,
-        },
-        .memContext = memContextCurrent(),
-    };
+        this = OBJ_NEW_ALLOC();
 
-    FUNCTION_TEST_RETURN((Variant *)this);
+        *this = (VariantUInt)
+        {
+            .pub =
+            {
+                .type = varTypeUInt,
+                .data = data,
+            },
+        };
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(VARIANT, (Variant *)this);
 }
 
 /**********************************************************************************************************************************/
@@ -564,7 +563,7 @@ varUInt(const Variant *this)
     ASSERT(this != NULL);
     ASSERT(varType(this) == varTypeUInt);
 
-    FUNCTION_TEST_RETURN(((VariantUInt *)this)->pub.data);
+    FUNCTION_TEST_RETURN(UINT, ((VariantUInt *)this)->pub.data);
 }
 
 unsigned int
@@ -647,7 +646,7 @@ varUIntForce(const Variant *this)
             THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[varType(this)], variantTypeName[varTypeUInt]);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(UINT, result);
 }
 
 /**********************************************************************************************************************************/
@@ -658,20 +657,24 @@ varNewUInt64(uint64_t data)
         FUNCTION_TEST_PARAM(UINT64, data);
     FUNCTION_TEST_END();
 
-    // Allocate memory for the variant and set the type and data
-    VariantUInt64 *this = memNew(sizeof(VariantUInt64));
+    VariantUInt64 *this = NULL;
 
-    *this = (VariantUInt64)
+    OBJ_NEW_BEGIN(VariantUInt64)
     {
-        .pub =
-        {
-            .type = varTypeUInt64,
-            .data = data,
-        },
-        .memContext = memContextCurrent(),
-    };
+        this = OBJ_NEW_ALLOC();
 
-    FUNCTION_TEST_RETURN((Variant *)this);
+        *this = (VariantUInt64)
+        {
+            .pub =
+            {
+                .type = varTypeUInt64,
+                .data = data,
+            },
+        };
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(VARIANT, (Variant *)this);
 }
 
 /**********************************************************************************************************************************/
@@ -685,7 +688,7 @@ varUInt64(const Variant *this)
     ASSERT(this != NULL);
     ASSERT(varType(this) == varTypeUInt64);
 
-    FUNCTION_TEST_RETURN(((VariantUInt64 *)this)->pub.data);
+    FUNCTION_TEST_RETURN(UINT64, ((VariantUInt64 *)this)->pub.data);
 }
 
 uint64_t
@@ -755,7 +758,7 @@ varUInt64Force(const Variant *this)
             THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[varType(this)], variantTypeName[varTypeUInt64]);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(UINT64, result);
 }
 
 /**********************************************************************************************************************************/
@@ -766,19 +769,23 @@ varNewKv(KeyValue *data)
         FUNCTION_TEST_PARAM(KEY_VALUE, data);
     FUNCTION_TEST_END();
 
-    // Allocate memory for the variant and set the type and data
-    VariantKeyValue *this = memNew(sizeof(VariantKeyValue));
+    VariantKeyValue *this = NULL;
 
-    *this = (VariantKeyValue)
+    OBJ_NEW_BEGIN(VariantKeyValue, .childQty = 1)
     {
-        .memContext = memContextCurrent(),
-        .type = varTypeKeyValue,
-    };
+        this = OBJ_NEW_ALLOC();
 
-    if (data != NULL)
-        this->data = kvMove(data, memContextCurrent());
+        *this = (VariantKeyValue)
+        {
+            .type = varTypeKeyValue,
+        };
 
-    FUNCTION_TEST_RETURN((Variant *)this);
+        if (data != NULL)
+            this->data = kvMove(data, memContextCurrent());
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(VARIANT, (Variant *)this);
 }
 
 /**********************************************************************************************************************************/
@@ -797,7 +804,7 @@ varKv(const Variant *this)
         result = ((VariantKeyValue *)this)->data;
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(KEY_VALUE, result);
 }
 
 /**********************************************************************************************************************************/
@@ -808,37 +815,66 @@ varNewStr(const String *data)
         FUNCTION_TEST_PARAM(STRING, data);
     FUNCTION_TEST_END();
 
-    // Allocate memory for the variant
-    VariantString *this = memNew(sizeof(VariantString) + (data != NULL ? sizeof(StringPub) + strSize(data) + 1 : 0));
+    VariantString *this = NULL;
 
-    *this = (VariantString)
+    // If the variant is larger than the extra allowed with a mem context then allocate the buffer separately
+    size_t allocExtra = sizeof(VariantString) + (data != NULL ? sizeof(StringPub) + strSize(data) + 1 : 0);
+
+    if (allocExtra > MEM_CONTEXT_ALLOC_EXTRA_MAX)
     {
-        .pub =
+        ASSERT(data != NULL);
+
+        OBJ_NEW_BEGIN(VariantString, .childQty = 1)
         {
-            .type = varTypeString,
-        },
-        .memContext = memContextCurrent(),
-    };
+            this = OBJ_NEW_ALLOC();
 
-    if (data != NULL)
-    {
-        // Point the String after the VariantString struct
-        StringPub *const pubData = (StringPub *)(this + 1);
-        this->pub.data = (String *)pubData;
+            *this = (VariantString)
+            {
+                .pub =
+                {
+                    .type = varTypeString,
+                    .data = strDup(data),
+                },
+            };
+        }
+        OBJ_NEW_END();
 
-        // Assign the string size and buffer (after StringPub struct)
-        *pubData = (StringPub)
-        {
-            .size = (unsigned int)strSize(data),
-            .buffer = (char *)(pubData + 1),
-        };
-
-        // Assign the string
-        strncpy(pubData->buffer, strZ(data), strSize(data));
-        pubData->buffer[strSize(data)] = '\0';
+        FUNCTION_TEST_RETURN(VARIANT, (Variant *)this);
     }
 
-    FUNCTION_TEST_RETURN((Variant *)this);
+    OBJ_NEW_EXTRA_BEGIN(VariantString, (uint16_t)(allocExtra))
+    {
+        this = OBJ_NEW_ALLOC();
+
+        *this = (VariantString)
+        {
+            .pub =
+            {
+                .type = varTypeString,
+            },
+        };
+
+        if (data != NULL)
+        {
+            // Point the String after the VariantString struct
+            StringPub *const pubData = (StringPub *)(this + 1);
+            this->pub.data = (String *)pubData;
+
+            // Assign the string size and buffer (after StringPub struct)
+            *pubData = (StringPub)
+            {
+                .size = (unsigned int)strSize(data),
+                .buffer = (char *)(pubData + 1),
+            };
+
+            // Assign the string
+            strncpy(pubData->buffer, strZ(data), strSize(data));
+            pubData->buffer[strSize(data)] = '\0';
+        }
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(VARIANT, (Variant *)this);
 }
 
 Variant *
@@ -848,7 +884,7 @@ varNewStrZ(const char *data)
         FUNCTION_TEST_PARAM(STRINGZ, data);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(varNewStr(data == NULL ? NULL : STR(data)));
+    FUNCTION_TEST_RETURN(VARIANT, varNewStr(data == NULL ? NULL : STR(data)));
 }
 
 /**********************************************************************************************************************************/
@@ -867,7 +903,7 @@ varStr(const Variant *this)
         result = ((VariantString *)this)->pub.data;
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 String *
@@ -931,7 +967,7 @@ varStrForce(const Variant *this)
             THROW_FMT(FormatError, "unable to force %s to %s", variantTypeName[varType(this)], variantTypeName[varTypeString]);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 /**********************************************************************************************************************************/
@@ -942,19 +978,23 @@ varNewVarLst(const VariantList *data)
         FUNCTION_TEST_PARAM(VARIANT_LIST, data);
     FUNCTION_TEST_END();
 
-    // Allocate memory for the variant and set the type and data
-    VariantVariantList *this = memNew(sizeof(VariantVariantList));
+    VariantVariantList *this = NULL;
 
-    *this = (VariantVariantList)
+    OBJ_NEW_BEGIN(VariantVariantList, .childQty = 1)
     {
-        .memContext = memContextCurrent(),
-        .type = varTypeVariantList,
-    };
+        this = OBJ_NEW_ALLOC();
 
-    if (data != NULL)
-        this->data = varLstDup(data);
+        *this = (VariantVariantList)
+        {
+            .type = varTypeVariantList,
+        };
 
-    FUNCTION_TEST_RETURN((Variant *)this);
+        if (data != NULL)
+            this->data = varLstDup(data);
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_TEST_RETURN(VARIANT, (Variant *)this);
 }
 
 /**********************************************************************************************************************************/
@@ -973,7 +1013,7 @@ varVarLst(const Variant *this)
         result = ((VariantVariantList *)this)->data;
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(VARIANT_LIST, result);
 }
 
 /**********************************************************************************************************************************/
@@ -1014,62 +1054,3 @@ varToLog(const Variant *this)
 
     return result;
 }
-
-/**********************************************************************************************************************************/
-void
-varFree(Variant *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(VARIANT, this);
-    FUNCTION_TEST_END();
-
-    if (this != NULL)
-    {
-        MemContext *memContext = NULL;
-
-        switch (varType(this))
-        {
-            case varTypeBool:
-                memContext = ((VariantBool *)this)->memContext;
-                break;
-
-            case varTypeInt:
-                memContext = ((VariantInt *)this)->memContext;
-                break;
-
-            case varTypeInt64:
-                memContext = ((VariantInt64 *)this)->memContext;
-                break;
-
-            case varTypeKeyValue:
-                memContext = ((VariantKeyValue *)this)->memContext;
-                kvFree(((VariantKeyValue *)this)->data);
-                break;
-
-            case varTypeString:
-                memContext = ((VariantString *)this)->memContext;
-                break;
-
-            case varTypeUInt:
-                memContext = ((VariantUInt *)this)->memContext;
-                break;
-
-            case varTypeUInt64:
-                memContext = ((VariantUInt64 *)this)->memContext;
-                break;
-
-            case varTypeVariantList:
-                memContext = ((VariantVariantList *)this)->memContext;
-                varLstFree(((VariantVariantList *)this)->data);
-                break;
-        }
-
-        MEM_CONTEXT_BEGIN(memContext)
-        {
-            memFree(this);
-        }
-        MEM_CONTEXT_END();
-    }
-
-    FUNCTION_TEST_RETURN_VOID();
-}
