@@ -151,7 +151,11 @@ bool varEq(const Variant *this1, const Variant *this2);
 /***********************************************************************************************************************************
 Destructor
 ***********************************************************************************************************************************/
-void varFree(Variant *this);
+__attribute__((always_inline)) static inline void
+varFree(Variant *const this)
+{
+    objFree(this);
+}
 
 /***********************************************************************************************************************************
 Macros for constant variants
