@@ -15,7 +15,7 @@ Command and Option Configuration
 /***********************************************************************************************************************************
 Data for the currently loaded configuration
 ***********************************************************************************************************************************/
-Config *configLocal = NULL;
+static Config *configLocal = NULL;
 
 /**********************************************************************************************************************************/
 void
@@ -43,7 +43,7 @@ ConfigCommand
 cfgCommand(void)
 {
     FUNCTION_TEST_VOID();
-    FUNCTION_TEST_RETURN(configLocal == NULL ? cfgCmdNone : configLocal->command);
+    FUNCTION_TEST_RETURN(ENUM, configLocal == NULL ? cfgCmdNone : configLocal->command);
 }
 
 ConfigCommandRole
@@ -51,7 +51,7 @@ cfgCommandRole(void)
 {
     FUNCTION_TEST_VOID();
     ASSERT(configLocal != NULL);
-    FUNCTION_TEST_RETURN(configLocal->commandRole);
+    FUNCTION_TEST_RETURN(ENUM, configLocal->commandRole);
 }
 
 void
@@ -77,7 +77,7 @@ cfgCommandHelp(void)
 {
     FUNCTION_TEST_VOID();
     ASSERT(configLocal != NULL);
-    FUNCTION_TEST_RETURN(configLocal->help);
+    FUNCTION_TEST_RETURN(BOOL, configLocal->help);
 }
 
 /**********************************************************************************************************************************/
@@ -92,15 +92,19 @@ cfgCommandJobRetry(void)
     unsigned int retryTotal = cfgOptionUInt(cfgOptJobRetry);
 
     if (retryTotal == 0)
-        FUNCTION_TEST_RETURN(NULL);
+        FUNCTION_TEST_RETURN(VARIANT_LIST, NULL);
 
     // Build retry list
     VariantList *result = varLstNew();
 
-    for (unsigned int retryIdx = 0; retryIdx < cfgOptionUInt(cfgOptJobRetry); retryIdx++)
-        varLstAdd(result, varNewUInt64(retryIdx == 0 ? 0 : cfgOptionUInt64(cfgOptJobRetryInterval)));
+    MEM_CONTEXT_BEGIN(lstMemContext((List *)result))
+    {
+        for (unsigned int retryIdx = 0; retryIdx < cfgOptionUInt(cfgOptJobRetry); retryIdx++)
+            varLstAdd(result, varNewUInt64(retryIdx == 0 ? 0 : cfgOptionUInt64(cfgOptJobRetryInterval)));
+    }
+    MEM_CONTEXT_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(VARIANT_LIST, result);
 }
 
 /**********************************************************************************************************************************/
@@ -112,15 +116,14 @@ cfgCommandName(void)
     ASSERT(configLocal != NULL);
     ASSERT(configLocal->command < cfgCmdNone);
 
-    FUNCTION_TEST_RETURN(cfgParseCommandName(configLocal->command));
+    FUNCTION_TEST_RETURN_CONST(STRINGZ, cfgParseCommandName(configLocal->command));
 }
 
 String *
 cfgCommandRoleName(void)
 {
     FUNCTION_TEST_VOID();
-
-    FUNCTION_TEST_RETURN(cfgParseCommandRoleName(cfgCommand(), cfgCommandRole(), COLON_STR));
+    FUNCTION_TEST_RETURN(STRING, cfgParseCommandRoleName(cfgCommand(), cfgCommandRole()));
 }
 
 /**********************************************************************************************************************************/
@@ -140,7 +143,7 @@ cfgCommandParam(void)
         MEM_CONTEXT_END();
     }
 
-    FUNCTION_TEST_RETURN(configLocal->paramList);
+    FUNCTION_TEST_RETURN(STRING_LIST, configLocal->paramList);
 }
 
 /**********************************************************************************************************************************/
@@ -149,7 +152,7 @@ cfgExe(void)
 {
     FUNCTION_TEST_VOID();
     ASSERT(configLocal != NULL);
-    FUNCTION_TEST_RETURN(configLocal->exe);
+    FUNCTION_TEST_RETURN(STRING, configLocal->exe);
 }
 
 /**********************************************************************************************************************************/
@@ -163,6 +166,7 @@ cfgLockRequired(void)
 
     // Local roles never take a lock and the remote role has special logic for locking
     FUNCTION_TEST_RETURN(
+        BOOL,
         // If a lock is required for the command and the role is main
         (configLocal->lockRequired && cfgCommandRole() == cfgCmdRoleMain) ||
         // Or any command when the role is async
@@ -178,7 +182,7 @@ cfgLockRemoteRequired(void)
     ASSERT(configLocal != NULL);
     ASSERT(configLocal->command != cfgCmdNone);
 
-    FUNCTION_TEST_RETURN(configLocal->lockRemoteRequired);
+    FUNCTION_TEST_RETURN(BOOL, configLocal->lockRemoteRequired);
 }
 
 /**********************************************************************************************************************************/
@@ -190,7 +194,7 @@ cfgLockType(void)
     ASSERT(configLocal != NULL);
     ASSERT(configLocal->command != cfgCmdNone);
 
-    FUNCTION_TEST_RETURN(configLocal->lockType);
+    FUNCTION_TEST_RETURN(ENUM, configLocal->lockType);
 }
 
 /**********************************************************************************************************************************/
@@ -203,6 +207,7 @@ cfgLogFile(void)
     ASSERT(configLocal->command != cfgCmdNone);
 
     FUNCTION_TEST_RETURN(
+        BOOL,
         // If the command always logs to a file
         configLocal->logFile ||
         // Or log-level-file was explicitly set as a param/env var
@@ -220,7 +225,7 @@ cfgLogLevelDefault(void)
     ASSERT(configLocal != NULL);
     ASSERT(configLocal->command != cfgCmdNone);
 
-    FUNCTION_TEST_RETURN(configLocal->logLevelDefault);
+    FUNCTION_TEST_RETURN(ENUM, configLocal->logLevelDefault);
 }
 
 /**********************************************************************************************************************************/
@@ -234,7 +239,38 @@ cfgOptionGroup(ConfigOption optionId)
     ASSERT(configLocal != NULL);
     ASSERT(optionId < CFG_OPTION_TOTAL);
 
-    FUNCTION_TEST_RETURN(configLocal->option[optionId].group);
+    FUNCTION_TEST_RETURN(BOOL, configLocal->option[optionId].group);
+}
+
+/**********************************************************************************************************************************/
+const char *
+cfgOptionGroupName(const ConfigOptionGroup groupId, const unsigned int groupIdx)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(ENUM, groupId);
+        FUNCTION_TEST_PARAM(UINT, groupIdx);
+    FUNCTION_TEST_END();
+
+    ASSERT(configLocal != NULL);
+    ASSERT(groupId < CFG_OPTION_GROUP_TOTAL);
+    ASSERT(groupIdx < configLocal->optionGroup[groupId].indexTotal);
+
+    // Generate names for the group the first time one is requested
+    ConfigOptionGroupData *const group = &configLocal->optionGroup[groupId];
+
+    if (group->indexName == NULL)
+    {
+        MEM_CONTEXT_BEGIN(configLocal->memContext)
+        {
+            group->indexName = memNew(sizeof(String *) * group->indexTotal);
+
+            for (unsigned int groupIdx = 0; groupIdx < group->indexTotal; groupIdx++)
+                group->indexName[groupIdx] = strNewFmt("%s%u", group->name, group->indexMap[groupIdx] + 1);
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_TEST_RETURN_CONST(STRINGZ, strZ(group->indexName[groupIdx]));
 }
 
 /**********************************************************************************************************************************/
@@ -249,7 +285,7 @@ cfgOptionGroupId(ConfigOption optionId)
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal->option[optionId].group);
 
-    FUNCTION_TEST_RETURN(configLocal->option[optionId].groupId);
+    FUNCTION_TEST_RETURN(UINT, configLocal->option[optionId].groupId);
 }
 
 /**********************************************************************************************************************************/
@@ -264,7 +300,7 @@ cfgOptionGroupIdxDefault(ConfigOptionGroup groupId)
     ASSERT(groupId < CFG_OPTION_GROUP_TOTAL);
     ASSERT(configLocal->optionGroup[groupId].indexDefaultExists);
 
-    FUNCTION_TEST_RETURN(configLocal->optionGroup[groupId].indexDefault);
+    FUNCTION_TEST_RETURN(UINT, configLocal->optionGroup[groupId].indexDefault);
 }
 
 /**********************************************************************************************************************************/
@@ -280,7 +316,7 @@ cfgOptionGroupIdxToKey(ConfigOptionGroup groupId, unsigned int groupIdx)
     ASSERT(groupId < CFG_OPTION_GROUP_TOTAL);
     ASSERT(groupIdx < configLocal->optionGroup[groupId].indexTotal);
 
-    FUNCTION_TEST_RETURN(configLocal->optionGroup[groupId].indexMap[groupIdx] + 1);
+    FUNCTION_TEST_RETURN(UINT, configLocal->optionGroup[groupId].indexMap[groupIdx] + 1);
 }
 
 /**********************************************************************************************************************************/
@@ -314,7 +350,7 @@ cfgOptionKeyToIdx(ConfigOption optionId, unsigned int key)
             THROW_FMT(AssertError, "key '%u' is not valid for '%s' option", key, configLocal->option[optionId].name);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(UINT, result);
 }
 
 /**********************************************************************************************************************************/
@@ -328,7 +364,7 @@ cfgOptionGroupIdxTotal(ConfigOptionGroup groupId)
     ASSERT(configLocal != NULL);
     ASSERT(groupId < CFG_OPTION_GROUP_TOTAL);
 
-    FUNCTION_TEST_RETURN(configLocal->optionGroup[groupId].indexTotal);
+    FUNCTION_TEST_RETURN(UINT, configLocal->optionGroup[groupId].indexTotal);
 }
 
 /**********************************************************************************************************************************/
@@ -342,7 +378,7 @@ cfgOptionGroupValid(ConfigOptionGroup groupId)
     ASSERT(configLocal != NULL);
     ASSERT(groupId < CFG_OPTION_GROUP_TOTAL);
 
-    FUNCTION_TEST_RETURN(configLocal->optionGroup[groupId].valid);
+    FUNCTION_TEST_RETURN(BOOL, configLocal->optionGroup[groupId].valid);
 }
 
 /**********************************************************************************************************************************/
@@ -358,8 +394,9 @@ cfgOptionIdxDefault(ConfigOption optionId)
     ASSERT(
         !configLocal->option[optionId].group || configLocal->optionGroup[configLocal->option[optionId].groupId].indexDefaultExists);
 
-    FUNCTION_TEST_RETURN(
-        configLocal->option[optionId].group ? configLocal->optionGroup[configLocal->option[optionId].groupId].indexDefault : 0);
+    const ConfigOptionData *const option = &configLocal->option[optionId];
+
+    FUNCTION_TEST_RETURN(UINT, option->group ? configLocal->optionGroup[option->groupId].indexDefault : 0);
 }
 
 /**********************************************************************************************************************************/
@@ -373,8 +410,9 @@ cfgOptionIdxTotal(ConfigOption optionId)
     ASSERT(configLocal != NULL);
     ASSERT(optionId < CFG_OPTION_TOTAL);
 
-    FUNCTION_TEST_RETURN(
-        configLocal->option[optionId].group ? configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal : 1);
+    const ConfigOptionData *const option = &configLocal->option[optionId];
+
+    FUNCTION_TEST_RETURN(UINT, option->group ? configLocal->optionGroup[option->groupId].indexTotal : 1);
 }
 
 /**********************************************************************************************************************************/
@@ -388,10 +426,12 @@ cfgOptionDefault(ConfigOption optionId)
     ASSERT(configLocal != NULL);
     ASSERT(optionId < CFG_OPTION_TOTAL);
 
-    if (configLocal->option[optionId].defaultValue == NULL)
-        configLocal->option[optionId].defaultValue = cfgParseOptionDefault(cfgCommand(), optionId);
+    ConfigOptionData *const option = &configLocal->option[optionId];
 
-    FUNCTION_TEST_RETURN(configLocal->option[optionId].defaultValue);
+    if (option->defaultValue == NULL)
+        option->defaultValue = cfgParseOptionDefault(cfgCommand(), optionId);
+
+    FUNCTION_TEST_RETURN_CONST(STRING, option->defaultValue);
 }
 
 void
@@ -413,16 +453,19 @@ cfgOptionDefaultSet(ConfigOption optionId, const Variant *defaultValue)
         defaultValue = varDup(defaultValue);
 
         // Set the default value
-        configLocal->option[optionId].defaultValue = varStr(defaultValue);
+        ConfigOptionData *const option = &configLocal->option[optionId];
+        option->defaultValue = varStr(defaultValue);
 
         // Copy the value to option indexes that are marked as default so the default can be retrieved quickly
         for (unsigned int optionIdx = 0; optionIdx < cfgOptionIdxTotal(optionId); optionIdx++)
         {
-            if (configLocal->option[optionId].index[optionIdx].source == cfgSourceDefault)
+            ConfigOptionValue *const optionValue = &option->index[optionIdx];
+
+            if (optionValue->source == cfgSourceDefault)
             {
-                configLocal->option[optionId].index[optionIdx].set = true;
-                configLocal->option[optionId].index[optionIdx].value.string = varStr(defaultValue);
-                configLocal->option[optionId].index[optionIdx].display = NULL;
+                optionValue->set = true;
+                optionValue->value.string = varStr(defaultValue);
+                optionValue->display = NULL;
             }
         }
     }
@@ -446,22 +489,22 @@ cfgOptionDisplayVar(const Variant *const value, const ConfigOptionType optionTyp
 
     if (varType(value) == varTypeString)
     {
-        FUNCTION_TEST_RETURN(varStr(value));
+        FUNCTION_TEST_RETURN_CONST(STRING, varStr(value));
     }
     else if (optionType == cfgOptTypeBoolean)
     {
-        FUNCTION_TEST_RETURN(varBool(value) ? TRUE_STR : FALSE_STR);
+        FUNCTION_TEST_RETURN_CONST(STRING, varBool(value) ? TRUE_STR : FALSE_STR);
     }
     else if (optionType == cfgOptTypeTime)
     {
-        FUNCTION_TEST_RETURN(strNewDbl((double)varInt64(value) / MSEC_PER_SEC));
+        FUNCTION_TEST_RETURN_CONST(STRING, strNewDbl((double)varInt64(value) / MSEC_PER_SEC));
     }
     else if (optionType == cfgOptTypeStringId)
     {
-        FUNCTION_TEST_RETURN(strIdToStr(varUInt64(value)));
+        FUNCTION_TEST_RETURN_CONST(STRING, strIdToStr(varUInt64(value)));
     }
 
-    FUNCTION_TEST_RETURN(varStrForce(value));
+    FUNCTION_TEST_RETURN(STRING, varStrForce(value));
 }
 
 const String *
@@ -474,10 +517,9 @@ cfgOptionIdxDisplay(const ConfigOption optionId, const unsigned int optionIdx)
 
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal != NULL);
-    ASSERT(
-        (!configLocal->option[optionId].group && optionIdx == 0) ||
-        (configLocal->option[optionId].group && optionIdx <
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal));
+    ASSERT_DECLARE(const bool group = configLocal->option[optionId].group);
+    ASSERT_DECLARE(const unsigned int indexTotal = configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal);
+    ASSERT((!group && optionIdx == 0) || (group && optionIdx < indexTotal));
 
     // Check that the option is valid for the current command
     if (!cfgOptionValid(optionId))
@@ -487,7 +529,7 @@ cfgOptionIdxDisplay(const ConfigOption optionId, const unsigned int optionIdx)
     ConfigOptionValue *const option = &configLocal->option[optionId].index[optionIdx];
 
     if (option->display != NULL)
-        FUNCTION_TEST_RETURN(option->display);
+        FUNCTION_TEST_RETURN_CONST(STRING, option->display);
 
     // Generate the display value based on the type
     MEM_CONTEXT_BEGIN(configLocal->memContext)
@@ -497,7 +539,7 @@ cfgOptionIdxDisplay(const ConfigOption optionId, const unsigned int optionIdx)
     MEM_CONTEXT_END();
 
 
-    FUNCTION_TEST_RETURN(option->display);
+    FUNCTION_TEST_RETURN_CONST(STRING, option->display);
 }
 
 const String *
@@ -507,98 +549,7 @@ cfgOptionDisplay(const ConfigOption optionId)
         FUNCTION_TEST_PARAM(ENUM, optionId);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(cfgOptionIdxDisplay(optionId, cfgOptionIdxDefault(optionId)));
-}
-
-/**********************************************************************************************************************************/
-String *
-cfgOptionHostPort(ConfigOption optionId, unsigned int *port)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(ENUM, optionId);
-        FUNCTION_TEST_PARAM_P(UINT, port);
-    FUNCTION_TEST_END();
-
-    FUNCTION_TEST_RETURN(cfgOptionIdxHostPort(optionId, cfgOptionIdxDefault(optionId), port));
-}
-
-String *
-cfgOptionIdxHostPort(ConfigOption optionId, unsigned int optionIdx, unsigned int *port)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(ENUM, optionId);
-        FUNCTION_TEST_PARAM(UINT, optionIdx);
-        FUNCTION_TEST_PARAM_P(UINT, port);
-    FUNCTION_TEST_END();
-
-    ASSERT(optionId < CFG_OPTION_TOTAL);
-    ASSERT(configLocal != NULL);
-    ASSERT(
-        (!configLocal->option[optionId].group && optionIdx == 0) ||
-        (configLocal->option[optionId].group && optionIdx <
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal));
-    ASSERT(port != NULL);
-
-    String *result = NULL;
-
-    // Proceed if option is valid and has a value
-    if (cfgOptionIdxTest(optionId, optionIdx))
-    {
-        MEM_CONTEXT_TEMP_BEGIN()
-        {
-            const String *host = cfgOptionIdxStr(optionId, optionIdx);
-
-            // If the host contains a colon then it has a port appended
-            if (strChr(host, ':') != -1)
-            {
-                const StringList *hostPart = strLstNewSplitZ(host, ":");
-
-                // More than one colon is invalid
-                if (strLstSize(hostPart) > 2)
-                {
-                    THROW_FMT(
-                        OptionInvalidError,
-                        "'%s' is not valid for option '%s'"
-                            "\nHINT: is more than one port specified?",
-                        strZ(host), cfgOptionIdxName(optionId, optionIdx));
-                }
-
-                // Set the host
-                MEM_CONTEXT_PRIOR_BEGIN()
-                {
-                    result = strDup(strLstGet(hostPart, 0));
-                }
-                MEM_CONTEXT_PRIOR_END();
-
-                // Set the port and error if it is not a positive integer
-                TRY_BEGIN()
-                {
-                    *port = cvtZToUInt(strZ(strLstGet(hostPart, 1)));
-                }
-                CATCH(FormatError)
-                {
-                    THROW_FMT(
-                        OptionInvalidError,
-                        "'%s' is not valid for option '%s'"
-                            "\nHINT: port is not a positive integer.",
-                        strZ(host), cfgOptionIdxName(optionId, optionIdx));
-                }
-                TRY_END();
-            }
-            // Else there is no port and just copy the host
-            else
-            {
-                MEM_CONTEXT_PRIOR_BEGIN()
-                {
-                    result = strDup(host);
-                }
-                MEM_CONTEXT_PRIOR_END();
-            }
-        }
-        MEM_CONTEXT_TEMP_END();
-    }
-
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN_CONST(STRING, cfgOptionIdxDisplay(optionId, cfgOptionIdxDefault(optionId)));
 }
 
 /***********************************************************************************************************************************
@@ -613,7 +564,7 @@ cfgOptionName(ConfigOption optionId)
 
     ASSERT(optionId < CFG_OPTION_TOTAL);
 
-    FUNCTION_TEST_RETURN(cfgOptionIdxName(optionId, cfgOptionIdxDefault(optionId)));
+    FUNCTION_TEST_RETURN_CONST(STRINGZ, cfgOptionIdxName(optionId, cfgOptionIdxDefault(optionId)));
 }
 
 const char *
@@ -626,24 +577,38 @@ cfgOptionIdxName(ConfigOption optionId, unsigned int optionIdx)
 
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal != NULL);
-    ASSERT(
-        (!configLocal->option[optionId].group && optionIdx == 0) ||
-        (configLocal->option[optionId].group && optionIdx <
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal));
+    ASSERT_DECLARE(const bool group = configLocal->option[optionId].group);
+    ASSERT_DECLARE(const unsigned int indexTotal = configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal);
+    ASSERT((!group && optionIdx == 0) || (group && optionIdx < indexTotal));
 
-    if (configLocal->option[optionId].group)
+    // If an indexed option
+    ConfigOptionData *const option = &configLocal->option[optionId];
+
+    if (option->group)
     {
-        // This is somewhat less than ideal since memory is being allocated with each call, rather than caching prior results. In
-        // practice the number of allocations should be quite small so we'll ignore this for now.
-        String *name = strNewFmt(
-            "%s%u%s", configLocal->optionGroup[configLocal->option[optionId].groupId].name,
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexMap[optionIdx] + 1,
-            configLocal->option[optionId].name + strlen(configLocal->optionGroup[configLocal->option[optionId].groupId].name));
+        const ConfigOptionGroupData *const group = &configLocal->optionGroup[option->groupId];
 
-        FUNCTION_TEST_RETURN(strZ(name));
+        // Generate indexed names for the option the first time one is requested
+        if (option->indexName == NULL)
+        {
+            MEM_CONTEXT_BEGIN(configLocal->memContext)
+            {
+                option->indexName = memNew(sizeof(String *) * group->indexTotal);
+
+                for (unsigned int optionIdx = 0; optionIdx < group->indexTotal; optionIdx++)
+                {
+                    option->indexName[optionIdx] = strNewFmt(
+                        "%s%u%s", group->name, group->indexMap[optionIdx] + 1, option->name + strlen(group->name));
+                }
+            }
+            MEM_CONTEXT_END();
+        }
+
+        FUNCTION_TEST_RETURN_CONST(STRINGZ, strZ(option->indexName[optionIdx]));
     }
 
-    FUNCTION_TEST_RETURN(configLocal->option[optionId].name);
+    // Else not indexed
+    FUNCTION_TEST_RETURN_CONST(STRINGZ, option->name);
 }
 
 /**********************************************************************************************************************************/
@@ -654,7 +619,7 @@ cfgOptionNegate(ConfigOption optionId)
         FUNCTION_TEST_PARAM(ENUM, optionId);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(cfgOptionIdxNegate(optionId, cfgOptionIdxDefault(optionId)));
+    FUNCTION_TEST_RETURN(BOOL, cfgOptionIdxNegate(optionId, cfgOptionIdxDefault(optionId)));
 }
 
 bool
@@ -667,12 +632,11 @@ cfgOptionIdxNegate(ConfigOption optionId, unsigned int optionIdx)
 
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal != NULL);
-    ASSERT(
-        (!configLocal->option[optionId].group && optionIdx == 0) ||
-        (configLocal->option[optionId].group && optionIdx <
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal));
+    ASSERT_DECLARE(const bool group = configLocal->option[optionId].group);
+    ASSERT_DECLARE(const unsigned int indexTotal = configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal);
+    ASSERT((!group && optionIdx == 0) || (group && optionIdx < indexTotal));
 
-    FUNCTION_TEST_RETURN(configLocal->option[optionId].index[optionIdx].negate);
+    FUNCTION_TEST_RETURN(BOOL, configLocal->option[optionId].index[optionIdx].negate);
 }
 
 /**********************************************************************************************************************************/
@@ -683,7 +647,7 @@ cfgOptionReset(ConfigOption optionId)
         FUNCTION_TEST_PARAM(ENUM, optionId);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(cfgOptionIdxReset(optionId, cfgOptionIdxDefault(optionId)));
+    FUNCTION_TEST_RETURN(BOOL, cfgOptionIdxReset(optionId, cfgOptionIdxDefault(optionId)));
 }
 
 bool
@@ -696,12 +660,11 @@ cfgOptionIdxReset(ConfigOption optionId, unsigned int optionIdx)
 
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal != NULL);
-    ASSERT(
-        (!configLocal->option[optionId].group && optionIdx == 0) ||
-        (configLocal->option[optionId].group && optionIdx <
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal));
+    ASSERT_DECLARE(const bool group = configLocal->option[optionId].group);
+    ASSERT_DECLARE(const unsigned int indexTotal = configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal);
+    ASSERT((!group && optionIdx == 0) || (group && optionIdx < indexTotal));
 
-    FUNCTION_TEST_RETURN(configLocal->option[optionId].index[optionIdx].reset);
+    FUNCTION_TEST_RETURN(BOOL, configLocal->option[optionId].index[optionIdx].reset);
 }
 
 /**********************************************************************************************************************************/
@@ -719,32 +682,32 @@ cfgOptionIdxInternal(
 
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal != NULL);
-    ASSERT(
-        (!configLocal->option[optionId].group && optionIdx == 0) ||
-        (configLocal->option[optionId].group && optionIdx <
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal));
+    ASSERT_DECLARE(const bool group = configLocal->option[optionId].group);
+    ASSERT_DECLARE(const unsigned int indexTotal = configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal);
+    ASSERT((!group && optionIdx == 0) || (group && optionIdx < indexTotal));
 
     // Check that the option is valid for the current command
     if (!cfgOptionValid(optionId))
         THROW_FMT(AssertError, "option '%s' is not valid for the current command", cfgOptionIdxName(optionId, optionIdx));
 
     // If the option is not NULL then check it is the requested type
-    const ConfigOptionValue *const result = &configLocal->option[optionId].index[optionIdx];
+    ConfigOptionData *const option = &configLocal->option[optionId];
+    ConfigOptionValue *const result = &option->index[optionIdx];
 
     if (result->set)
     {
-        if (configLocal->option[optionId].dataType != typeRequested)
+        if (option->dataType != typeRequested)
         {
             THROW_FMT(
                 AssertError, "option '%s' is type %u but %u was requested", cfgOptionIdxName(optionId, optionIdx),
-                configLocal->option[optionId].dataType, typeRequested);
+                option->dataType, typeRequested);
         }
     }
     // Else check the option is allowed to be NULL
     else if (!nullAllowed)
         THROW_FMT(AssertError, "option '%s' is null but non-null was requested", cfgOptionIdxName(optionId, optionIdx));
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN_TYPE_P(ConfigOptionValue, result);
 }
 
 Variant *
@@ -756,39 +719,42 @@ cfgOptionIdxVar(const ConfigOption optionId, const unsigned int optionIdx)
     FUNCTION_TEST_END();
 
     ASSERT(configLocal != NULL);
-    ASSERT(
-        (!configLocal->option[optionId].group && optionIdx == 0) ||
-        (configLocal->option[optionId].group && optionIdx <
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal));
+    ASSERT_DECLARE(const bool group = configLocal->option[optionId].group);
+    ASSERT_DECLARE(const unsigned int indexTotal = configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal);
+    ASSERT((!group && optionIdx == 0) || (group && optionIdx < indexTotal));
 
-    if (configLocal->option[optionId].index[optionIdx].set)
+    const ConfigOptionData *const option = &configLocal->option[optionId];
+
+    if (option->index[optionIdx].set)
     {
-        switch (configLocal->option[optionId].dataType)
+        const ConfigOptionValueType *const optionValueType = &option->index[optionIdx].value;
+
+        switch (option->dataType)
         {
             case cfgOptDataTypeBoolean:
-                FUNCTION_TEST_RETURN(varNewBool(configLocal->option[optionId].index[optionIdx].value.boolean));
+                FUNCTION_TEST_RETURN(VARIANT, varNewBool(optionValueType->boolean));
 
             case cfgOptDataTypeHash:
-                FUNCTION_TEST_RETURN(varNewKv(kvDup(configLocal->option[optionId].index[optionIdx].value.keyValue)));
+                FUNCTION_TEST_RETURN(VARIANT, varNewKv(kvDup(optionValueType->keyValue)));
 
             case cfgOptDataTypeInteger:
-                FUNCTION_TEST_RETURN(varNewInt64(configLocal->option[optionId].index[optionIdx].value.integer));
+                FUNCTION_TEST_RETURN(VARIANT, varNewInt64(optionValueType->integer));
 
             case cfgOptDataTypeList:
-                FUNCTION_TEST_RETURN(varNewVarLst(configLocal->option[optionId].index[optionIdx].value.list));
+                FUNCTION_TEST_RETURN(VARIANT, varNewVarLst(optionValueType->list));
 
             case cfgOptDataTypeStringId:
-                FUNCTION_TEST_RETURN(varNewUInt64(configLocal->option[optionId].index[optionIdx].value.stringId));
+                FUNCTION_TEST_RETURN(VARIANT, varNewUInt64(optionValueType->stringId));
 
             default:
-                ASSERT(configLocal->option[optionId].dataType == cfgOptDataTypeString);
+                ASSERT(option->dataType == cfgOptDataTypeString);
                 break;
         }
 
-        FUNCTION_TEST_RETURN(varNewStr(configLocal->option[optionId].index[optionIdx].value.string));
+        FUNCTION_TEST_RETURN(VARIANT, varNewStr(optionValueType->string));
     }
 
-    FUNCTION_TEST_RETURN(NULL);
+    FUNCTION_TEST_RETURN(VARIANT, NULL);
 }
 
 bool
@@ -961,25 +927,28 @@ cfgOptionIdxSet(ConfigOption optionId, unsigned int optionIdx, ConfigSource sour
 
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal != NULL);
-    ASSERT(
-        (!configLocal->option[optionId].group && optionIdx == 0) ||
-        (configLocal->option[optionId].group && optionIdx <
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal));
+    ASSERT_DECLARE(const bool group = configLocal->option[optionId].group);
+    ASSERT_DECLARE(const unsigned int indexTotal = configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal);
+    ASSERT((!group && optionIdx == 0) || (group && optionIdx < indexTotal));
 
     // Set the source
-    configLocal->option[optionId].index[optionIdx].source = source;
+    ConfigOptionData *const option = &configLocal->option[optionId];
+    ConfigOptionValue *const optionValue = &option->index[optionIdx];
+    optionValue->source = source;
 
     // Only set value if it is not null
+    ConfigOptionValueType *const optionValueType = &optionValue->value;
+
     if (value != NULL)
     {
-        switch (configLocal->option[optionId].dataType)
+        switch (option->dataType)
         {
             case cfgOptDataTypeBoolean:
-                configLocal->option[optionId].index[optionIdx].value.boolean = varBool(value);
+                optionValueType->boolean = varBool(value);
                 break;
 
             case cfgOptDataTypeInteger:
-                configLocal->option[optionId].index[optionIdx].value.integer = varInt64(value);
+                optionValueType->integer = varInt64(value);
                 break;
 
             case cfgOptDataTypeString:
@@ -988,7 +957,7 @@ cfgOptionIdxSet(ConfigOption optionId, unsigned int optionIdx, ConfigSource sour
                 {
                     MEM_CONTEXT_BEGIN(configLocal->memContext)
                     {
-                        configLocal->option[optionId].index[optionIdx].value.string = strDup(varStr(value));
+                        optionValueType->string = strDup(varStr(value));
                     }
                     MEM_CONTEXT_END();
                 }
@@ -1004,29 +973,30 @@ cfgOptionIdxSet(ConfigOption optionId, unsigned int optionIdx, ConfigSource sour
             case cfgOptDataTypeStringId:
             {
                 if (varType(value) == varTypeUInt64)
-                    configLocal->option[optionId].index[optionIdx].value.stringId = varUInt64(value);
+                    optionValueType->stringId = varUInt64(value);
                 else
-                    configLocal->option[optionId].index[optionIdx].value.stringId = strIdFromStr(varStr(value));
+                    optionValueType->stringId = strIdFromStr(varStr(value));
 
                 break;
             }
 
             default:
-                THROW_FMT(AssertError, "set not available for option data type %u", configLocal->option[optionId].dataType);
+                THROW_FMT(AssertError, "set not available for option data type %u", option->dataType);
         }
 
-        configLocal->option[optionId].index[optionIdx].set = true;
+        optionValue->set = true;
     }
     else
     {
-        configLocal->option[optionId].index[optionIdx].set = false;
+        optionValue->set = false;
 
         // Pointer values need to be set to null since they can be accessed when the option is not set, e.g. cfgOptionStrNull().
-        configLocal->option[optionId].index[optionIdx].value.string = NULL;
+        // Setting string to NULL suffices to set the other pointers in the union to NULL.
+        optionValueType->string = NULL;
     }
 
     // Clear the display value, which will be generated when needed
-    configLocal->option[optionId].index[optionIdx].display = NULL;
+    optionValue->display = NULL;
 
     FUNCTION_TEST_RETURN_VOID();
 }
@@ -1039,7 +1009,7 @@ cfgOptionSource(ConfigOption optionId)
         FUNCTION_TEST_PARAM(ENUM, optionId);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(cfgOptionIdxSource(optionId, cfgOptionIdxDefault(optionId)));
+    FUNCTION_TEST_RETURN(ENUM, cfgOptionIdxSource(optionId, cfgOptionIdxDefault(optionId)));
 }
 
 ConfigSource
@@ -1052,12 +1022,11 @@ cfgOptionIdxSource(ConfigOption optionId, unsigned int optionIdx)
 
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal != NULL);
-    ASSERT(
-        (!configLocal->option[optionId].group && optionIdx == 0) ||
-        (configLocal->option[optionId].group && optionIdx <
-            configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal));
+    ASSERT_DECLARE(const bool group = configLocal->option[optionId].group);
+    ASSERT_DECLARE(const unsigned int indexTotal = configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal);
+    ASSERT((!group && optionIdx == 0) || (group && optionIdx < indexTotal));
 
-    FUNCTION_TEST_RETURN(configLocal->option[optionId].index[optionIdx].source);
+    FUNCTION_TEST_RETURN(ENUM, configLocal->option[optionId].index[optionIdx].source);
 }
 
 /**********************************************************************************************************************************/
@@ -1068,7 +1037,7 @@ cfgOptionTest(ConfigOption optionId)
         FUNCTION_TEST_PARAM(ENUM, optionId);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(cfgOptionIdxTest(optionId, cfgOptionIdxDefault(optionId)));
+    FUNCTION_TEST_RETURN(BOOL, cfgOptionIdxTest(optionId, cfgOptionIdxDefault(optionId)));
 }
 
 bool
@@ -1081,13 +1050,11 @@ cfgOptionIdxTest(ConfigOption optionId, unsigned int optionIdx)
 
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal != NULL);
-    ASSERT(
-        !cfgOptionValid(optionId) ||
-        ((!configLocal->option[optionId].group && optionIdx == 0) ||
-         (configLocal->option[optionId].group && optionIdx <
-          configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal)));
+    ASSERT_DECLARE(const bool group = configLocal->option[optionId].group);
+    ASSERT_DECLARE(const unsigned int indexTotal = configLocal->optionGroup[configLocal->option[optionId].groupId].indexTotal);
+    ASSERT(!cfgOptionValid(optionId) || ((!group && optionIdx == 0) || (group && optionIdx < indexTotal)));
 
-    FUNCTION_TEST_RETURN(cfgOptionValid(optionId) && configLocal->option[optionId].index[optionIdx].set);
+    FUNCTION_TEST_RETURN(BOOL, cfgOptionValid(optionId) && configLocal->option[optionId].index[optionIdx].set);
 }
 
 /**********************************************************************************************************************************/
@@ -1101,7 +1068,7 @@ cfgOptionValid(ConfigOption optionId)
     ASSERT(optionId < CFG_OPTION_TOTAL);
     ASSERT(configLocal != NULL);
 
-    FUNCTION_TEST_RETURN(configLocal->option[optionId].valid);
+    FUNCTION_TEST_RETURN(BOOL, configLocal->option[optionId].valid);
 }
 
 void
