@@ -6,7 +6,7 @@ Configuration Load
 
 #include <sys/types.h>
 
-#include "common/type/string.h"
+#include "config/config.h"
 
 /***********************************************************************************************************************************
 Functions
@@ -14,6 +14,9 @@ Functions
 // Load the configuration
 void cfgLoad(unsigned int argListSize, const char *argList[]);
 
+// Generate log file path and name. Only the command role is configurable here because log settings may vary between commands.
+String *cfgLoadLogFileName(ConfigCommandRole commandRole);
+
 // Attempt to set the log file and turn file logging off if the file cannot be opened
 void cfgLoadLogFile(void);
 
