@@ -54,6 +54,7 @@ dbFreeResource(THIS_VOID)
     pckWriteU32P(protocolCommandParam(command), this->remoteIdx);
 
     protocolClientExecute(this->remoteClient, command, false);
+    protocolCommandFree(command);
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -75,7 +76,7 @@ dbNew(PgClient *client, ProtocolClient *remoteClient, const Storage *const stora
 
     Db *this = NULL;
 
-    OBJ_NEW_BEGIN(Db)
+    OBJ_NEW_BEGIN(Db, .childQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
         this = OBJ_NEW_ALLOC();
 
@@ -100,35 +101,46 @@ dbNew(PgClient *client, ProtocolClient *remoteClient, const Storage *const stora
 /***********************************************************************************************************************************
 Execute a query
 ***********************************************************************************************************************************/
-static VariantList *
-dbQuery(Db *this, const String *query)
+static Pack *
+dbQuery(Db *this, const PgClientQueryResult resultType, const String *const query)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
+        FUNCTION_LOG_PARAM(STRING_ID, resultType);
         FUNCTION_LOG_PARAM(STRING, query);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
+    ASSERT(resultType != 0);
     ASSERT(query != NULL);
 
-    VariantList *result = NULL;
+    Pack *result = NULL;
 
     // Query remotely
     if (this->remoteClient != NULL)
     {
-        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_QUERY);
-        PackWrite *const param = protocolCommandParam(command);
+        MEM_CONTEXT_TEMP_BEGIN()
+        {
+            ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_QUERY);
+            PackWrite *const param = protocolCommandParam(command);
 
-        pckWriteU32P(param, this->remoteIdx);
-        pckWriteStrP(param, query);
+            pckWriteU32P(param, this->remoteIdx);
+            pckWriteStrIdP(param, resultType);
+            pckWriteStrP(param, query);
 
-        result = varVarLst(jsonToVar(pckReadStrP(protocolClientExecute(this->remoteClient, command, true))));
+            MEM_CONTEXT_PRIOR_BEGIN()
+            {
+                result = pckReadPackP(protocolClientExecute(this->remoteClient, command, true));
+            }
+            MEM_CONTEXT_PRIOR_END();
+        }
+        MEM_CONTEXT_TEMP_END();
     }
     // Else locally
     else
-        result = pgClientQuery(this->client, query);
+        result = pgClientQuery(this->client, query, resultType);
 
-    FUNCTION_LOG_RETURN(VARIANT_LIST, result);
+    FUNCTION_LOG_RETURN(PACK, result);
 }
 
 /***********************************************************************************************************************************
@@ -145,7 +157,7 @@ dbExec(Db *this, const String *command)
     ASSERT(this != NULL);
     ASSERT(command != NULL);
 
-    CHECK(AssertError, dbQuery(this, command) == NULL, "exec returned data");
+    CHECK(AssertError, dbQuery(this, pgClientQueryResultNone, command) == NULL, "exec returned data");
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -153,8 +165,8 @@ dbExec(Db *this, const String *command)
 /***********************************************************************************************************************************
 Execute a query that returns a single row and column
 ***********************************************************************************************************************************/
-static Variant *
-dbQueryColumn(Db *this, const String *query)
+static PackRead *
+dbQueryColumn(Db *const this, const String *const query)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
@@ -164,19 +176,14 @@ dbQueryColumn(Db *this, const String *query)
     ASSERT(this != NULL);
     ASSERT(query != NULL);
 
-    VariantList *result = dbQuery(this, query);
-
-    CHECK(AssertError, varLstSize(result) == 1, "query must return one column");
-    CHECK(AssertError, varLstSize(varVarLst(varLstGet(result, 0))) == 1, "query must return one row");
-
-    FUNCTION_LOG_RETURN(VARIANT, varLstGet(varVarLst(varLstGet(result, 0)), 0));
+    FUNCTION_LOG_RETURN(PACK_READ, pckReadNew(dbQuery(this, pgClientQueryResultColumn, query)));
 }
 
 /***********************************************************************************************************************************
 Execute a query that returns a single row
 ***********************************************************************************************************************************/
-static VariantList *
-dbQueryRow(Db *this, const String *query)
+static PackRead *
+dbQueryRow(Db *const this, const String *const query)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
@@ -186,11 +193,7 @@ dbQueryRow(Db *this, const String *query)
     ASSERT(this != NULL);
     ASSERT(query != NULL);
 
-    VariantList *result = dbQuery(this, query);
-
-    CHECK(AssertError, varLstSize(result) == 1, "query must return one row");
-
-    FUNCTION_LOG_RETURN(VARIANT_LIST, varVarLst(varLstGet(result, 0)));
+    FUNCTION_LOG_RETURN(PACK_READ, pckReadNew(dbQuery(this, pgClientQueryResultRow, query)));
 }
 
 /***********************************************************************************************************************************
@@ -208,7 +211,7 @@ dbIsInRecovery(Db *const this)
     bool result = false;
 
     if (dbPgVersion(this) >= PG_VERSION_HOT_STANDBY)
-        result = varBool(dbQueryColumn(this, STRDEF("select pg_catalog.pg_is_in_recovery()")));
+        result = pckReadBoolP(dbQueryColumn(this, STRDEF("select pg_catalog.pg_is_in_recovery()")));
 
     FUNCTION_LOG_RETURN(BOOL, result);
 }
@@ -251,9 +254,9 @@ dbOpen(Db *this)
         // Set client encoding to UTF8.  This is the only encoding (other than ASCII) that we can safely work with.
         dbExec(this, STRDEF("set client_encoding = 'UTF8'"));
 
-        // Query the version and data_directory
-        VariantList *row = dbQueryRow(
-            this,
+        // Query the version and data_directory. Be sure the update the total in the null check below when adding/removing columns.
+        const Pack *const row = dbQuery(
+            this, pgClientQueryResultRow,
             STRDEF(
                 "select (select setting from pg_catalog.pg_settings where name = 'server_version_num')::int4,"
                 " (select setting from pg_catalog.pg_settings where name = 'data_directory')::text,"
@@ -262,9 +265,11 @@ dbOpen(Db *this)
                 " (select setting from pg_catalog.pg_settings where name = 'checkpoint_timeout')::int4"));
 
         // Check that none of the return values are null, which indicates the user cannot select some rows in pg_settings
-        for (unsigned int columnIdx = 0; columnIdx < varLstSize(row); columnIdx++)
+        PackRead *read = pckReadNew(row);
+
+        for (unsigned int columnIdx = 0; columnIdx < 5; columnIdx++)
         {
-            if (varLstGet(row, columnIdx) == NULL)
+            if (pckReadNullP(read, .id = columnIdx + 1))
             {
                 THROW(
                     DbQueryError,
@@ -274,25 +279,27 @@ dbOpen(Db *this)
             }
         }
 
+        // Restart the read to get the data
+        read = pckReadNew(row);
+
         // Strip the minor version off since we don't need it.  In the future it might be a good idea to warn users when they are
         // running an old minor version.
-        this->pub.pgVersion = varUIntForce(varLstGet(row, 0)) / 100 * 100;
+        this->pub.pgVersion = (unsigned int)pckReadI32P(read) / 100 * 100;
 
         // Store the data directory that PostgreSQL is running in, the archive mode, and archive command. These can be compared to
         // the configured pgBackRest directory, and archive settings checked for validity, when validating the configuration.
         // Also store the checkpoint timeout to warn in case a backup is requested without using the start-fast option.
         MEM_CONTEXT_BEGIN(this->pub.memContext)
         {
-            this->pub.pgDataPath = strDup(varStr(varLstGet(row, 1)));
-            this->pub.archiveMode = strDup(varStr(varLstGet(row, 2)));
-            this->pub.archiveCommand = strDup(varStr(varLstGet(row, 3)));
-            this->pub.checkpointTimeout = (TimeMSec)(varUIntForce(varLstGet(row, 4))) * MSEC_PER_SEC;
+            this->pub.pgDataPath = pckReadStrP(read);
+            this->pub.archiveMode = pckReadStrP(read);
+            this->pub.archiveCommand = pckReadStrP(read);
+            this->pub.checkpointTimeout = (TimeMSec)pckReadI32P(read) * MSEC_PER_SEC;
         }
         MEM_CONTEXT_END();
 
         // Set application name to help identify the backup session
-        if (dbPgVersion(this) >= PG_VERSION_APPLICATION_NAME)
-            dbExec(this, strNewFmt("set application_name = '%s'", strZ(this->applicationName)));
+        dbExec(this, strNewFmt("set application_name = '%s'", strZ(this->applicationName)));
 
         // There is no need to have parallelism enabled in a backup session. In particular, 9.6 marks pg_stop_backup() as
         // parallel-safe but an error will be thrown if pg_stop_backup() is run in a worker.
@@ -325,26 +332,26 @@ dbBackupStartQuery(unsigned int pgVersion, bool startFast)
         strNew(),
         "select lsn::text as lsn,\n"
         "       pg_catalog.pg_%sfile_name(lsn)::text as wal_segment_name\n"
-        "  from pg_catalog.pg_start_backup('" PROJECT_NAME " backup started at ' || current_timestamp",
-        strZ(pgWalName(pgVersion)));
+        "  from pg_catalog.pg_%s('" PROJECT_NAME " backup started at ' || current_timestamp",
+        strZ(pgWalName(pgVersion)), pgVersion >= PG_VERSION_15 ? "backup_start" : "start_backup");
 
     // Start backup after immediate checkpoint
     if (startFast)
     {
-        strCatFmt(result, ", " TRUE_Z);
+        strCatZ(result, ", " TRUE_Z);
     }
     // Else start backup at the next scheduled checkpoint
-    else if (pgVersion >= PG_VERSION_84)
-        strCatFmt(result, ", " FALSE_Z);
+    else
+        strCatZ(result, ", " FALSE_Z);
 
     // Use non-exclusive backup mode when available
-    if (pgVersion >= PG_VERSION_96)
-        strCatFmt(result, ", " FALSE_Z);
+    if (pgVersion >= PG_VERSION_96 && pgVersion <= PG_VERSION_14)
+        strCatZ(result, ", " FALSE_Z);
 
     // Complete query
-    strCatFmt(result, ") as lsn");
+    strCatZ(result, ") as lsn");
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 DbBackupStartResult
@@ -365,7 +372,7 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
     {
         // Acquire the backup advisory lock to make sure that backups are not running from multiple backup servers against the same
         // database cluster.  This lock helps make the stop-auto option safe.
-        if (!varBool(dbQueryColumn(this, STRDEF("select pg_catalog.pg_try_advisory_lock(" PG_BACKUP_ADVISORY_LOCK ")::bool"))))
+        if (!pckReadBoolP(dbQueryColumn(this, STRDEF("select pg_catalog.pg_try_advisory_lock(" PG_BACKUP_ADVISORY_LOCK ")::bool"))))
         {
             THROW(
                 LockAcquireError,
@@ -381,7 +388,7 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
             // Feature is not needed for PostgreSQL >= 9.6 since backups are run in non-exclusive mode
             if (dbPgVersion(this) < PG_VERSION_96)
             {
-                if (varBool(dbQueryColumn(this, STRDEF("select pg_catalog.pg_is_in_backup()::bool"))))
+                if (pckReadBoolP(dbQueryColumn(this, STRDEF("select pg_catalog.pg_is_in_backup()::bool"))))
                 {
                     LOG_WARN(
                         "the cluster is already in backup mode but no " PROJECT_NAME " backup process is running."
@@ -407,7 +414,7 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
 
         if (archiveCheck)
         {
-            walSegmentCheck = varStr(
+            walSegmentCheck = pckReadStrP(
                 dbQueryColumn(
                     this,
                     strNewFmt(
@@ -417,12 +424,12 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
         }
 
         // Start backup
-        VariantList *row = dbQueryRow(this, dbBackupStartQuery(dbPgVersion(this), startFast));
+        PackRead *const read = dbQueryRow(this, dbBackupStartQuery(dbPgVersion(this), startFast));
 
         // Make sure the backup start checkpoint was written to pg_control. This helps ensure that we have a consistent view of the
         // storage with PostgreSQL.
         const PgControl pgControl = pgControlFromFile(this->storage);
-        const String *const lsnStart = varStr(varLstGet(row, 0));
+        const String *const lsnStart = pckReadStrP(read);
 
         if (pgControl.checkpoint < pgLsnFromStr(lsnStart))
         {
@@ -432,7 +439,7 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
         }
 
         // If archive check then make sure WAL segment was switched on start backup
-        const String *const walSegmentName = varStr(varLstGet(row, 1));
+        const String *const walSegmentName = pckReadStrP(read);
 
         if (archiveCheck && strEq(walSegmentCheck, walSegmentName))
         {
@@ -484,7 +491,7 @@ dbBackupStopQuery(unsigned int pgVersion)
         "       pg_catalog.pg_%sfile_name(lsn)::text as wal_segment_name",
         strZ(pgWalName(pgVersion)));
 
-    // For PostgreSQL >= 9.6 the backup label and tablespace map are returned from pg_stop_backup
+    // For PostgreSQL >= 9.6 the backup label and tablespace map are returned
     if (pgVersion >= PG_VERSION_96)
     {
         strCatZ(
@@ -495,26 +502,32 @@ dbBackupStopQuery(unsigned int pgVersion)
     }
 
     // Build stop backup function
-    strCatZ(
+    strCatFmt(
         result,
         "\n"
-        "  from pg_catalog.pg_stop_backup(");
+        "  from pg_catalog.pg_%s(",
+        pgVersion >= PG_VERSION_15 ? "backup_stop" : "stop_backup");
 
     // Use non-exclusive backup mode when available
-    if (pgVersion >= PG_VERSION_96)
-        strCatFmt(result, FALSE_Z);
+    if (pgVersion >= PG_VERSION_96 && pgVersion <= PG_VERSION_14)
+        strCatZ(result, FALSE_Z);
 
-    // Disable archive checking in pg_stop_backup() since we do this elsewhere
+    // Disable archive checking since we do this elsewhere
     if (pgVersion >= PG_VERSION_10)
-        strCatFmt(result, ", " FALSE_Z);
+    {
+        if (pgVersion <= PG_VERSION_14)
+            strCatZ(result, ", ");
+
+        strCatZ(result, FALSE_Z);
+    }
 
     // Complete query
-    strCatFmt(result, ")");
+    strCatZ(result, ")");
 
     if (pgVersion < PG_VERSION_96)
-        strCatFmt(result, " as lsn");
+        strCatZ(result, " as lsn");
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 DbBackupStopResult
@@ -531,24 +544,28 @@ dbBackupStop(Db *this)
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Stop backup
-        VariantList *row = dbQueryRow(this, dbBackupStopQuery(dbPgVersion(this)));
-
-        // Check if the tablespace map is empty
-        bool tablespaceMapEmpty =
-            dbPgVersion(this) >= PG_VERSION_96 ? strSize(strTrim(strDup(varStr(varLstGet(row, 3))))) == 0 : false;
+        PackRead *const read = dbQueryRow(this, dbBackupStopQuery(dbPgVersion(this)));
 
         // Return results
         MEM_CONTEXT_PRIOR_BEGIN()
         {
-            result.lsn = strDup(varStr(varLstGet(row, 0)));
-            result.walSegmentName = strDup(varStr(varLstGet(row, 1)));
+            result.lsn = pckReadStrP(read);
+            result.walSegmentName = pckReadStrP(read);
 
             if (dbPgVersion(this) >= PG_VERSION_96)
             {
-                result.backupLabel = strDup(varStr(varLstGet(row, 2)));
+                result.backupLabel = pckReadStrP(read);
 
-                if (!tablespaceMapEmpty)
-                    result.tablespaceMap = strDup(varStr(varLstGet(row, 3)));
+                // Return the tablespace map if it is not empty
+                String *const tablespaceMap = pckReadStrP(read);
+                String *const tablespaceMapTrim = strTrim(strDup(tablespaceMap));
+
+                if (!strEmpty(tablespaceMapTrim))
+                    result.tablespaceMap = tablespaceMap;
+                else
+                    strFree(tablespaceMap);
+
+                strFree(tablespaceMapTrim);
             }
         }
         MEM_CONTEXT_PRIOR_END();
@@ -559,7 +576,7 @@ dbBackupStop(Db *this)
 }
 
 /**********************************************************************************************************************************/
-VariantList *
+Pack *
 dbList(Db *this)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -567,7 +584,12 @@ dbList(Db *this)
     FUNCTION_LOG_END();
 
     FUNCTION_LOG_RETURN(
-        VARIANT_LIST, dbQuery(this, STRDEF("select oid::oid, datname::text, datlastsysoid::oid from pg_catalog.pg_database")));
+        PACK,
+        dbQuery(
+            this, pgClientQueryResultAny,
+            STRDEF(
+                "select oid::oid, datname::text, (select oid::oid from pg_catalog.pg_database where datname = 'template0')"
+                    " as datlastsysoid from pg_catalog.pg_database")));
 }
 
 /**********************************************************************************************************************************/
@@ -634,8 +656,8 @@ dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targe
                 strZ(replayLsnFunction));
 
             // Execute the query and get replayLsn
-            VariantList *row = dbQueryRow(this, query);
-            replayLsn = varStr(varLstGet(row, 0));
+            PackRead *read = dbQueryRow(this, query);
+            replayLsn = pckReadStrP(read);
 
             // Error when replayLsn is null which indicates that this is not a standby.  This should have been sorted out before we
             // connected but it's possible that the standby was promoted in the meantime.
@@ -648,10 +670,10 @@ dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targe
                     strZ(replayLsnFunction));
             }
 
-            targetReached = varBool(varLstGet(row, 1));
+            targetReached = pckReadBoolP(read);
 
             // If the target has not been reached but progress is being made then reset the timer
-            if (!targetReached && varLstSize(row) > 2 && varBool(varLstGet(row, 2)))
+            if (!targetReached && pckReadBoolP(read, .defaultValue = true))
                 wait = waitNew(timeout);
 
             protocolKeepAlive();
@@ -689,9 +711,9 @@ dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targe
                     lsnName, strZ(targetLsn), lsnName);
 
                 // Execute the query and get checkpointLsn
-                VariantList *row = dbQueryRow(this, query);
-                targetReached = varBool(varLstGet(row, 0));
-                checkpointLsn = varStr(varLstGet(row, 1));
+                PackRead *const read = dbQueryRow(this, query);
+                targetReached = pckReadBoolP(read);
+                checkpointLsn = pckReadStrP(read);
 
                 protocolKeepAlive();
             }
@@ -729,27 +751,21 @@ dbPing(Db *const this, const bool force)
 
         if (force || timeNow - this->pingTimeLast > DB_PING_SEC)
         {
-            // Make sure recovery state has not changed for versions that support recovery
-            if (dbPgVersion(this) >= PG_VERSION_90)
+            // Make sure recovery state has not changed
+            if (dbIsInRecovery(this) != dbIsStandby(this))
             {
-                if (dbIsInRecovery(this) != dbIsStandby(this))
+                // If this is the standby then it must have been promoted
+                if (dbIsStandby(this))
                 {
-                    // If this is the standby then it must have been promoted
-                    if (dbIsStandby(this))
-                    {
-                        THROW(
-                            DbMismatchError,
-                            "standby is no longer is recovery\n"
-                            "HINT: was the standby promoted during the backup?");
-                    }
-                    // Else if a primary then something has gone seriously wrong
-                    else
-                        THROW(AssertError, "primary has switched to recovery");
+                    THROW(
+                        DbMismatchError,
+                        "standby is no longer in recovery\n"
+                        "HINT: was the standby promoted during the backup?");
                 }
+                // Else if a primary then something has gone seriously wrong
+                else
+                    THROW(AssertError, "primary has switched to recovery");
             }
-            // Else just make sure the cluster is still running
-            else
-                dbTimeMSec(this);
 
             this->pingTimeLast = timeNow;
         }
@@ -760,14 +776,16 @@ dbPing(Db *const this, const bool force)
 }
 
 /**********************************************************************************************************************************/
-VariantList *
+Pack *
 dbTablespaceList(Db *this)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
     FUNCTION_LOG_END();
 
-    FUNCTION_LOG_RETURN(VARIANT_LIST, dbQuery(this, STRDEF("select oid::oid, spcname::text from pg_catalog.pg_tablespace")));
+    FUNCTION_LOG_RETURN(
+        PACK,
+        dbQuery(this, pgClientQueryResultAny, STRDEF("select oid::oid, spcname::text from pg_catalog.pg_tablespace")));
 }
 
 /**********************************************************************************************************************************/
@@ -778,8 +796,16 @@ dbTimeMSec(Db *this)
         FUNCTION_LOG_PARAM(DB, this);
     FUNCTION_LOG_END();
 
-    FUNCTION_LOG_RETURN(
-        TIME_MSEC, varUInt64Force(dbQueryColumn(this, STRDEF("select (extract(epoch from clock_timestamp()) * 1000)::bigint"))));
+    TimeMSec result;
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        result = (TimeMSec)pckReadI64P(
+            dbQueryColumn(this, STRDEF("select (extract(epoch from clock_timestamp()) * 1000)::bigint")));
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN(TIME_MSEC, result);
 }
 
 /**********************************************************************************************************************************/
@@ -803,7 +829,7 @@ dbWalSwitch(Db *this)
 
         // Request a WAL segment switch
         const char *walName = strZ(pgWalName(dbPgVersion(this)));
-        const String *walFileName = varStr(
+        const String *walFileName = pckReadStrP(
             dbQueryColumn(this, strNewFmt("select pg_catalog.pg_%sfile_name(pg_catalog.pg_switch_%s())::text", walName, walName)));
 
         // Copy WAL segment name to the prior context
