@@ -19,19 +19,6 @@ Info Handler
 #include "storage/helper.h"
 #include "version.h"
 
-/***********************************************************************************************************************************
-Internal constants
-***********************************************************************************************************************************/
-#define INFO_SECTION_BACKREST                                       "backrest"
-    STRING_STATIC(INFO_SECTION_BACKREST_STR,                            INFO_SECTION_BACKREST);
-STRING_STATIC(INFO_SECTION_CIPHER_STR,                              "cipher");
-
-STRING_STATIC(INFO_KEY_CIPHER_PASS_STR,                             "cipher-pass");
-#define INFO_KEY_CHECKSUM                                           "backrest-checksum"
-    STRING_STATIC(INFO_KEY_CHECKSUM_STR,                            INFO_KEY_CHECKSUM);
-STRING_EXTERN(INFO_KEY_FORMAT_STR,                                  INFO_KEY_FORMAT);
-STRING_EXTERN(INFO_KEY_VERSION_STR,                                 INFO_KEY_VERSION);
-
 /***********************************************************************************************************************************
 Object types
 ***********************************************************************************************************************************/
@@ -84,7 +71,7 @@ BUFFER_STRDEF_STATIC(INFO_CHECKSUM_KEY_VALUE_END_BUF, ":");
 #define INFO_CHECKSUM_KEY_VALUE(checksum, key, value)                                                                              \
     do                                                                                                                             \
     {                                                                                                                              \
-        ioFilterProcessIn(checksum, BUFSTR(jsonFromStr(key)));                                                                     \
+        ioFilterProcessIn(checksum, BUFSTR(jsonFromVar(VARSTR(key))));                                                             \
         ioFilterProcessIn(checksum, INFO_CHECKSUM_KEY_VALUE_END_BUF);                                                              \
         ioFilterProcessIn(checksum, BUFSTR(value));                                                                                \
     }                                                                                                                              \
@@ -121,7 +108,7 @@ infoNewInternal(void)
         .memContext = memContextCurrent(),
     };
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(INFO, this);
 }
 
 /**********************************************************************************************************************************/
@@ -134,7 +121,7 @@ infoNew(const String *cipherPass)
 
     Info *this = NULL;
 
-    OBJ_NEW_BEGIN(Info)
+    OBJ_NEW_BEGIN(Info, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         this = infoNewInternal();
 
@@ -150,6 +137,11 @@ infoNew(const String *cipherPass)
 /***********************************************************************************************************************************
 Load and validate the info file (or copy)
 ***********************************************************************************************************************************/
+#define INFO_SECTION_BACKREST                                       "backrest"
+#define INFO_KEY_CHECKSUM                                           "backrest-checksum"
+#define INFO_SECTION_CIPHER                                         "cipher"
+#define INFO_KEY_CIPHER_PASS                                        "cipher-pass"
+
 typedef struct InfoLoadData
 {
     MemContext *memContext;                                         // Mem context to use for storing data in this structure
@@ -158,18 +150,17 @@ typedef struct InfoLoadData
     Info *info;                                                     // Info object
     String *sectionLast;                                            // The last section seen during load
     IoFilter *checksumActual;                                       // Checksum calculated from the file
-    String *checksumExpected;                                       // Checksum found in ini file
+    const String *checksumExpected;                                 // Checksum found in ini file
 } InfoLoadData;
 
 static void
-infoLoadCallback(void *data, const String *section, const String *key, const String *value, const Variant *valueVar)
+infoLoadCallback(void *const data, const String *const section, const String *const key, const String *const value)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM_P(VOID, data);
         FUNCTION_TEST_PARAM(STRING, section);
         FUNCTION_TEST_PARAM(STRING, key);
         FUNCTION_TEST_PARAM(STRING, value);
-        FUNCTION_TEST_PARAM(VARIANT, valueVar);
     FUNCTION_TEST_END();
 
     ASSERT(data != NULL);
@@ -177,10 +168,10 @@ infoLoadCallback(void *data, const String *section, const String *key, const Str
     ASSERT(key != NULL);
     ASSERT(value != NULL);
 
-    InfoLoadData *loadData = (InfoLoadData *)data;
+    InfoLoadData *const loadData = (InfoLoadData *)data;
 
     // Calculate checksum
-    if (!(strEq(section, INFO_SECTION_BACKREST_STR) && strEq(key, INFO_KEY_CHECKSUM_STR)))
+    if (!(strEqZ(section, INFO_SECTION_BACKREST) && strEqZ(key, INFO_KEY_CHECKSUM)))
     {
         if (loadData->sectionLast == NULL || !strEq(section, loadData->sectionLast))
         {
@@ -202,53 +193,49 @@ infoLoadCallback(void *data, const String *section, const String *key, const Str
     }
 
     // Process backrest section
-    if (strEq(section, INFO_SECTION_BACKREST_STR))
+    if (strEqZ(section, INFO_SECTION_BACKREST))
     {
-        ASSERT(valueVar != NULL);
-
         // Validate format
-        if (strEq(key, INFO_KEY_FORMAT_STR))
+        if (strEqZ(key, INFO_KEY_FORMAT))
         {
-            if (varUInt64(valueVar) != REPOSITORY_FORMAT)
-                THROW_FMT(FormatError, "expected format %d but found %" PRIu64, REPOSITORY_FORMAT, varUInt64(valueVar));
+            if (varUInt64(jsonToVar(value)) != REPOSITORY_FORMAT)
+                THROW_FMT(FormatError, "expected format %d but found %" PRIu64, REPOSITORY_FORMAT, varUInt64(jsonToVar(value)));
         }
         // Store pgBackRest version
-        else if (strEq(key, INFO_KEY_VERSION_STR))
+        else if (strEqZ(key, INFO_KEY_VERSION))
         {
             MEM_CONTEXT_BEGIN(loadData->info->memContext)
             {
-                loadData->info->pub.backrestVersion = strDup(varStr(valueVar));
+                loadData->info->pub.backrestVersion = varStr(jsonToVar(value));
             }
             MEM_CONTEXT_END();
         }
         // Store checksum to be validated later
-        else if (strEq(key, INFO_KEY_CHECKSUM_STR))
+        else if (strEqZ(key, INFO_KEY_CHECKSUM))
         {
             MEM_CONTEXT_BEGIN(loadData->memContext)
             {
-                loadData->checksumExpected = strDup(varStr(valueVar));
+                loadData->checksumExpected = varStr(jsonToVar(value));
             }
             MEM_CONTEXT_END();
         }
     }
     // Process cipher section
-    else if (strEq(section, INFO_SECTION_CIPHER_STR))
+    else if (strEqZ(section, INFO_SECTION_CIPHER))
     {
-        ASSERT(valueVar != NULL);
-
         // No validation needed for cipher-pass, just store it
-        if (strEq(key, INFO_KEY_CIPHER_PASS_STR))
+        if (strEqZ(key, INFO_KEY_CIPHER_PASS))
         {
             MEM_CONTEXT_BEGIN(loadData->info->memContext)
             {
-                loadData->info->pub.cipherPass = strDup(varStr(valueVar));
+                loadData->info->pub.cipherPass = varStr(jsonToVar(value));
             }
             MEM_CONTEXT_END();
         }
     }
     // Else pass to callback for processing
     else
-        loadData->callbackFunction(loadData->callbackData, section, key, valueVar);
+        loadData->callbackFunction(loadData->callbackData, section, key, value);
 
     FUNCTION_TEST_RETURN_VOID();
 }
@@ -269,7 +256,7 @@ infoNewLoad(IoRead *read, InfoLoadNewCallback *callbackFunction, void *callbackD
 
     Info *this = NULL;
 
-    OBJ_NEW_BEGIN(Info)
+    OBJ_NEW_BEGIN(Info, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         this = infoNewInternal();
 
@@ -282,7 +269,7 @@ infoNewLoad(IoRead *read, InfoLoadNewCallback *callbackFunction, void *callbackD
                 .callbackFunction = callbackFunction,
                 .callbackData = callbackData,
                 .info = this,
-                .checksumActual = cryptoHashNew(HASH_TYPE_SHA1_STR),
+                .checksumActual = cryptoHashNew(hashTypeSha1),
             };
 
             INFO_CHECKSUM_BEGIN(data.checksumActual);
@@ -320,27 +307,31 @@ infoNewLoad(IoRead *read, InfoLoadNewCallback *callbackFunction, void *callbackD
 
 /**********************************************************************************************************************************/
 bool
-infoSaveSection(InfoSave *infoSaveData, const String *section, const String *sectionNext)
+infoSaveSection(InfoSave *const infoSaveData, const char *const section, const String *const sectionNext)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(INFO_SAVE, infoSaveData);
-        FUNCTION_TEST_PARAM(STRING, section);
+        FUNCTION_TEST_PARAM(STRINGZ, section);
         FUNCTION_TEST_PARAM(STRING, sectionNext);
     FUNCTION_TEST_END();
 
+    ASSERT(infoSaveData != NULL);
+    ASSERT(section != NULL);
+
     FUNCTION_TEST_RETURN(
-        (infoSaveData->sectionLast == NULL || strCmp(section, infoSaveData->sectionLast) > 0) &&
-            (sectionNext == NULL || strCmp(section, sectionNext) < 0));
+        BOOL,
+        (infoSaveData->sectionLast == NULL || strCmpZ(infoSaveData->sectionLast, section) < 0) &&
+            (sectionNext == NULL || strCmpZ(sectionNext, section) > 0));
 }
 
 /**********************************************************************************************************************************/
 void
-infoSaveValue(InfoSave *infoSaveData, const String *section, const String *key, const String *jsonValue)
+infoSaveValue(InfoSave *const infoSaveData, const char *const section, const char *const key, const String *const jsonValue)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(INFO_SAVE, infoSaveData);
-        FUNCTION_TEST_PARAM(STRING, section);
-        FUNCTION_TEST_PARAM(STRING, key);
+        FUNCTION_TEST_PARAM(STRINGZ, section);
+        FUNCTION_TEST_PARAM(STRINGZ, key);
         FUNCTION_TEST_PARAM(STRING, jsonValue);
     FUNCTION_TEST_END();
 
@@ -353,7 +344,7 @@ infoSaveValue(InfoSave *infoSaveData, const String *section, const String *key,
     ASSERT(strZ(jsonValue)[0] != '[');
 
     // Save section
-    if (infoSaveData->sectionLast == NULL || !strEq(section, infoSaveData->sectionLast))
+    if (infoSaveData->sectionLast == NULL || !strEqZ(infoSaveData->sectionLast, section))
     {
         if (infoSaveData->sectionLast != NULL)
         {
@@ -361,15 +352,15 @@ infoSaveValue(InfoSave *infoSaveData, const String *section, const String *key,
             ioWriteLine(infoSaveData->write, BUFSTRDEF(""));
         }
 
-        INFO_CHECKSUM_SECTION(infoSaveData->checksum, section);
+        INFO_CHECKSUM_SECTION(infoSaveData->checksum, STR(section));
 
         ioWrite(infoSaveData->write, BRACKETL_BUF);
-        ioWrite(infoSaveData->write, BUFSTR(section));
+        ioWrite(infoSaveData->write, BUFSTRZ(section));
         ioWriteLine(infoSaveData->write, BRACKETR_BUF);
 
         MEM_CONTEXT_BEGIN(infoSaveData->memContext)
         {
-            infoSaveData->sectionLast = strDup(section);
+            infoSaveData->sectionLast = strNewZ(section);
         }
         MEM_CONTEXT_END();
     }
@@ -377,9 +368,9 @@ infoSaveValue(InfoSave *infoSaveData, const String *section, const String *key,
         INFO_CHECKSUM_KEY_VALUE_NEXT(infoSaveData->checksum);
 
     // Save key/value
-    INFO_CHECKSUM_KEY_VALUE(infoSaveData->checksum, key, jsonValue);
+    INFO_CHECKSUM_KEY_VALUE(infoSaveData->checksum, STR(key), jsonValue);
 
-    ioWrite(infoSaveData->write, BUFSTR(key));
+    ioWrite(infoSaveData->write, BUFSTRZ(key));
     ioWrite(infoSaveData->write, EQ_BUF);
     ioWriteLine(infoSaveData->write, BUFSTR(jsonValue));
 
@@ -413,19 +404,19 @@ infoSave(Info *this, IoWrite *write, InfoSaveCallback *callbackFunction, void *c
         ioWriteOpen(data.write);
 
         // Begin checksum calculation
-        data.checksum = cryptoHashNew(HASH_TYPE_SHA1_STR);
+        data.checksum = cryptoHashNew(hashTypeSha1);
         INFO_CHECKSUM_BEGIN(data.checksum);
 
         // Add version and format
-        callbackFunction(callbackData, INFO_SECTION_BACKREST_STR, &data);
-        infoSaveValue(&data, INFO_SECTION_BACKREST_STR, INFO_KEY_FORMAT_STR, jsonFromUInt(REPOSITORY_FORMAT));
-        infoSaveValue(&data, INFO_SECTION_BACKREST_STR, INFO_KEY_VERSION_STR, jsonFromStr(STRDEF(PROJECT_VERSION)));
+        callbackFunction(callbackData, STRDEF(INFO_SECTION_BACKREST), &data);
+        infoSaveValue(&data, INFO_SECTION_BACKREST, INFO_KEY_FORMAT, jsonFromVar(VARUINT(REPOSITORY_FORMAT)));
+        infoSaveValue(&data, INFO_SECTION_BACKREST, INFO_KEY_VERSION, jsonFromVar(VARSTRDEF(PROJECT_VERSION)));
 
         // Add cipher passphrase if defined
         if (infoCipherPass(this) != NULL)
         {
-            callbackFunction(callbackData, INFO_SECTION_CIPHER_STR, &data);
-            infoSaveValue(&data, INFO_SECTION_CIPHER_STR, INFO_KEY_CIPHER_PASS_STR, jsonFromStr(infoCipherPass(this)));
+            callbackFunction(callbackData, STRDEF(INFO_SECTION_CIPHER), &data);
+            infoSaveValue(&data, INFO_SECTION_CIPHER, INFO_KEY_CIPHER_PASS, jsonFromVar(VARSTR(infoCipherPass(this))));
         }
 
         // Flush out any additional sections
@@ -435,7 +426,7 @@ infoSave(Info *this, IoWrite *write, InfoSaveCallback *callbackFunction, void *c
         INFO_CHECKSUM_END(data.checksum);
 
         ioWrite(data.write, BUFSTRDEF("\n[" INFO_SECTION_BACKREST "]\n" INFO_KEY_CHECKSUM "="));
-        ioWriteLine(data.write, BUFSTR(jsonFromStr(pckReadStrP(pckReadNew(ioFilterResult(data.checksum))))));
+        ioWriteLine(data.write, BUFSTR(jsonFromVar(VARSTR(pckReadStrP(pckReadNew(ioFilterResult(data.checksum)))))));
 
         // Close the file
         ioWriteClose(data.write);
