@@ -52,7 +52,7 @@ infoArchiveNewInternal(void)
         },
     };
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(INFO_ARCHIVE, this);
 }
 
 /**********************************************************************************************************************************/
@@ -69,7 +69,7 @@ infoArchiveNew(unsigned int pgVersion, uint64_t pgSystemId, const String *cipher
 
     InfoArchive *this = NULL;
 
-    OBJ_NEW_BEGIN(InfoArchive)
+    OBJ_NEW_BEGIN(InfoArchive, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         this = infoArchiveNewInternal();
 
@@ -94,7 +94,7 @@ infoArchiveNewLoad(IoRead *read)
 
     InfoArchive *this = NULL;
 
-    OBJ_NEW_BEGIN(InfoArchive)
+    OBJ_NEW_BEGIN(InfoArchive, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         this = infoArchiveNewInternal();
         this->pub.infoPg = infoPgNewLoad(read, infoPgArchive, NULL, NULL);
@@ -213,7 +213,7 @@ typedef struct InfoArchiveLoadFileData
 } InfoArchiveLoadFileData;
 
 static bool
-infoArchiveLoadFileCallback(void *data, unsigned int try)
+infoArchiveLoadFileCallback(void *const data, const unsigned int try)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM_P(VOID, data);
@@ -222,24 +222,28 @@ infoArchiveLoadFileCallback(void *data, unsigned int try)
 
     ASSERT(data != NULL);
 
-    InfoArchiveLoadFileData *loadData = (InfoArchiveLoadFileData *)data;
+    InfoArchiveLoadFileData *const loadData = data;
     bool result = false;
 
     if (try < 2)
     {
-        // Construct filename based on try
-        const String *fileName = try == 0 ? loadData->fileName : strNewFmt("%s" INFO_COPY_EXT, strZ(loadData->fileName));
-
-        // Attempt to load the file
-        IoRead *read = storageReadIo(storageNewReadP(loadData->storage, fileName));
-        cipherBlockFilterGroupAdd(ioReadFilterGroup(read), loadData->cipherType, cipherModeDecrypt, loadData->cipherPass);
-
-        MEM_CONTEXT_BEGIN(loadData->memContext)
+        MEM_CONTEXT_TEMP_BEGIN()
         {
-            loadData->infoArchive = infoArchiveNewLoad(read);
-            result = true;
+            // Construct filename based on try
+            const String *const fileName = try == 0 ? loadData->fileName : strNewFmt("%s" INFO_COPY_EXT, strZ(loadData->fileName));
+
+            // Attempt to load the file
+            IoRead *const read = storageReadIo(storageNewReadP(loadData->storage, fileName));
+            cipherBlockFilterGroupAdd(ioReadFilterGroup(read), loadData->cipherType, cipherModeDecrypt, loadData->cipherPass);
+
+            MEM_CONTEXT_BEGIN(loadData->memContext)
+            {
+                loadData->infoArchive = infoArchiveNewLoad(read);
+                result = true;
+            }
+            MEM_CONTEXT_END();
         }
-        MEM_CONTEXT_END();
+        MEM_CONTEXT_TEMP_END();
     }
 
     FUNCTION_LOG_RETURN(BOOL, result);
