@@ -22,19 +22,6 @@ PostgreSQL Info Handler
 #include "postgres/version.h"
 #include "storage/helper.h"
 
-/***********************************************************************************************************************************
-Internal constants
-***********************************************************************************************************************************/
-STRING_STATIC(INFO_SECTION_DB_STR,                                          "db");
-STRING_STATIC(INFO_SECTION_DB_HISTORY_STR,                                  "db:history");
-
-STRING_STATIC(INFO_KEY_DB_ID_STR,                                           INFO_KEY_DB_ID);
-VARIANT_STRDEF_EXTERN(INFO_KEY_DB_ID_VAR,                                   INFO_KEY_DB_ID);
-VARIANT_STRDEF_STATIC(INFO_KEY_DB_CATALOG_VERSION_VAR,                      "db-catalog-version");
-VARIANT_STRDEF_STATIC(INFO_KEY_DB_CONTROL_VERSION_VAR,                      "db-control-version");
-VARIANT_STRDEF_STATIC(INFO_KEY_DB_SYSTEM_ID_VAR,                            "db-system-id");
-VARIANT_STRDEF_STATIC(INFO_KEY_DB_VERSION_VAR,                              "db-version");
-
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
@@ -68,7 +55,7 @@ infoPgNewInternal(InfoPgType type)
         .type = type,
     };
 
-    FUNCTION_TEST_RETURN(this);
+    FUNCTION_TEST_RETURN(INFO_PG, this);
 }
 
 /**********************************************************************************************************************************/
@@ -82,7 +69,7 @@ infoPgNew(InfoPgType type, const String *cipherPassSub)
 
     InfoPg *this = NULL;
 
-    OBJ_NEW_BEGIN(InfoPg)
+    OBJ_NEW_BEGIN(InfoPg, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         this = infoPgNewInternal(type);
         this->pub.info = infoNew(cipherPassSub);
@@ -93,6 +80,14 @@ infoPgNew(InfoPgType type, const String *cipherPassSub)
 }
 
 /**********************************************************************************************************************************/
+#define INFO_SECTION_DB                                                     "db"
+#define INFO_SECTION_DB_HISTORY                                             "db:history"
+
+#define INFO_KEY_DB_CATALOG_VERSION                                         "db-catalog-version"
+#define INFO_KEY_DB_CONTROL_VERSION                                         "db-control-version"
+#define INFO_KEY_DB_SYSTEM_ID                                               "db-system-id"
+#define INFO_KEY_DB_VERSION                                                 "db-version"
+
 typedef struct InfoPgLoadData
 {
     InfoLoadNewCallback *callbackFunction;                          // Callback function for child object
@@ -102,13 +97,13 @@ typedef struct InfoPgLoadData
 } InfoPgLoadData;
 
 static void
-infoPgLoadCallback(void *data, const String *section, const String *key, const Variant *value)
+infoPgLoadCallback(void *const data, const String *const section, const String *const key, const String *const value)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM_P(VOID, data);
         FUNCTION_TEST_PARAM(STRING, section);
         FUNCTION_TEST_PARAM(STRING, key);
-        FUNCTION_TEST_PARAM(VARIANT, value);
+        FUNCTION_TEST_PARAM(STRING, value);
     FUNCTION_TEST_END();
 
     ASSERT(data != NULL);
@@ -116,31 +111,32 @@ infoPgLoadCallback(void *data, const String *section, const String *key, const V
     ASSERT(key != NULL);
     ASSERT(value != NULL);
 
-    InfoPgLoadData *loadData = (InfoPgLoadData *)data;
+    InfoPgLoadData *const loadData = data;
 
     // Process db section
-    if (strEq(section, INFO_SECTION_DB_STR))
+    if (strEqZ(section, INFO_SECTION_DB))
     {
-        if (strEq(key, INFO_KEY_DB_ID_STR))
-            loadData->currentId = varUIntForce(value);
+        if (strEqZ(key, INFO_KEY_DB_ID))
+            loadData->currentId = varUIntForce(jsonToVar(value));
     }
     // Process db:history section
-    else if (strEq(section, INFO_SECTION_DB_HISTORY_STR))
+    else if (strEqZ(section, INFO_SECTION_DB_HISTORY))
     {
-        // Get db values that are common to all info files
-        const KeyValue *pgDataKv = varKv(value);
+        InfoPgData infoPgData = {.id = cvtZToUInt(strZ(key))};
 
-        InfoPgData infoPgData =
-        {
-            .id = cvtZToUInt(strZ(key)),
-            .version = pgVersionFromStr(varStr(kvGet(pgDataKv, INFO_KEY_DB_VERSION_VAR))),
-            .catalogVersion =
-                loadData->infoPg->type == infoPgBackup ? varUIntForce(kvGet(pgDataKv, INFO_KEY_DB_CATALOG_VERSION_VAR)) : 0,
+        JsonRead *const json = jsonReadNew(value);
+        jsonReadObjectBegin(json);
 
-            // This is different in archive.info due to a typo that can't be fixed without a format version bump
-            .systemId = varUInt64Force(
-                kvGet(pgDataKv, loadData->infoPg->type == infoPgArchive ? INFO_KEY_DB_ID_VAR : INFO_KEY_DB_SYSTEM_ID_VAR)),
-        };
+        // Catalog version
+        if (loadData->infoPg->type == infoPgBackup)
+            infoPgData.catalogVersion = jsonReadUInt(jsonReadKeyRequireZ(json, INFO_KEY_DB_CATALOG_VERSION));
+
+        // System id
+        infoPgData.systemId = jsonReadUInt64(
+                jsonReadKeyRequireZ(json, loadData->infoPg->type == infoPgArchive ? INFO_KEY_DB_ID : INFO_KEY_DB_SYSTEM_ID));
+
+        // PostgreSQL version
+        infoPgData.version = pgVersionFromStr(jsonReadStr(jsonReadKeyRequireZ(json, INFO_KEY_DB_VERSION)));
 
         // Insert at beginning of list so the history is reverse ordered
         lstInsert(loadData->infoPg->pub.history, 0, &infoPgData);
@@ -168,7 +164,7 @@ infoPgNewLoad(IoRead *read, InfoPgType type, InfoLoadNewCallback *callbackFuncti
 
     InfoPg *this = NULL;
 
-    OBJ_NEW_BEGIN(InfoPg)
+    OBJ_NEW_BEGIN(InfoPg, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         this = infoPgNewInternal(type);
 
@@ -274,7 +270,7 @@ typedef struct InfoPgSaveData
 } InfoPgSaveData;
 
 static void
-infoPgSaveCallback(void *data, const String *sectionNext, InfoSave *infoSaveData)
+infoPgSaveCallback(void *const data, const String *const sectionNext, InfoSave *const infoSaveData)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM_P(VOID, data);
@@ -285,56 +281,57 @@ infoPgSaveCallback(void *data, const String *sectionNext, InfoSave *infoSaveData
     ASSERT(data != NULL);
     ASSERT(infoSaveData != NULL);
 
-    InfoPgSaveData *saveData = (InfoPgSaveData *)data;
+    InfoPgSaveData *const saveData = (InfoPgSaveData *)data;
 
-    if (infoSaveSection(infoSaveData, INFO_SECTION_DB_STR, sectionNext))
+    if (infoSaveSection(infoSaveData, INFO_SECTION_DB, sectionNext))
     {
         if (saveData->callbackFunction != NULL)
-            saveData->callbackFunction(saveData->callbackData, INFO_SECTION_DB_STR, infoSaveData);
+            saveData->callbackFunction(saveData->callbackData, STRDEF(INFO_SECTION_DB), infoSaveData);
 
         InfoPgData pgData = infoPgDataCurrent(saveData->infoPg);
 
         // These need to be saved because older pgBackRest versions expect them
         if (saveData->infoPg->type == infoPgBackup)
         {
+            infoSaveValue(infoSaveData, INFO_SECTION_DB, INFO_KEY_DB_CATALOG_VERSION, jsonFromVar(VARUINT(pgData.catalogVersion)));
             infoSaveValue(
-                infoSaveData, INFO_SECTION_DB_STR, varStr(INFO_KEY_DB_CATALOG_VERSION_VAR), jsonFromUInt(pgData.catalogVersion));
-            infoSaveValue(
-                infoSaveData, INFO_SECTION_DB_STR, varStr(INFO_KEY_DB_CONTROL_VERSION_VAR),
-                jsonFromUInt(pgControlVersion(pgData.version)));
+                infoSaveData, INFO_SECTION_DB, INFO_KEY_DB_CONTROL_VERSION, jsonFromVar(VARUINT(pgControlVersion(pgData.version))));
         }
 
-        infoSaveValue(infoSaveData, INFO_SECTION_DB_STR, varStr(INFO_KEY_DB_ID_VAR), jsonFromUInt(pgData.id));
-        infoSaveValue(infoSaveData, INFO_SECTION_DB_STR, varStr(INFO_KEY_DB_SYSTEM_ID_VAR), jsonFromUInt64(pgData.systemId));
-        infoSaveValue(
-            infoSaveData, INFO_SECTION_DB_STR, varStr(INFO_KEY_DB_VERSION_VAR), jsonFromStr(pgVersionToStr(pgData.version)));
+        infoSaveValue(infoSaveData, INFO_SECTION_DB, INFO_KEY_DB_ID, jsonFromVar(VARUINT(pgData.id)));
+        infoSaveValue(infoSaveData, INFO_SECTION_DB, INFO_KEY_DB_SYSTEM_ID, jsonFromVar(VARUINT64(pgData.systemId)));
+        infoSaveValue(infoSaveData, INFO_SECTION_DB, INFO_KEY_DB_VERSION, jsonFromVar(VARSTR(pgVersionToStr(pgData.version))));
     }
 
-    if (infoSaveSection(infoSaveData, INFO_SECTION_DB_HISTORY_STR, sectionNext))
+    if (infoSaveSection(infoSaveData, INFO_SECTION_DB_HISTORY, sectionNext))
     {
         if (saveData->callbackFunction != NULL)
-            saveData->callbackFunction(saveData->callbackData, INFO_SECTION_DB_HISTORY_STR, infoSaveData);
+            saveData->callbackFunction(saveData->callbackData, STRDEF(INFO_SECTION_DB_HISTORY), infoSaveData);
 
         // Set the db history section in reverse so oldest history is first instead of last to be consistent with load
         for (unsigned int pgDataIdx = infoPgDataTotal(saveData->infoPg) - 1; (int)pgDataIdx >= 0; pgDataIdx--)
         {
-            InfoPgData pgData = infoPgData(saveData->infoPg, pgDataIdx);
-
-            KeyValue *pgDataKv = kvNew();
-            kvPut(pgDataKv, INFO_KEY_DB_VERSION_VAR, VARSTR(pgVersionToStr(pgData.version)));
+            const InfoPgData pgData = infoPgData(saveData->infoPg, pgDataIdx);
+            JsonWrite *const json = jsonWriteObjectBegin(jsonWriteNewP());
 
+            // These need to be saved because older pgBackRest versions expect them
             if (saveData->infoPg->type == infoPgBackup)
             {
-                kvPut(pgDataKv, INFO_KEY_DB_SYSTEM_ID_VAR, VARUINT64(pgData.systemId));
-
-                // These need to be saved because older pgBackRest versions expect them
-                kvPut(pgDataKv, INFO_KEY_DB_CATALOG_VERSION_VAR, VARUINT(pgData.catalogVersion));
-                kvPut(pgDataKv, INFO_KEY_DB_CONTROL_VERSION_VAR, VARUINT(pgControlVersion(pgData.version)));
+                jsonWriteUInt(jsonWriteKeyZ(json, INFO_KEY_DB_CATALOG_VERSION), pgData.catalogVersion);
+                jsonWriteUInt(jsonWriteKeyZ(json, INFO_KEY_DB_CONTROL_VERSION), pgControlVersion(pgData.version));
             }
-            else
-                kvPut(pgDataKv, INFO_KEY_DB_ID_VAR, VARUINT64(pgData.systemId));
 
-            infoSaveValue(infoSaveData, INFO_SECTION_DB_HISTORY_STR, varStrForce(VARUINT(pgData.id)), jsonFromKv(pgDataKv));
+            if (saveData->infoPg->type == infoPgArchive)
+                jsonWriteUInt64(jsonWriteKeyZ(json, INFO_KEY_DB_ID), pgData.systemId);
+
+            if (saveData->infoPg->type == infoPgBackup)
+                jsonWriteUInt64(jsonWriteKeyZ(json, INFO_KEY_DB_SYSTEM_ID), pgData.systemId);
+
+            jsonWriteStr(jsonWriteKeyZ(json, INFO_KEY_DB_VERSION), pgVersionToStr(pgData.version));
+
+            infoSaveValue(
+                infoSaveData, INFO_SECTION_DB_HISTORY, strZ(varStrForce(VARUINT(pgData.id))),
+                jsonWriteResult(jsonWriteObjectEnd(json)));
         }
     }
 
@@ -377,7 +374,7 @@ infoPgSave(InfoPg *this, IoWrite *write, InfoSaveCallback *callbackFunction, voi
 
 /**********************************************************************************************************************************/
 String *
-infoPgArchiveId(const InfoPg *this, unsigned int pgDataIdx)
+infoPgArchiveId(const InfoPg *const this, const unsigned int pgDataIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(INFO_PG, this);
@@ -386,9 +383,14 @@ infoPgArchiveId(const InfoPg *this, unsigned int pgDataIdx)
 
     ASSERT(this != NULL);
 
-    InfoPgData pgData = infoPgData(this, pgDataIdx);
+    const InfoPgData pgData = infoPgData(this, pgDataIdx);
+    String *const version = pgVersionToStr(pgData.version);
 
-    FUNCTION_LOG_RETURN(STRING, strNewFmt("%s-%u", strZ(pgVersionToStr(pgData.version)), pgData.id));
+    String *const result = strNewFmt("%s-%u", strZ(version), pgData.id);
+
+    strFree(version);
+
+    FUNCTION_LOG_RETURN(STRING, result);
 }
 
 /**********************************************************************************************************************************/
