@@ -13,9 +13,13 @@ nothing is missing or corrupt.  It is also useful for reporting, e.g. size of ba
 /***********************************************************************************************************************************
 Constants
 ***********************************************************************************************************************************/
-#define BACKUP_MANIFEST_FILE                                        "backup.manifest"
+#define BACKUP_MANIFEST_EXT                                         ".manifest"
+#define BACKUP_MANIFEST_FILE                                        "backup" BACKUP_MANIFEST_EXT
     STRING_DECLARE(BACKUP_MANIFEST_FILE_STR);
 
+#define MANIFEST_PATH_BUNDLE                                        "bundle"
+    STRING_DECLARE(MANIFEST_PATH_BUNDLE_STR);
+
 #define MANIFEST_TARGET_PGDATA                                      "pg_data"
     STRING_DECLARE(MANIFEST_TARGET_PGDATA_STR);
 #define MANIFEST_TARGET_PGTBLSPC                                    "pg_tblspc"
@@ -30,7 +34,7 @@ typedef struct Manifest Manifest;
 #include "common/compress/helper.h"
 #include "common/crypto/common.h"
 #include "common/crypto/hash.h"
-#include "common/type/variantList.h"
+#include "common/type/variant.h"
 #include "common/type/object.h"
 #include "info/info.h"
 #include "info/infoBackup.h"
@@ -49,6 +53,7 @@ typedef struct ManifestData
     time_t backupTimestampStart;                                    // When did the backup start?
     time_t backupTimestampStop;                                     // When did the backup stop?
     BackupType backupType;                                          // Type of backup: full, diff, incr
+    bool bundle;                                                    // Does the backup bundle files?
 
     // ??? Note that these fields are redundant and verbose since storing the start/stop lsn as a uint64 would be sufficient.
     // However, we currently lack the functions to transform these values back and forth so this will do for now.
@@ -83,7 +88,7 @@ typedef struct ManifestDb
 {
     const String *name;                                             // Db name (must be first member in struct)
     unsigned int id;                                                // Db oid
-    unsigned int lastSystemId;                                      // Highest oid used by system objects in this database
+    unsigned int lastSystemId;                                      // Highest oid used by system objects (deprecated - do not use)
 } ManifestDb;
 
 /***********************************************************************************************************************************
@@ -92,15 +97,16 @@ File type
 typedef struct ManifestFile
 {
     const String *name;                                             // File name (must be first member in struct)
-    bool primary:1;                                                 // Should this file be copied from the primary?
     bool checksumPage:1;                                            // Does this file have page checksums?
     bool checksumPageError:1;                                       // Is there an error in the page checksum?
     mode_t mode;                                                    // File mode
     char checksumSha1[HASH_TYPE_SHA1_SIZE_HEX + 1];                 // SHA1 checksum
-    const VariantList *checksumPageErrorList;                       // List of page checksum errors if there are any
+    const String *checksumPageErrorList;                            // List of page checksum errors if there are any
     const String *user;                                             // User name
     const String *group;                                            // Group name
     const String *reference;                                        // Reference to a prior backup
+    uint64_t bundleId;                                              // Bundle id
+    uint64_t bundleOffset;                                          // Bundle offset
     uint64_t size;                                                  // Original size
     uint64_t sizeRepo;                                              // Size in repo
     time_t timestamp;                                               // Original timestamp
@@ -152,8 +158,8 @@ Constructors
 ***********************************************************************************************************************************/
 // Build a new manifest for a PostgreSQL data directory
 Manifest *manifestNewBuild(
-    const Storage *storagePg, unsigned int pgVersion, unsigned int pgCatalogVersion, bool online, bool checksumPage,
-    const StringList *excludeList, const VariantList *tablespaceList);
+    const Storage *storagePg, unsigned int pgVersion, unsigned int pgCatalogVersion, bool online, bool checksumPage, bool bundle,
+    const StringList *excludeList, const Pack *tablespaceList);
 
 // Load a manifest from IO
 Manifest *manifestNewLoad(IoRead *read);
@@ -208,7 +214,7 @@ void manifestBuildIncr(Manifest *this, const Manifest *prior, BackupType type, c
 // Set remaining values before the final save
 void manifestBuildComplete(
     Manifest *this, time_t timestampStart, const String *lsnStart, const String *archiveStart, time_t timestampStop,
-    const String *lsnStop, const String *archiveStop, unsigned int pgId, uint64_t pgSystemId, const VariantList *dbList,
+    const String *lsnStop, const String *archiveStop, unsigned int pgId, uint64_t pgSystemId, const Pack *dbList,
     bool optionArchiveCheck, bool optionArchiveCopy, size_t optionBufferSize, unsigned int optionCompressLevel,
     unsigned int optionCompressLevelNetwork, bool optionHardLink, unsigned int optionProcessMax, bool optionStandby);
 
@@ -259,21 +265,52 @@ manifestDbTotal(const Manifest *const this)
 /***********************************************************************************************************************************
 File functions and getters/setters
 ***********************************************************************************************************************************/
-__attribute__((always_inline)) static inline const ManifestFile *
-manifestFile(const Manifest *const this, const unsigned int fileIdx)
+typedef struct ManifestFilePack ManifestFilePack;
+
+// Unpack file pack returned by manifestFilePackGet()
+ManifestFile manifestFileUnpack(const Manifest *manifest, const ManifestFilePack *filePack);
+
+// Get file in pack format by index
+__attribute__((always_inline)) static inline const ManifestFilePack *
+manifestFilePackGet(const Manifest *const this, const unsigned int fileIdx)
 {
-    return lstGet(THIS_PUB(Manifest)->fileList, fileIdx);
+    return *(ManifestFilePack **)lstGet(THIS_PUB(Manifest)->fileList, fileIdx);
 }
 
-void manifestFileAdd(Manifest *this, const ManifestFile *file);
-const ManifestFile *manifestFileFind(const Manifest *this, const String *name);
+// Get file name
+__attribute__((always_inline)) static inline const String *
+manifestFileNameGet(const Manifest *const this, const unsigned int fileIdx)
+{
+    return (const String *)manifestFilePackGet(this, fileIdx);
+}
 
-// If the file requested is not found in the list, return the default passed rather than throw an error
-__attribute__((always_inline)) static inline const ManifestFile *
-manifestFileFindDefault(const Manifest *const this, const String *const name, const ManifestFile *const fileDefault)
+// Get file by index
+__attribute__((always_inline)) static inline ManifestFile
+manifestFile(const Manifest *const this, const unsigned int fileIdx)
+{
+    return manifestFileUnpack(this, manifestFilePackGet(this, fileIdx));
+}
+
+// Add a file
+void manifestFileAdd(Manifest *this, ManifestFile *file);
+
+// Find file in pack format by name
+const ManifestFilePack *manifestFilePackFind(const Manifest *this, const String *name);
+
+// Find file by name
+__attribute__((always_inline)) static inline ManifestFile
+manifestFileFind(const Manifest *const this, const String *const name)
 {
     ASSERT_INLINE(name != NULL);
-    return lstFindDefault(THIS_PUB(Manifest)->fileList, &name, (void *)fileDefault);
+    return manifestFileUnpack(this, manifestFilePackFind(this, name));
+}
+
+// Does the file exist?
+__attribute__((always_inline)) static inline bool
+manifestFileExists(const Manifest *const this, const String *const name)
+{
+    ASSERT_INLINE(name != NULL);
+    return lstFindDefault(THIS_PUB(Manifest)->fileList, &name, NULL) != NULL;
 }
 
 void manifestFileRemove(const Manifest *this, const String *name);
@@ -287,7 +324,7 @@ manifestFileTotal(const Manifest *const this)
 // Update a file with new data
 void manifestFileUpdate(
     Manifest *this, const String *name, uint64_t size, uint64_t sizeRepo, const char *checksumSha1, const Variant *reference,
-    bool checksumPage, bool checksumPageError, const VariantList *checksumPageErrorList);
+    bool checksumPage, bool checksumPageError, const String *checksumPageErrorList, uint64_t bundleId, uint64_t bundleOffset);
 
 /***********************************************************************************************************************************
 Link functions and getters/setters
