@@ -10,7 +10,7 @@ PostgreSQL Interface
 #include "common/memContext.h"
 #include "common/regExp.h"
 #include "postgres/interface.h"
-#include "postgres/interface/version.h"
+#include "postgres/interface/static.vendor.h"
 #include "postgres/version.h"
 #include "storage/helper.h"
 
@@ -19,7 +19,7 @@ Defines for various Postgres paths and files
 ***********************************************************************************************************************************/
 STRING_EXTERN(PG_FILE_PGVERSION_STR,                                PG_FILE_PGVERSION);
 STRING_EXTERN(PG_FILE_POSTGRESQLAUTOCONF_STR,                       PG_FILE_POSTGRESQLAUTOCONF);
-STRING_EXTERN(PG_FILE_POSTMASTERPID_STR,                            PG_FILE_POSTMASTERPID);
+STRING_EXTERN(PG_FILE_POSTMTRPID_STR,                               PG_FILE_POSTMTRPID);
 STRING_EXTERN(PG_FILE_RECOVERYCONF_STR,                             PG_FILE_RECOVERYCONF);
 STRING_EXTERN(PG_FILE_RECOVERYDONE_STR,                             PG_FILE_RECOVERYDONE);
 STRING_EXTERN(PG_FILE_RECOVERYSIGNAL_STR,                           PG_FILE_RECOVERYSIGNAL);
@@ -43,18 +43,15 @@ STRING_STATIC(PG_NAME_LSN_STR,                                      "lsn");
 STRING_STATIC(PG_NAME_LOCATION_STR,                                 "location");
 
 /***********************************************************************************************************************************
-Control file size.  The control file is actually 8192 bytes but only the first 512 bytes are used to prevent torn pages even on
-really old storage with 512-byte sectors.  This is true across all versions of PostgreSQL.
+The control file is 8192 bytes but only the first 512 bytes are used to prevent torn pages even on really old storage with 512-byte
+sectors. This is true across all versions of PostgreSQL.
 ***********************************************************************************************************************************/
-#define PG_CONTROL_SIZE                                             ((unsigned int)(8 * 1024))
-#define PG_CONTROL_DATA_SIZE                                        ((unsigned int)(512))
+#define PG_CONTROL_DATA_SIZE                                        512
 
 /***********************************************************************************************************************************
 PostgreSQL interface definitions
 
-Each supported version of PostgreSQL must have interface files named postgres/interface/vXXX.c/h that implement the functions
-specified in the interface structure below.  The functions are documented here rather than in the interface files so that a change
-in wording does not need to be propagated through N source files.
+The interface functions are documented here rather than the auto-generated file.
 ***********************************************************************************************************************************/
 typedef struct PgInterface
 {
@@ -75,246 +72,10 @@ typedef struct PgInterface
 
     // Convert WAL header to a common data structure
     PgWal (*wal)(const unsigned char *);
-
-#ifdef DEBUG
-    // Catalog version for testing
-    unsigned int catalogVersion;
-
-    // Create pg_control for testing
-    void (*controlTest)(PgControl, unsigned char *);
-
-    // Create WAL header for testing
-    void (*walTest)(PgWal, unsigned char *);
-#endif
 } PgInterface;
 
-static const PgInterface pgInterface[] =
-{
-    {
-        .version = PG_VERSION_13,
-
-        .controlIs = pgInterfaceControlIs130,
-        .control = pgInterfaceControl130,
-        .controlVersion = pgInterfaceControlVersion130,
-
-        .walIs = pgInterfaceWalIs130,
-        .wal = pgInterfaceWal130,
-
-#ifdef DEBUG
-        .catalogVersion = 202007201,
-
-        .controlTest = pgInterfaceControlTest130,
-        .walTest = pgInterfaceWalTest130,
-#endif
-    },
-    {
-        .version = PG_VERSION_12,
-
-        .controlIs = pgInterfaceControlIs120,
-        .control = pgInterfaceControl120,
-        .controlVersion = pgInterfaceControlVersion120,
-
-        .walIs = pgInterfaceWalIs120,
-        .wal = pgInterfaceWal120,
-
-#ifdef DEBUG
-        .catalogVersion = 201909212,
-
-        .controlTest = pgInterfaceControlTest120,
-        .walTest = pgInterfaceWalTest120,
-#endif
-    },
-    {
-        .version = PG_VERSION_11,
-
-        .controlIs = pgInterfaceControlIs110,
-        .control = pgInterfaceControl110,
-        .controlVersion = pgInterfaceControlVersion110,
-
-        .walIs = pgInterfaceWalIs110,
-        .wal = pgInterfaceWal110,
-
-#ifdef DEBUG
-        .catalogVersion = 201809051,
-
-        .controlTest = pgInterfaceControlTest110,
-        .walTest = pgInterfaceWalTest110,
-#endif
-    },
-    {
-        .version = PG_VERSION_10,
-
-        .controlIs = pgInterfaceControlIs100,
-        .control = pgInterfaceControl100,
-        .controlVersion = pgInterfaceControlVersion100,
-
-        .walIs = pgInterfaceWalIs100,
-        .wal = pgInterfaceWal100,
-
-#ifdef DEBUG
-        .catalogVersion = 201707211,
-
-        .controlTest = pgInterfaceControlTest100,
-        .walTest = pgInterfaceWalTest100,
-#endif
-    },
-    {
-        .version = PG_VERSION_96,
-
-        .controlIs = pgInterfaceControlIs096,
-        .control = pgInterfaceControl096,
-        .controlVersion = pgInterfaceControlVersion096,
-
-        .walIs = pgInterfaceWalIs096,
-        .wal = pgInterfaceWal096,
-
-#ifdef DEBUG
-        .catalogVersion = 201608131,
-
-        .controlTest = pgInterfaceControlTest096,
-        .walTest = pgInterfaceWalTest096,
-#endif
-    },
-    {
-        .version = PG_VERSION_95,
-
-        .controlIs = pgInterfaceControlIs095,
-        .control = pgInterfaceControl095,
-        .controlVersion = pgInterfaceControlVersion095,
-
-        .walIs = pgInterfaceWalIs095,
-        .wal = pgInterfaceWal095,
-
-#ifdef DEBUG
-        .catalogVersion = 201510051,
-
-        .controlTest = pgInterfaceControlTest095,
-        .walTest = pgInterfaceWalTest095,
-#endif
-    },
-    {
-        .version = PG_VERSION_94,
-
-        .controlIs = pgInterfaceControlIs094,
-        .control = pgInterfaceControl094,
-        .controlVersion = pgInterfaceControlVersion094,
-
-        .walIs = pgInterfaceWalIs094,
-        .wal = pgInterfaceWal094,
-
-#ifdef DEBUG
-        .catalogVersion = 201409291,
-
-        .controlTest = pgInterfaceControlTest094,
-        .walTest = pgInterfaceWalTest094,
-#endif
-    },
-    {
-        .version = PG_VERSION_93,
-
-        .controlIs = pgInterfaceControlIs093,
-        .control = pgInterfaceControl093,
-        .controlVersion = pgInterfaceControlVersion093,
-
-        .walIs = pgInterfaceWalIs093,
-        .wal = pgInterfaceWal093,
-
-#ifdef DEBUG
-        .catalogVersion = 201306121,
-
-        .controlTest = pgInterfaceControlTest093,
-        .walTest = pgInterfaceWalTest093,
-#endif
-    },
-    {
-        .version = PG_VERSION_92,
-
-        .controlIs = pgInterfaceControlIs092,
-        .control = pgInterfaceControl092,
-        .controlVersion = pgInterfaceControlVersion092,
-
-        .walIs = pgInterfaceWalIs092,
-        .wal = pgInterfaceWal092,
-
-#ifdef DEBUG
-        .catalogVersion = 201204301,
-
-        .controlTest = pgInterfaceControlTest092,
-        .walTest = pgInterfaceWalTest092,
-#endif
-    },
-    {
-        .version = PG_VERSION_91,
-
-        .controlIs = pgInterfaceControlIs091,
-        .control = pgInterfaceControl091,
-        .controlVersion = pgInterfaceControlVersion091,
-
-        .walIs = pgInterfaceWalIs091,
-        .wal = pgInterfaceWal091,
-
-#ifdef DEBUG
-        .catalogVersion = 201105231,
-
-        .controlTest = pgInterfaceControlTest091,
-        .walTest = pgInterfaceWalTest091,
-#endif
-    },
-    {
-        .version = PG_VERSION_90,
-
-        .controlIs = pgInterfaceControlIs090,
-        .control = pgInterfaceControl090,
-        .controlVersion = pgInterfaceControlVersion090,
-
-        .walIs = pgInterfaceWalIs090,
-        .wal = pgInterfaceWal090,
-
-#ifdef DEBUG
-        .catalogVersion = 201008051,
-
-        .controlTest = pgInterfaceControlTest090,
-        .walTest = pgInterfaceWalTest090,
-#endif
-    },
-    {
-        .version = PG_VERSION_84,
-
-        .controlIs = pgInterfaceControlIs084,
-        .control = pgInterfaceControl084,
-        .controlVersion = pgInterfaceControlVersion084,
-
-        .walIs = pgInterfaceWalIs084,
-        .wal = pgInterfaceWal084,
-
-#ifdef DEBUG
-        .catalogVersion = 200904091,
-
-        .controlTest = pgInterfaceControlTest084,
-        .walTest = pgInterfaceWalTest084,
-#endif
-    },
-    {
-        .version = PG_VERSION_83,
-
-        .controlIs = pgInterfaceControlIs083,
-        .control = pgInterfaceControl083,
-        .controlVersion = pgInterfaceControlVersion083,
-
-        .walIs = pgInterfaceWalIs083,
-        .wal = pgInterfaceWal083,
-
-#ifdef DEBUG
-        .catalogVersion = 200711281,
-
-        .controlTest = pgInterfaceControlTest083,
-        .walTest = pgInterfaceWalTest083,
-#endif
-    },
-};
-
-// Total PostgreSQL versions in pgInterface
-#define PG_INTERFACE_SIZE                                           (sizeof(pgInterface) / sizeof(PgInterface))
+// Include auto-generated interfaces
+#include "postgres/interface.auto.c.inc"
 
 /***********************************************************************************************************************************
 These pg_control fields are common to all versions of PostgreSQL, so we can use them to generate error messages when the pg_control
@@ -339,7 +100,7 @@ pgInterfaceVersion(unsigned int pgVersion)
 
     const PgInterface *result = NULL;
 
-    for (unsigned int interfaceIdx = 0; interfaceIdx < PG_INTERFACE_SIZE; interfaceIdx++)
+    for (unsigned int interfaceIdx = 0; interfaceIdx < LENGTH_OF(pgInterface); interfaceIdx++)
     {
         if (pgInterface[interfaceIdx].version == pgVersion)
         {
@@ -352,7 +113,40 @@ pgInterfaceVersion(unsigned int pgVersion)
     if (result == NULL)
         THROW_FMT(AssertError, "invalid " PG_NAME " version %u", pgVersion);
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN_TYPE_CONST_P(PgInterface, result);
+}
+
+/**********************************************************************************************************************************/
+bool
+pgDbIsTemplate(const String *const name)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, name);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(BOOL, strEqZ(name, "template0") || strEqZ(name, "template1"));
+}
+
+/**********************************************************************************************************************************/
+bool
+pgDbIsSystem(const String *const name)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, name);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(BOOL, strEqZ(name, "postgres") || pgDbIsTemplate(name));
+}
+
+/**********************************************************************************************************************************/
+bool
+pgDbIsSystemId(const unsigned int id)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(UINT, id);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(BOOL, id < FirstNormalObjectId);
 }
 
 /***********************************************************************************************************************************
@@ -377,7 +171,7 @@ pgWalSegmentSizeCheck(unsigned int pgVersion, unsigned int walSegmentSize)
 }
 
 /**********************************************************************************************************************************/
-PgControl
+static PgControl
 pgControlFromBuffer(const Buffer *controlFile)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
@@ -389,7 +183,7 @@ pgControlFromBuffer(const Buffer *controlFile)
     // Search for the version of PostgreSQL that uses this control file
     const PgInterface *interface = NULL;
 
-    for (unsigned int interfaceIdx = 0; interfaceIdx < PG_INTERFACE_SIZE; interfaceIdx++)
+    for (unsigned int interfaceIdx = 0; interfaceIdx < LENGTH_OF(pgInterface); interfaceIdx++)
     {
         if (pgInterface[interfaceIdx].controlIs(bufPtrConst(controlFile)))
         {
@@ -456,7 +250,7 @@ pgControlVersion(unsigned int pgVersion)
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgInterfaceVersion(pgVersion)->controlVersion());
+    FUNCTION_TEST_RETURN(UINT32, pgInterfaceVersion(pgVersion)->controlVersion());
 }
 
 /***********************************************************************************************************************************
@@ -488,7 +282,7 @@ pgWalFromBuffer(const Buffer *walBuffer)
     // Search for the version of PostgreSQL that uses this WAL magic
     const PgInterface *interface = NULL;
 
-    for (unsigned int interfaceIdx = 0; interfaceIdx < PG_INTERFACE_SIZE; interfaceIdx++)
+    for (unsigned int interfaceIdx = 0; interfaceIdx < LENGTH_OF(pgInterface); interfaceIdx++)
     {
         if (pgInterface[interfaceIdx].walIs(bufPtrConst(walBuffer)))
         {
@@ -551,22 +345,19 @@ pgTablespaceId(unsigned int pgVersion, unsigned int pgCatalogVersion)
 
     String *result = NULL;
 
-    if (pgVersion >= PG_VERSION_90)
+    MEM_CONTEXT_TEMP_BEGIN()
     {
-        MEM_CONTEXT_TEMP_BEGIN()
+        String *pgVersionStr = pgVersionToStr(pgVersion);
+
+        MEM_CONTEXT_PRIOR_BEGIN()
         {
-            String *pgVersionStr = pgVersionToStr(pgVersion);
-
-            MEM_CONTEXT_PRIOR_BEGIN()
-            {
-                result = strNewFmt("PG_%s_%u", strZ(pgVersionStr), pgCatalogVersion);
-            }
-            MEM_CONTEXT_PRIOR_END();
+            result = strNewFmt("PG_%s_%u", strZ(pgVersionStr), pgCatalogVersion);
         }
-        MEM_CONTEXT_TEMP_END();
+        MEM_CONTEXT_PRIOR_END();
     }
+    MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 /**********************************************************************************************************************************/
@@ -583,13 +374,13 @@ pgLsnFromStr(const String *lsn)
     {
         StringList *lsnPart = strLstNewSplit(lsn, FSLASH_STR);
 
-        CHECK(strLstSize(lsnPart) == 2);
+        CHECK(FormatError, strLstSize(lsnPart) == 2, "lsn requires two parts");
 
         result = (cvtZToUInt64Base(strZ(strLstGet(lsnPart, 0)), 16) << 32) + cvtZToUInt64Base(strZ(strLstGet(lsnPart, 1)), 16);
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(UINT64, result);
 }
 
 String *
@@ -599,7 +390,7 @@ pgLsnToStr(uint64_t lsn)
         FUNCTION_TEST_PARAM(UINT64, lsn);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(strNewFmt("%x/%x", (unsigned int)(lsn >> 32), (unsigned int)(lsn & 0xFFFFFFFF)));
+    FUNCTION_TEST_RETURN(STRING, strNewFmt("%x/%x", (unsigned int)(lsn >> 32), (unsigned int)(lsn & 0xFFFFFFFF)));
 }
 
 /**********************************************************************************************************************************/
@@ -613,11 +404,11 @@ pgLsnToWalSegment(uint32_t timeline, uint64_t lsn, unsigned int walSegmentSize)
     FUNCTION_TEST_END();
 
     FUNCTION_TEST_RETURN(
-        strNewFmt("%08X%08X%08X", timeline, (unsigned int)(lsn >> 32), (unsigned int)(lsn & 0xFFFFFFFF) / walSegmentSize));
+        STRING, strNewFmt("%08X%08X%08X", timeline, (unsigned int)(lsn >> 32), (unsigned int)(lsn & 0xFFFFFFFF) / walSegmentSize));
 }
 
 uint64_t
-pgLsnFromWalSegment(const String *walSegment, unsigned int walSegmentSize)
+pgLsnFromWalSegment(const String *const walSegment, const unsigned int walSegmentSize)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(STRING, walSegment);
@@ -629,14 +420,30 @@ pgLsnFromWalSegment(const String *walSegment, unsigned int walSegmentSize)
     ASSERT(walSegmentSize > 0);
 
     FUNCTION_TEST_RETURN(
-        (cvtZToUInt64Base(strZ(strSubN(walSegment, 8, 8)), 16) << 32) +
-        (cvtZToUInt64Base(strZ(strSubN(walSegment, 16, 8)), 16) * walSegmentSize));
+        UINT64,
+        (cvtZSubNToUInt64Base(strZ(walSegment), 8, 8, 16) << 32) +
+        (cvtZSubNToUInt64Base(strZ(walSegment), 16, 8, 16) * walSegmentSize));
+}
+
+/**********************************************************************************************************************************/
+uint32_t
+pgTimelineFromWalSegment(const String *const walSegment)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, walSegment);
+    FUNCTION_TEST_END();
+
+    ASSERT(walSegment != NULL);
+    ASSERT(strSize(walSegment) == 24);
+
+    FUNCTION_TEST_RETURN(UINT32, cvtZSubNToUIntBase(strZ(walSegment), 0, 8, 16));
 }
 
 /**********************************************************************************************************************************/
 StringList *
 pgLsnRangeToWalSegmentList(
-    unsigned int pgVersion, uint32_t timeline, uint64_t lsnStart, uint64_t lsnStop, unsigned int walSegmentSize)
+    const unsigned int pgVersion, const uint32_t timeline, const uint64_t lsnStart, const uint64_t lsnStop,
+    const unsigned int walSegmentSize)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(UINT, pgVersion);
@@ -652,26 +459,24 @@ pgLsnRangeToWalSegmentList(
     ASSERT(walSegmentSize != 0);
     ASSERT(pgVersion > PG_VERSION_92 || walSegmentSize == PG_WAL_SEGMENT_SIZE_DEFAULT);
 
-    StringList *result = NULL;
+    StringList *const result = strLstNew();
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        result = strLstNew();
-
         // Skip the FF segment when PostgreSQL <= 9.2 (in this case segment size should always be 16MB)
-        bool skipFF = pgVersion <= PG_VERSION_92;
+        const bool skipFF = pgVersion <= PG_VERSION_92;
 
         // Calculate the start and stop segments
         unsigned int startMajor = (unsigned int)(lsnStart >> 32);
         unsigned int startMinor = (unsigned int)(lsnStart & 0xFFFFFFFF) / walSegmentSize;
 
-        unsigned int stopMajor = (unsigned int)(lsnStop >> 32);
-        unsigned int stopMinor = (unsigned int)(lsnStop & 0xFFFFFFFF) / walSegmentSize;
+        const unsigned int stopMajor = (unsigned int)(lsnStop >> 32);
+        const unsigned int stopMinor = (unsigned int)(lsnStop & 0xFFFFFFFF) / walSegmentSize;
 
-        unsigned int minorPerMajor = 0xFFFFFFFF / walSegmentSize;
+        const unsigned int minorPerMajor = 0xFFFFFFFF / walSegmentSize;
 
         // Create list
-        strLstAdd(result, strNewFmt("%08X%08X%08X", timeline, startMajor, startMinor));
+        strLstAddFmt(result, "%08X%08X%08X", timeline, startMajor, startMinor);
 
         while (!(startMajor == stopMajor && startMinor == stopMinor))
         {
@@ -683,14 +488,12 @@ pgLsnRangeToWalSegmentList(
                 startMinor = 0;
             }
 
-            strLstAdd(result, strNewFmt("%08X%08X%08X", timeline, startMajor, startMinor));
+            strLstAddFmt(result, "%08X%08X%08X", timeline, startMajor, startMinor);
         }
-
-        strLstMove(result, memContextPrior());
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING_LIST, result);
 }
 
 /**********************************************************************************************************************************/
@@ -701,7 +504,7 @@ pgLsnName(unsigned int pgVersion)
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgVersion >= PG_VERSION_WAL_RENAME ? PG_NAME_LSN_STR : PG_NAME_LOCATION_STR);
+    FUNCTION_TEST_RETURN_CONST(STRING, pgVersion >= PG_VERSION_WAL_RENAME ? PG_NAME_LSN_STR : PG_NAME_LOCATION_STR);
 }
 
 /***********************************************************************************************************************************
@@ -714,7 +517,7 @@ pgWalName(unsigned int pgVersion)
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgVersion >= PG_VERSION_WAL_RENAME ? PG_NAME_WAL_STR : PG_NAME_XLOG_STR);
+    FUNCTION_TEST_RETURN_CONST(STRING, pgVersion >= PG_VERSION_WAL_RENAME ? PG_NAME_WAL_STR : PG_NAME_XLOG_STR);
 }
 
 /**********************************************************************************************************************************/
@@ -725,7 +528,7 @@ pgWalPath(unsigned int pgVersion)
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgVersion >= PG_VERSION_WAL_RENAME ? PG_PATH_PGWAL_STR : PG_PATH_PGXLOG_STR);
+    FUNCTION_TEST_RETURN_CONST(STRING, pgVersion >= PG_VERSION_WAL_RENAME ? PG_PATH_PGWAL_STR : PG_PATH_PGXLOG_STR);
 }
 
 /**********************************************************************************************************************************/
@@ -736,81 +539,12 @@ pgXactPath(unsigned int pgVersion)
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgVersion >= PG_VERSION_WAL_RENAME ? PG_PATH_PGXACT_STR : PG_PATH_PGCLOG_STR);
+    FUNCTION_TEST_RETURN_CONST(STRING, pgVersion >= PG_VERSION_WAL_RENAME ? PG_PATH_PGXACT_STR : PG_PATH_PGCLOG_STR);
 }
 
-/**********************************************************************************************************************************/
-#ifdef DEBUG
-
-unsigned int
-pgCatalogTestVersion(unsigned int pgVersion)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(UINT, pgVersion);
-    FUNCTION_TEST_END();
-
-    FUNCTION_TEST_RETURN(pgInterfaceVersion(pgVersion)->catalogVersion);
-}
-
-#endif
-
-/**********************************************************************************************************************************/
-#ifdef DEBUG
-
-Buffer *
-pgControlTestToBuffer(PgControl pgControl)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(PG_CONTROL, pgControl);
-    FUNCTION_TEST_END();
-
-    // Set defaults if values are not passed
-    pgControl.pageSize = pgControl.pageSize == 0 ? PG_PAGE_SIZE_DEFAULT : pgControl.pageSize;
-    pgControl.walSegmentSize = pgControl.walSegmentSize == 0 ? PG_WAL_SEGMENT_SIZE_DEFAULT : pgControl.walSegmentSize;
-    pgControl.catalogVersion = pgControl.catalogVersion == 0 ?
-        pgInterfaceVersion(pgControl.version)->catalogVersion : pgControl.catalogVersion;
-
-    // Create the buffer and clear it
-    Buffer *result = bufNew(PG_CONTROL_SIZE);
-    memset(bufPtr(result), 0, bufSize(result));
-    bufUsedSet(result, bufSize(result));
-
-    // Generate pg_control
-    pgInterfaceVersion(pgControl.version)->controlTest(pgControl, bufPtr(result));
-
-    FUNCTION_TEST_RETURN(result);
-}
-
-#endif
-
-/**********************************************************************************************************************************/
-#ifdef DEBUG
-
-void
-pgWalTestToBuffer(PgWal pgWal, Buffer *walBuffer)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(PG_WAL, pgWal);
-        FUNCTION_TEST_PARAM(BUFFER, walBuffer);
-    FUNCTION_TEST_END();
-
-    ASSERT(walBuffer != NULL);
-
-    // Set default WAL segment size if not specified
-    if (pgWal.size == 0)
-        pgWal.size = PG_WAL_SEGMENT_SIZE_DEFAULT;
-
-    // Generate WAL
-    pgInterfaceVersion(pgWal.version)->walTest(pgWal, bufPtr(walBuffer));
-
-    FUNCTION_TEST_RETURN_VOID();
-}
-
-#endif
-
 /**********************************************************************************************************************************/
 unsigned int
-pgVersionFromStr(const String *version)
+pgVersionFromStr(const String *const version)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(STRING, version);
@@ -818,33 +552,19 @@ pgVersionFromStr(const String *version)
 
     ASSERT(version != NULL);
 
-    unsigned int result = 0;
+    // If format not number.number (9.4) or number only (10) then error. No check for valid/supported PG version is on purpose.
+    if (!regExpMatchOne(STRDEF("^[0-9]+[.]*[0-9]+$"), version))
+        THROW_FMT(AssertError, "version %s format is invalid", strZ(version));
 
-    MEM_CONTEXT_TEMP_BEGIN()
-    {
-        // If format is not number.number (9.4) or number only (10) then error
-        if (!regExpMatchOne(STRDEF("^[0-9]+[.]*[0-9]+$"), version))
-            THROW_FMT(AssertError, "version %s format is invalid", strZ(version));
+    // If there is no dot then only the major version is needed
+    const int idxStart = strChr(version, '.');
 
-        // If there is a dot set the major and minor versions, else just the major
-        int idxStart = strChr(version, '.');
-        unsigned int major;
-        unsigned int minor = 0;
+    if (idxStart == -1)
+        FUNCTION_LOG_RETURN(UINT, cvtZToUInt(strZ(version)) * 10000);
 
-        if (idxStart != -1)
-        {
-            major = cvtZToUInt(strZ(strSubN(version, 0, (size_t)idxStart)));
-            minor = cvtZToUInt(strZ(strSub(version, (size_t)idxStart + 1)));
-        }
-        else
-            major = cvtZToUInt(strZ(version));
-
-        // No check to see if valid/supported PG version is on purpose
-        result = major * 10000 + minor * 100;
-    }
-    MEM_CONTEXT_TEMP_END();
-
-    FUNCTION_LOG_RETURN(UINT, result);
+    // Major and minor version are needed
+    FUNCTION_LOG_RETURN(
+        UINT, cvtZSubNToUInt(strZ(version), 0, (size_t)idxStart) * 10000 + cvtZToUInt(strZ(version) + (size_t)idxStart + 1) * 100);
 }
 
 String *
