@@ -10,7 +10,7 @@ PostgreSQL Page Interface
 /***********************************************************************************************************************************
 Include the page checksum code
 ***********************************************************************************************************************************/
-#include "postgres/interface/pageChecksum.vendor.c"
+#include "postgres/interface/pageChecksum.vendor.c.inc"
 
 /**********************************************************************************************************************************/
 uint16_t
