@@ -47,7 +47,7 @@ protocolParallelNew(TimeMSec timeout, ParallelJobCallback *callbackFunction, voi
 
     ProtocolParallel *this = NULL;
 
-    OBJ_NEW_BEGIN(ProtocolParallel)
+    OBJ_NEW_BEGIN(ProtocolParallel, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
     {
         this = OBJ_NEW_ALLOC();
 
@@ -105,11 +105,11 @@ protocolParallelProcess(ProtocolParallel *this)
         // If called for the first time, initialize processing
         if (this->state == protocolParallelJobStatePending)
         {
-            MEM_CONTEXT_BEGIN(objMemContext(this))
+            MEM_CONTEXT_OBJ_BEGIN(this)
             {
                 this->clientJobList = memNewPtrArray(lstSize(this->clientList));
             }
-            MEM_CONTEXT_END();
+            MEM_CONTEXT_OBJ_END();
 
             this->state = protocolParallelJobStateRunning;
         }
