@@ -27,7 +27,7 @@ protocolParallelJobNew(const Variant *key, ProtocolCommand *command)
 
     ProtocolParallelJob *this = NULL;
 
-    OBJ_NEW_BEGIN(ProtocolParallelJob)
+    OBJ_NEW_BEGIN(ProtocolParallelJob, .childQty = MEM_CONTEXT_QTY_MAX)
     {
         this = OBJ_NEW_ALLOC();
 
@@ -61,12 +61,12 @@ protocolParallelJobErrorSet(ProtocolParallelJob *this, int code, const String *m
     ASSERT(code != 0);
     ASSERT(message != NULL);
 
-    MEM_CONTEXT_BEGIN(objMemContext(this))
+    MEM_CONTEXT_OBJ_BEGIN(this)
     {
         this->pub.code = code;
         this->pub.message = strDup(message);
     }
-    MEM_CONTEXT_END();
+    MEM_CONTEXT_OBJ_END();
 
     FUNCTION_LOG_RETURN_VOID();
 }
