@@ -142,6 +142,7 @@ storageAzureAuth(
             // Generate string to sign
             const String *contentLength = httpHeaderGet(httpHeader, HTTP_HEADER_CONTENT_LENGTH_STR);
             const String *contentMd5 = httpHeaderGet(httpHeader, HTTP_HEADER_CONTENT_MD5_STR);
+            const String *const range = httpHeaderGet(httpHeader, HTTP_HEADER_RANGE_STR);
 
             const String *stringToSign = strNewFmt(
                 "%s\n"                                                  // verb
@@ -155,17 +156,18 @@ storageAzureAuth(
                 "\n"                                                    // If-Match
                 "\n"                                                    // If-None-Match
                 "\n"                                                    // If-Unmodified-Since
-                "\n"                                                    // range
+                "%s\n"                                                  // range
                 "%s"                                                    // Canonicalized headers
                 "/%s%s"                                                 // Canonicalized account/path
                 "%s",                                                   // Canonicalized query
                 strZ(verb), strEq(contentLength, ZERO_STR) ? "" : strZ(contentLength), contentMd5 == NULL ? "" : strZ(contentMd5),
-                strZ(dateTime), strZ(headerCanonical), strZ(this->account), strZ(path), strZ(queryCanonical));
+                strZ(dateTime), range == NULL ? "" : strZ(range), strZ(headerCanonical), strZ(this->account), strZ(path),
+                strZ(queryCanonical));
 
             // Generate authorization header
             httpHeaderPut(
                 httpHeader, HTTP_HEADER_AUTHORIZATION_STR, strNewFmt("SharedKey %s:%s", strZ(this->account),
-                strZ(strNewEncode(encodeBase64, cryptoHmacOne(HASH_TYPE_SHA256_STR, this->sharedKey, BUFSTR(stringToSign))))));
+                strZ(strNewEncode(encodeBase64, cryptoHmacOne(hashTypeSha256, this->sharedKey, BUFSTR(stringToSign))))));
         }
         // SAS authentication
         else
@@ -214,8 +216,7 @@ storageAzureRequestAsync(StorageAzure *this, const String *verb, StorageAzureReq
         if (param.content != NULL)
         {
             httpHeaderAdd(
-                requestHeader, HTTP_HEADER_CONTENT_MD5_STR,
-                strNewEncode(encodeBase64, cryptoHashOne(HASH_TYPE_MD5_STR, param.content)));
+                requestHeader, HTTP_HEADER_CONTENT_MD5_STR, strNewEncode(encodeBase64, cryptoHashOne(hashTypeMd5, param.content)));
         }
 
         // Encode path
@@ -287,12 +288,13 @@ storageAzureRequest(StorageAzure *this, const String *verb, StorageAzureRequestP
         FUNCTION_LOG_PARAM(BOOL, param.contentIo);
     FUNCTION_LOG_END();
 
-    FUNCTION_LOG_RETURN(
-        HTTP_RESPONSE,
-        storageAzureResponseP(
-            storageAzureRequestAsyncP(
-                this, verb, .path = param.path, .header = param.header, .query = param.query, .content = param.content),
-            .allowMissing = param.allowMissing, .contentIo = param.contentIo));
+    HttpRequest *const request = storageAzureRequestAsyncP(
+        this, verb, .path = param.path, .header = param.header, .query = param.query, .content = param.content);
+    HttpResponse *const result = storageAzureResponseP(request, .allowMissing = param.allowMissing, .contentIo = param.contentIo);
+
+    httpRequestFree(request);
+
+    FUNCTION_LOG_RETURN(HTTP_RESPONSE, result);
 }
 
 /***********************************************************************************************************************************
@@ -301,7 +303,7 @@ General function for listing files to be used by other list routines
 static void
 storageAzureListInternal(
     StorageAzure *this, const String *path, StorageInfoLevel level, const String *expression, bool recurse,
-    StorageInfoListCallback callback, void *callbackData)
+    StorageListCallback callback, void *callbackData)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE_AZURE, this);
@@ -486,7 +488,7 @@ storageAzureInfo(THIS_VOID, const String *file, StorageInfoLevel level, StorageI
     ASSERT(file != NULL);
 
     // Attempt to get file info
-    HttpResponse *httpResponse = storageAzureRequestP(this, HTTP_VERB_HEAD_STR, .path = file, .allowMissing = true);
+    HttpResponse *const httpResponse = storageAzureRequestP(this, HTTP_VERB_HEAD_STR, .path = file, .allowMissing = true);
 
     // Does the file exist?
     StorageInfo result = {.level = level, .exists = httpResponseCodeOk(httpResponse)};
@@ -498,14 +500,30 @@ storageAzureInfo(THIS_VOID, const String *file, StorageInfoLevel level, StorageI
         result.timeModified = httpDateToTime(httpHeaderGet(httpResponseHeader(httpResponse), HTTP_HEADER_LAST_MODIFIED_STR));
     }
 
+    httpResponseFree(httpResponse);
+
     FUNCTION_LOG_RETURN(STORAGE_INFO, result);
 }
 
 /**********************************************************************************************************************************/
-static bool
-storageAzureInfoList(
-    THIS_VOID, const String *path, StorageInfoLevel level, StorageInfoListCallback callback, void *callbackData,
-    StorageInterfaceInfoListParam param)
+static void
+storageAzureListCallback(void *const callbackData, const StorageInfo *const info)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM_P(VOID, callbackData);
+        FUNCTION_TEST_PARAM(STORAGE_INFO, info);
+    FUNCTION_TEST_END();
+
+    ASSERT(callbackData != NULL);
+    ASSERT(info != NULL);
+
+    storageLstAdd(callbackData, info);
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
+static StorageList *
+storageAzureList(THIS_VOID, const String *const path, const StorageInfoLevel level, const StorageInterfaceListParam param)
 {
     THIS(StorageAzure);
 
@@ -513,18 +531,17 @@ storageAzureInfoList(
         FUNCTION_LOG_PARAM(STORAGE_AZURE, this);
         FUNCTION_LOG_PARAM(STRING, path);
         FUNCTION_LOG_PARAM(ENUM, level);
-        FUNCTION_LOG_PARAM(FUNCTIONP, callback);
-        FUNCTION_LOG_PARAM_P(VOID, callbackData);
         FUNCTION_LOG_PARAM(STRING, param.expression);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(path != NULL);
-    ASSERT(callback != NULL);
 
-    storageAzureListInternal(this, path, level, param.expression, false, callback, callbackData);
+    StorageList *const result = storageLstNew(level);
 
-    FUNCTION_LOG_RETURN(BOOL, true);
+    storageAzureListInternal(this, path, level, param.expression, false, storageAzureListCallback, result);
+
+    FUNCTION_LOG_RETURN(STORAGE_LIST, result);
 }
 
 /**********************************************************************************************************************************/
@@ -537,13 +554,14 @@ storageAzureNewRead(THIS_VOID, const String *file, bool ignoreMissing, StorageIn
         FUNCTION_LOG_PARAM(STORAGE_AZURE, this);
         FUNCTION_LOG_PARAM(STRING, file);
         FUNCTION_LOG_PARAM(BOOL, ignoreMissing);
-        (void)param;                                                // No parameters are used
+        FUNCTION_LOG_PARAM(UINT64, param.offset);
+        FUNCTION_LOG_PARAM(VARIANT, param.limit);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(file != NULL);
 
-    FUNCTION_LOG_RETURN(STORAGE_READ, storageReadAzureNew(this, file, ignoreMissing));
+    FUNCTION_LOG_RETURN(STORAGE_READ, storageReadAzureNew(this, file, ignoreMissing, param.offset, param.limit));
 }
 
 /**********************************************************************************************************************************/
@@ -578,7 +596,7 @@ typedef struct StorageAzurePathRemoveData
 } StorageAzurePathRemoveData;
 
 static void
-storageAzurePathRemoveCallback(void *callbackData, const StorageInfo *info)
+storageAzurePathRemoveCallback(void *const callbackData, const StorageInfo *const info)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM_P(VOID, callbackData);
@@ -588,13 +606,12 @@ storageAzurePathRemoveCallback(void *callbackData, const StorageInfo *info)
     ASSERT(callbackData != NULL);
     ASSERT(info != NULL);
 
-    StorageAzurePathRemoveData *data = callbackData;
+    StorageAzurePathRemoveData *const data = callbackData;
 
     // Get response from prior async request
     if (data->request != NULL)
     {
-        storageAzureResponseP(data->request, .allowMissing = true);
-
+        httpResponseFree(storageAzureResponseP(data->request, .allowMissing = true));
         httpRequestFree(data->request);
         data->request = NULL;
     }
@@ -664,7 +681,7 @@ storageAzureRemove(THIS_VOID, const String *file, StorageInterfaceRemoveParam pa
     ASSERT(file != NULL);
     ASSERT(!param.errorOnMissing);
 
-    storageAzureRequestP(this, HTTP_VERB_DELETE_STR, file, .allowMissing = true);
+    httpResponseFree(storageAzureRequestP(this, HTTP_VERB_DELETE_STR, file, .allowMissing = true));
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -673,7 +690,7 @@ storageAzureRemove(THIS_VOID, const String *file, StorageInterfaceRemoveParam pa
 static const StorageInterface storageInterfaceAzure =
 {
     .info = storageAzureInfo,
-    .infoList = storageAzureInfoList,
+    .list = storageAzureList,
     .newRead = storageAzureNewRead,
     .newWrite = storageAzureNewWrite,
     .pathRemove = storageAzurePathRemove,
@@ -714,7 +731,7 @@ storageAzureNew(
 
     Storage *this = NULL;
 
-    OBJ_NEW_BEGIN(StorageAzure)
+    OBJ_NEW_BEGIN(StorageAzure, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
     {
         StorageAzure *driver = OBJ_NEW_ALLOC();
 
@@ -724,7 +741,7 @@ storageAzureNew(
             .container = strDup(container),
             .account = strDup(account),
             .blockSize = blockSize,
-            .host = uriStyle == storageAzureUriStyleHost ? strNewFmt("%s.%s", strZ(account), strZ(endpoint)) : endpoint,
+            .host = uriStyle == storageAzureUriStyleHost ? strNewFmt("%s.%s", strZ(account), strZ(endpoint)) : strDup(endpoint),
             .pathPrefix = uriStyle == storageAzureUriStyleHost ?
                 strNewFmt("/%s", strZ(container)) : strNewFmt("/%s/%s", strZ(account), strZ(container)),
         };
