@@ -10,6 +10,6 @@ GCS Storage Read
 /***********************************************************************************************************************************
 Constructors
 ***********************************************************************************************************************************/
-StorageRead *storageReadGcsNew(StorageGcs *storage, const String *name, bool ignoreMissing);
+StorageRead *storageReadGcsNew(StorageGcs *storage, const String *name, bool ignoreMissing, uint64_t offset, const Variant *limit);
 
 #endif
