@@ -71,8 +71,6 @@ storageReadPosixOpen(THIS_VOID)
     ASSERT(this != NULL);
     ASSERT(this->fd == -1);
 
-    bool result = false;
-
     // Open the file
     this->fd = open(strZ(this->interface.name), O_RDONLY, 0);
 
@@ -87,15 +85,22 @@ storageReadPosixOpen(THIS_VOID)
         else
             THROW_SYS_ERROR_FMT(FileOpenError, STORAGE_ERROR_READ_OPEN, strZ(this->interface.name));                // {vm_covered}
     }
-
-    // On success set free callback to ensure the file descriptor is freed
-    if (this->fd != -1)
+    // Else success
+    else
     {
-        memContextCallbackSet(THIS_MEM_CONTEXT(), storageReadPosixFreeResource, this);
-        result = true;
+        // Set free callback to ensure the file descriptor is freed
+        memContextCallbackSet(objMemContext(this), storageReadPosixFreeResource, this);
+
+        // Seek to offset
+        if (this->interface.offset != 0)
+        {
+            THROW_ON_SYS_ERROR_FMT(
+                lseek(this->fd, (off_t)this->interface.offset, SEEK_SET) == -1, FileOpenError, STORAGE_ERROR_READ_SEEK,
+                this->interface.offset, strZ(this->interface.name));
+        }
     }
 
-    FUNCTION_LOG_RETURN(BOOL, result);
+    FUNCTION_LOG_RETURN(BOOL, this->fd != -1);
 }
 
 /***********************************************************************************************************************************
@@ -160,7 +165,7 @@ storageReadPosixClose(THIS_VOID)
 
     ASSERT(this != NULL);
 
-    memContextCallbackClear(THIS_MEM_CONTEXT());
+    memContextCallbackClear(objMemContext(this));
     storageReadPosixFreeResource(this);
     this->fd = -1;
 
@@ -181,7 +186,7 @@ storageReadPosixEof(THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->eof);
+    FUNCTION_TEST_RETURN(BOOL, this->eof);
 }
 
 /***********************************************************************************************************************************
@@ -198,16 +203,19 @@ storageReadPosixFd(const THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->fd);
+    FUNCTION_TEST_RETURN(INT, this->fd);
 }
 
 /**********************************************************************************************************************************/
 StorageRead *
-storageReadPosixNew(StoragePosix *storage, const String *name, bool ignoreMissing, const Variant *limit)
+storageReadPosixNew(
+    StoragePosix *const storage, const String *const name, const bool ignoreMissing, const uint64_t offset,
+    const Variant *const limit)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(STRING, name);
         FUNCTION_LOG_PARAM(BOOL, ignoreMissing);
+        FUNCTION_LOG_PARAM(UINT64, offset);
         FUNCTION_LOG_PARAM(VARIANT, limit);
     FUNCTION_LOG_END();
 
@@ -215,7 +223,7 @@ storageReadPosixNew(StoragePosix *storage, const String *name, bool ignoreMissin
 
     StorageRead *this = NULL;
 
-    OBJ_NEW_BEGIN(StorageReadPosix)
+    OBJ_NEW_BEGIN(StorageReadPosix, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
         StorageReadPosix *driver = OBJ_NEW_ALLOC();
 
@@ -234,6 +242,7 @@ storageReadPosixNew(StoragePosix *storage, const String *name, bool ignoreMissin
                 .type = STORAGE_POSIX_TYPE,
                 .name = strDup(name),
                 .ignoreMissing = ignoreMissing,
+                .offset = offset,
                 .limit = varDup(limit),
 
                 .ioInterface = (IoReadInterface)
