@@ -14,7 +14,6 @@ Remote Storage Protocol Handler
 #include "common/log.h"
 #include "common/regExp.h"
 #include "common/type/pack.h"
-#include "common/type/json.h"
 #include "config/config.h"
 #include "protocol/helper.h"
 #include "storage/remote/protocol.h"
@@ -34,7 +33,7 @@ static struct
 Set filter group based on passed filters
 ***********************************************************************************************************************************/
 static void
-storageRemoteFilterGroup(IoFilterGroup *filterGroup, const Pack *const filterPack)
+storageRemoteFilterGroup(IoFilterGroup *const filterGroup, const Pack *const filterPack)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(IO_FILTER_GROUP, filterGroup);
@@ -44,46 +43,50 @@ storageRemoteFilterGroup(IoFilterGroup *filterGroup, const Pack *const filterPac
     ASSERT(filterGroup != NULL);
     ASSERT(filterPack != NULL);
 
-    PackRead *const filterList = pckReadNew(filterPack);
-
-    while (!pckReadNullP(filterList))
+    MEM_CONTEXT_TEMP_BEGIN()
     {
-        const StringId filterKey = pckReadStrIdP(filterList);
-        const Pack *const filterParam = pckReadPackP(filterList);
+        PackRead *const filterList = pckReadNew(filterPack);
 
-        IoFilter *filter = compressFilterPack(filterKey, filterParam);
-
-        if (filter != NULL)
-            ioFilterGroupAdd(filterGroup, filter);
-        else
+        while (!pckReadNullP(filterList))
         {
-            switch (filterKey)
+            const StringId filterKey = pckReadStrIdP(filterList);
+            const Pack *const filterParam = pckReadPackP(filterList);
+
+            IoFilter *filter = compressFilterPack(filterKey, filterParam);
+
+            if (filter != NULL)
+                ioFilterGroupAdd(filterGroup, filter);
+            else
             {
-                case CIPHER_BLOCK_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, cipherBlockNewPack(filterParam));
-                    break;
+                switch (filterKey)
+                {
+                    case CIPHER_BLOCK_FILTER_TYPE:
+                        ioFilterGroupAdd(filterGroup, cipherBlockNewPack(filterParam));
+                        break;
 
-                case CRYPTO_HASH_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, cryptoHashNewPack(filterParam));
-                    break;
+                    case CRYPTO_HASH_FILTER_TYPE:
+                        ioFilterGroupAdd(filterGroup, cryptoHashNewPack(filterParam));
+                        break;
 
-                case PAGE_CHECKSUM_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, pageChecksumNewPack(filterParam));
-                    break;
+                    case PAGE_CHECKSUM_FILTER_TYPE:
+                        ioFilterGroupAdd(filterGroup, pageChecksumNewPack(filterParam));
+                        break;
 
-                case SINK_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, ioSinkNew());
-                    break;
+                    case SINK_FILTER_TYPE:
+                        ioFilterGroupAdd(filterGroup, ioSinkNew());
+                        break;
 
-                case SIZE_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, ioSizeNew());
-                    break;
+                    case SIZE_FILTER_TYPE:
+                        ioFilterGroupAdd(filterGroup, ioSizeNew());
+                        break;
 
-                default:
-                    THROW_FMT(AssertError, "unable to add filter '%s'", strZ(strIdToStr(filterKey)));
+                    default:
+                        THROW_FMT(AssertError, "unable to add filter '%s'", strZ(strIdToStr(filterKey)));
+                }
             }
         }
     }
+    MEM_CONTEXT_TEMP_END();
 
     FUNCTION_TEST_RETURN_VOID();
 }
@@ -111,7 +114,7 @@ storageRemoteFeatureProtocol(PackRead *const param, ProtocolServer *const server
         {
             MEM_CONTEXT_PRIOR_BEGIN()
             {
-                MEM_CONTEXT_NEW_BEGIN("StorageRemoteProtocol")
+                MEM_CONTEXT_NEW_BEGIN(StorageRemoteProtocol, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
                 {
                     storageRemoteProtocolLocal.memContext = memContextCurrent();
                     storageRemoteProtocolLocal.driver = storageDriver(storage);
@@ -214,7 +217,7 @@ storageRemoteInfoProtocolPut(
         data->userIdLast = info->userId;
         data->groupIdLast = info->groupId;
 
-        if (!strEq(info->user, data->user) && info->user != NULL)                                                   // {vm_covered}
+        if (info->user != NULL && !strEq(info->user, data->user))                                                   // {vm_covered}
         {
             strFree(data->user);
 
@@ -225,7 +228,7 @@ storageRemoteInfoProtocolPut(
             MEM_CONTEXT_END();
         }
 
-        if (!strEq(info->group, data->group) && info->group != NULL)                                                // {vm_covered}
+        if (info->group != NULL && !strEq(info->group, data->group))                                                // {vm_covered}
         {
             strFree(data->group);
 
@@ -277,35 +280,8 @@ storageRemoteInfoProtocol(PackRead *const param, ProtocolServer *const server)
 }
 
 /**********************************************************************************************************************************/
-typedef struct StorageRemoteProtocolInfoListCallbackData
-{
-    ProtocolServer *const server;
-    StorageRemoteInfoProtocolWriteData writeData;
-} StorageRemoteProtocolInfoListCallbackData;
-
-static void
-storageRemoteProtocolInfoListCallback(void *dataVoid, const StorageInfo *info)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_LOG_PARAM_P(VOID, dataVoid);
-        FUNCTION_LOG_PARAM(STORAGE_INFO, info);
-    FUNCTION_TEST_END();
-
-    ASSERT(dataVoid != NULL);
-    ASSERT(info != NULL);
-
-    StorageRemoteProtocolInfoListCallbackData *data = dataVoid;
-
-    PackWrite *write = protocolPackNew();
-    pckWriteStrP(write, info->name);
-    storageRemoteInfoProtocolPut(&data->writeData, write, info);
-    protocolServerDataPut(data->server, write);
-
-    FUNCTION_TEST_RETURN_VOID();
-}
-
 void
-storageRemoteInfoListProtocol(PackRead *const param, ProtocolServer *const server)
+storageRemoteListProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(PACK_READ, param);
@@ -320,15 +296,27 @@ storageRemoteInfoListProtocol(PackRead *const param, ProtocolServer *const serve
     {
         const String *const path = pckReadStrP(param);
         const StorageInfoLevel level = (StorageInfoLevel)pckReadU32P(param);
+        StorageRemoteInfoProtocolWriteData writeData = {.memContext = memContextCurrent()};
+        StorageList *const list = storageInterfaceListP(storageRemoteProtocolLocal.driver, path, level);
 
-        StorageRemoteProtocolInfoListCallbackData data = {.server = server, .writeData = {.memContext = memContextCurrent()}};
+        // Put list
+        if (list != NULL)
+        {
+            for (unsigned int listIdx = 0; listIdx < storageLstSize(list); listIdx++)
+            {
+                const StorageInfo info = storageLstGet(list, listIdx);
 
-        const bool result = storageInterfaceInfoListP(
-            storageRemoteProtocolLocal.driver, path, level, storageRemoteProtocolInfoListCallback, &data);
+                PackWrite *const write = protocolPackNew();
+                pckWriteStrP(write, info.name);
+                storageRemoteInfoProtocolPut(&writeData, write, &info);
+                protocolServerDataPut(server, write);
+                pckWriteFree(write);
+            }
+        }
 
         // Indicate whether or not the path was found
         PackWrite *write = protocolPackNew();
-        pckWriteBoolP(write, result, .defaultWrite = true);
+        pckWriteBoolP(write, list != NULL, .defaultWrite = true);
         protocolServerDataPut(server, write);
 
         protocolServerDataEndPut(server);
@@ -355,12 +343,13 @@ storageRemoteOpenReadProtocol(PackRead *const param, ProtocolServer *const serve
     {
         const String *file = pckReadStrP(param);
         bool ignoreMissing = pckReadBoolP(param);
-        const Variant *limit = jsonToVar(pckReadStrP(param));
+        const uint64_t offset = pckReadU64P(param);
+        const Variant *const limit = pckReadNullP(param) ? NULL : VARUINT64(pckReadU64P(param));
         const Pack *const filter = pckReadPackP(param);
 
         // Create the read object
         IoRead *fileRead = storageReadIo(
-            storageInterfaceNewReadP(storageRemoteProtocolLocal.driver, file, ignoreMissing, .limit = limit));
+            storageInterfaceNewReadP(storageRemoteProtocolLocal.driver, file, ignoreMissing, .offset = offset, .limit = limit));
 
         // Set filter group based on passed filters
         storageRemoteFilterGroup(ioReadFilterGroup(fileRead), filter);
