@@ -11,7 +11,6 @@ Remote Storage Read
 #include "common/io/read.h"
 #include "common/log.h"
 #include "common/type/convert.h"
-#include "common/type/json.h"
 #include "common/type/object.h"
 #include "storage/remote/protocol.h"
 #include "storage/remote/read.h"
@@ -74,7 +73,13 @@ storageReadRemoteOpen(THIS_VOID)
 
         pckWriteStrP(param, this->interface.name);
         pckWriteBoolP(param, this->interface.ignoreMissing);
-        pckWriteStrP(param, jsonFromVar(this->interface.limit));
+        pckWriteU64P(param, this->interface.offset);
+
+        if (this->interface.limit == NULL)
+            pckWriteNullP(param);
+        else
+            pckWriteU64P(param, varUInt64(this->interface.limit));
+
         pckWritePackP(param, ioFilterGroupParamAll(ioReadFilterGroup(storageReadIo(this->read))));
 
         protocolClientCommandPut(this->client, command, false);
@@ -135,12 +140,12 @@ storageReadRemote(THIS_VOID, Buffer *buffer, bool block)
                     // If binary then read the next block
                     if (pckReadType(read) == pckTypeBin)
                     {
-                        MEM_CONTEXT_BEGIN(THIS_MEM_CONTEXT())
+                        MEM_CONTEXT_OBJ_BEGIN(this)
                         {
                             this->block = pckReadBinP(read);
                             this->remaining = bufUsed(this->block);
                         }
-                        MEM_CONTEXT_END();
+                        MEM_CONTEXT_OBJ_END();
                     }
                     // Else read is complete and get the filter list
                     else
@@ -201,14 +206,14 @@ storageReadRemoteEof(THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->eof);
+    FUNCTION_TEST_RETURN(BOOL, this->eof);
 }
 
 /**********************************************************************************************************************************/
 StorageRead *
 storageReadRemoteNew(
-    StorageRemote *storage, ProtocolClient *client, const String *name, bool ignoreMissing, bool compressible,
-    unsigned int compressLevel, const Variant *limit)
+    StorageRemote *const storage, ProtocolClient *const client, const String *const name, const bool ignoreMissing,
+    const bool compressible, const unsigned int compressLevel, const uint64_t offset, const Variant *const limit)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(STORAGE_REMOTE, storage);
@@ -217,6 +222,7 @@ storageReadRemoteNew(
         FUNCTION_LOG_PARAM(BOOL, ignoreMissing);
         FUNCTION_LOG_PARAM(BOOL, compressible);
         FUNCTION_LOG_PARAM(UINT, compressLevel);
+        FUNCTION_LOG_PARAM(UINT64, offset);
         FUNCTION_LOG_PARAM(VARIANT, limit);
     FUNCTION_LOG_END();
 
@@ -226,7 +232,7 @@ storageReadRemoteNew(
 
     StorageReadRemote *this = NULL;
 
-    OBJ_NEW_BEGIN(StorageReadRemote)
+    OBJ_NEW_BEGIN(StorageReadRemote, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
     {
         this = OBJ_NEW_ALLOC();
 
@@ -242,6 +248,7 @@ storageReadRemoteNew(
                 .compressible = compressible,
                 .compressLevel = compressLevel,
                 .ignoreMissing = ignoreMissing,
+                .offset = offset,
                 .limit = varDup(limit),
 
                 .ioInterface = (IoReadInterface)
