@@ -31,6 +31,9 @@ S3 HTTP headers
 STRING_STATIC(S3_HEADER_CONTENT_SHA256_STR,                         "x-amz-content-sha256");
 STRING_STATIC(S3_HEADER_DATE_STR,                                   "x-amz-date");
 STRING_STATIC(S3_HEADER_TOKEN_STR,                                  "x-amz-security-token");
+STRING_STATIC(S3_HEADER_SRVSDENC_STR,                               "x-amz-server-side-encryption");
+STRING_STATIC(S3_HEADER_SRVSDENC_KMS_STR,                           "aws:kms");
+STRING_STATIC(S3_HEADER_SRVSDENC_KMSKEYID_STR,                      "x-amz-server-side-encryption-aws-kms-key-id");
 
 /***********************************************************************************************************************************
 S3 query tokens
@@ -46,15 +49,15 @@ STRING_STATIC(S3_QUERY_VALUE_LIST_TYPE_2_STR,                       "2");
 /***********************************************************************************************************************************
 XML tags
 ***********************************************************************************************************************************/
-STRING_STATIC(S3_XML_TAG_CODE_STR,                                  "Code");
 STRING_STATIC(S3_XML_TAG_COMMON_PREFIXES_STR,                       "CommonPrefixes");
 STRING_STATIC(S3_XML_TAG_CONTENTS_STR,                              "Contents");
 STRING_STATIC(S3_XML_TAG_DELETE_STR,                                "Delete");
 STRING_STATIC(S3_XML_TAG_ERROR_STR,                                 "Error");
+STRING_STATIC(S3_XML_TAG_IS_TRUNCATED_STR,                          "IsTruncated");
 STRING_STATIC(S3_XML_TAG_KEY_STR,                                   "Key");
 STRING_STATIC(S3_XML_TAG_LAST_MODIFIED_STR,                         "LastModified");
-STRING_STATIC(S3_XML_TAG_MESSAGE_STR,                               "Message");
-STRING_STATIC(S3_XML_TAG_NEXT_CONTINUATION_TOKEN_STR,               "NextContinuationToken");
+#define S3_XML_TAG_NEXT_CONTINUATION_TOKEN                          "NextContinuationToken"
+    STRING_STATIC(S3_XML_TAG_NEXT_CONTINUATION_TOKEN_STR,           S3_XML_TAG_NEXT_CONTINUATION_TOKEN);
 STRING_STATIC(S3_XML_TAG_OBJECT_STR,                                "Object");
 STRING_STATIC(S3_XML_TAG_PREFIX_STR,                                "Prefix");
 STRING_STATIC(S3_XML_TAG_QUIET_STR,                                 "Quiet");
@@ -90,6 +93,7 @@ struct StorageS3
     String *accessKey;                                              // Access key
     String *secretAccessKey;                                        // Secret access key
     String *securityToken;                                          // Security token, if any
+    const String *kmsKeyId;                                         // Server-side encryption key
     size_t partSize;                                                // Part size for multi-part upload
     unsigned int deleteMax;                                         // Maximum objects that can be deleted in one request
     StorageS3UriStyle uriStyle;                                     // Path or host style URIs
@@ -129,7 +133,7 @@ storageS3DateTime(time_t authTime)
         strftime(buffer, sizeof(buffer), "%Y%m%dT%H%M%SZ", gmtime_r(&authTime, &timePart)) != ISO_8601_DATE_TIME_SIZE, AssertError,
         "unable to format date");
 
-    FUNCTION_TEST_RETURN(strNewZ(buffer));
+    FUNCTION_TEST_RETURN(STRING, strNewZ(buffer));
 }
 
 /***********************************************************************************************************************************
@@ -200,7 +204,7 @@ storageS3Auth(
         // Generate string to sign
         const String *stringToSign = strNewFmt(
             AWS4_HMAC_SHA256 "\n%s\n%s/%s/" S3 "/" AWS4_REQUEST "\n%s", strZ(dateTime), strZ(date), strZ(this->region),
-            strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA256_STR, BUFSTR(canonicalRequest)))));
+            strZ(bufHex(cryptoHashOne(hashTypeSha256, BUFSTR(canonicalRequest)))));
 
         // Generate signing key.  This key only needs to be regenerated every seven days but we'll do it once a day to keep the
         // logic simple.  It's a relatively expensive operation so we'd rather not do it for every request.
@@ -208,24 +212,24 @@ storageS3Auth(
         if (!strEq(date, this->signingKeyDate))
         {
             const Buffer *dateKey = cryptoHmacOne(
-                HASH_TYPE_SHA256_STR, BUFSTR(strNewFmt(AWS4 "%s", strZ(this->secretAccessKey))), BUFSTR(date));
-            const Buffer *regionKey = cryptoHmacOne(HASH_TYPE_SHA256_STR, dateKey, BUFSTR(this->region));
-            const Buffer *serviceKey = cryptoHmacOne(HASH_TYPE_SHA256_STR, regionKey, S3_BUF);
+                hashTypeSha256, BUFSTR(strNewFmt(AWS4 "%s", strZ(this->secretAccessKey))), BUFSTR(date));
+            const Buffer *regionKey = cryptoHmacOne(hashTypeSha256, dateKey, BUFSTR(this->region));
+            const Buffer *serviceKey = cryptoHmacOne(hashTypeSha256, regionKey, S3_BUF);
 
             // Switch to the object context so signing key and date are not lost
-            MEM_CONTEXT_BEGIN(THIS_MEM_CONTEXT())
+            MEM_CONTEXT_OBJ_BEGIN(this)
             {
-                this->signingKey = cryptoHmacOne(HASH_TYPE_SHA256_STR, serviceKey, AWS4_REQUEST_BUF);
+                this->signingKey = cryptoHmacOne(hashTypeSha256, serviceKey, AWS4_REQUEST_BUF);
                 this->signingKeyDate = strDup(date);
             }
-            MEM_CONTEXT_END();
+            MEM_CONTEXT_OBJ_END();
         }
 
         // Generate authorization header
         const String *authorization = strNewFmt(
             AWS4_HMAC_SHA256 " Credential=%s/%s/%s/" S3 "/" AWS4_REQUEST ",SignedHeaders=%s,Signature=%s",
             strZ(this->accessKey), strZ(date), strZ(this->region), strZ(signedHeaders),
-            strZ(bufHex(cryptoHmacOne(HASH_TYPE_SHA256_STR, this->signingKey, BUFSTR(stringToSign)))));
+            strZ(bufHex(cryptoHmacOne(hashTypeSha256, this->signingKey, BUFSTR(stringToSign)))));
 
         httpHeaderPut(httpHeader, HTTP_HEADER_AUTHORIZATION_STR, authorization);
     }
@@ -239,23 +243,24 @@ Convert YYYY-MM-DDTHH:MM:SS.MSECZ format to time_t. This format is very nearly I
 which are discarded here.
 ***********************************************************************************************************************************/
 static time_t
-storageS3CvtTime(const String *time)
+storageS3CvtTime(const String *const time)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(STRING, time);
     FUNCTION_TEST_END();
 
     FUNCTION_TEST_RETURN(
+        TIME,
         epochFromParts(
-            cvtZToInt(strZ(strSubN(time, 0, 4))), cvtZToInt(strZ(strSubN(time, 5, 2))),
-            cvtZToInt(strZ(strSubN(time, 8, 2))), cvtZToInt(strZ(strSubN(time, 11, 2))),
-            cvtZToInt(strZ(strSubN(time, 14, 2))), cvtZToInt(strZ(strSubN(time, 17, 2))), 0));
+            cvtZSubNToInt(strZ(time), 0, 4), cvtZSubNToInt(strZ(time), 5, 2), cvtZSubNToInt(strZ(time), 8, 2),
+            cvtZSubNToInt(strZ(time), 11, 2), cvtZSubNToInt(strZ(time), 14, 2), cvtZSubNToInt(strZ(time), 17, 2), 0));
 }
 
 /***********************************************************************************************************************************
 Automatically get credentials for an associated IAM role
 
 Documentation for the response format is found at: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/iam-roles-for-amazon-ec2.html
+Documentation for IMDSv2 tokens: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instancedata-data-retrieval.html
 ***********************************************************************************************************************************/
 STRING_STATIC(S3_CREDENTIAL_HOST_STR,                               "169.254.169.254");
 #define S3_CREDENTIAL_PORT                                          80
@@ -271,86 +276,107 @@ VARIANT_STRDEF_STATIC(S3_JSON_TAG_TOKEN_VAR,                        "Token");
 VARIANT_STRDEF_STATIC(S3_JSON_VALUE_SUCCESS_VAR,                    "Success");
 
 static void
-storageS3AuthAuto(StorageS3 *const this, const HttpHeader *const header)
+storageS3AuthAuto(StorageS3 *const this, HttpHeader *const header)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(HTTP_HEADER, header);
     FUNCTION_LOG_END();
 
-    // If the role was not set explicitly or retrieved previously then retrieve it
-    if (this->credRole == NULL)
+    MEM_CONTEXT_TEMP_BEGIN()
     {
-        // Request the role
+        // Retrieve the IMDSv2 token. Do not store the token because it will likely be expired by the time the credentials expire
+        // since credentials have a much longer default expiration. The TTL needs to be long enough to survive retries of the two
+        // metadata queries.
+        HttpHeader *const tokenHeader = httpHeaderDup(header, NULL);
+        httpHeaderAdd(
+            tokenHeader, STRDEF("x-aws-ec2-metadata-token-ttl-seconds"),
+            strNewFmt("%" PRIu64, httpClientTimeout(this->credHttpClient) / 1000 * 3));
+
         HttpRequest *request = httpRequestNewP(
-            this->credHttpClient, HTTP_VERB_GET_STR, STRDEF(S3_CREDENTIAL_PATH), .header = header);
+            this->credHttpClient, HTTP_VERB_PUT_STR, STRDEF("/latest/api/token"), .header = tokenHeader);
         HttpResponse *response = httpRequestResponse(request, true);
 
-        // Not found likely means no role is associated with this instance
+        // Set IMDSv2 token on success. If the token request fails for any reason assume that IMDSv2 is not supported. If the
+        // instance only supports IMDSv2 then subsequent metadata requests will fail because of the missing token.
+        if (httpResponseCodeOk(response))
+            httpHeaderAdd(header, STRDEF("x-aws-ec2-metadata-token"), strNewBuf(httpResponseContent(response)));
+
+        // If the role was not set explicitly or retrieved previously then retrieve it
+        if (this->credRole == NULL)
+        {
+            // Request the role
+            HttpRequest *request = httpRequestNewP(
+                this->credHttpClient, HTTP_VERB_GET_STR, STRDEF(S3_CREDENTIAL_PATH), .header = header);
+            HttpResponse *response = httpRequestResponse(request, true);
+
+            // Not found likely means no role is associated with this instance
+            if (httpResponseCode(response) == HTTP_RESPONSE_CODE_NOT_FOUND)
+            {
+                THROW(
+                    ProtocolError,
+                    "role to retrieve temporary credentials not found\n"
+                        "HINT: is a valid IAM role associated with this instance?");
+            }
+            // Else an error that we can't handle
+            else if (!httpResponseCodeOk(response))
+                httpRequestError(request, response);
+
+            // Get role from the text response
+            MEM_CONTEXT_OBJ_BEGIN(this)
+            {
+                this->credRole = strNewBuf(httpResponseContent(response));
+            }
+            MEM_CONTEXT_OBJ_END();
+        }
+
+        // Retrieve the temp credentials
+        request = httpRequestNewP(
+            this->credHttpClient, HTTP_VERB_GET_STR, strNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(this->credRole)), .header = header);
+        response = httpRequestResponse(request, true);
+
+        // Not found likely means the role is not valid
         if (httpResponseCode(response) == HTTP_RESPONSE_CODE_NOT_FOUND)
         {
-            THROW(
+            THROW_FMT(
                 ProtocolError,
-                "role to retrieve temporary credentials not found\n"
-                    "HINT: is a valid IAM role associated with this instance?");
+                "role '%s' not found\n"
+                    "HINT: is '%s' a valid IAM role associated with this instance?",
+                strZ(this->credRole), strZ(this->credRole));
         }
         // Else an error that we can't handle
         else if (!httpResponseCodeOk(response))
             httpRequestError(request, response);
 
-        // Get role from the text response
-        MEM_CONTEXT_BEGIN(THIS_MEM_CONTEXT())
+        // Get credentials from the JSON response
+        KeyValue *credential = varKv(jsonToVar(strNewBuf(httpResponseContent(response))));
+
+        MEM_CONTEXT_OBJ_BEGIN(this)
         {
-            this->credRole = strNewBuf(httpResponseContent(response));
+            // Check the code field for errors
+            const Variant *code = kvGetDefault(credential, S3_JSON_TAG_CODE_VAR, VARSTRDEF("code field is missing"));
+            CHECK(FormatError, code != NULL, "error code missing");
+
+            if (!varEq(code, S3_JSON_VALUE_SUCCESS_VAR))
+                THROW_FMT(FormatError, "unable to retrieve temporary credentials: %s", strZ(varStr(code)));
+
+            // Make sure the required values are present
+            CHECK(FormatError, kvGet(credential, S3_JSON_TAG_ACCESS_KEY_ID_VAR) != NULL, "access key missing");
+            CHECK(FormatError, kvGet(credential, S3_JSON_TAG_SECRET_ACCESS_KEY_VAR) != NULL, "secret access key missing");
+            CHECK(FormatError, kvGet(credential, S3_JSON_TAG_TOKEN_VAR) != NULL, "token missing");
+
+            // Copy credentials
+            this->accessKey = strDup(varStr(kvGet(credential, S3_JSON_TAG_ACCESS_KEY_ID_VAR)));
+            this->secretAccessKey = strDup(varStr(kvGet(credential, S3_JSON_TAG_SECRET_ACCESS_KEY_VAR)));
+            this->securityToken = strDup(varStr(kvGet(credential, S3_JSON_TAG_TOKEN_VAR)));
         }
-        MEM_CONTEXT_END();
+        MEM_CONTEXT_OBJ_END();
+
+        // Update expiration time
+        CHECK(FormatError, kvGet(credential, S3_JSON_TAG_EXPIRATION_VAR) != NULL, "expiration missing");
+        this->credExpirationTime = storageS3CvtTime(varStr(kvGet(credential, S3_JSON_TAG_EXPIRATION_VAR)));
     }
-
-    // Retrieve the temp credentials
-    HttpRequest *request = httpRequestNewP(
-        this->credHttpClient, HTTP_VERB_GET_STR, strNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(this->credRole)), .header = header);
-    HttpResponse *response = httpRequestResponse(request, true);
-
-    // Not found likely means the role is not valid
-    if (httpResponseCode(response) == HTTP_RESPONSE_CODE_NOT_FOUND)
-    {
-        THROW_FMT(
-            ProtocolError,
-            "role '%s' not found\n"
-                "HINT: is '%s' a valid IAM role associated with this instance?",
-            strZ(this->credRole), strZ(this->credRole));
-    }
-    // Else an error that we can't handle
-    else if (!httpResponseCodeOk(response))
-        httpRequestError(request, response);
-
-    // Get credentials from the JSON response
-    KeyValue *credential = jsonToKv(strNewBuf(httpResponseContent(response)));
-
-    MEM_CONTEXT_BEGIN(THIS_MEM_CONTEXT())
-    {
-        // Check the code field for errors
-        const Variant *code = kvGetDefault(credential, S3_JSON_TAG_CODE_VAR, VARSTRDEF("code field is missing"));
-        CHECK(FormatError, code != NULL, "error code missing");
-
-        if (!varEq(code, S3_JSON_VALUE_SUCCESS_VAR))
-            THROW_FMT(FormatError, "unable to retrieve temporary credentials: %s", strZ(varStr(code)));
-
-        // Make sure the required values are present
-        CHECK(FormatError, kvGet(credential, S3_JSON_TAG_ACCESS_KEY_ID_VAR) != NULL, "access key missing");
-        CHECK(FormatError, kvGet(credential, S3_JSON_TAG_SECRET_ACCESS_KEY_VAR) != NULL, "secret access key missing");
-        CHECK(FormatError, kvGet(credential, S3_JSON_TAG_TOKEN_VAR) != NULL, "token missing");
-
-        // Copy credentials
-        this->accessKey = strDup(varStr(kvGet(credential, S3_JSON_TAG_ACCESS_KEY_ID_VAR)));
-        this->secretAccessKey = strDup(varStr(kvGet(credential, S3_JSON_TAG_SECRET_ACCESS_KEY_VAR)));
-        this->securityToken = strDup(varStr(kvGet(credential, S3_JSON_TAG_TOKEN_VAR)));
-    }
-    MEM_CONTEXT_END();
-
-    // Update expiration time
-    CHECK(FormatError, kvGet(credential, S3_JSON_TAG_EXPIRATION_VAR) != NULL, "expiration missing");
-    this->credExpirationTime = storageS3CvtTime(varStr(kvGet(credential, S3_JSON_TAG_EXPIRATION_VAR)));
+    MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -371,41 +397,45 @@ storageS3AuthWebId(StorageS3 *const this, const HttpHeader *const header)
         FUNCTION_LOG_PARAM(HTTP_HEADER, header);
     FUNCTION_LOG_END();
 
-    // Get credentials
-    HttpQuery *const query = httpQueryNewP();
-    httpQueryAdd(query, STRDEF("Action"), STRDEF("AssumeRoleWithWebIdentity"));
-    httpQueryAdd(query, STRDEF("RoleArn"), this->credRole);
-    httpQueryAdd(query, STRDEF("RoleSessionName"), STRDEF(PROJECT_NAME));
-    httpQueryAdd(query, STRDEF("Version"), STRDEF("2011-06-15"));
-    httpQueryAdd(query, STRDEF("WebIdentityToken"), this->webIdToken);
-
-    HttpRequest *const request = httpRequestNewP(
-        this->credHttpClient, HTTP_VERB_GET_STR, FSLASH_STR, .header = header, .query = query);
-    HttpResponse *const response = httpRequestResponse(request, true);
-
-    CHECK(FormatError, httpResponseCode(response) != HTTP_RESPONSE_CODE_NOT_FOUND, "invalid response code");
-
-    // Copy credentials
-    const XmlNode *const xmlCred =
-        xmlNodeChild(
-            xmlNodeChild(
-                xmlDocumentRoot(xmlDocumentNewBuf(httpResponseContent(response))), STRDEF("AssumeRoleWithWebIdentityResult"), true),
-            STRDEF("Credentials"), true);
-
-    const XmlNode *const accessKeyNode = xmlNodeChild(xmlCred, STRDEF("AccessKeyId"), true);
-    const XmlNode *const secretAccessKeyNode = xmlNodeChild(xmlCred, STRDEF("SecretAccessKey"), true);
-    const XmlNode *const securityTokenNode = xmlNodeChild(xmlCred, STRDEF("SessionToken"), true);
-
-    MEM_CONTEXT_BEGIN(THIS_MEM_CONTEXT())
+    MEM_CONTEXT_TEMP_BEGIN()
     {
-        this->accessKey = xmlNodeContent(accessKeyNode);
-        this->secretAccessKey = xmlNodeContent(secretAccessKeyNode);
-        this->securityToken = xmlNodeContent(securityTokenNode);
-    }
-    MEM_CONTEXT_END();
+        // Get credentials
+        HttpQuery *const query = httpQueryNewP();
+        httpQueryAdd(query, STRDEF("Action"), STRDEF("AssumeRoleWithWebIdentity"));
+        httpQueryAdd(query, STRDEF("RoleArn"), this->credRole);
+        httpQueryAdd(query, STRDEF("RoleSessionName"), STRDEF(PROJECT_NAME));
+        httpQueryAdd(query, STRDEF("Version"), STRDEF("2011-06-15"));
+        httpQueryAdd(query, STRDEF("WebIdentityToken"), this->webIdToken);
 
-    // Update expiration time
-    this->credExpirationTime = storageS3CvtTime(xmlNodeContent(xmlNodeChild(xmlCred, STRDEF("Expiration"), true)));
+        HttpRequest *const request = httpRequestNewP(
+            this->credHttpClient, HTTP_VERB_GET_STR, FSLASH_STR, .header = header, .query = query);
+        HttpResponse *const response = httpRequestResponse(request, true);
+
+        CHECK(FormatError, httpResponseCode(response) != HTTP_RESPONSE_CODE_NOT_FOUND, "invalid response code");
+
+        // Copy credentials
+        const XmlNode *const xmlCred =
+            xmlNodeChild(
+                xmlNodeChild(
+                    xmlDocumentRoot(xmlDocumentNewBuf(httpResponseContent(response))), STRDEF("AssumeRoleWithWebIdentityResult"), true),
+                STRDEF("Credentials"), true);
+
+        const XmlNode *const accessKeyNode = xmlNodeChild(xmlCred, STRDEF("AccessKeyId"), true);
+        const XmlNode *const secretAccessKeyNode = xmlNodeChild(xmlCred, STRDEF("SecretAccessKey"), true);
+        const XmlNode *const securityTokenNode = xmlNodeChild(xmlCred, STRDEF("SessionToken"), true);
+
+        MEM_CONTEXT_OBJ_BEGIN(this)
+        {
+            this->accessKey = xmlNodeContent(accessKeyNode);
+            this->secretAccessKey = xmlNodeContent(secretAccessKeyNode);
+            this->securityToken = xmlNodeContent(securityTokenNode);
+        }
+        MEM_CONTEXT_OBJ_END();
+
+        // Update expiration time
+        this->credExpirationTime = storageS3CvtTime(xmlNodeContent(xmlNodeChild(xmlCred, STRDEF("Expiration"), true)));
+    }
+    MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -420,8 +450,10 @@ storageS3RequestAsync(StorageS3 *this, const String *verb, const String *path, S
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, verb);
         FUNCTION_LOG_PARAM(STRING, path);
+        FUNCTION_LOG_PARAM(HTTP_HEADER, param.header);
         FUNCTION_LOG_PARAM(HTTP_QUERY, param.query);
         FUNCTION_LOG_PARAM(BUFFER, param.content);
+        FUNCTION_LOG_PARAM(BOOL, param.sseKms);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
@@ -432,7 +464,8 @@ storageS3RequestAsync(StorageS3 *this, const String *verb, const String *path, S
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        HttpHeader *requestHeader = httpHeaderNew(this->headerRedactList);
+        HttpHeader *requestHeader = param.header == NULL ?
+            httpHeaderNew(this->headerRedactList) : httpHeaderDup(param.header, this->headerRedactList);
 
         // Set content length
         httpHeaderAdd(
@@ -443,8 +476,14 @@ storageS3RequestAsync(StorageS3 *this, const String *verb, const String *path, S
         if (param.content != NULL)
         {
             httpHeaderAdd(
-                requestHeader, HTTP_HEADER_CONTENT_MD5_STR,
-                strNewEncode(encodeBase64, cryptoHashOne(HASH_TYPE_MD5_STR, param.content)));
+                requestHeader, HTTP_HEADER_CONTENT_MD5_STR, strNewEncode(encodeBase64, cryptoHashOne(hashTypeMd5, param.content)));
+        }
+
+        // Set KMS headers when requested
+        if (param.sseKms && this->kmsKeyId != NULL)
+        {
+            httpHeaderPut(requestHeader, S3_HEADER_SRVSDENC_STR, S3_HEADER_SRVSDENC_KMS_STR);
+            httpHeaderPut(requestHeader, S3_HEADER_SRVSDENC_KMSKEYID_STR, this->kmsKeyId);
         }
 
         // When using path-style URIs the bucket name needs to be prepended
@@ -493,7 +532,7 @@ storageS3RequestAsync(StorageS3 *this, const String *verb, const String *path, S
         storageS3Auth(
             this, verb, path, param.query, storageS3DateTime(time(NULL)), requestHeader,
             param.content == NULL || bufEmpty(param.content) ?
-                HASH_TYPE_SHA256_ZERO_STR : bufHex(cryptoHashOne(HASH_TYPE_SHA256_STR, param.content)));
+                HASH_TYPE_SHA256_ZERO_STR : bufHex(cryptoHashOne(hashTypeSha256, param.content)));
 
         // Send request
         MEM_CONTEXT_PRIOR_BEGIN()
@@ -545,17 +584,22 @@ storageS3Request(StorageS3 *this, const String *verb, const String *path, Storag
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, verb);
         FUNCTION_LOG_PARAM(STRING, path);
+        FUNCTION_LOG_PARAM(HTTP_HEADER, param.header);
         FUNCTION_LOG_PARAM(HTTP_QUERY, param.query);
         FUNCTION_LOG_PARAM(BUFFER, param.content);
         FUNCTION_LOG_PARAM(BOOL, param.allowMissing);
         FUNCTION_LOG_PARAM(BOOL, param.contentIo);
+        FUNCTION_LOG_PARAM(BOOL, param.sseKms);
     FUNCTION_LOG_END();
 
-    FUNCTION_LOG_RETURN(
-        HTTP_RESPONSE,
-        storageS3ResponseP(
-            storageS3RequestAsyncP(this, verb, path, .query = param.query, .content = param.content),
-            .allowMissing = param.allowMissing, .contentIo = param.contentIo));
+    HttpRequest *const request = storageS3RequestAsyncP(
+        this, verb, path, .header = param.header, .query = param.query, .content = param.content, .sseKms = param.sseKms);
+    HttpResponse *const result = storageS3ResponseP(
+        request, .allowMissing = param.allowMissing, .contentIo = param.contentIo);
+
+    httpRequestFree(request);
+
+    FUNCTION_LOG_RETURN(HTTP_RESPONSE, result);
 }
 
 /***********************************************************************************************************************************
@@ -564,7 +608,7 @@ General function for listing files to be used by other list routines
 static void
 storageS3ListInternal(
     StorageS3 *this, const String *path, StorageInfoLevel level, const String *expression, bool recurse,
-    StorageInfoListCallback callback, void *callbackData)
+    StorageListCallback callback, void *callbackData)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
@@ -640,22 +684,23 @@ storageS3ListInternal(
                 }
                 // Else get the response immediately from a sync request
                 else
-                    response = storageS3RequestP(this, HTTP_VERB_GET_STR, FSLASH_STR, query);
+                    response = storageS3RequestP(this, HTTP_VERB_GET_STR, FSLASH_STR, .query = query);
 
                 XmlNode *xmlRoot = xmlDocumentRoot(xmlDocumentNewBuf(httpResponseContent(response)));
 
-                // If a continuation token exists then send an async request to get more data
-                const String *continuationToken = xmlNodeContent(
-                    xmlNodeChild(xmlRoot, S3_XML_TAG_NEXT_CONTINUATION_TOKEN_STR, false));
-
-                if (continuationToken != NULL)
+                // If list is truncated then send an async request to get more data
+                if (strEq(xmlNodeContent(xmlNodeChild(xmlRoot, S3_XML_TAG_IS_TRUNCATED_STR, true)), TRUE_STR))
                 {
-                    httpQueryPut(query, S3_QUERY_CONTINUATION_TOKEN_STR, continuationToken);
+                    const String *const nextContinuationToken = xmlNodeContent(
+                        xmlNodeChild(xmlRoot, S3_XML_TAG_NEXT_CONTINUATION_TOKEN_STR, true));
+                    CHECK(FormatError, !strEmpty(nextContinuationToken), S3_XML_TAG_NEXT_CONTINUATION_TOKEN " may not be empty");
+
+                    httpQueryPut(query, S3_QUERY_CONTINUATION_TOKEN_STR, nextContinuationToken);
 
                     // Store request in the outer temp context
                     MEM_CONTEXT_PRIOR_BEGIN()
                     {
-                        request = storageS3RequestAsyncP(this, HTTP_VERB_GET_STR, FSLASH_STR, query);
+                        request = storageS3RequestAsyncP(this, HTTP_VERB_GET_STR, FSLASH_STR, .query = query);
                     }
                     MEM_CONTEXT_PRIOR_END();
                 }
@@ -728,7 +773,7 @@ storageS3ListInternal(
 
 /**********************************************************************************************************************************/
 static StorageInfo
-storageS3Info(THIS_VOID, const String *file, StorageInfoLevel level, StorageInterfaceInfoParam param)
+storageS3Info(THIS_VOID, const String *const file, const StorageInfoLevel level, const StorageInterfaceInfoParam param)
 {
     THIS(StorageS3);
 
@@ -743,7 +788,7 @@ storageS3Info(THIS_VOID, const String *file, StorageInfoLevel level, StorageInte
     ASSERT(file != NULL);
 
     // Attempt to get file info
-    HttpResponse *httpResponse = storageS3RequestP(this, HTTP_VERB_HEAD_STR, file, .allowMissing = true);
+    HttpResponse *const httpResponse = storageS3RequestP(this, HTTP_VERB_HEAD_STR, file, .allowMissing = true);
 
     // Does the file exist?
     StorageInfo result = {.level = level, .exists = httpResponseCodeOk(httpResponse)};
@@ -751,7 +796,7 @@ storageS3Info(THIS_VOID, const String *file, StorageInfoLevel level, StorageInte
     // Add basic level info if requested and the file exists (no need to add type info since file is default type)
     if (result.level >= storageInfoLevelBasic && result.exists)
     {
-        const HttpHeader *httpHeader = httpResponseHeader(httpResponse);
+        const HttpHeader *const httpHeader = httpResponseHeader(httpResponse);
 
         const String *const contentLength = httpHeaderGet(httpHeader, HTTP_HEADER_CONTENT_LENGTH_STR);
         CHECK(FormatError, contentLength != NULL, "content length missing");
@@ -762,14 +807,30 @@ storageS3Info(THIS_VOID, const String *file, StorageInfoLevel level, StorageInte
         result.timeModified = httpDateToTime(lastModified);
     }
 
+    httpResponseFree(httpResponse);
+
     FUNCTION_LOG_RETURN(STORAGE_INFO, result);
 }
 
 /**********************************************************************************************************************************/
-static bool
-storageS3InfoList(
-    THIS_VOID, const String *path, StorageInfoLevel level, StorageInfoListCallback callback, void *callbackData,
-    StorageInterfaceInfoListParam param)
+static void
+storageS3ListCallback(void *const callbackData, const StorageInfo *const info)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM_P(VOID, callbackData);
+        FUNCTION_TEST_PARAM(STORAGE_INFO, info);
+    FUNCTION_TEST_END();
+
+    ASSERT(callbackData != NULL);
+    ASSERT(info != NULL);
+
+    storageLstAdd(callbackData, info);
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
+static StorageList *
+storageS3List(THIS_VOID, const String *const path, const StorageInfoLevel level, const StorageInterfaceListParam param)
 {
     THIS(StorageS3);
 
@@ -777,18 +838,17 @@ storageS3InfoList(
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, path);
         FUNCTION_LOG_PARAM(ENUM, level);
-        FUNCTION_LOG_PARAM(FUNCTIONP, callback);
-        FUNCTION_LOG_PARAM_P(VOID, callbackData);
         FUNCTION_LOG_PARAM(STRING, param.expression);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(path != NULL);
-    ASSERT(callback != NULL);
 
-    storageS3ListInternal(this, path, level, param.expression, false, callback, callbackData);
+    StorageList *const result = storageLstNew(level);
 
-    FUNCTION_LOG_RETURN(BOOL, true);
+    storageS3ListInternal(this, path, level, param.expression, false, storageS3ListCallback, result);
+
+    FUNCTION_LOG_RETURN(STORAGE_LIST, result);
 }
 
 /**********************************************************************************************************************************/
@@ -801,13 +861,14 @@ storageS3NewRead(THIS_VOID, const String *file, bool ignoreMissing, StorageInter
         FUNCTION_LOG_PARAM(STORAGE_S3, this);
         FUNCTION_LOG_PARAM(STRING, file);
         FUNCTION_LOG_PARAM(BOOL, ignoreMissing);
-        (void)param;                                                // No parameters are used
+        FUNCTION_LOG_PARAM(UINT64, param.offset);
+        FUNCTION_LOG_PARAM(VARIANT, param.limit);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(file != NULL);
 
-    FUNCTION_LOG_RETURN(STORAGE_READ, storageReadS3New(this, file, ignoreMissing));
+    FUNCTION_LOG_RETURN(STORAGE_READ, storageReadS3New(this, file, ignoreMissing, param.offset, param.limit));
 }
 
 /**********************************************************************************************************************************/
@@ -844,7 +905,7 @@ typedef struct StorageS3PathRemoveData
 } StorageS3PathRemoveData;
 
 static HttpRequest *
-storageS3PathRemoveInternal(StorageS3 *this, HttpRequest *request, XmlDocument *xml)
+storageS3PathRemoveInternal(StorageS3 *const this, HttpRequest *const request, XmlDocument *const xml)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(STORAGE_S3, this);
@@ -857,24 +918,27 @@ storageS3PathRemoveInternal(StorageS3 *this, HttpRequest *request, XmlDocument *
     // Get response for async request
     if (request != NULL)
     {
-        const Buffer *response = httpResponseContent(storageS3ResponseP(request));
-
-        // Nothing is returned when there are no errors
-        if (!bufEmpty(response))
+        MEM_CONTEXT_TEMP_BEGIN()
         {
-            XmlNodeList *errorList = xmlNodeChildList(xmlDocumentRoot(xmlDocumentNewBuf(response)), S3_XML_TAG_ERROR_STR);
+            const Buffer *const response = httpResponseContent(storageS3ResponseP(request));
 
-            if (xmlNodeLstSize(errorList) > 0)
+            // Nothing is returned when there are no errors
+            if (!bufEmpty(response))
             {
-                XmlNode *error = xmlNodeLstGet(errorList, 0);
+                XmlNodeList *errorList = xmlNodeChildList(xmlDocumentRoot(xmlDocumentNewBuf(response)), S3_XML_TAG_ERROR_STR);
 
-                THROW_FMT(
-                    FileRemoveError, STORAGE_ERROR_PATH_REMOVE_FILE ": [%s] %s",
-                    strZ(xmlNodeContent(xmlNodeChild(error, S3_XML_TAG_KEY_STR, true))),
-                    strZ(xmlNodeContent(xmlNodeChild(error, S3_XML_TAG_CODE_STR, true))),
-                    strZ(xmlNodeContent(xmlNodeChild(error, S3_XML_TAG_MESSAGE_STR, true))));
+                // Attempt to remove errored files one at a time rather than retrying the batch
+                for (unsigned int errorIdx = 0; errorIdx < xmlNodeLstSize(errorList); errorIdx++)
+                {
+                    storageS3RequestP(
+                        this, HTTP_VERB_DELETE_STR,
+                        strNewFmt(
+                            "/%s",
+                            strZ(xmlNodeContent(xmlNodeChild(xmlNodeLstGet(errorList, errorIdx), S3_XML_TAG_KEY_STR, true)))));
+                }
             }
         }
+        MEM_CONTEXT_TEMP_END();
 
         httpRequestFree(request);
     }
@@ -884,12 +948,16 @@ storageS3PathRemoveInternal(StorageS3 *this, HttpRequest *request, XmlDocument *
 
     if (xml != NULL)
     {
-        result = storageS3RequestAsyncP(
-            this, HTTP_VERB_POST_STR, FSLASH_STR, .query = httpQueryAdd(httpQueryNewP(), S3_QUERY_DELETE_STR, EMPTY_STR),
-            .content = xmlDocumentBuf(xml));
+        HttpQuery *const query = httpQueryAdd(httpQueryNewP(), S3_QUERY_DELETE_STR, EMPTY_STR);
+        Buffer *const content = xmlDocumentBuf(xml);
+
+        result = storageS3RequestAsyncP(this, HTTP_VERB_POST_STR, FSLASH_STR, .query = query, .content = content);
+
+        httpQueryFree(query);
+        bufFree(content);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(HTTP_REQUEST, result);
 }
 
 static void
@@ -920,9 +988,14 @@ storageS3PathRemoveCallback(void *callbackData, const StorageInfo *info)
         }
 
         // Add to delete list
-        xmlNodeContentSet(
-            xmlNodeAdd(xmlNodeAdd(xmlDocumentRoot(data->xml), S3_XML_TAG_OBJECT_STR), S3_XML_TAG_KEY_STR),
-            strNewFmt("%s%s", strZ(data->path), strZ(info->name)));
+        MEM_CONTEXT_TEMP_BEGIN()
+        {
+            xmlNodeContentSet(
+                xmlNodeAdd(xmlNodeAdd(xmlDocumentRoot(data->xml), S3_XML_TAG_OBJECT_STR), S3_XML_TAG_KEY_STR),
+                strNewFmt("%s%s", strZ(data->path), strZ(info->name)));
+        }
+        MEM_CONTEXT_TEMP_END();
+
         data->size++;
 
         // Delete list when it is full
@@ -983,7 +1056,7 @@ storageS3PathRemove(THIS_VOID, const String *path, bool recurse, StorageInterfac
 
 /**********************************************************************************************************************************/
 static void
-storageS3Remove(THIS_VOID, const String *file, StorageInterfaceRemoveParam param)
+storageS3Remove(THIS_VOID, const String *const file, const StorageInterfaceRemoveParam param)
 {
     THIS(StorageS3);
 
@@ -997,7 +1070,7 @@ storageS3Remove(THIS_VOID, const String *file, StorageInterfaceRemoveParam param
     ASSERT(file != NULL);
     ASSERT(!param.errorOnMissing);
 
-    storageS3RequestP(this, HTTP_VERB_DELETE_STR, file);
+    httpResponseFree(storageS3RequestP(this, HTTP_VERB_DELETE_STR, file));
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -1006,7 +1079,7 @@ storageS3Remove(THIS_VOID, const String *file, StorageInterfaceRemoveParam param
 static const StorageInterface storageInterfaceS3 =
 {
     .info = storageS3Info,
-    .infoList = storageS3InfoList,
+    .list = storageS3List,
     .newRead = storageS3NewRead,
     .newWrite = storageS3NewWrite,
     .pathRemove = storageS3PathRemove,
@@ -1017,9 +1090,9 @@ Storage *
 storageS3New(
     const String *path, bool write, StoragePathExpressionCallback pathExpressionFunction, const String *bucket,
     const String *endPoint, StorageS3UriStyle uriStyle, const String *region, StorageS3KeyType keyType, const String *accessKey,
-    const String *secretAccessKey, const String *securityToken, const String *credRole, const String *const webIdToken,
-    size_t partSize, const String *host, unsigned int port, TimeMSec timeout, bool verifyPeer, const String *caFile,
-    const String *caPath)
+    const String *secretAccessKey, const String *securityToken, const String *const kmsKeyId, const String *credRole,
+    const String *const webIdToken, size_t partSize, const String *host, unsigned int port, TimeMSec timeout, bool verifyPeer,
+    const String *caFile, const String *caPath)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING, path);
@@ -1033,6 +1106,7 @@ storageS3New(
         FUNCTION_TEST_PARAM(STRING, accessKey);
         FUNCTION_TEST_PARAM(STRING, secretAccessKey);
         FUNCTION_TEST_PARAM(STRING, securityToken);
+        FUNCTION_TEST_PARAM(STRING, kmsKeyId);
         FUNCTION_TEST_PARAM(STRING, credRole);
         FUNCTION_TEST_PARAM(STRING, webIdToken);
         FUNCTION_LOG_PARAM(SIZE, partSize);
@@ -1052,7 +1126,7 @@ storageS3New(
 
     Storage *this = NULL;
 
-    OBJ_NEW_BEGIN(StorageS3)
+    OBJ_NEW_BEGIN(StorageS3, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
     {
         StorageS3 *driver = OBJ_NEW_ALLOC();
 
@@ -1062,6 +1136,7 @@ storageS3New(
             .bucket = strDup(bucket),
             .region = strDup(region),
             .keyType = keyType,
+            .kmsKeyId = strDup(kmsKeyId),
             .partSize = partSize,
             .deleteMax = STORAGE_S3_DELETE_MAX,
             .uriStyle = uriStyle,
