@@ -15,6 +15,7 @@ in the description of each function.
 
 #include "common/type/param.h"
 #include "storage/info.h"
+#include "storage/list.h"
 #include "storage/read.h"
 #include "storage/write.h"
 
@@ -30,6 +31,7 @@ Error messages
 #define STORAGE_ERROR_READ_CLOSE                                    "unable to close file '%s' after read"
 #define STORAGE_ERROR_READ_OPEN                                     "unable to open file '%s' for read"
 #define STORAGE_ERROR_READ_MISSING                                  "unable to open missing file '%s' for read"
+#define STORAGE_ERROR_READ_SEEK                                     "unable to seek to %" PRIu64 " in file '%s'"
 
 #define STORAGE_ERROR_INFO                                          "unable to get info for path/file '%s'"
 #define STORAGE_ERROR_INFO_MISSING                                  "unable to get info for missing path/file '%s'"
@@ -56,6 +58,11 @@ Path expression callback function type - used to modify paths based on expressio
 ***********************************************************************************************************************************/
 typedef String *StoragePathExpressionCallback(const String *expression, const String *path);
 
+/***********************************************************************************************************************************
+Storage info callback function type - used to return storage info
+***********************************************************************************************************************************/
+typedef void (*StorageListCallback)(void *callbackData, const StorageInfo *info);
+
 /***********************************************************************************************************************************
 Required interface functions
 ***********************************************************************************************************************************/
@@ -89,6 +96,9 @@ typedef struct StorageInterfaceNewReadParam
     // Is the file compressible? This is used when the file must be moved across a network and temporary compression is helpful.
     bool compressible;
 
+    // Where to start reading in the file
+    const uint64_t offset;
+
     // Limit bytes read from the file. NULL for no limit.
     const Variant *limit;
 } StorageInterfaceNewReadParam;
@@ -139,30 +149,29 @@ typedef StorageWrite *StorageInterfaceNewWrite(void *thisVoid, const String *fil
     STORAGE_COMMON_INTERFACE(thisVoid).newWrite(thisVoid, file, (StorageInterfaceNewWriteParam){VAR_PARAM_INIT, __VA_ARGS__})
 
 // ---------------------------------------------------------------------------------------------------------------------------------
-// Get info for a path and all paths/files in the path (does not recurse)
+// Get info for all files/links/paths in a path (does not recurse or return info about the path passed to the function)
 //
 // See storageInterfaceInfoP() for usage of the level parameter.
-typedef struct StorageInterfaceInfoListParam
+typedef struct StorageInterfaceListParam
 {
     VAR_PARAM_HEADER;
 
     // Regular expression used to filter the results. The expression is always checked in the callback passed to
-    // storageInterfaceInfoListP() so checking the expression in the driver is entirely optional. The driver should only use the
+    // storageInterfaceIterP() so checking the expression in the driver is entirely optional. The driver should only use the
     // expression if it can improve performance or limit network transfer.
     //
     // Partial matching of the expression is fine as long as nothing that should match is excluded, e.g. it is OK to prefix match
     // using the prefix returned from regExpPrefix(). This may cause extra results to be sent to the callback but won't exclude
     // anything that matches the expression exactly.
     const String *expression;
-} StorageInterfaceInfoListParam;
+} StorageInterfaceListParam;
 
-typedef bool StorageInterfaceInfoList(
-    void *thisVoid, const String *path, StorageInfoLevel level, void (*callback)(void *callbackData, const StorageInfo *info),
-    void *callbackData, StorageInterfaceInfoListParam param);
+typedef StorageList *StorageInterfaceList(
+    void *thisVoid, const String *path, StorageInfoLevel level, StorageInterfaceListParam param);
 
-#define storageInterfaceInfoListP(thisVoid, path, level, callback, callbackData, ...)                                              \
-    STORAGE_COMMON_INTERFACE(thisVoid).infoList(                                                                                   \
-        thisVoid, path, level, callback, callbackData, (StorageInterfaceInfoListParam){VAR_PARAM_INIT, __VA_ARGS__})
+#define storageInterfaceListP(thisVoid, path, level, ...)                                                                          \
+    STORAGE_COMMON_INTERFACE(thisVoid).list(                                                                                       \
+        thisVoid, path, level, (StorageInterfaceListParam){VAR_PARAM_INIT, __VA_ARGS__})
 
 // ---------------------------------------------------------------------------------------------------------------------------------
 // Remove a path (and optionally recurse)
@@ -259,7 +268,7 @@ typedef struct StorageInterface
 
     // Required functions
     StorageInterfaceInfo *info;
-    StorageInterfaceInfoList *infoList;
+    StorageInterfaceList *list;
     StorageInterfaceNewRead *newRead;
     StorageInterfaceNewWrite *newWrite;
     StorageInterfacePathRemove *pathRemove;
