@@ -62,7 +62,6 @@ sub new
         $self->{strLogLevelTest},
         $self->{strLogLevelTestFile},
         $self->{bLogTimestamp},
-        $self->{bLogForce},
         $self->{bShowOutputAsync},
         $self->{bNoCleanup},
         $self->{iRetry},
@@ -96,7 +95,6 @@ sub new
             {name => 'strLogLevelTest'},
             {name => 'strLogLevelTestFile'},
             {name => 'bLogTimestamp'},
-            {name => 'bLogForce'},
             {name => 'bShowOutputAsync'},
             {name => 'bNoCleanup'},
             {name => 'iRetry'},
@@ -172,8 +170,8 @@ sub run
         &log($self->{bDryRun} && !$self->{bVmOut} || $self->{bShowOutputAsync} ? INFO : DETAIL, "${strTest}" .
              (!($self->{bDryRun} || !$self->{bVmOut}) || $self->{bShowOutputAsync} ? "\n" : ''));
 
-        my $strVmTestPath = '/home/' . TEST_USER . "/test/${strImage}";
         my $strHostTestPath = "$self->{strTestPath}/${strImage}";
+        my $strVmTestPath = $strHostTestPath;
 
         # Don't create the container if this is a dry run unless output from the VM is required. Output can be requested
         # to get more information about the specific tests that will be run.
@@ -302,7 +300,6 @@ sub run
                     "\n" .
                     "CFLAGS_TEST =" .
                         " \\\n\t" . (($self->{bOptimize} && ($self->{bProfile} || $bPerformance)) ? '-O2' : '-O0') .
-                        " \\\n\t-DDEBUG_MEM" .
                         (!$self->{bDebugTestTrace} && $self->{bDebug} ? " \\\n\t-DDEBUG_TEST_TRACE" : '') .
                         ($bCoverage ? " \\\n\t-fprofile-arcs -ftest-coverage" : '') .
                         ($self->{oTest}->{&TEST_VM} eq VM_NONE ? '' : " \\\n\t-DTEST_CONTAINER_REQUIRED") .
@@ -612,7 +609,9 @@ sub run
                 $strTestDepend .= " ${strTestFile}";
 
                 # Determine where the project exe is located
-                my $strProjectExePath = "$self->{strTestPath}/bin/$self->{oTest}->{&TEST_VM}/" . PROJECT_EXE;
+                my $strProjectExePath =
+                    "$self->{strTestPath}/bin/$self->{oTest}->{&TEST_VM}/" . ($self->{oTest}->{&TEST_VM} eq VM_NONE ? 'src/' : '') .
+                    PROJECT_EXE;
 
                 # Is this test running in a container?
                 my $strContainer = $self->{oTest}->{&TEST_VM} eq VM_NONE ? 'false' : 'true';
@@ -722,12 +721,14 @@ sub run
                         '') .
                     $self->{strMakeCmd} . " -j $self->{iBuildMax} -s 2>&1 && \\\n" .
                     "rm ${strBuildProcessingFile} && \\\n" .
+                    # Allow stderr to be copied to stderr and stdout
+                    "exec 3>&1 && \\\n" .
                     # Test with valgrind when requested
                     ($self->{bValgrindUnit} && $self->{oTest}->{&TEST_TYPE} ne TESTDEF_PERFORMANCE ?
                         'valgrind -q --gen-suppressions=all' .
                         ($self->{oStorageTest}->exists($strValgrindSuppress) ? " --suppressions=${strValgrindSuppress}" : '') .
                         " --exit-on-first-error=yes --leak-check=full --leak-resolution=high --error-exitcode=25" . ' ' : '') .
-                        "./test.bin 2>&1" .
+                        "./test.bin 2>&1 1>&3 | tee /dev/stderr" .
                     ($self->{oTest}->{&TEST_VM} ne VM_NONE ? "'" : '');
             }
             else
@@ -747,7 +748,6 @@ sub run
                     ($self->{bLogTimestamp} ? '' : ' --no-log-timestamp') .
                     ' --pgsql-bin=' . $self->{oTest}->{&TEST_PGSQL_BIN} .
                     ($self->{strTimeZone} ? " --tz='$self->{strTimeZone}'" : '') .
-                    ($self->{bLogForce} ? ' --log-force' : '') .
                     ($self->{bDryRun} ? ' --dry-run' : '') .
                     ($self->{bDryRun} ? ' --vm-out' : '') .
                     ($self->{bNoCleanup} ? " --no-cleanup" : '');
@@ -810,7 +810,7 @@ sub end
         if ($iExitStatus == 0 && $self->{oTest}->{&TEST_C} && $self->{bProfile})
         {
             executeTest(
-                ($self->{oTest}->{&TEST_VM} ne VM_NONE  ? 'docker exec -i -u ' . TEST_USER . " ${strImage} " : '') .
+                ($self->{oTest}->{&TEST_VM} ne VM_NONE ? 'docker exec -i -u ' . TEST_USER . " ${strImage} " : '') .
                     "gprof $self->{strGCovPath}/test.bin $self->{strGCovPath}/gmon.out > $self->{strGCovPath}/gprof.txt");
 
             $self->{oStorageTest}->pathCreate(
@@ -833,20 +833,20 @@ sub end
         my $fTestElapsedTime = ceil((gettimeofday() - $self->{oProcess}{start_time}) * 100) / 100;
 
         # Output error
-        if ($iExitStatus != 0)
+        if ($iExitStatus != 0 || (defined($oExecDone->{strErrorLog}) && $oExecDone->{strErrorLog} ne ''))
         {
             # Get stdout
             my $strOutput = trim($oExecDone->{strOutLog}) ? "STDOUT:\n" . trim($oExecDone->{strOutLog}) : '';
 
             # Get stderr
-            if (trim($oExecDone->{strSuppressedErrorLog}) ne '')
+            if (defined($oExecDone->{strErrorLog}) && trim($oExecDone->{strErrorLog}) ne '')
             {
                 if ($strOutput ne '')
                 {
                     $strOutput .= "\n";
                 }
 
-                $strOutput .= "STDERR:\n" . trim($oExecDone->{strSuppressedErrorLog});
+                $strOutput .= "STDERR:\n" . trim($oExecDone->{strErrorLog});
             }
 
             # If no stdout or stderr output something rather than a blank line
