@@ -180,10 +180,7 @@ sub new
     $self->{strLockPath} = $self->testPath() . '/' . HOST_PATH_LOCK;
 
     # Set conf file
-    $self->{strBackRestConfig} =  $self->testPath() . '/' . PROJECT_CONF;
-
-    # Set LogTest object
-    $self->{oLogTest} = $$oParam{oLogTest};
+    $self->{strBackRestConfig} = $self->testPath() . '/' . PROJECT_CONF;
 
     # Set synthetic
     $self->{bSynthetic} = defined($$oParam{bSynthetic}) && $$oParam{bSynthetic} ? true : false;
@@ -341,8 +338,7 @@ sub backupBegin
         (defined($oParam->{iRepo}) ? ' --repo=' . $oParam->{iRepo} : '') .
         ($strType ne 'incr' ? " --type=${strType}" : '') .
         ' --stanza=' . (defined($oParam->{strStanza}) ? $oParam->{strStanza} : $self->stanza()) . ' backup',
-        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus},
-         oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, bLogOutput => $self->synthetic()});
 
     $oExecuteBackup->begin();
 
@@ -413,7 +409,8 @@ sub backupEnd
         # Make sure tablespace links are correct
         if ($self->hasLink())
         {
-            if ($strType eq CFGOPTVAL_BACKUP_TYPE_FULL || $self->hardLink())
+            if (($strType eq CFGOPTVAL_BACKUP_TYPE_FULL || $self->hardLink()) &&
+                !$oExpectedManifest->{&MANIFEST_SECTION_BACKUP}{'backup-bundle'})
             {
                 my $hTablespaceManifest = storageTest()->manifest(
                     $self->repoBackupPath("${strBackup}/" . MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGTBLSPC));
@@ -508,39 +505,6 @@ sub backupEnd
         }
     }
 
-    # Add files to expect log
-    if (defined($self->{oLogTest}) && (!defined($$oParam{bSupplemental}) || $$oParam{bSupplemental}))
-    {
-        my $oHostGroup = hostGroupGet();
-
-        if (defined($oHostGroup->hostGet(HOST_DB_PRIMARY, true)))
-        {
-            $self->{oLogTest}->supplementalAdd($oHostGroup->hostGet(HOST_DB_PRIMARY)->testPath() . '/' . PROJECT_CONF);
-        }
-
-        if (defined($oHostGroup->hostGet(HOST_DB_STANDBY, true)))
-        {
-            $self->{oLogTest}->supplementalAdd($oHostGroup->hostGet(HOST_DB_STANDBY)->testPath() . '/' . PROJECT_CONF);
-        }
-
-        if (defined($oHostGroup->hostGet(HOST_BACKUP, true)))
-        {
-            $self->{oLogTest}->supplementalAdd($oHostGroup->hostGet(HOST_BACKUP)->testPath() . '/' . PROJECT_CONF);
-        }
-
-        if ($self->synthetic() && $bManifestCompare)
-        {
-            $self->{oLogTest}->supplementalAdd(
-                $self->repoBackupPath("${strBackup}/" . FILE_MANIFEST), undef,
-                ${storageRepo()->get(
-                    storageRepo()->openRead(
-                        $self->repoBackupPath("${strBackup}/" . FILE_MANIFEST),
-                        {strCipherPass => $self->cipherPassManifest()}))});
-            $self->{oLogTest}->supplementalAdd(
-                $self->repoBackupPath(FILE_BACKUP_INFO), undef, ${storageRepo->get($self->repoBackupPath(FILE_BACKUP_INFO))});
-        }
-    }
-
     # Return from function and log return values if any
     return logDebugReturn
     (
@@ -702,69 +666,54 @@ sub manifestDefault
     my $self = shift;
     my $oExpectedManifest = shift;
 
-    # Set defaults for subkeys that tend to repeat
+    # Defaults for subkeys that tend to repeat
+    my $strDefaultUser = $oExpectedManifest->{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_USER};
+    my $strDefaultGroup = $oExpectedManifest->{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_GROUP};
+    my $strDefaultPathMode = $oExpectedManifest->{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_MODE};
+    my $strDefaultFileMode = sprintf('%04o', oct($strDefaultPathMode) & (S_IRUSR | S_IWUSR | S_IRGRP));
+
+    # Remove subkeys that match the defaults
     foreach my $strSection (&MANIFEST_SECTION_TARGET_FILE, &MANIFEST_SECTION_TARGET_PATH, &MANIFEST_SECTION_TARGET_LINK)
     {
-        foreach my $strSubKey (&MANIFEST_SUBKEY_USER, &MANIFEST_SUBKEY_GROUP, &MANIFEST_SUBKEY_MODE, &MANIFEST_SUBKEY_PRIMARY)
+        next if !defined($oExpectedManifest->{$strSection});
+
+        foreach my $strFile (keys(%{$oExpectedManifest->{$strSection}}))
         {
-            my %oDefault;
-            my $iSectionTotal = 0;
-
-            next if !defined($oExpectedManifest->{$strSection});
-
-            foreach my $strFile (keys(%{$oExpectedManifest->{$strSection}}))
+            if (defined($oExpectedManifest->{$strSection}{$strFile}{&MANIFEST_SUBKEY_USER}) &&
+                $oExpectedManifest->{$strSection}{$strFile}{&MANIFEST_SUBKEY_USER} eq $strDefaultUser)
             {
-                my $strValue = $oExpectedManifest->{$strSection}{$strFile}{$strSubKey};
-
-                if (defined($strValue))
-                {
-                    if (defined($oDefault{$strValue}))
-                    {
-                        $oDefault{$strValue}++;
-                    }
-                    else
-                    {
-                        $oDefault{$strValue} = 1;
-                    }
-                }
-
-                $iSectionTotal++;
+                delete($oExpectedManifest->{$strSection}{$strFile}{&MANIFEST_SUBKEY_USER});
             }
 
-            my $strMaxValue;
-            my $iMaxValueTotal = 0;
-
-            foreach my $strValue (sort(keys(%oDefault)))
+            if (defined($oExpectedManifest->{$strSection}{$strFile}{&MANIFEST_SUBKEY_GROUP}) &&
+                $oExpectedManifest->{$strSection}{$strFile}{&MANIFEST_SUBKEY_GROUP} eq $strDefaultGroup)
             {
-                if ($oDefault{$strValue} > $iMaxValueTotal)
-                {
-                    $iMaxValueTotal = $oDefault{$strValue};
-                    $strMaxValue = $strValue;
-                }
+                delete($oExpectedManifest->{$strSection}{$strFile}{&MANIFEST_SUBKEY_GROUP});
             }
 
-            if (defined($strMaxValue) > 0 && $iMaxValueTotal > $iSectionTotal * MANIFEST_DEFAULT_MATCH_FACTOR)
+            if (defined($oExpectedManifest->{$strSection}{$strFile}{&MANIFEST_SUBKEY_MODE}) &&
+                $oExpectedManifest->{$strSection}{$strFile}{&MANIFEST_SUBKEY_MODE} eq
+                    ($strSection eq MANIFEST_SECTION_TARGET_PATH ? $strDefaultPathMode : $strDefaultFileMode))
             {
-                if ($strSubKey eq MANIFEST_SUBKEY_PRIMARY)
-                {
-                    $oExpectedManifest->{"${strSection}:default"}{$strSubKey} = $strMaxValue ? JSON::PP::true : JSON::PP::false;
-                }
-                else
-                {
-                    $oExpectedManifest->{"${strSection}:default"}{$strSubKey} = $strMaxValue;
-                }
-
-                foreach my $strFile (keys(%{$oExpectedManifest->{$strSection}}))
-                {
-                    if (defined($oExpectedManifest->{$strSection}{$strFile}{$strSubKey}) &&
-                        $oExpectedManifest->{$strSection}{$strFile}{$strSubKey} eq $strMaxValue)
-                    {
-                        delete($oExpectedManifest->{$strSection}{$strFile}{$strSubKey});
-                    }
-                }
+                delete($oExpectedManifest->{$strSection}{$strFile}{&MANIFEST_SUBKEY_MODE});
             }
         }
     }
+
+    # Write defaults
+    $oExpectedManifest->{&MANIFEST_SECTION_TARGET_FILE . ':default'}{&MANIFEST_SUBKEY_USER} = $strDefaultUser;
+    $oExpectedManifest->{&MANIFEST_SECTION_TARGET_FILE . ':default'}{&MANIFEST_SUBKEY_GROUP} = $strDefaultGroup;
+    $oExpectedManifest->{&MANIFEST_SECTION_TARGET_FILE . ':default'}{&MANIFEST_SUBKEY_MODE} = $strDefaultFileMode;
+
+    if (defined($oExpectedManifest->{&MANIFEST_SECTION_TARGET_LINK}))
+    {
+        $oExpectedManifest->{&MANIFEST_SECTION_TARGET_LINK . ':default'}{&MANIFEST_SUBKEY_USER} = $strDefaultUser;
+        $oExpectedManifest->{&MANIFEST_SECTION_TARGET_LINK . ':default'}{&MANIFEST_SUBKEY_GROUP} = $strDefaultGroup;
+    }
+
+    $oExpectedManifest->{&MANIFEST_SECTION_TARGET_PATH . ':default'}{&MANIFEST_SUBKEY_USER} = $strDefaultUser;
+    $oExpectedManifest->{&MANIFEST_SECTION_TARGET_PATH . ':default'}{&MANIFEST_SUBKEY_GROUP} = $strDefaultGroup;
+    $oExpectedManifest->{&MANIFEST_SECTION_TARGET_PATH . ':default'}{&MANIFEST_SUBKEY_MODE} = $strDefaultPathMode;
 }
 
 ####################################################################################################################################
@@ -819,8 +768,7 @@ sub check
         (defined($$oParam{iTimeout}) ? " --archive-timeout=$$oParam{iTimeout}" : '') .
         (defined($$oParam{strOptionalParam}) ? " $$oParam{strOptionalParam}" : '') .
         ' --stanza=' . $self->stanza() . ' check',
-        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest},
-         bLogOutput => $self->synthetic()});
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, bLogOutput => $self->synthetic()});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
@@ -864,8 +812,7 @@ sub expire
         (defined($$oParam{strOptionalParam}) ? " $$oParam{strOptionalParam}" : '') .
         ' --repo=' . (defined($oParam->{iRepo}) ? $oParam->{iRepo} : '1') .
         '  --stanza=' . $self->stanza() . ' expire',
-        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest},
-         bLogOutput => $self->synthetic()});
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, bLogOutput => $self->synthetic()});
 }
 
 ####################################################################################################################################
@@ -900,7 +847,7 @@ sub info
         ' --log-level-console=warn' .
         (defined($$oParam{strStanza}) ? " --stanza=$$oParam{strStanza}" : '') .
         (defined($$oParam{strOutput}) ? " --output=$$oParam{strOutput}" : '') . ' info',
-        {strComment => $strComment, oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
+        {strComment => $strComment, bLogOutput => $self->synthetic()});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
@@ -938,30 +885,16 @@ sub stanzaCreate
         ' --stanza=' . $self->stanza() .
         (defined($$oParam{strOptionalParam}) ? " $$oParam{strOptionalParam}" : '') .
         ' stanza-create',
-        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest},
-         bLogOutput => $self->synthetic()});
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, bLogOutput => $self->synthetic()});
 
     if (storageRepo()->exists('backup/' . $self->stanza() . qw{/} . FILE_BACKUP_INFO))
     {
-        # If the info file was created, then add it to the expect log
-        if (defined($self->{oLogTest}) && $self->synthetic())
-        {
-            $self->{oLogTest}->supplementalAdd(
-                $self->repoBackupPath(FILE_BACKUP_INFO), undef, ${storageRepo()->get($self->repoBackupPath(FILE_BACKUP_INFO))});
-        }
-
         # Get the passphrase for accessing the manifest file
         $self->{strCipherPassManifest} = (new pgBackRestTest::Env::BackupInfo($self->repoBackupPath()))->cipherPassSub();
     }
 
     if (storageRepo()->exists('archive/' . $self->stanza() . qw{/} . ARCHIVE_INFO_FILE))
     {
-        # If the info file was created, then add it to the expect log
-        if (defined($self->{oLogTest}) && $self->synthetic())
-        {
-            $self->{oLogTest}->supplementalAdd(
-                $self->repoArchivePath(ARCHIVE_INFO_FILE), undef, ${storageRepo()->get($self->repoArchivePath(ARCHIVE_INFO_FILE))});
-        }
 
         # Get the passphrase for accessing the archived files
         $self->{strCipherPassArchive} =
@@ -1004,23 +937,7 @@ sub stanzaUpgrade
         ' --stanza=' . $self->stanza() .
         (defined($$oParam{strOptionalParam}) ? " $$oParam{strOptionalParam}" : '') .
         ' stanza-upgrade',
-        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest},
-         bLogOutput => $self->synthetic()});
-
-    # If the info file was created, then add it to the expect log
-    if (defined($self->{oLogTest}) && $self->synthetic() &&
-        storageRepo()->exists('backup/' . $self->stanza() . qw{/} . FILE_BACKUP_INFO))
-    {
-        $self->{oLogTest}->supplementalAdd(
-            $self->repoBackupPath(FILE_BACKUP_INFO), undef, ${storageRepo()->get($self->repoBackupPath(FILE_BACKUP_INFO))});
-    }
-
-    if (defined($self->{oLogTest}) && $self->synthetic() &&
-        storageRepo()->exists('archive/' . $self->stanza() . qw{/} . ARCHIVE_INFO_FILE))
-    {
-        $self->{oLogTest}->supplementalAdd(
-            $self->repoArchivePath(ARCHIVE_INFO_FILE), undef, ${storageRepo()->get($self->repoArchivePath(ARCHIVE_INFO_FILE))});
-    }
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, bLogOutput => $self->synthetic()});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
@@ -1059,18 +976,7 @@ sub stanzaDelete
         ' --stanza=' . $self->stanza() .
         (defined($$oParam{strOptionalParam}) ? " $$oParam{strOptionalParam}" : '') .
         ' stanza-delete',
-        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest},
-         bLogOutput => $self->synthetic()});
-
-    if (defined($self->{oLogTest}) && $self->synthetic())
-    {
-        $self->{oLogTest}->logAdd(
-            'list backup', $self->stanza() . ' must not exist for successful delete',
-            join("\n", storageRepo()->list('backup')));
-        $self->{oLogTest}->logAdd(
-            'list archive', $self->stanza() . ' must not exist for successful delete',
-            join("\n", storageRepo()->list('archive')));
-    }
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, bLogOutput => $self->synthetic()});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
@@ -1104,7 +1010,7 @@ sub start
         $self->backrestExe() .
         ' --config=' . $self->backrestConfig() .
         (defined($$oParam{strStanza}) ? " --stanza=$$oParam{strStanza}" : '') . ' start',
-        {strComment => $strComment, oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
+        {strComment => $strComment, bLogOutput => $self->synthetic()});
 }
 
 ####################################################################################################################################
@@ -1136,7 +1042,7 @@ sub stop
         ' --config=' . $self->backrestConfig() .
         (defined($$oParam{strStanza}) ? " --stanza=$$oParam{strStanza}" : '') .
         (defined($$oParam{bForce}) && $$oParam{bForce} ? ' --force' : '') . ' stop',
-        {strComment => $strComment, oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
+        {strComment => $strComment, bLogOutput => $self->synthetic()});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
@@ -1189,6 +1095,14 @@ sub configCreate
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{'log-timestamp'} = 'n';
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{'buffer-size'} = '64k';
 
+    if ($oParam->{bBundle})
+    {
+        $oParamHash{&CFGDEF_SECTION_GLOBAL}{'repo1-bundle'} = 'y';
+        # Set bundle size/limit smaller for testing and because FakeGCS does not do multi-part upload
+        $oParamHash{&CFGDEF_SECTION_GLOBAL}{'repo1-bundle-size'} = '1MiB';
+        $oParamHash{&CFGDEF_SECTION_GLOBAL}{'repo1-bundle-limit'} = '64KiB';
+    }
+
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{'log-path'} = $self->logPath();
     $oParamHash{&CFGDEF_SECTION_GLOBAL}{'lock-path'} = $self->lockPath();
 
@@ -1910,8 +1824,7 @@ sub restore
         (defined($strTargetAction) && $strTargetAction ne 'pause' ? " --target-action=${strTargetAction}" : '') .
         (defined($iRepo) ? " --repo=${iRepo}" : '') .
         " --stanza=" . $self->stanza() . ' restore',
-        {strComment => $strComment, iExpectedExitStatus => $iExpectedExitStatus, oLogTest => $self->{oLogTest},
-         bLogOutput => $self->synthetic()},
+        {strComment => $strComment, iExpectedExitStatus => $iExpectedExitStatus, bLogOutput => $self->synthetic()},
         $strUser);
 
     if (!defined($iExpectedExitStatus))
@@ -1922,13 +1835,6 @@ sub restore
         {
             $self->restoreCompare($strBackupExpected, dclone($rhExpectedManifest), $bTablespace);
         }
-
-        if (defined($self->{oLogTest}))
-        {
-            $self->{oLogTest}->supplementalAdd(
-                $rhExpectedManifest->{&MANIFEST_SECTION_BACKUP_TARGET}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_PATH} .
-                "/recovery.conf");
-        }
     }
 }
 
@@ -2085,9 +1991,13 @@ sub restoreCompare
                 ${$oExpectedManifestRef}{&MANIFEST_SECTION_TARGET_FILE}{$strName}{size});
         }
 
-        # Remove repo-size from the manifest.  ??? This could be improved to get actual sizes from the backup.
+        # Remove repo-size, bno, bni from the manifest
         $oActualManifest->remove(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_REPO_SIZE);
         delete($oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{$strName}{&MANIFEST_SUBKEY_REPO_SIZE});
+        $oActualManifest->remove(MANIFEST_SECTION_TARGET_FILE, $strName, "bni");
+        delete($oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{$strName}{"bni"});
+        $oActualManifest->remove(MANIFEST_SECTION_TARGET_FILE, $strName, "bno");
+        delete($oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{$strName}{"bno"});
 
         if ($oActualManifest->get(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_SIZE) != 0)
         {
@@ -2149,6 +2059,8 @@ sub restoreCompare
                           ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_ARCHIVE_COPY});
     $oActualManifest->set(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_BACKUP_STANDBY, undef,
                           ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_BACKUP_STANDBY});
+    $oActualManifest->set(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_BACKUP_STANDBY, undef,
+                          ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_BACKUP_STANDBY});
     $oActualManifest->set(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_BUFFER_SIZE, undef,
                           ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_BUFFER_SIZE});
     $oActualManifest->set(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_COMPRESS, undef,
@@ -2216,6 +2128,13 @@ sub restoreCompare
             MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TYPE, undef,
             $oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TYPE});
 
+        if (defined($oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP}{'backup-bundle'}))
+        {
+            $oActualManifest->set(
+                MANIFEST_SECTION_BACKUP, 'backup-bundle', undef,
+                $oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP}{'backup-bundle'});
+        }
+
         $oActualManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_LSN_START, undef,
                               ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_LSN_START});
         $oActualManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_LSN_STOP, undef,
@@ -2246,7 +2165,11 @@ sub restoreCompare
     # Delete the list of DBs
     delete($$oExpectedManifestRef{&MANIFEST_SECTION_DB});
 
-    $self->manifestDefault($oExpectedManifestRef);
+    # Only update defaults if the expect manifest is synthetic. If loaded from a file the defaults will already be correct.
+    if ($self->synthetic())
+    {
+        $self->manifestDefault($oExpectedManifestRef);
+    }
 
     # Newer Perls will change this variable to a number whenever a numeric comparison is performed. It is expected to be a string so
     # make sure it is one before saving.
