@@ -64,7 +64,6 @@ sub new
             strImage => containerRepo() . ':' . testRunGet()->vm() . "-test",
             bTls => $oParam->{bTls},
             strBackupDestination => $$oParam{strBackupDestination},
-            oLogTest => $$oParam{oLogTest},
             bStandby => $$oParam{bStandby},
             bRepoLocal => $oParam->{bRepoLocal},
             bRepoEncrypt => $oParam->{bRepoEncrypt},
@@ -100,9 +99,13 @@ sub new
 
             if (!defined($$oParam{bStandby}) || !$$oParam{bStandby})
             {
-                &log(WARN, "Testing against ${strDbVersionActual} ${strDevVersion} version");
+                &log(WARN, 'Testing against ' . trim($strOutLog) . " ${strDevVersion}");
             }
         }
+        elsif (!defined($$oParam{bStandby}) || !$$oParam{bStandby})
+        {
+            &log(INFO, 'Testing against ' . trim($strOutLog));
+        }
 
         # Don't run unit tests for unsupported versions
         my @stryVersionSupport = versionSupport();
@@ -389,9 +392,9 @@ sub clusterStart
     my $bArchiveEnabled = defined($$hParam{bArchiveEnabled}) ? $$hParam{bArchiveEnabled} : true;
 
     # Make sure postgres is not running
-    if (-e $self->dbBasePath() . '/postmas'.'ter.pid')
+    if (-e $self->dbBasePath() . '/' . DB_FILE_POSTMTRPID)
     {
-        confess 'postmas'.'ter.pid exists';
+        confess DB_FILE_POSTMTRPID . ' exists';
     }
 
     # Create the archive command
@@ -429,10 +432,7 @@ sub clusterStart
         $strCommand .= " -c archive_command=true";
     }
 
-    if ($self->pgVersion() >= PG_VERSION_90)
-    {
-        $strCommand .= ' -c wal_level=hot_standby -c hot_standby=' . ($bHotStandby ? 'on' : 'off');
-    }
+    $strCommand .= ' -c wal_level=hot_standby -c hot_standby=' . ($bHotStandby ? 'on' : 'off');
 
     # Force parallel mode on to make sure we are disabling it and there are no issues. This is important for testing that 9.6
     # works since pg_stop_backup() is marked parallel safe and will error if run in a worker.
@@ -470,6 +470,7 @@ sub clusterStop
 
     # Set defaults
     my $bIgnoreLogError = defined($$hParam{bIgnoreLogError}) ? $$hParam{bIgnoreLogError} : false;
+    my $bStop = defined($hParam->{bStop}) ? $$hParam{bStop} : true;
 
     # Disconnect user session
     $self->sqlDisconnect();
@@ -484,7 +485,7 @@ sub clusterStop
     }
 
     # If pg process is running then stop the cluster
-    if (-e $self->dbBasePath() . '/' . DB_FILE_POSTMTRPID)
+    if ($bStop && -e $self->dbBasePath() . '/' . DB_FILE_POSTMTRPID)
     {
         $self->executeSimple($self->pgBinPath() . '/pg_ctl stop -D ' . $self->dbBasePath() . ' -w -s -m fast');
     }
