@@ -229,7 +229,7 @@ sub run
 
         # Create hosts, file object, and config
         my ($oHostDbPrimary, $oHostDbStandby, $oHostBackup) = $self->setup(
-            true, $self->expect(),
+            true,
             {bHostBackup => $bRemote, bTls => $bTls, strStorage => $strStorage, bRepoEncrypt => $bEncrypt,
                 strCompressType => NONE});
 
@@ -254,7 +254,7 @@ sub run
         $oManifest{&INI_SECTION_BACKREST}{&INI_KEY_VERSION} = PROJECT_VERSION;
         $oManifest{&INI_SECTION_BACKREST}{&INI_KEY_FORMAT} = REPOSITORY_FORMAT;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_ARCHIVE_CHECK} = JSON::PP::false;
-        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_ARCHIVE_COPY} = JSON::PP::true;
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_ARCHIVE_COPY} = JSON::PP::false;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_BACKUP_STANDBY} = JSON::PP::false;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_BUFFER_SIZE} = 16384;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_CHECKSUM_PAGE} = JSON::PP::true;
@@ -332,10 +332,10 @@ sub run
             pageBuild($tBasePage, 0, 0x1b99) .
             pageBuild($tBasePage, 1, 0x1b9a) .
             pageBuild($tBasePage, 2, 0x1b97) .
-            pageBuild($tBasePage, 0, 0x8170, 0xFFFFFFFF, 0xFFFFFFFF);
+            ("\0" x 8192);
 
         $oHostDbPrimary->manifestFileCreate(
-            \%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/33000', $tPageValid, '7a16d165e4775f7c92e8cdf60c0af57313f0bf90',
+            \%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/33000', $tPageValid, '1d73a0052828531770e7c155aeb22338e017e196',
             $lTime);
 
         my $iBlockOffset = 32767 * 131072;
@@ -344,11 +344,11 @@ sub run
             pageBuild($tBasePage, $iBlockOffset + 0, 0xf7de) .
             pageBuild($tBasePage, $iBlockOffset + 1, 0xf7df) .
             ("\0" x 8192) .
-            pageBuild($tBasePage, 0, 0x8170, 0xFFFFFFFF, 0xFFFFFFFF);
+            ("\0" x 8192);
 
         $oHostDbPrimary->manifestFileCreate(
             \%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/33000.32767', $tPageValidSeg32767,
-            '6e99b589e550e68e934fd235ccba59fe5b592a9e', $lTime);
+            '1d11c42e6080e805a7b12bf9f83f4def548d92ac', $lTime);
 
         my $tPageInvalid33001 =
             pageBuild($tBasePage, 1, 0x1b9a) .
@@ -430,7 +430,7 @@ sub run
 
             # Unlog and temp files to ignore (unlog _init will NOT be ignored)
             $oHostDbPrimary->manifestFileCreate(\%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/44000_init', $tPageValid,
-                '7a16d165e4775f7c92e8cdf60c0af57313f0bf90', $lTime);
+                '1d73a0052828531770e7c155aeb22338e017e196', $lTime);
             $oHostDbPrimary->dbFileCreate(\%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/44000', 'IGNORE');
             $oHostDbPrimary->dbFileCreate(\%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/t333_44000', 'IGNORE');
         }
@@ -439,8 +439,8 @@ sub run
         #---------------------------------------------------------------------------------------------------------------------------
         if ($self->runCurrent() == 1)
         {
-            $oHostDbPrimary->executeSimple($self->backrestExe() . " version", {oLogTest => $self->expect()});
-            $oHostDbPrimary->executeSimple($self->backrestExe() . " help version", {oLogTest => $self->expect()});
+            $oHostDbPrimary->executeSimple($self->backrestExe() . " version");
+            $oHostDbPrimary->executeSimple($self->backrestExe() . " help version");
         }
 
         # Full backup
@@ -621,7 +621,7 @@ sub run
             {oExpectedManifest => \%oManifest,
                 strOptionalParam => '--force --checksum-page' . ($bDeltaBackup ? ' --delta' : '')});
 
-        # Remove postmas'.'ter.pid so restore will succeed (the rest will be cleaned up by the delta)
+        # Remove pid so restore will succeed (the rest will be cleaned up by the delta)
         storageTest->remove($oHostDbPrimary->dbBasePath() . '/' . DB_FILE_POSTMTRPID);
 
         # Restore - tests various mode, extra files/paths, missing files/paths
@@ -682,15 +682,13 @@ sub run
                         {&MANIFEST_SUBKEY_USER => undef, &MANIFEST_SUBKEY_GROUP => undef}}},
                 false);
 
-            delete($oManifest{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_USER});
-            delete($oManifest{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_GROUP});
+            $oManifest{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_USER} = TEST_USER;
+            $oManifest{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_GROUP} = TEST_GROUP;
 
-            delete(
-                $oManifest{&MANIFEST_SECTION_TARGET_FILE}{MANIFEST_TARGET_PGDATA . '/base/1/' . DB_FILE_PGVERSION}
-                  {&MANIFEST_SUBKEY_USER});
-            delete(
-                $oManifest{&MANIFEST_SECTION_TARGET_FILE}{MANIFEST_TARGET_PGDATA . '/base/16384/' . DB_FILE_PGVERSION}
-                {&MANIFEST_SUBKEY_GROUP});
+            $oManifest{&MANIFEST_SECTION_TARGET_FILE}{MANIFEST_TARGET_PGDATA . '/base/1/' . DB_FILE_PGVERSION}
+                {&MANIFEST_SUBKEY_USER} = TEST_USER;
+            $oManifest{&MANIFEST_SECTION_TARGET_FILE}{MANIFEST_TARGET_PGDATA . '/base/16384/' . DB_FILE_PGVERSION}
+                {&MANIFEST_SUBKEY_GROUP} = TEST_GROUP;
 
             $oHostDbPrimary->restore(
                 'fix permissions', $strFullBackup,
@@ -737,6 +735,13 @@ sub run
         $strType = CFGOPTVAL_BACKUP_TYPE_INCR;
         $oHostDbPrimary->manifestReference(\%oManifest, $strFullBackup);
 
+        # Fix mode on base path so defaults are simpler in backups
+        if (!$bRemote)
+        {
+            $oHostDbPrimary->executeSimple('chmod 700 ' . $oHostDbPrimary->dbBasePath(), undef, 'root');
+            $oManifest{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_MODE} = '0700';
+        }
+
         # Add tablespace 1
         $oHostDbPrimary->manifestTablespaceCreate(\%oManifest, 1);
         $oHostDbPrimary->manifestPathCreate(\%oManifest, MANIFEST_TARGET_PGTBLSPC . '/1', '16384');
@@ -779,7 +784,8 @@ sub run
             $oHostBackup->manifestMunge(
                 basename($strResumePath),
                 {&MANIFEST_SECTION_TARGET_FILE =>
-                    {(&MANIFEST_TARGET_PGDATA . '/badchecksum.txt') => {&MANIFEST_SUBKEY_CHECKSUM => BOGUS}}},
+                    {(&MANIFEST_TARGET_PGDATA . '/badchecksum.txt') =>
+                        {&MANIFEST_SUBKEY_CHECKSUM => 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'}}},
                 false);
         }
         # Change contents of resumed file without changing size so it will throw a nasty error about the repo having been corrupted
@@ -845,12 +851,6 @@ sub run
 
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = $strStorage eq S3 ? 2 : 1;
 
-        if (!$bRemote)
-        {
-            # Remove the size-changed test file to avoid expect log churn
-            $oHostDbPrimary->manifestFileRemove(\%oManifest, MANIFEST_TARGET_PGDATA, 'changesize.txt');
-        }
-
         # Drop tablespace 11
         #---------------------------------------------------------------------------------------------------------------------------
         $strType = CFGOPTVAL_BACKUP_TYPE_DIFF;
@@ -875,13 +875,6 @@ sub run
         $oRemapHash{&MANIFEST_TARGET_PGTBLSPC . '/1'} = $oHostDbPrimary->tablespacePath(1, 2);
         $oRemapHash{&MANIFEST_TARGET_PGTBLSPC . '/2'} = $oHostDbPrimary->tablespacePath(2, 2);
 
-        # At this point the $PG_DATA permissions have been reset to 0700
-        if (!$bRemote)
-        {
-            $oManifest{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_MODE} = '0777';
-            $oManifest{&MANIFEST_SECTION_TARGET_PATH}{&MANIFEST_TARGET_PGTBLSPC}{&MANIFEST_SUBKEY_MODE} = '0777';
-        }
-
         $oHostDbPrimary->restore(
             'remap all paths', $strBackup, {rhExpectedManifest => \%oManifest, rhRemapHash => \%oRemapHash});
 
@@ -962,10 +955,12 @@ sub run
             $oHostBackup->configUpdate({&CFGDEF_SECTION_GLOBAL => {'compress-type' => $strCompressType}});
         }
 
-        # Enable hardlinks (except for s3) to ensure a warning is raised
+        # Enable hardlinks (except for s3) to show they can be enabled after a full backup
         if ($strStorage eq POSIX)
         {
             $oHostBackup->configUpdate({&CFGDEF_SECTION_GLOBAL => {'repo1-hardlink' => 'y'}});
+            $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_HARDLINK} = JSON::PP::true;
+            $oHostBackup->{bHardLink} = true;
         }
 
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = 1;
@@ -989,16 +984,10 @@ sub run
         #---------------------------------------------------------------------------------------------------------------------------
         $strType = CFGOPTVAL_BACKUP_TYPE_FULL;
 
-        # Now the compression and hardlink changes will take effect
+        # Now the compression changes will take effect
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_COMPRESS} = JSON::PP::true;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_COMPRESS_TYPE} = $strCompressType;
 
-        if ($strStorage eq POSIX)
-        {
-            $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_HARDLINK} = JSON::PP::true;
-            $oHostBackup->{bHardLink} = true;
-        }
-
         $oHostDbPrimary->manifestReference(\%oManifest);
 
         $oHostDbPrimary->manifestFileCreate(
@@ -1052,7 +1041,7 @@ sub run
 
         # Restore checksum values for next test
         $oManifest{&MANIFEST_SECTION_TARGET_FILE}{'pg_data/base/32768/33000'}{&MANIFEST_SUBKEY_CHECKSUM} =
-            '7a16d165e4775f7c92e8cdf60c0af57313f0bf90';
+            '1d73a0052828531770e7c155aeb22338e017e196';
         $oManifest{&MANIFEST_SECTION_TARGET_FILE}{'pg_data/base/32768/33001'}{&MANIFEST_SUBKEY_CHECKSUM} =
             '6bf316f11d28c28914ea9be92c00de9bea6d9a6b';
         $oManifest{&MANIFEST_SECTION_TARGET_FILE}{'pg_tblspc/2/PG_9.4_201409291/32768/tablespace2.txt'}
@@ -1105,9 +1094,7 @@ sub run
         #---------------------------------------------------------------------------------------------------------------------------
         if (!$bRemote && $strStorage eq POSIX)
         {
-            executeTest(
-                'ls -1Rtr ' . $oHostBackup->repoBackupPath(PATH_BACKUP_HISTORY),
-                {oLogTest => $self->expect(), bRemote => $bRemote});
+            executeTest('ls -1Rtr ' . $oHostBackup->repoBackupPath(PATH_BACKUP_HISTORY), {bRemote => $bRemote});
         }
 
         # Test backup from standby warning that standby not configured so option reset
