@@ -52,20 +52,19 @@ sub run
 
     foreach my $rhRun
     (
-        {pg => PG_VERSION_83, repoDest => HOST_DB_PRIMARY, tls => 0, storage => POSIX, encrypt => 0, compress => NONE, repo => 2},
-        {pg => PG_VERSION_84, repoDest =>     HOST_BACKUP, tls => 1, storage => AZURE, encrypt => 1, compress =>   GZ, repo => 1},
-        {pg => PG_VERSION_90, repoDest => HOST_DB_PRIMARY, tls => 0, storage =>   GCS, encrypt => 1, compress =>  BZ2, repo => 2},
-        {pg => PG_VERSION_91, repoDest => HOST_DB_STANDBY, tls => 1, storage =>   GCS, encrypt => 0, compress =>   GZ, repo => 1},
-        {pg => PG_VERSION_92, repoDest => HOST_DB_STANDBY, tls => 0, storage => POSIX, encrypt => 1, compress => NONE, repo => 1},
-        {pg => PG_VERSION_93, repoDest =>     HOST_BACKUP, tls => 0, storage => AZURE, encrypt => 0, compress => NONE, repo => 2},
-        {pg => PG_VERSION_94, repoDest => HOST_DB_STANDBY, tls => 0, storage => POSIX, encrypt => 1, compress =>  LZ4, repo => 1},
-        {pg => PG_VERSION_95, repoDest =>     HOST_BACKUP, tls => 1, storage =>    S3, encrypt => 0, compress =>  BZ2, repo => 1},
-        {pg => PG_VERSION_96, repoDest =>     HOST_BACKUP, tls => 0, storage => POSIX, encrypt => 0, compress => NONE, repo => 2},
-        {pg => PG_VERSION_10, repoDest => HOST_DB_STANDBY, tls => 1, storage =>    S3, encrypt => 1, compress =>   GZ, repo => 2},
-        {pg => PG_VERSION_11, repoDest =>     HOST_BACKUP, tls => 1, storage => AZURE, encrypt => 0, compress =>  ZST, repo => 2},
-        {pg => PG_VERSION_12, repoDest =>     HOST_BACKUP, tls => 0, storage =>    S3, encrypt => 1, compress =>  LZ4, repo => 1},
-        {pg => PG_VERSION_13, repoDest => HOST_DB_STANDBY, tls => 1, storage =>   GCS, encrypt => 0, compress =>  ZST, repo => 1},
-        {pg => PG_VERSION_14, repoDest =>     HOST_BACKUP, tls => 0, storage => POSIX, encrypt => 1, compress =>  LZ4, repo => 2},
+        {pg => '9.0', repoDest => HOST_DB_PRIMARY, tls => 0, storage =>   GCS, encrypt => 1, compress =>  BZ2, repo => 2, bnd => 1},
+        {pg => '9.1', repoDest => HOST_DB_STANDBY, tls => 1, storage =>   GCS, encrypt => 0, compress =>   GZ, repo => 1, bnd => 0},
+        {pg => '9.2', repoDest => HOST_DB_STANDBY, tls => 0, storage => POSIX, encrypt => 1, compress => NONE, repo => 1, bnd => 1},
+        {pg => '9.3', repoDest =>     HOST_BACKUP, tls => 0, storage => AZURE, encrypt => 0, compress => NONE, repo => 2, bnd => 0},
+        {pg => '9.4', repoDest => HOST_DB_STANDBY, tls => 0, storage => POSIX, encrypt => 1, compress =>  LZ4, repo => 1, bnd => 1},
+        {pg => '9.5', repoDest =>     HOST_BACKUP, tls => 1, storage =>    S3, encrypt => 0, compress =>  BZ2, repo => 1, bnd => 0},
+        {pg => '9.6', repoDest =>     HOST_BACKUP, tls => 0, storage => POSIX, encrypt => 0, compress => NONE, repo => 2, bnd => 1},
+        {pg =>  '10', repoDest => HOST_DB_STANDBY, tls => 1, storage =>    S3, encrypt => 1, compress =>   GZ, repo => 2, bnd => 0},
+        {pg =>  '11', repoDest =>     HOST_BACKUP, tls => 1, storage => AZURE, encrypt => 0, compress =>  ZST, repo => 2, bnd => 1},
+        {pg =>  '12', repoDest =>     HOST_BACKUP, tls => 0, storage =>    S3, encrypt => 1, compress =>  LZ4, repo => 1, bnd => 0},
+        {pg =>  '13', repoDest => HOST_DB_STANDBY, tls => 1, storage =>   GCS, encrypt => 0, compress =>  ZST, repo => 1, bnd => 1},
+        {pg =>  '14', repoDest =>     HOST_BACKUP, tls => 0, storage => POSIX, encrypt => 1, compress =>  LZ4, repo => 2, bnd => 0},
+        {pg =>  '15', repoDest => HOST_DB_STANDBY, tls => 0, storage => AZURE, encrypt => 0, compress => NONE, repo => 2, bnd => 1},
     )
     {
         # Only run tests for this pg version
@@ -80,23 +79,22 @@ sub run
         my $bRepoEncrypt = $rhRun->{encrypt};
         my $strCompressType = $rhRun->{compress};
         my $iRepoTotal = $rhRun->{repo};
+        my $bBundle = $rhRun->{bnd};
 
-        # Use a specific VM and version of PostgreSQL for expect testing. This version will also be used to run tests that are not
-        # version specific.
-        my $bExpectVersion = $self->vm() eq VM_EXPECT && $self->pgVersion() eq PG_VERSION_96;
+        # Some tests are not version specific so only run them on a single version of PostgreSQL
+        my $bNonVersionSpecific = $self->pgVersion() eq PG_VERSION_96;
 
         # Increment the run, log, and decide whether this unit test should be run
-        next if (!$self->begin(
+        next if !$self->begin(
             "bkp ${bHostBackup}, sby ${bHostStandby}, tls ${bTls}, dst ${strBackupDestination}, cmp ${strCompressType}" .
-                ", storage ${strStorage}, enc ${bRepoEncrypt}",
-            $bExpectVersion));
+                ", storage ${strStorage}, enc ${bRepoEncrypt}");
 
         # Create hosts, file object, and config
         my ($oHostDbPrimary, $oHostDbStandby, $oHostBackup) = $self->setup(
-            false, $self->expect(),
+            false,
             {bHostBackup => $bHostBackup, bStandby => $bHostStandby, bTls => $bTls, strBackupDestination => $strBackupDestination,
-             strCompressType => $strCompressType, bArchiveAsync => false, strStorage => $strStorage,
-             bRepoEncrypt => $bRepoEncrypt, iRepoTotal => $iRepoTotal});
+                strCompressType => $strCompressType, bArchiveAsync => false, strStorage => $strStorage,
+                bRepoEncrypt => $bRepoEncrypt, iRepoTotal => $iRepoTotal, bBundle => $bBundle});
 
         # Some commands will fail because of the bogus host created when a standby is present. These options reset the bogus host
         # so it won't interfere with commands that won't tolerate a connection failure.
@@ -209,9 +207,9 @@ sub run
         }
 
         # Make a new backup with expire-auto disabled then run the expire command and compare backup numbers to ensure that expire
-        # was really disabled. This test is not version specific so is run on only the expect version.
+        # was really disabled. This test is not version specific so is run on only one version.
         #---------------------------------------------------------------------------------------------------------------------------
-        if ($bExpectVersion)
+        if ($bNonVersionSpecific)
         {
             my $oBackupInfo = new pgBackRestTest::Env::BackupInfo($oHostBackup->repoBackupPath());
             push(my @backupLst1, $oBackupInfo->list());
@@ -349,9 +347,9 @@ sub run
         # Execute stop and make sure the backup fails
         #---------------------------------------------------------------------------------------------------------------------------
         # Restart the cluster to check for any errors before continuing since the stop tests will definitely create errors and the
-        # logs will to be deleted to avoid causing issues further down the line. This test is not version specific so is run on only
-        # the expect version.
-        if ($bExpectVersion)
+        # logs will need to be deleted to avoid causing issues further down the line. This test is not version specific so is run on
+        # only one version.
+        if ($bNonVersionSpecific)
         {
             confess "test must be performed on posix storage" if $strStorage ne POSIX;
 
@@ -508,7 +506,7 @@ sub run
         $oHostDbPrimary->sqlWalRotate();
 
         # Get the SHA1 and path of the table for the database that will not be restored
-        $strDb1TablePath =  $oHostDbPrimary->dbBasePath(). "/base/" .
+        $strDb1TablePath = $oHostDbPrimary->dbBasePath(). "/base/" .
             $oHostDbPrimary->sqlSelectOne("select oid from pg_database where datname='test1'") . "/" .
             $oHostDbPrimary->sqlSelectOne("select relfilenode from pg_class where relname='test1_zeroed'", {strDb => 'test1'});
         $strDb1TableSha1 = storageTest()->hashSize($strDb1TablePath);
@@ -572,34 +570,22 @@ sub run
         # The tablespace path should exist and have files in it
         my $strTablespacePath = $oHostDbPrimary->tablespacePath(1);
 
-        # Version <= 8.4 always places a PG_VERSION file in the tablespace
-        if ($oHostDbPrimary->pgVersion() <= PG_VERSION_84)
-        {
-            if (!storageTest()->exists("${strTablespacePath}/" . DB_FILE_PGVERSION))
-            {
-                confess &log(ASSERT, "unable to find '" . DB_FILE_PGVERSION . "' in tablespace path '${strTablespacePath}'");
-            }
-        }
-        # Version >= 9.0 creates a special path using the version and catalog number
-        else
-        {
-            # Backup info will have the catalog number
-            my $oBackupInfo = new pgBackRestDoc::Common::Ini(
-                storageRepo(), $oHostBackup->repoBackupPath(FILE_BACKUP_INFO),
-                {bLoad => false, strContent => ${storageRepo()->get($oHostBackup->repoBackupPath(FILE_BACKUP_INFO))}});
+        # Backup info will have the catalog number
+        my $oBackupInfo = new pgBackRestDoc::Common::Ini(
+            storageRepo(), $oHostBackup->repoBackupPath(FILE_BACKUP_INFO),
+            {bLoad => false, strContent => ${storageRepo()->get($oHostBackup->repoBackupPath(FILE_BACKUP_INFO))}});
 
-            # Construct the special path
-            $strTablespacePath .=
-                '/PG_' . $oHostDbPrimary->pgVersion() . qw{_} . $oBackupInfo->get(INFO_BACKUP_SECTION_DB, INFO_BACKUP_KEY_CATALOG);
+        # Construct the special path
+        $strTablespacePath .=
+            '/PG_' . $oHostDbPrimary->pgVersion() . qw{_} . $oBackupInfo->get(INFO_BACKUP_SECTION_DB, INFO_BACKUP_KEY_CATALOG);
 
-            # Check that path exists
-            if (!storageTest()->pathExists($strTablespacePath))
-            {
-                confess &log(ASSERT, "unable to find tablespace path '${strTablespacePath}'");
-            }
+        # Check that path exists
+        if (!storageTest()->pathExists($strTablespacePath))
+        {
+            confess &log(ASSERT, "unable to find tablespace path '${strTablespacePath}'");
         }
 
-        # Make sure there are some files in the tablespace path (exclude PG_VERSION if <= 8.4 since that was tested above)
+        # Make sure there are some files in the tablespace path
         if (grep(!/^PG\_VERSION$/i, storageTest()->list($strTablespacePath)) == 0)
         {
             confess &log(ASSERT, "no files found in tablespace path '${strTablespacePath}'");
@@ -614,15 +600,7 @@ sub run
         $oHostDbPrimary->sqlExecute('drop database test2', {bAutoCommit => true});
 
         # The test table lives in ts1 so it needs to be moved or dropped
-        if ($oHostDbPrimary->pgVersion() >= PG_VERSION_90)
-        {
-            $oHostDbPrimary->sqlExecute('alter table test set tablespace pg_default');
-        }
-        # Drop for older versions
-        else
-        {
-            $oHostDbPrimary->sqlExecute('drop table test');
-        }
+        $oHostDbPrimary->sqlExecute('alter table test set tablespace pg_default');
 
         # And drop the tablespace
         $oHostDbPrimary->sqlExecute('drop database test3', {bAutoCommit => true});
@@ -753,34 +731,30 @@ sub run
 
         # Restore (restore type = default, timeline = created by type = xid, inclusive recovery)
         #---------------------------------------------------------------------------------------------------------------------------
-        if ($oHostDbPrimary->pgVersion() >= PG_VERSION_84)
-        {
-            &log(INFO, '    testing recovery type = ' . CFGOPTVAL_RESTORE_TYPE_DEFAULT);
+        &log(INFO, '    testing recovery type = ' . CFGOPTVAL_RESTORE_TYPE_DEFAULT);
 
-            $oHostDbPrimary->clusterStop();
-
-            # The timeline to use for this test is subject to change based on tests being added or removed above.  The best thing
-            # would be to automatically grab the timeline after the restore, but since this test has been stable for a long time
-            # it does not seem worth the effort to automate.
-            $oHostDbPrimary->restore(
-                undef, $strIncrBackup,
-                {bDelta => true,
-                    strType => $oHostDbPrimary->pgVersion() >= PG_VERSION_90 ?
-                        CFGOPTVAL_RESTORE_TYPE_STANDBY : CFGOPTVAL_RESTORE_TYPE_DEFAULT,
-                    strTargetTimeline => 4, iRepo => $iRepoTotal});
-
-            $oHostDbPrimary->clusterStart({bHotStandby => true});
-            $oHostDbPrimary->sqlSelectOneTest('select message from test', $strTimelineMessage, {iTimeout => 120});
-        }
-
-        # Stop clusters to catch any errors in the postgres log
-        #---------------------------------------------------------------------------------------------------------------------------
         $oHostDbPrimary->clusterStop();
 
-        # Stanza-delete --force without access to pgbackrest on database host. This test is not version specific so is run on only
-        # the expect version.
+        # The timeline to use for this test is subject to change based on tests being added or removed above.  The best thing
+        # would be to automatically grab the timeline after the restore, but since this test has been stable for a long time
+        # it does not seem worth the effort to automate.
+        $oHostDbPrimary->restore(
+            undef, $strIncrBackup,
+            {bDelta => true, strType => CFGOPTVAL_RESTORE_TYPE_STANDBY, strTargetTimeline => 4, iRepo => $iRepoTotal});
+
+        $oHostDbPrimary->clusterStart({bHotStandby => true});
+        $oHostDbPrimary->sqlSelectOneTest('select message from test', $strTimelineMessage, {iTimeout => 120});
+
+        # Stop clusters to catch any errors in the postgres log. Stopping the cluster has started consistently running out of memory
+        # on PostgreSQL 9.1. This seems to have happened after pulling in new packages at some point so it might be build related.
+        # Stopping the cluster is not critical for 9.1 so skip it.
         #---------------------------------------------------------------------------------------------------------------------------
-        if ($bExpectVersion)
+        $oHostDbPrimary->clusterStop({bStop => $oHostDbPrimary->pgVersion() != PG_VERSION_91});
+
+        # Stanza-delete --force without access to pgbackrest on database host. This test is not version specific so is run on only
+        # one version.
+        #---------------------------------------------------------------------------------------------------------------------------
+        if ($bNonVersionSpecific)
         {
             # Make sure this test has a backup host to work with
             confess "test must run with backup dst = " . HOST_BACKUP if !$bHostBackup;
