@@ -11,14 +11,6 @@ Harness for PostgreSQL Interface
 /***********************************************************************************************************************************
 Interface definition
 ***********************************************************************************************************************************/
-uint32_t hrnPgInterfaceCatalogVersion083(void);
-void hrnPgInterfaceControl083(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal083(PgWal pgWal, unsigned char *buffer);
-
-uint32_t hrnPgInterfaceCatalogVersion084(void);
-void hrnPgInterfaceControl084(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal084(PgWal pgWal, unsigned char *buffer);
-
 uint32_t hrnPgInterfaceCatalogVersion090(void);
 void hrnPgInterfaceControl090(PgControl pgControl, unsigned char *buffer);
 void hrnPgInterfaceWal090(PgWal pgWal, unsigned char *buffer);
@@ -67,6 +59,10 @@ uint32_t hrnPgInterfaceCatalogVersion140(void);
 void hrnPgInterfaceControl140(PgControl pgControl, unsigned char *buffer);
 void hrnPgInterfaceWal140(PgWal pgWal, unsigned char *buffer);
 
+uint32_t hrnPgInterfaceCatalogVersion150(void);
+void hrnPgInterfaceControl150(PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal150(PgWal pgWal, unsigned char *buffer);
+
 typedef struct HrnPgInterface
 {
     // Version of PostgreSQL supported by this interface
@@ -84,6 +80,13 @@ typedef struct HrnPgInterface
 
 static const HrnPgInterface hrnPgInterface[] =
 {
+    {
+        .version = PG_VERSION_15,
+
+        .catalogVersion = hrnPgInterfaceCatalogVersion150,
+        .control = hrnPgInterfaceControl150,
+        .wal = hrnPgInterfaceWal150,
+    },
     {
         .version = PG_VERSION_14,
 
@@ -167,20 +170,6 @@ static const HrnPgInterface hrnPgInterface[] =
         .control = hrnPgInterfaceControl090,
         .wal = hrnPgInterfaceWal090,
     },
-    {
-        .version = PG_VERSION_84,
-
-        .catalogVersion = hrnPgInterfaceCatalogVersion084,
-        .control = hrnPgInterfaceControl084,
-        .wal = hrnPgInterfaceWal084,
-    },
-    {
-        .version = PG_VERSION_83,
-
-        .catalogVersion = hrnPgInterfaceCatalogVersion083,
-        .control = hrnPgInterfaceControl083,
-        .wal = hrnPgInterfaceWal083,
-    },
 };
 
 /***********************************************************************************************************************************
@@ -195,7 +184,7 @@ hrnPgInterfaceVersion(unsigned int pgVersion)
 
     const HrnPgInterface *result = NULL;
 
-    for (unsigned int interfaceIdx = 0; interfaceIdx < sizeof(hrnPgInterface) / sizeof(HrnPgInterface); interfaceIdx++)
+    for (unsigned int interfaceIdx = 0; interfaceIdx < LENGTH_OF(hrnPgInterface); interfaceIdx++)
     {
         if (hrnPgInterface[interfaceIdx].version == pgVersion)
         {
