@@ -314,7 +314,7 @@ testRun(void)
 
         memset(bufPtr(walBuffer1), 0, bufSize(walBuffer1));
         hrnPgWalToBuffer((PgWal){.version = PG_VERSION_11, .systemId = 1}, walBuffer1);
-        const char *walBuffer1Sha1 = strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA1_STR, walBuffer1)));
+        const char *walBuffer1Sha1 = strZ(bufHex(cryptoHashOne(hashTypeSha1, walBuffer1)));
 
         HRN_STORAGE_PUT(storagePgWrite(), "pg_wal/000000010000000100000001", walBuffer1);
 
@@ -335,38 +335,39 @@ testRun(void)
         TEST_RESULT_LOG("P00   INFO: pushed WAL file '000000010000000100000001' to the archive");
 
         TEST_STORAGE_EXISTS(
-            storageRepoIdxWrite(0), strZ(strNewFmt(STORAGE_REPO_ARCHIVE "/11-1/000000010000000100000001-%s.gz", walBuffer1Sha1)),
+            storageRepoIdxWrite(0), zNewFmt(STORAGE_REPO_ARCHIVE "/11-1/000000010000000100000001-%s.gz", walBuffer1Sha1),
             .remove = true, .comment = "check repo for WAL file, then remove");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("generate valid WAL and push them");
+        TEST_TITLE("generate valid WAL and push them, with parameter --no-archive-mode-check to suppress duplicate WAL warning");
 
         argListTemp = strLstDup(argList);
+        hrnCfgArgRawNegate(argListTemp, cfgOptArchiveModeCheck);
         strLstAddZ(argListTemp, "pg_wal/000000010000000100000001");
         HRN_CFG_LOAD(cfgCmdArchivePush, argListTemp);
 
         memset(bufPtr(walBuffer1), 0, bufSize(walBuffer1));
         hrnPgWalToBuffer((PgWal){.version = PG_VERSION_11}, walBuffer1);
 
-        // Check sha1 checksum against fixed values once to make sure they are not getting munged. After this we'll calculate them
-        // directly from the buffers to reduce the cost of maintaining checksums.
-        walBuffer1Sha1 = TEST_64BIT() ?
-            (TEST_BIG_ENDIAN() ? "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" : "858a9ef24b79468eb2a61543b58140addfede0fc") :
-            "044ec0576dc4e59d460aa3a8ac796ba4874ddff3";
-
         HRN_STORAGE_PUT(storagePgWrite(), "pg_wal/000000010000000100000001", walBuffer1);
 
         TEST_RESULT_VOID(cmdArchivePush(), "push the WAL segment");
         TEST_RESULT_LOG("P00   INFO: pushed WAL file '000000010000000100000001' to the archive");
 
-        TEST_STORAGE_EXISTS(
-            storageRepoIdxWrite(0), strZ(strNewFmt(STORAGE_REPO_ARCHIVE "/11-1/000000010000000100000001-%s.gz", walBuffer1Sha1)),
+        // Check sha1 checksum against fixed values once to make sure they are not getting munged. After this we'll calculate them
+        // directly from the buffers to reduce the cost of maintaining checksums.
+        TEST_STORAGE_LIST(
+            storageRepoIdx(0), STORAGE_REPO_ARCHIVE "/11-1/0000000100000001",
+            zNewFmt(
+                "000000010000000100000001-%s.gz\n",
+                TEST_64BIT() ?
+                    (TEST_BIG_ENDIAN() ? "4dc9df63290935f68f43b7d02005716a98800ce0" : "858a9ef24b79468eb2a61543b58140addfede0fc") :
+                    "044ec0576dc4e59d460aa3a8ac796ba4874ddff3"),
             .comment = "check repo for WAL file");
 
+        // No warning emitted re WAL file already existing with the same checksum due to --no-archive-mode-check
         TEST_RESULT_VOID(cmdArchivePush(), "push the WAL segment again");
         TEST_RESULT_LOG(
-            "P00   WARN: WAL file '000000010000000100000001' already exists in the repo1 archive with the same checksum\n"
-            "            HINT: this is valid in some recovery scenarios but may also indicate a problem.\n"
             "P00   INFO: pushed WAL file '000000010000000100000001' to the archive");
 
         // Now create a new WAL buffer with a different checksum to test checksum errors
@@ -374,7 +375,7 @@ testRun(void)
         bufUsedSet(walBuffer2, bufSize(walBuffer2));
         memset(bufPtr(walBuffer2), 0xFF, bufSize(walBuffer2));
         hrnPgWalToBuffer((PgWal){.version = PG_VERSION_11}, walBuffer2);
-        const char *walBuffer2Sha1 = strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA1_STR, walBuffer2)));
+        const char *walBuffer2Sha1 = strZ(bufHex(cryptoHashOne(hashTypeSha1, walBuffer2)));
 
         HRN_STORAGE_PUT(storagePgWrite(), "pg_wal/000000010000000100000001", walBuffer2);
 
@@ -396,8 +397,7 @@ testRun(void)
         // Create tmp file to make it look like a prior push failed partway through to ensure that retries work
         HRN_STORAGE_PUT_Z(
             storageTest,
-            strZ(
-                strNewFmt("repo/archive/test/11-1/0000000100000001/000000010000000100000002-%s.gz.pgbackrest.tmp", walBuffer2Sha1)),
+            zNewFmt("repo/archive/test/11-1/0000000100000001/000000010000000100000002-%s.gz.pgbackrest.tmp", walBuffer2Sha1),
             "PARTIAL", .comment = "write WAL tmp file");
 
         TEST_RESULT_VOID(cmdArchivePush(), "push the WAL segment");
@@ -405,9 +405,7 @@ testRun(void)
 
         TEST_STORAGE_EXISTS(
             storageRepoIdxWrite(0),
-            strZ(
-                strNewFmt(
-                    STORAGE_REPO_ARCHIVE "/11-1/0000000100000001/000000010000000100000002-%s.gz", walBuffer2Sha1)),
+            zNewFmt(STORAGE_REPO_ARCHIVE "/11-1/0000000100000001/000000010000000100000002-%s.gz", walBuffer2Sha1),
             .comment = "check repo for WAL file");
         TEST_RESULT_BOOL(
             storageExistsP(
@@ -461,6 +459,14 @@ testRun(void)
         // Remove old repo
         HRN_STORAGE_PATH_REMOVE(storageTest, "repo", .errorOnMissing = true, .recurse = true);
 
+        // Create an artificially small WAL file to save time on encryption
+        walBuffer2 = bufNew(1024);
+        bufUsedSet(walBuffer2, bufSize(walBuffer2));
+        memset(bufPtr(walBuffer2), 0xFF, bufSize(walBuffer2));
+        hrnPgWalToBuffer((PgWal){.version = PG_VERSION_11}, walBuffer2);
+        walBuffer2Sha1 = strZ(bufHex(cryptoHashOne(hashTypeSha1, walBuffer2)));
+        HRN_STORAGE_PUT(storageTest, "pg/pg_wal/000000010000000100000002", walBuffer2, .comment = "write WAL");
+
         argListTemp = strLstNew();
         hrnCfgArgRawZ(argListTemp, cfgOptStanza, "test");
         hrnCfgArgKeyRawZ(argListTemp, cfgOptPgPath, 1, TEST_PATH "/pg");
@@ -500,11 +506,11 @@ testRun(void)
         TEST_RESULT_LOG("P00   INFO: pushed WAL file '000000010000000100000002' to the archive");
 
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo2/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            storageTest, zNewFmt("repo2/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1),
             .remove = true, .comment = "check repo2 for WAL file then remove");
 
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo3/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            storageTest, zNewFmt("repo3/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1),
             .remove = true, .comment = "check repo3 for WAL file then remove");
 
         // -------------------------------------------------------------------------------------------------------------------------
@@ -512,18 +518,17 @@ testRun(void)
 
         HRN_STORAGE_MODE(storageTest, "repo2/archive/test/11-1/0000000100000001", .mode = 0500);
 
-        TEST_ERROR(
+        TEST_ERROR_FMT(
             cmdArchivePush(), CommandError,
-            strZ(
-                strNewFmt(
-                    "archive-push command encountered error(s):\n"
-                    "repo2: [FileOpenError] unable to open file '" TEST_PATH "/repo2/archive/test/11-1/0000000100000001"
-                        "/000000010000000100000002-%s' for write: [13] Permission denied", walBuffer2Sha1)));
+            "archive-push command encountered error(s):\n"
+            "repo2: [FileOpenError] unable to open file '" TEST_PATH "/repo2/archive/test/11-1/0000000100000001"
+                "/000000010000000100000002-%s' for write: [13] Permission denied",
+            walBuffer2Sha1);
 
         TEST_STORAGE_LIST_EMPTY(storageTest, "repo2/archive/test/11-1/0000000100000001", .comment = "check repo2 for no WAL file");
         TEST_STORAGE_LIST(
-            storageTest, "repo3/archive/test/11-1/0000000100000001",
-            strZ(strNewFmt("000000010000000100000002-%s\n", walBuffer2Sha1)), .comment = "check repo3 for WAL file");
+            storageTest, "repo3/archive/test/11-1/0000000100000001", zNewFmt("000000010000000100000002-%s\n", walBuffer2Sha1),
+            .comment = "check repo3 for WAL file");
 
         HRN_STORAGE_MODE(storageTest, "repo2/archive/test/11-1/0000000100000001");
 
@@ -537,8 +542,8 @@ testRun(void)
             "P00   INFO: pushed WAL file '000000010000000100000002' to the archive");
 
         TEST_STORAGE_LIST(
-            storageTest, "repo2/archive/test/11-1/0000000100000001",
-            strZ(strNewFmt("000000010000000100000002-%s\n", walBuffer2Sha1)), .comment = "check repo2 for WAL file");
+            storageTest, "repo2/archive/test/11-1/0000000100000001", zNewFmt("000000010000000100000002-%s\n", walBuffer2Sha1),
+            .comment = "check repo2 for WAL file");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("WAL already exists in both repos");
@@ -555,10 +560,10 @@ testRun(void)
         TEST_TITLE("push succeeds on one repo when other repo fails to load archive.info");
 
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo2/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            storageTest, zNewFmt("repo2/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1),
             .remove = true);
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo3/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            storageTest, zNewFmt("repo3/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1),
             .remove = true);
         HRN_STORAGE_MODE(storageTest, "repo2", .mode = 0200);
 
@@ -577,7 +582,7 @@ testRun(void)
 
         // Make sure WAL got pushed to repo3
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo3/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            storageTest, zNewFmt("repo3/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1),
             .remove = true);
 
         HRN_STORAGE_MODE(storageTest, "repo2");
@@ -595,7 +600,7 @@ testRun(void)
 
         // Make sure WAL got pushed to repo3
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo3/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            storageTest, zNewFmt("repo3/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1),
             .remove = true);
 
         HRN_STORAGE_MODE(storageTest, "repo2/archive/test/11-1");
@@ -608,7 +613,7 @@ testRun(void)
 
         // Install local command handler shim
         static const ProtocolServerHandler testLocalHandlerList[] = {PROTOCOL_SERVER_HANDLER_ARCHIVE_PUSH_LIST};
-        hrnProtocolLocalShimInstall(testLocalHandlerList, PROTOCOL_SERVER_HANDLER_LIST_SIZE(testLocalHandlerList));
+        hrnProtocolLocalShimInstall(testLocalHandlerList, LENGTH_OF(testLocalHandlerList));
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("command must be run on the pg host");
@@ -654,7 +659,8 @@ testRun(void)
 
         TEST_ERROR(
             cmdArchivePush(), ArchiveTimeoutError,
-            "unable to push WAL file 'bogus' to the archive asynchronously after 1 second(s)");
+            "unable to push WAL file 'bogus' to the archive asynchronously after 1 second(s)\n"
+            "HINT: check '" HRN_PATH "/test-archive-push-async.log' for errors.");
 
         // Create pg_control and archive.info for next set of tests
         // -------------------------------------------------------------------------------------------------------------------------
@@ -728,7 +734,8 @@ testRun(void)
 
                 TEST_ERROR(
                     cmdArchivePush(), ArchiveTimeoutError,
-                    "unable to push WAL file '000000010000000100000001' to the archive asynchronously after 1 second(s)");
+                    "unable to push WAL file '000000010000000100000001' to the archive asynchronously after 1 second(s)\n"
+                    "HINT: check '" HRN_PATH "/test-archive-push-async.log' for errors.");
 
                 // Notify child to release lock
                 HRN_FORK_PARENT_NOTIFY_PUT(0);
@@ -751,7 +758,7 @@ testRun(void)
         bufUsedSet(walBuffer1, bufSize(walBuffer1));
         memset(bufPtr(walBuffer1), 0xFF, bufSize(walBuffer1));
         hrnPgWalToBuffer((PgWal){.version = PG_VERSION_94}, walBuffer1);
-        const char *walBuffer1Sha1 = strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA1_STR, walBuffer1)));
+        const char *walBuffer1Sha1 = strZ(bufHex(cryptoHashOne(hashTypeSha1, walBuffer1)));
 
         HRN_STORAGE_PUT(storagePgWrite(),"pg_xlog/000000010000000100000001", walBuffer1);
 
@@ -759,7 +766,7 @@ testRun(void)
         TEST_RESULT_LOG("P00   INFO: pushed WAL file '000000010000000100000001' to the archive asynchronously");
 
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1)),
+            storageTest, zNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1),
             .comment = "check repo for WAL file");
 
         // -------------------------------------------------------------------------------------------------------------------------
@@ -817,7 +824,7 @@ testRun(void)
         HRN_STORAGE_PUT_EMPTY(storagePgWrite(), "pg_xlog/archive_status/000000010000000100000001.ready");
 
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1)),
+            storageTest, zNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1),
             .comment = "check repo1 for WAL 1 file");
 
         // Create a ready file for WAL 2 but don't create the segment yet -- this will test the file error
@@ -835,11 +842,11 @@ testRun(void)
             TEST_PATH "/pg/pg_xlog/000000010000000100000002");
 
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1)),
+            storageTest, zNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1),
             .comment = "check repo1 for WAL 1 file");
 
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo3/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1)),
+            storageTest, zNewFmt("repo3/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1),
             .comment = "check repo3 for WAL 1 file");
 
         TEST_STORAGE_LIST(
@@ -856,7 +863,7 @@ testRun(void)
         bufUsedSet(walBuffer2, bufSize(walBuffer2));
         memset(bufPtr(walBuffer2), 0x0C, bufSize(walBuffer2));
         hrnPgWalToBuffer((PgWal){.version = PG_VERSION_94}, walBuffer2);
-        const char *walBuffer2Sha1 = strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA1_STR, walBuffer2)));
+        const char *walBuffer2Sha1 = strZ(bufHex(cryptoHashOne(hashTypeSha1, walBuffer2)));
 
         HRN_STORAGE_PUT(storagePgWrite(), "pg_xlog/000000010000000100000002", walBuffer2);
 
@@ -870,10 +877,10 @@ testRun(void)
             "P01 DETAIL: pushed WAL file '000000010000000100000002' to the archive");
 
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            storageTest, zNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1),
             .comment = "check repo1 for WAL 2 file");
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo3/archive/test/9.4-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            storageTest, zNewFmt("repo3/archive/test/9.4-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1),
             .comment = "check repo3 for WAL 2 file");
 
         TEST_STORAGE_LIST(
@@ -905,7 +912,7 @@ testRun(void)
         bufUsedSet(walBuffer3, bufSize(walBuffer3));
         memset(bufPtr(walBuffer3), 0x44, bufSize(walBuffer3));
         hrnPgWalToBuffer((PgWal){.version = PG_VERSION_94}, walBuffer3);
-        const char *walBuffer3Sha1 = strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA1_STR, walBuffer3)));
+        const char *walBuffer3Sha1 = strZ(bufHex(cryptoHashOne(hashTypeSha1, walBuffer3)));
 
         HRN_STORAGE_PUT(storagePgWrite(), "pg_xlog/000000010000000100000003", walBuffer3);
 
@@ -918,10 +925,10 @@ testRun(void)
             "P01 DETAIL: pushed WAL file '000000010000000100000003' to the archive");
 
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000003-%s", walBuffer3Sha1)),
+            storageTest, zNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000003-%s", walBuffer3Sha1),
             .comment = "check repo1 for WAL 3 file");
         TEST_STORAGE_EXISTS(
-            storageTest, strZ(strNewFmt("repo3/archive/test/9.4-1/0000000100000001/000000010000000100000003-%s", walBuffer3Sha1)),
+            storageTest, zNewFmt("repo3/archive/test/9.4-1/0000000100000001/000000010000000100000003-%s", walBuffer3Sha1),
             .comment = "check repo3 for WAL 3 file");
 
         // Remove the ready file to prevent WAL 3 from being considered for the next test
