@@ -51,7 +51,7 @@ testRun(void)
 
         TEST_ERROR(cmdCheck(), ConfigError, "no database found\nHINT: check indexed pg-path/pg-host configurations");
         TEST_RESULT_LOG(
-            "P00   WARN: unable to check pg-1: [DbConnectError] unable to connect to 'dbname='postgres' port=5432': error");
+            "P00   WARN: unable to check pg1: [DbConnectError] unable to connect to 'dbname='postgres' port=5432': error");
 
         //--------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("standby only, repo local - fail to find primary database");
@@ -367,24 +367,6 @@ testRun(void)
         TEST_RESULT_VOID(checkPrimary(dbGroup), "primary == NULL");
     }
 
-    // *****************************************************************************************************************************
-    if (testBegin("checkManifest()"))
-    {
-        argList = strLstNew();
-        hrnCfgArgRawZ(argList, cfgOptStanza, "test1");
-        hrnCfgArgRawZ(argList, cfgOptPgPath, TEST_PATH "/pg");
-        hrnCfgArgKeyRawZ(argList, cfgOptPgHost, 5, "localhost");
-        hrnCfgArgKeyRawZ(argList, cfgOptPgHostCmd, 5, "pgbackrest-bogus");
-        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 5, "/path/to/pg5");
-        hrnCfgArgKeyRawZ(argList, cfgOptPgHostUser, 5, TEST_USER);
-        HRN_CFG_LOAD(cfgCmdCheck, argList);
-
-        // Placeholder test for manifest
-        TEST_ERROR(
-            checkManifest(), UnknownError,
-            "remote-0 process on 'localhost' terminated unexpectedly [127]: bash: pgbackrest-bogus: command not found");
-    }
-
     // *****************************************************************************************************************************
     if (testBegin("checkDbConfig(), checkArchiveCommand()"))
     {
