@@ -1,7 +1,7 @@
 /***********************************************************************************************************************************
 Test Server Command
 ***********************************************************************************************************************************/
-#include "common/exit.h"
+#include "command/exit.h"
 #include "storage/posix/storage.h"
 #include "storage/remote/storage.h"
 
@@ -130,6 +130,7 @@ testRun(void)
                         StringList *argList = strLstNew();
                         hrnCfgArgRawZ(argList, cfgOptConfig, TEST_PATH "/pgbackrest.conf");
                         hrnCfgArgRawFmt(argList, cfgOptTlsServerPort, "%u", hrnServerPort(0));
+                        hrnCfgArgRawZ(argList, cfgOptLogLevelStderr, CFGOPTVAL_ARCHIVE_MODE_OFF_Z);
                         HRN_CFG_LOAD(cfgCmdServer, argList);
 
                         // Init exit signal handlers
