@@ -54,10 +54,10 @@ testIoWriteOpen(void *driver)
 }
 
 static void
-testIoWrite(void *driver, const Buffer *buffer)
+testIoWrite(void *const driver, const Buffer *const buffer)
 {
     ASSERT(driver == (void *)999);
-    ASSERT(strEq(strNewBuf(buffer), STRDEF("ABC")));
+    ASSERT(strncmp((const char *)bufPtrConst(buffer), "ABC", bufSize(buffer)) == 0);
 }
 
 static bool testIoWriteCloseCalled = false;
@@ -106,7 +106,11 @@ ioTestFilterSizeResult(THIS_VOID)
     pckWriteU64P(packWrite, this->size);
     pckWriteEndP(packWrite);
 
-    return pckWriteResult(packWrite);
+    Pack *const result = pckDup(pckWriteResult(packWrite));
+
+    pckWriteFree(packWrite);
+
+    return result;
 }
 
 static IoFilter *
@@ -114,7 +118,7 @@ ioTestFilterSizeNew(const StringId type)
 {
     IoFilter *this = NULL;
 
-    OBJ_NEW_BEGIN(IoTestFilterSize)
+    OBJ_NEW_BEGIN(IoTestFilterSize, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
     {
         IoTestFilterSize *driver = OBJ_NEW_ALLOC();
         *driver = (IoTestFilterSize){0};
@@ -140,7 +144,7 @@ typedef struct IoTestFilterMultiply
 } IoTestFilterMultiply;
 
 static void
-ioTestFilterMultiplyProcess(THIS_VOID, const Buffer *input, Buffer *output)
+ioTestFilterMultiplyProcess(THIS_VOID, const Buffer *const input, Buffer *const output)
 {
     THIS(IoTestFilterMultiply);
 
@@ -163,7 +167,7 @@ ioTestFilterMultiplyProcess(THIS_VOID, const Buffer *input, Buffer *output)
         else
         {
             char flushZ[] = {this->flushChar, 0};
-            bufCat(output, bufNewC(flushZ, 1));
+            bufCat(output, BUF(flushZ, 1));
             this->flushTotal--;
         }
     }
@@ -171,7 +175,12 @@ ioTestFilterMultiplyProcess(THIS_VOID, const Buffer *input, Buffer *output)
     {
         if (this->multiplyBuffer == NULL)
         {
-            this->multiplyBuffer = bufNew(bufUsed(input) * this->multiplier);
+            MEM_CONTEXT_OBJ_BEGIN(this)
+            {
+                this->multiplyBuffer = bufNew(bufUsed(input) * this->multiplier);
+            }
+            MEM_CONTEXT_OBJ_END();
+
             const unsigned char *inputPtr = bufPtrConst(input);
             unsigned char *bufferPtr = bufPtr(this->multiplyBuffer);
 
@@ -214,7 +223,7 @@ ioTestFilterMultiplyNew(const StringId type, unsigned int multiplier, unsigned i
 {
     IoFilter *this = NULL;
 
-    OBJ_NEW_BEGIN(IoTestFilterMultiply)
+    OBJ_NEW_BEGIN(IoTestFilterMultiply, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
     {
         IoTestFilterMultiply *driver = OBJ_NEW_ALLOC();
 
@@ -413,8 +422,7 @@ testRun(void)
         // Mixed line and buffer read
         // -------------------------------------------------------------------------------------------------------------------------
         ioBufferSizeSet(5);
-        read = ioBufferReadNew(BUFSTRDEF("AAAAAA123\n1234\n\n12\nBDDDEFF"));
-        ioReadOpen(read);
+        read = ioBufferReadNewOpen(BUFSTRDEF("AAAAAA123\n1234\n\n12\nBDDDEFF"));
         buffer = bufNew(6);
 
         // Start with a small read
@@ -459,21 +467,48 @@ testRun(void)
 
         // Error if buffer is full and there is no linefeed
         ioBufferSizeSet(10);
-        read = ioBufferReadNew(BUFSTRDEF("0123456789"));
-        ioReadOpen(read);
+        read = ioBufferReadNewOpen(BUFSTRDEF("0123456789"));
         TEST_ERROR(ioReadLine(read), FileReadError, "unable to find line in 10 byte buffer");
 
         // Read line without eof
-        read = ioBufferReadNew(BUFSTRDEF("1234"));
-        ioReadOpen(read);
+        read = ioBufferReadNewOpen(BUFSTRDEF("1234"));
         TEST_RESULT_STR_Z(ioReadLineParam(read, true), "1234", "read line without eof");
 
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("ioCopyP()");
+
+        ioBufferSizeSet(4);
+
+        bufferRead = ioBufferReadNewOpen(BUFSTRDEF("a test string"));
+
+        buffer = bufNew(0);
+        IoWrite *bufferWrite = ioBufferWriteNewOpen(buffer);
+
+        TEST_RESULT_VOID(ioCopyP(bufferRead, bufferWrite), "copy buffer");
+        TEST_RESULT_VOID(ioWriteClose(bufferWrite), "close write");
+
+        TEST_RESULT_STR_Z(strNewBuf(buffer), "a test string", "check buffer");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("ioCopyP() with limit");
+
+        ioBufferSizeSet(4);
+
+        bufferRead = ioBufferReadNewOpen(BUFSTRDEF("a test string"));
+
+        buffer = bufNew(0);
+        bufferWrite = ioBufferWriteNewOpen(buffer);
+
+        TEST_RESULT_VOID(ioCopyP(bufferRead, bufferWrite, .limit = VARUINT64(6)), "copy buffer");
+        TEST_RESULT_VOID(ioWriteClose(bufferWrite), "close write");
+
+        TEST_RESULT_STR_Z(strNewBuf(buffer), "a test", "check buffer");
+
         // Read IO into a buffer
         // -------------------------------------------------------------------------------------------------------------------------
         ioBufferSizeSet(8);
 
-        bufferRead = ioBufferReadNew(BUFSTRDEF("a test string"));
-        ioReadOpen(bufferRead);
+        bufferRead = ioBufferReadNewOpen(BUFSTRDEF("a test string"));
 
         TEST_RESULT_STR_Z(strNewBuf(ioReadBuf(bufferRead)), "a test string", "read into buffer");
 
