@@ -13,14 +13,11 @@ testFree(void *thisVoid)
 {
     MemContext *this = thisVoid;
 
-    TEST_RESULT_BOOL(memContextFreeing(this), true, "state should be freeing before memContextFree() in callback");
-    memContextFree(this);
-    TEST_RESULT_INT(this->state, memContextStateFreeing, "state should still be freeing after memContextFree() in callback");
+    TEST_RESULT_BOOL(this->active, false, "mem context inactive in callback");
 
-    TEST_ERROR(memContextCallbackSet(this, testFree, this), AssertError, "cannot assign callback to inactive context");
-
-    TEST_ERROR(memContextSwitch(this), AssertError, "cannot switch to inactive context");
-    TEST_ERROR(memContextName(this), AssertError, "cannot get name for inactive context");
+    TEST_ERROR(memContextFree(this), AssertError, "assertion 'this->active' failed");
+    TEST_ERROR(memContextCallbackSet(this, testFree, this), AssertError, "assertion 'this->active' failed");
+    TEST_ERROR(memContextSwitch(this), AssertError, "assertion 'this->active' failed");
 
     memContextCallbackArgument = this;
 
@@ -76,13 +73,15 @@ testRun(void)
     {
         TEST_TITLE("struct size");
 
-        TEST_RESULT_UINT(sizeof(MemContext), TEST_64BIT() ? 72 : 48, "MemContext size");
+        TEST_RESULT_UINT(sizeof(MemContext), TEST_64BIT() ? 24 : 16, "MemContext size");
+        TEST_RESULT_UINT(sizeof(MemContextChildMany), TEST_64BIT() ? 16 : 12, "MemContextChildMany size");
+        TEST_RESULT_UINT(sizeof(MemContextAllocMany), TEST_64BIT() ? 16 : 12, "MemContextAllocMany size");
+        TEST_RESULT_UINT(sizeof(MemContextCallbackOne), TEST_64BIT() ? 16 : 8, "MemContextCallbackOne size");
 
         // -------------------------------------------------------------------------------------------------------------------------
         // Make sure top context was created
-        TEST_RESULT_Z(memContextName(memContextTop()), "TOP", "top context should exist");
-        TEST_RESULT_INT(memContextTop()->contextChildListSize, 0, "top context should init with zero children");
-        TEST_RESULT_PTR(memContextTop()->contextChildList, NULL, "top context child list empty");
+        TEST_RESULT_Z(memContextTop()->name, "TOP", "top context should exist");
+        TEST_RESULT_BOOL(memContextTop()->childInitialized, false, "top context should init with zero children");
 
         // Current context should equal top context
         TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "top context == current context");
@@ -90,11 +89,13 @@ testRun(void)
         // Context name length errors
         TEST_ERROR(memContextNewP(""), AssertError, "assertion 'name[0] != '\\0'' failed");
 
-        MemContext *memContext = memContextNewP("test1");
+        MemContext *memContext = memContextNewP(
+            "test1", .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1);
         memContextKeep();
-        TEST_RESULT_Z(memContextName(memContext), "test1", "test1 context name");
+        TEST_RESULT_Z(memContext->name, "test1", "test1 context name");
         TEST_RESULT_PTR(memContext->contextParent, memContextTop(), "test1 context parent is top");
-        TEST_RESULT_INT(memContextTop()->contextChildListSize, MEM_CONTEXT_INITIAL_SIZE, "initial top context child list size");
+        TEST_RESULT_INT(
+            memContextChildMany(memContextTop())->listSize, MEM_CONTEXT_INITIAL_SIZE, "initial top context child list size");
 
         memContextSwitch(memContext);
 
@@ -103,71 +104,92 @@ testRun(void)
         TEST_RESULT_PTR(memContext, memContextCurrent(), "current context is now test1");
 
         // Create enough mem contexts to use up the initially allocated block
+        memContextSwitch(memContextTop());
+
         for (int contextIdx = 1; contextIdx < MEM_CONTEXT_INITIAL_SIZE; contextIdx++)
         {
-            memContextSwitch(memContextTop());
-            memContextNewP("test-filler");
+            memContextNewP("test-filler", .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1);
             memContextKeep();
-            TEST_RESULT_BOOL(
-                memContextTop()->contextChildList[contextIdx]->state == memContextStateActive, true, "new context is active");
-            TEST_RESULT_Z(memContextName(memContextTop()->contextChildList[contextIdx]), "test-filler", "new context name");
+            TEST_RESULT_PTR_NE(memContextChildMany(memContextTop())->list[contextIdx], NULL, "new context exists");
+            TEST_RESULT_BOOL(memContextChildMany(memContextTop())->list[contextIdx]->active, true, "new context is active");
+            TEST_RESULT_Z(memContextChildMany(memContextTop())->list[contextIdx]->name, "test-filler", "new context name");
         }
 
         // This forces the child context array to grow
-        memContext = memContextNewP("test5", .allocExtra = 16);
+        memContext = memContextNewP(
+            "test5", .allocExtra = 16, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1);
         TEST_RESULT_PTR(memContextAllocExtra(memContext), memContext + 1, "mem context alloc extra");
         TEST_RESULT_PTR(memContextFromAllocExtra(memContext + 1), memContext, "mem context from alloc extra");
         TEST_RESULT_PTR(memContextConstFromAllocExtra(memContext + 1), memContext, "const mem context from alloc extra");
         memContextKeep();
-        TEST_RESULT_INT(memContextTop()->contextChildListSize, MEM_CONTEXT_INITIAL_SIZE * 2, "increased child context list size");
-        TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, MEM_CONTEXT_INITIAL_SIZE + 1, "check context free idx");
+        TEST_RESULT_INT(
+            memContextChildMany(memContextTop())->listSize, MEM_CONTEXT_INITIAL_SIZE * 2, "increased child context list size");
+        TEST_RESULT_UINT(memContextChildMany(memContextTop())->freeIdx, MEM_CONTEXT_INITIAL_SIZE + 1, "check context free idx");
 
         // Free a context
-        memContextFree(memContextTop()->contextChildList[1]);
-        TEST_RESULT_PTR(memContextTop()->contextChildList[1], NULL, "child context NULL after free");
+        memContextFree(memContextChildMany(memContextTop())->list[1]);
+        TEST_RESULT_PTR(memContextChildMany(memContextTop())->list[1], NULL, "child context NULL after free");
         TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "current context is top");
-        TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, 1, "check context free idx");
+        TEST_RESULT_UINT(memContextChildMany(memContextTop())->freeIdx, 1, "check context free idx");
 
         // Create a new context and it should end up in the same spot
-        memContextNewP("test-reuse");
+        MemContext *const memContextOneChild = memContextNewP("test-reuse", .childQty = 1);
         memContextKeep();
         TEST_RESULT_BOOL(
-            memContextTop()->contextChildList[1]->state == memContextStateActive,
-            true, "new context in same index as freed context is active");
-        TEST_RESULT_Z(memContextName(memContextTop()->contextChildList[1]), "test-reuse", "new context name");
-        TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, 2, "check context free idx");
+            memContextChildMany(memContextTop())->list[1]->active, true, "new context in same index as freed context is active");
+        TEST_RESULT_Z(memContextChildMany(memContextTop())->list[1]->name, "test-reuse", "new context name");
+        TEST_RESULT_UINT(memContextChildMany(memContextTop())->freeIdx, 2, "check context free idx");
 
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("child context in context with only one child allowed");
+
+        TEST_RESULT_VOID(memContextSwitch(memContextOneChild), "switch");
+        MemContext *memContextSingleChild;
+        TEST_ASSIGN(memContextSingleChild, memContextNewP("single-child", .childQty = 1, .allocQty = 1), "new");
+        TEST_RESULT_VOID(memContextKeep(), "keep");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("child context freed in context with only one child allowed");
+
+        TEST_RESULT_VOID(memContextSwitch(memContextSingleChild), "switch");
+        TEST_RESULT_VOID(memContextNewP("free-child", .childQty = 1), "new");
+        TEST_RESULT_VOID(memContextDiscard(), "discard");
+
+        TEST_RESULT_VOID(memContextSwitch(memContextTop()), "switch to top");
+
+        // -------------------------------------------------------------------------------------------------------------------------
         // Next context will be at the end
         memContextNewP("test-at-end");
         memContextKeep();
-        TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, MEM_CONTEXT_INITIAL_SIZE + 2, "check context free idx");
+        TEST_RESULT_UINT(memContextChildMany(memContextTop())->freeIdx, MEM_CONTEXT_INITIAL_SIZE + 2, "check context free idx");
 
         // Create a child context to test recursive free
-        memContextSwitch(memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]);
-        memContextNewP("test-reuse");
+        memContextSwitch(memContextChildMany(memContextTop())->list[MEM_CONTEXT_INITIAL_SIZE]);
+        memContextNewP("test-reuse", .childQty = 1);
         memContextKeep();
         TEST_RESULT_PTR_NE(
-            memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]->contextChildList, NULL, "context child list is allocated");
+            memContextChildMany(memContextChildMany(memContextTop())->list[MEM_CONTEXT_INITIAL_SIZE])->list, NULL,
+            "context child list is allocated");
         TEST_RESULT_INT(
-            memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]->contextChildListSize, MEM_CONTEXT_INITIAL_SIZE,
-            "context child list initial size");
+            memContextChildMany(memContextChildMany(memContextTop())->list[MEM_CONTEXT_INITIAL_SIZE])->listSize,
+            MEM_CONTEXT_INITIAL_SIZE, "context child list initial size");
 
         // This test will change if the contexts above change
-        TEST_RESULT_UINT(memContextSize(memContextTop()), TEST_64BIT() ? 688 : 448, "check size");
+        TEST_RESULT_UINT(memContextSize(memContextTop()), TEST_64BIT() ? 584 : 376, "check size");
 
         TEST_ERROR(
-            memContextFree(memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]),
-            AssertError, "cannot free current context 'test5'");
+            memContextFree(memContextChildMany(memContextTop())->list[MEM_CONTEXT_INITIAL_SIZE]), AssertError,
+            "cannot free current context 'test5'");
 
-        memContextSwitch(memContextTop());
-        // memContextFree(memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]);
-        memContextFree(memContextTop());
+        TEST_RESULT_VOID(memContextSwitch(memContextTop()), "switch to top");
+        TEST_RESULT_VOID(memContextFree(memContextTop()), "free top");
 
-        MemContext *noAllocation = memContextNewP("empty");
-        memContextKeep();
-        noAllocation->allocListSize = 0;
-        free(noAllocation->allocList);
-        TEST_RESULT_VOID(memContextFree(noAllocation), "free context with no allocations");
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("alloc extra not aligned");
+
+        TEST_ASSIGN(memContext, memContextNewP("test-alloc", .allocExtra = 7), "no aligned");
+        TEST_RESULT_UINT(memContext->allocExtra, 8, "check");
+        TEST_RESULT_VOID(memContextDiscard(), "discard");
     }
 
     // *****************************************************************************************************************************
@@ -185,7 +207,7 @@ testRun(void)
         memContextSwitch(memContextTop());
         memNewPtrArray(1);
 
-        MemContext *memContext = memContextNewP("test-alloc");
+        MemContext *memContext = memContextNewP("test-alloc", .allocQty = MEM_CONTEXT_QTY_MAX);
         TEST_ERROR(memContextSwitchBack(), AssertError, "current context expected but new context 'test-alloc' found");
         memContextKeep();
         memContextSwitch(memContext);
@@ -195,7 +217,7 @@ testRun(void)
             memNew(sizeof(size_t));
 
             TEST_RESULT_INT(
-                memContextCurrent()->allocListSize,
+                memContextAllocMany(memContextCurrent())->listSize,
                 allocIdx == MEM_CONTEXT_ALLOC_INITIAL_SIZE ? MEM_CONTEXT_ALLOC_INITIAL_SIZE * 2 : MEM_CONTEXT_ALLOC_INITIAL_SIZE,
                 "allocation list size");
         }
@@ -215,46 +237,83 @@ testRun(void)
         TEST_RESULT_INT(expectedTotal, sizeof(size_t), "all bytes are 0xFE in original portion");
 
         // Free memory
-        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, MEM_CONTEXT_ALLOC_INITIAL_SIZE + 2, "check alloc free idx");
-        TEST_RESULT_VOID(memFree(MEM_CONTEXT_ALLOC_BUFFER(memContextCurrent()->allocList[0])), "free allocation");
-        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 0, "check alloc free idx");
+        TEST_RESULT_UINT(
+            memContextAllocMany(memContextCurrent())->freeIdx, MEM_CONTEXT_ALLOC_INITIAL_SIZE + 2, "check alloc free idx");
+        TEST_RESULT_VOID(memFree(MEM_CONTEXT_ALLOC_BUFFER(memContextAllocMany(memContextCurrent())->list[0])), "free allocation");
+        TEST_RESULT_UINT(memContextAllocMany(memContextCurrent())->freeIdx, 0, "check alloc free idx");
 
-        TEST_RESULT_VOID(memFree(MEM_CONTEXT_ALLOC_BUFFER(memContextCurrent()->allocList[1])), "free allocation");
-        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 0, "check alloc free idx");
+        TEST_RESULT_VOID(memFree(MEM_CONTEXT_ALLOC_BUFFER(memContextAllocMany(memContextCurrent())->list[1])), "free allocation");
+        TEST_RESULT_UINT(memContextAllocMany(memContextCurrent())->freeIdx, 0, "check alloc free idx");
 
         TEST_RESULT_VOID(memNew(3), "new allocation");
-        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 1, "check alloc free idx");
+        TEST_RESULT_UINT(memContextAllocMany(memContextCurrent())->freeIdx, 1, "check alloc free idx");
 
         TEST_RESULT_VOID(memNew(3), "new allocation");
-        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 2, "check alloc free idx");
+        TEST_RESULT_UINT(memContextAllocMany(memContextCurrent())->freeIdx, 2, "check alloc free idx");
 
         TEST_RESULT_VOID(memNew(3), "new allocation");
-        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, MEM_CONTEXT_ALLOC_INITIAL_SIZE + 3, "check alloc free idx");
+        TEST_RESULT_UINT(
+            memContextAllocMany(memContextCurrent())->freeIdx, MEM_CONTEXT_ALLOC_INITIAL_SIZE + 3, "check alloc free idx");
 
         // This test will change if the allocations above change
-        TEST_RESULT_UINT(memContextSize(memContextCurrent()), TEST_64BIT() ? 241 : 165, "check size");
+        TEST_RESULT_UINT(memContextSize(memContextCurrent()), TEST_64BIT() ? 209 : 145, "check size");
 
         TEST_ERROR(
             memFree(NULL), AssertError,
-            "assertion '((MemContextAlloc *)buffer - 1) != NULL"
-                " && (uintptr_t)((MemContextAlloc *)buffer - 1) != (uintptr_t)-sizeof(MemContextAlloc)"
-                " && ((MemContextAlloc *)buffer - 1)->allocIdx <"
-                " memContextStack[memContextCurrentStackIdx].memContext->allocListSize"
-                " && memContextStack[memContextCurrentStackIdx].memContext->allocList[((MemContextAlloc *)buffer - 1)->allocIdx]'"
-                " failed");
+            "assertion 'alloc != NULL && "
+                "(uintptr_t)alloc != (uintptr_t)-sizeof(MemContextAlloc) && "
+                "alloc->allocIdx < memContextAllocMany(memContextStack[memContextCurrentStackIdx].memContext)->listSize && "
+                "memContextAllocMany(memContextStack[memContextCurrentStackIdx].memContext)->list[alloc->allocIdx]' failed");
         memFree(buffer);
 
         memContextSwitch(memContextTop());
         memContextFree(memContext);
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("mem context with one allocation");
+
+        TEST_ASSIGN(memContext, memContextNewP("test-alloc", .allocQty = 1), "new");
+        TEST_RESULT_VOID(memContextKeep(), "keep new");
+        TEST_RESULT_VOID(memContextSwitch(memContext), "switch to new");
+
+        TEST_ASSIGN(buffer, memNew(100), "new");
+        TEST_ASSIGN(buffer, memResize(buffer, 150), "resize");
+        TEST_RESULT_VOID(memFree(buffer), "free");
+
+        TEST_ASSIGN(buffer, memNew(200), "new");
+
+        // This test will change if the allocations above change
+        TEST_RESULT_UINT(memContextSize(memContextCurrent()), TEST_64BIT() ? 240 : 228, "check size");
+
+        TEST_RESULT_VOID(memContextSwitch(memContextTop()), "switch to top");
+        TEST_RESULT_VOID(memContextFree(memContext), "context free");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("mem context with one allocation freed before context free");
+
+        TEST_ASSIGN(memContext, memContextNewP("test-alloc", .allocQty = 1), "new");
+        TEST_RESULT_VOID(memContextKeep(), "keep new");
+        TEST_RESULT_VOID(memContextSwitch(memContext), "switch to new");
+
+        TEST_ASSIGN(buffer, memNew(100), "new");
+        TEST_RESULT_VOID(memFree(buffer), "free");
+
+        // This test will change if the allocations above change
+        TEST_RESULT_UINT(memContextSize(memContextCurrent()), TEST_64BIT() ? 32 : 20, "check size");
+
+        TEST_RESULT_VOID(memContextSwitch(memContextTop()), "switch to top");
+        TEST_RESULT_VOID(memContextFree(memContext), "context free");
     }
 
     // *****************************************************************************************************************************
     if (testBegin("memContextCallbackSet()"))
     {
         TEST_ERROR(
-            memContextCallbackSet(memContextTop(), testFree, NULL), AssertError, "top context may not have a callback");
+            memContextCallbackSet(memContextTop(), testFree, NULL), AssertError,
+            "assertion 'this->callbackQty != memQtyNone' failed");
 
-        MemContext *memContext = memContextNewP("test-callback");
+        MemContext *memContext = memContextNewP(
+            "test-callback", .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1);
         memContextKeep();
         memContextCallbackSet(memContext, testFree, memContext);
         TEST_ERROR(
@@ -270,7 +329,8 @@ testRun(void)
 
         // Now test with an error
         // -------------------------------------------------------------------------------------------------------------------------
-        memContext = memContextNewP("test-callback-error");
+        memContext = memContextNewP(
+            "test-callback-error", .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1);
         TEST_RESULT_VOID(memContextKeep(), "keep mem context");
         testFreeThrow = true;
         TEST_RESULT_VOID(memContextCallbackSet(memContext, testFree, memContext), "    set callback");
@@ -281,40 +341,41 @@ testRun(void)
     if (testBegin("MEM_CONTEXT_BEGIN() and MEM_CONTEXT_END()"))
     {
         memContextSwitch(memContextTop());
-        MemContext *memContext = memContextNewP("test-block");
+        MemContext *memContext = memContextNewP(
+            "test-block", .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1);
         memContextKeep();
 
         // Check normal block
         MEM_CONTEXT_BEGIN(memContext)
         {
-            TEST_RESULT_Z(memContextName(memContextCurrent()), "test-block", "context is now test-block");
+            TEST_RESULT_Z(memContextCurrent()->name, "test-block", "context is now test-block");
         }
         MEM_CONTEXT_END();
 
-        TEST_RESULT_Z(memContextName(memContextCurrent()), "TOP", "context is now top");
+        TEST_RESULT_Z(memContextCurrent()->name, "TOP", "context is now top");
 
         // Check block that errors
         TEST_ERROR(
             MEM_CONTEXT_BEGIN(memContext)
             {
-                TEST_RESULT_Z(memContextName(memContextCurrent()), "test-block", "context is now test-block");
+                TEST_RESULT_Z(memContextCurrent()->name, "test-block", "context is now test-block");
                 THROW(AssertError, "error in test block");
             }
             MEM_CONTEXT_END(),
             AssertError, "error in test block");
 
-        TEST_RESULT_Z(memContextName(memContextCurrent()), "TOP", "context is now top");
+        TEST_RESULT_Z(memContextCurrent()->name, "TOP", "context is now top");
 
         // Reset temp mem context after a single interaction
         // -------------------------------------------------------------------------------------------------------------------------
         MEM_CONTEXT_TEMP_RESET_BEGIN()
         {
-            TEST_RESULT_PTR(MEM_CONTEXT_TEMP()->allocList, NULL, "nothing allocated");
+            TEST_RESULT_BOOL(MEM_CONTEXT_TEMP()->allocInitialized, false, "nothing allocated");
             memNew(99);
-            TEST_RESULT_PTR_NE(MEM_CONTEXT_TEMP()->allocList[0], NULL, "1 allocation");
+            TEST_RESULT_PTR_NE(memContextAllocMany(MEM_CONTEXT_TEMP())->list[0], NULL, "1 allocation");
 
             MEM_CONTEXT_TEMP_RESET(1);
-            TEST_RESULT_PTR(MEM_CONTEXT_TEMP()->allocList, NULL, "nothing allocated");
+            TEST_RESULT_BOOL(MEM_CONTEXT_TEMP()->allocInitialized, false, "nothing allocated");
         }
         MEM_CONTEXT_TEMP_END();
     }
@@ -329,41 +390,41 @@ testRun(void)
 
         TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "context is now 'TOP'");
 
-        MEM_CONTEXT_NEW_BEGIN(memContextTestName)
+        MEM_CONTEXT_NEW_BEGIN(test-new-block, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
         {
             memContext = MEM_CONTEXT_NEW();
             TEST_RESULT_PTR(memContext, memContextCurrent(), "new mem context is current");
-            TEST_RESULT_Z(memContextName(memContext), memContextTestName, "check context name");
+            TEST_RESULT_Z(memContext->name, memContextTestName, "check context name");
         }
         MEM_CONTEXT_NEW_END();
 
-        TEST_RESULT_Z(memContextName(memContextCurrent()), "TOP", "context name is now 'TOP'");
+        TEST_RESULT_Z(memContextCurrent()->name, "TOP", "context name is now 'TOP'");
         TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "context is now 'TOP'");
-        TEST_RESULT_BOOL(memContext->state == memContextStateActive, true, "new mem context is still active");
+        TEST_RESULT_BOOL(memContext->active, true, "new mem context is still active");
         memContextFree(memContext);
 
         // ------------------------------------------------------------------------------------------------------------------------
         // Failed context new block
         memContextTestName = "test-new-failed-block";
-        bool bCatch = false;
+        bool catch = false;
 
         TRY_BEGIN()
         {
-            MEM_CONTEXT_NEW_BEGIN(memContextTestName)
+            MEM_CONTEXT_NEW_BEGIN(test-new-failed-block, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
             {
                 memContext = MEM_CONTEXT_NEW();
-                TEST_RESULT_Z(memContextName(memContext), memContextTestName, "check mem context name");
+                TEST_RESULT_Z(memContext->name, memContextTestName, "check mem context name");
                 THROW(AssertError, "create failed");
             }
             MEM_CONTEXT_NEW_END();
         }
         CATCH(AssertError)
         {
-            bCatch = true;
+            catch = true;
         }
         TRY_END();
 
-        TEST_RESULT_BOOL(bCatch, true, "new context error was caught");
+        TEST_RESULT_BOOL(catch, true, "new context error was caught");
         TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "context is now 'TOP'");
     }
 
@@ -378,36 +439,36 @@ testRun(void)
         void *mem = NULL;
         void *mem2 = NULL;
 
-        MEM_CONTEXT_NEW_BEGIN("outer")
+        MEM_CONTEXT_NEW_BEGIN("outer", .childQty = MEM_CONTEXT_QTY_MAX)
         {
             MEM_CONTEXT_TEMP_BEGIN()
             {
-                memContextNewP("not-to-be-moved");
+                memContextNewP("not-to-be-moved", .childQty = MEM_CONTEXT_QTY_MAX);
                 memContextKeep();
 
-                MEM_CONTEXT_NEW_BEGIN("inner")
+                MEM_CONTEXT_NEW_BEGIN("inner", .allocQty = MEM_CONTEXT_QTY_MAX)
                 {
                     memContext = MEM_CONTEXT_NEW();
                     mem = memNew(sizeof(int));
                 }
                 MEM_CONTEXT_NEW_END();
 
-                TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER(memContext->allocList[0]), mem, "check memory allocation");
-                TEST_RESULT_PTR(memContextCurrent()->contextChildList[1], memContext, "check memory context");
+                TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER(memContextAllocMany(memContext)->list[0]), mem, "check memory allocation");
+                TEST_RESULT_PTR(memContextChildMany(memContextCurrent())->list[1], memContext, "check memory context");
 
                 // Null out the mem context in the parent so the move will fail
-                memContextCurrent()->contextChildList[1] = NULL;
+                memContextChildMany(memContextCurrent())->list[1] = NULL;
                 TEST_ERROR(
                     memContextMove(memContext, memContextPrior()), AssertError,
-                    "assertion 'this->contextParent->contextChildList[this->contextParentIdx] == this' failed");
+                    "assertion 'memContextChildMany(this->contextParent)->list[this->contextParentIdx] == this' failed");
 
                 // Set it back so the move will succeed
-                memContextCurrent()->contextChildList[1] = memContext;
+                memContextChildMany(memContextCurrent())->list[1] = memContext;
                 TEST_RESULT_VOID(memContextMove(memContext, memContextPrior()), "move context");
                 TEST_RESULT_VOID(memContextMove(memContext, memContextPrior()), "move context again");
 
                 // Move another context
-                MEM_CONTEXT_NEW_BEGIN("inner2")
+                MEM_CONTEXT_NEW_BEGIN("inner2", .allocQty = MEM_CONTEXT_QTY_MAX)
                 {
                     memContext2 = MEM_CONTEXT_NEW();
                     mem2 = memNew(sizeof(int));
@@ -418,13 +479,36 @@ testRun(void)
             }
             MEM_CONTEXT_TEMP_END();
 
-            TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER(memContext->allocList[0]), mem, "check memory allocation");
-            TEST_RESULT_PTR(memContextCurrent()->contextChildList[1], memContext, "check memory context");
+            TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER(memContextAllocMany(memContext)->list[0]), mem, "check memory allocation");
+            TEST_RESULT_PTR(memContextChildMany(memContextCurrent())->list[1], memContext, "check memory context");
 
-            TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER(memContext2->allocList[0]), mem2, "check memory allocation 2");
-            TEST_RESULT_PTR(memContextCurrent()->contextChildList[2], memContext2, "check memory context 2");
+            TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER(memContextAllocMany(memContext2)->list[0]), mem2, "check memory allocation 2");
+            TEST_RESULT_PTR(memContextChildMany(memContextCurrent())->list[2], memContext2, "check memory context 2");
         }
         MEM_CONTEXT_NEW_END();
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("outer and inner contexts allow one child");
+
+        MemContext *memContextParent1;
+        TEST_ASSIGN(memContextParent1, memContextNewP("parent1", .childQty = 1), "new parent1");
+        TEST_RESULT_VOID(memContextKeep(), "keep parent1");
+
+        TEST_RESULT_VOID(memContextSwitch(memContextParent1), "switch to parent1");
+
+        MemContext *memContextChild;
+        TEST_ASSIGN(memContextChild, memContextNewP("child", .allocQty = 0), "new child");
+        TEST_RESULT_VOID(memContextKeep(), "keep child");
+
+        TEST_RESULT_VOID(memContextSwitch(memContextTop()), "switch to top");
+
+        MemContext *memContextParent2;
+        TEST_ASSIGN(memContextParent2, memContextNewP("parent2", .childQty = 1), "new parent2");
+        TEST_RESULT_VOID(memContextKeep(), "keep parent2");
+
+        TEST_RESULT_VOID(memContextMove(memContextChild, memContextParent2), "move");
+        TEST_RESULT_PTR(memContextChildOne(memContextParent1)->context, NULL, "check parent1");
+        TEST_RESULT_PTR(memContextChildOne(memContextParent2)->context, memContextChild, "check parent2");
     }
 
     memContextFree(memContextTop());
