@@ -26,14 +26,11 @@ testRun(void)
         TEST_TITLE("write pack");
 
         Buffer *pack = bufNew(0);
-
-        IoWrite *write = ioBufferWriteNew(pack);
-        ioWriteOpen(write);
+        IoWrite *write = ioBufferWriteNewOpen(pack);
+        PackWrite *packWrite = NULL;
 
         ioBufferSizeSet(3);
 
-        PackWrite *packWrite = NULL;
-
         MEM_CONTEXT_TEMP_BEGIN()
         {
             TEST_ASSIGN(packWrite, pckWriteMove(pckWriteNewIo(write), memContextPrior()), "move new write");
@@ -227,8 +224,7 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("read pack");
 
-        IoRead *read = ioBufferReadNew(pack);
-        ioReadOpen(read);
+        IoRead *read = ioBufferReadNewOpen(pack);
 
         PackRead *packRead = NULL;
 
@@ -277,7 +273,7 @@ testRun(void)
 
         while (pckReadNext(packRead))
         {
-            TEST_RESULT_UINT(pckReadU64P(packRead, .id = pckReadId(packRead)), value, strZ(strNewFmt("read %u", value)));
+            TEST_RESULT_UINT(pckReadU64P(packRead, .id = pckReadId(packRead)), value, zNewFmt("read %u", value));
             value++;
         }
 
