@@ -41,7 +41,8 @@ testRun(void)
         TEST_RESULT_VOID(CHECK_SIZE(555), "valid size");
         TEST_ERROR(CHECK_SIZE(STRING_SIZE_MAX + 1), AssertError, "string size must be <= 1073741824 bytes");
 
-        String *string = strNewZ("static string");
+        String *string = NULL;
+        TEST_ASSIGN(string, strNewZ("static string"), "new");
         TEST_RESULT_BOOL(string->pub.buffer == (char *)(string + 1), true, "string has fixed buffer");
         TEST_RESULT_STR_Z(string, "static string", "new with static string");
         TEST_RESULT_UINT(strSize(string), 13, "check size");
@@ -49,6 +50,8 @@ testRun(void)
         TEST_RESULT_UINT(strlen(strZ(string)), 13, "check size with strlen()");
         TEST_RESULT_INT(strZNull(string)[2], 'a', "check character");
 
+        TEST_RESULT_UINT(strSize(strNewBuf(bufNew(0))), 0, "new string from empty buffer");
+
         TEST_RESULT_VOID(strFree(string), "free string");
 
         // -------------------------------------------------------------------------------------------------------------------------
@@ -87,6 +90,15 @@ testRun(void)
         TEST_TITLE("strNewEncode()");
 
         TEST_RESULT_STR_Z(strNewEncode(encodeBase64, BUFSTRDEF("zz")), "eno=", "encode base64");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("fixed string large enough to need separate allocation");
+
+        char *compare = memNew(MEM_CONTEXT_ALLOC_EXTRA_MAX + 1);
+        memset(compare, 'A', MEM_CONTEXT_ALLOC_EXTRA_MAX);
+        compare[MEM_CONTEXT_ALLOC_EXTRA_MAX] = '\0';
+
+        TEST_RESULT_STR_Z(strNewZN(compare, MEM_CONTEXT_ALLOC_EXTRA_MAX), compare, "compare");
     }
 
     // *****************************************************************************************************************************
@@ -241,8 +253,15 @@ testRun(void)
     }
 
     // *****************************************************************************************************************************
-    if (testBegin("strReplaceChr()"))
+    if (testBegin("strReplace() and strReplaceChr()"))
     {
+        TEST_RESULT_STR_Z(strReplace(strNewZ(""), STRDEF("A"), STRDEF("B")), "", "replace none");
+        TEST_RESULT_STR_Z(strReplace(strCatZ(strNew(), "ABC"), STRDEF("ABC"), STRDEF("DEF")), "DEF", "replace all");
+        TEST_RESULT_STR_Z(strReplace(strCatZ(strNew(), "ABCXABC"), STRDEF("ABC"), STRDEF("DEF")), "DEFXDEF", "replace multiple");
+        TEST_RESULT_STR_Z(strReplace(strCatZ(strNew(), "XABCX"), STRDEF("ABC"), STRDEF("DEFGHI")), "XDEFGHIX", "replace larger");
+        TEST_RESULT_STR_Z(
+            strReplace(strCatZ(strNew(), "XABCXABCX"), STRDEF("ABC"), STRDEF("ABCD")), "XABCDXABCDX", "replace common substring");
+
         TEST_RESULT_STR_Z(strReplaceChr(strNewZ("ABCD"), 'B', 'R'), "ARCD", "replace chr");
     }
 
@@ -269,7 +288,7 @@ testRun(void)
     }
 
     // *****************************************************************************************************************************
-    if (testBegin("strChr() and strTrunc()"))
+    if (testBegin("strChr() and strTrunc*()"))
     {
         TEST_RESULT_INT(strChr(STRDEF("abcd"), 'c'), 2, "c found");
         TEST_RESULT_INT(strChr(STRDEF("abcd"), 'C'), -1, "capital C not found");
@@ -278,17 +297,17 @@ testRun(void)
 
         String *val = strCatZ(strNew(), "abcdef");
         TEST_ERROR(
-            strTrunc(val, (int)(strSize(val) + 1)), AssertError,
+            strTruncIdx(val, (int)(strSize(val) + 1)), AssertError,
             "assertion 'idx >= 0 && (size_t)idx <= strSize(this)' failed");
-        TEST_ERROR(strTrunc(val, -1), AssertError, "assertion 'idx >= 0 && (size_t)idx <= strSize(this)' failed");
+        TEST_ERROR(strTruncIdx(val, -1), AssertError, "assertion 'idx >= 0 && (size_t)idx <= strSize(this)' failed");
 
-        TEST_RESULT_STR_Z(strTrunc(val, strChr(val, 'd')), "abc", "simple string truncated");
+        TEST_RESULT_STR_Z(strTruncIdx(val, strChr(val, 'd')), "abc", "simple string truncated");
         strCatZ(val, "\r\n to end");
-        TEST_RESULT_STR_Z(strTrunc(val, strChr(val, 'n')), "abc\r\n to e", "complex string truncated");
-        TEST_RESULT_STR_Z(strTrunc(val, strChr(val, 'a')), "", "complete string truncated - empty string");
+        TEST_RESULT_STR_Z(strTruncIdx(val, strChr(val, 'n')), "abc\r\n to e", "complex string truncated");
+        TEST_RESULT_STR_Z(strTruncIdx(val, strChr(val, 'a')), "", "complete string truncated - empty string");
 
         TEST_RESULT_UINT(strSize(val), 0, "0 size");
-        TEST_RESULT_STR_Z(strTrunc(val, 0), "", "test coverage of empty string - no error thrown for index 0");
+        TEST_RESULT_STR_Z(strTrunc(val), "", "test coverage of empty string - no error thrown for index 0");
     }
 
     // *****************************************************************************************************************************
@@ -339,8 +358,7 @@ testRun(void)
                 else
                 {
                     TEST_RESULT_STR(
-                        strLstAdd(list, strNewFmt("STR%02d", listIdx)), strNewFmt("STR%02d", listIdx),
-                        strZ(strNewFmt("add item %d", listIdx)));
+                        strLstAddFmt(list, "STR%02d", listIdx), strNewFmt("STR%02d", listIdx), zNewFmt("add item %d", listIdx));
                 }
             }
 
@@ -355,11 +373,9 @@ testRun(void)
         for (unsigned int listIdx = 0; listIdx < strLstSize(list); listIdx++)
         {
             if (listIdx == 0)
-            {
                 TEST_RESULT_STR(strLstGet(list, listIdx), NULL, "check null item");
-            }
             else
-                TEST_RESULT_STR(strLstGet(list, listIdx), strNewFmt("STR%02u", listIdx), strZ(strNewFmt("check item %u", listIdx)));
+                TEST_RESULT_STR(strLstGet(list, listIdx), strNewFmt("STR%02u", listIdx), zNewFmt("check item %u", listIdx));
         }
 
         TEST_RESULT_VOID(strLstFree(list), "free string list");
@@ -416,7 +432,7 @@ testRun(void)
             if (listIdx == 0)
                 strLstAdd(list, NULL);
             else
-                strLstAdd(list, strNewFmt("STR%02d", listIdx));
+                strLstAddFmt(list, "STR%02d", listIdx);
         }
 
         // Check pointer
@@ -426,13 +442,9 @@ testRun(void)
         for (unsigned int listIdx = 0; listIdx < strLstSize(list); listIdx++)
         {
             if (listIdx == 0)
-            {
                 TEST_RESULT_PTR(szList[listIdx], NULL, "check null item");
-            }
             else
-            {
-                TEST_RESULT_Z_STR(szList[listIdx], strNewFmt("STR%02u", listIdx), strZ(strNewFmt("check item %u", listIdx)));
-            }
+                TEST_RESULT_Z_STR(szList[listIdx], strNewFmt("STR%02u", listIdx), zNewFmt("check item %u", listIdx));
         }
 
         TEST_RESULT_PTR(szList[strLstSize(list)], NULL, "check null terminator");
@@ -444,8 +456,8 @@ testRun(void)
     if (testBegin("strLstExists()"))
     {
         StringList *list = strLstNew();
-        strLstAddZ(list, "A");
-        strLstAddZ(list, "C");
+        strLstAddSub(list, STRDEF("AX"), 1);
+        strLstAddSubN(list, STRDEF("XC"), 1, 1);
 
         TEST_RESULT_BOOL(strLstExists(list, STRDEF("B")), false, "string does not exist");
         TEST_RESULT_BOOL(strLstExists(list, STRDEF("C")), true, "string exists");
@@ -458,8 +470,8 @@ testRun(void)
 
         TEST_RESULT_STR_Z(strLstJoin(list, ", "), "", "empty list");
 
-        strLstAdd(list, STRDEF("item1"));
-        strLstAddZ(list, "item2");
+        strLstAddZSub(list, "item1X", 5);
+        strLstAddZSubN(list, "Xitem2X", 1, 5);
 
         TEST_RESULT_STR_Z(strLstJoin(list, ", "), "item1, item2", "list");
 
@@ -693,6 +705,12 @@ testRun(void)
         TEST_RESULT_Z(buffer, "ab", "    check buffer");
     }
 
+    // *****************************************************************************************************************************
+    if (testBegin("z*()"))
+    {
+        TEST_RESULT_Z(zNewFmt("id=%d", 777), "id=777", "format");
+    }
+
     // *****************************************************************************************************************************
     if (testBegin("strLstToLog()"))
     {
