@@ -73,7 +73,7 @@ testRun(void)
         TEST_ASSIGN(infoBackup, infoBackupNewLoad(ioBufferReadNew(contentCompare)), "load backup info");
         TEST_RESULT_PTR(infoBackupPg(infoBackup), infoBackup->pub.infoPg, "infoPg set");
         TEST_RESULT_STR(infoBackupCipherPass(infoBackup), NULL, "cipher sub not set");
-        TEST_RESULT_INT(infoBackupDataTotal(infoBackup),  0, "infoBackupDataTotal returns 0");
+        TEST_RESULT_INT(infoBackupDataTotal(infoBackup), 0, "infoBackupDataTotal returns 0");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("check cipher pass");
@@ -271,7 +271,6 @@ testRun(void)
             "\n"                                                                                                                   \
             "[target:file:default]\n"                                                                                              \
             "group=\"group1\"\n"                                                                                                   \
-            "mas""ter=false\n"                                                                                                     \
             "mode=\"0600\"\n"                                                                                                      \
             "user=\"user1\"\n"
 
@@ -400,7 +399,7 @@ testRun(void)
             TEST_MANIFEST_DB                                                                                                       \
             "\n"                                                                                                                   \
             "[target:file]\n"                                                                                                      \
-            "pg_data/PG_VERSION={\"checksum\":\"184473f470864e067ee3a22e64b47b0a1c356f29\",\"mas""ter\":true"                      \
+            "pg_data/PG_VERSION={\"checksum\":\"184473f470864e067ee3a22e64b47b0a1c356f29\""                                        \
                 ",\"reference\":\"20190818-084502F_20190819-084506D\",\"size\":4,\"timestamp\":1565282114}\n"                      \
             "pg_data/base/16384/17000={\"checksum\":\"e0101dd8ffb910c9c202ca35b5f828bcb9697bed\",\"checksum-page\":false"          \
                 ",\"checksum-page-error\":[1],\"repo-size\":4096,\"size\":8192,\"timestamp\":1565282114}\n"                        \
@@ -410,10 +409,10 @@ testRun(void)
                 ",\"reference\":\"20190818-084502F\",\"size\":1073741824,\"timestamp\":1565282116}\n"                              \
             "pg_data/base/32768/33000.32767={\"checksum\":\"6e99b589e550e68e934fd235ccba59fe5b592a9e\",\"checksum-page\":true"     \
                 ",\"reference\":\"20190818-084502F_20190819-084506I\",\"size\":32768,\"timestamp\":1565282114}\n"                  \
-            "pg_data/postgresql.conf={\"checksum\":\"6721d92c9fcdf4248acff1f9a1377127d9064807\",\"mas""ter\":true,\"size\":4457"   \
+            "pg_data/postgresql.conf={\"checksum\":\"6721d92c9fcdf4248acff1f9a1377127d9064807\",\"size\":4457"                     \
                 ",\"timestamp\":1565282114}\n"                                                                                     \
-            "pg_data/special={\"mas""ter\":true,\"mode\":\"0640\",\"size\":0,\"timestamp\":1565282120,\"user\":false}\n"           \
-            "pg_data/dupref={\"mas""ter\":true,\"mode\":\"0640\",\"reference\":\"20190818-084502F\",\"size\":0"                    \
+            "pg_data/special={\"mode\":\"0640\",\"size\":0,\"timestamp\":1565282120,\"user\":false}\n"                             \
+            "pg_data/dupref={\"mode\":\"0640\",\"reference\":\"20190818-084502F\",\"size\":0"                                      \
                 ",\"timestamp\":1565282120,\"user\":false}\n"                                                                      \
             TEST_MANIFEST_FILE_DEFAULT                                                                                             \
             "\n"                                                                                                                   \
@@ -507,7 +506,7 @@ testRun(void)
             TEST_MANIFEST_DB
             "\n"
             "[target:file]\n"
-            "pg_data/PG_VERSION={\"checksum\":\"184473f470864e067ee3a22e64b47b0a1c356f29\",\"mas""ter\":true"
+            "pg_data/PG_VERSION={\"checksum\":\"184473f470864e067ee3a22e64b47b0a1c356f29\""
                 ",\"reference\":\"20190818-084502F_20190819-084506D\",\"size\":4,\"timestamp\":1569256970}\n"
             "pg_data/backup_label={\"checksum\":\"e0101dd8ffb910c9c202ca35b5f828bcb9697bed\",\"checksum-page\":false"
                 ",\"checksum-page-error\":[1],\"size\":249,\"timestamp\":1569257013}\n"
@@ -785,7 +784,7 @@ testRun(void)
             "[target:path]\n"
             "pg_data={}\n"
             TEST_MANIFEST_PATH_DEFAULT,
-            .comment =  "write manifest for dependent backup to be added to full already in backup.info");
+            .comment = "write manifest for dependent backup to be added to full already in backup.info");
 
         TEST_RESULT_VOID(
             infoBackupSaveFile(infoBackup, storageRepoWrite(), INFO_BACKUP_PATH_FILE_STR, cipherTypeNone, NULL),
@@ -842,12 +841,11 @@ testRun(void)
         const Buffer *contentLoad = harnessInfoChecksumZ(
             "[backup:current]\n"
             "20200317-181416F={\"backrest-format\":5,\"backrest-version\":\"2.25dev\","
-            "\"backup-archive-start\":\"000000080000000000000020\",\"backup-archive-stop\":\"000000080000000000000020\","
             "\"backup-info-repo-size\":3687611,\"backup-info-repo-size-delta\":3687611,\"backup-info-size\":31230816,"
             "\"backup-info-size-delta\":31230816,\"backup-timestamp-start\":1584468856,\"backup-timestamp-stop\":1584468864,"
             "\"backup-type\":\"full\",\"db-id\":1,\"option-archive-check\":true,\"option-archive-copy\":false,"
             "\"option-backup-standby\":false,\"option-checksum-page\":true,\"option-compress\":true,\"option-hardlink\":false,"
-            "\"option-online\":true}\n"
+            "\"option-online\":false}\n"
             "20200317-181625F={\"backrest-format\":5,\"backrest-version\":\"2.25dev\","
             "\"backup-archive-start\":\"000000010000000000000038\",\"backup-archive-stop\":\"000000010000000000000038\","
             "\"backup-info-repo-size\":3768898,\"backup-info-repo-size-delta\":3768898,\"backup-info-size\":31533937,"
