@@ -21,8 +21,6 @@ Test protocol server command handlers
 ***********************************************************************************************************************************/
 #define TEST_PROTOCOL_COMMAND_ASSERT                                STRID5("assert", 0x2922ce610)
 
-static unsigned int testCommandAssertProtocolTotal = 0;
-
 __attribute__((__noreturn__)) static void
 testCommandAssertProtocol(PackRead *const param, ProtocolServer *const server)
 {
@@ -34,14 +32,15 @@ testCommandAssertProtocol(PackRead *const param, ProtocolServer *const server)
     ASSERT(param == NULL);
     ASSERT(server != NULL);
 
-    testCommandAssertProtocolTotal++;
-    hrnErrorThrowP(.message = testCommandAssertProtocolTotal <= 3 ? NULL : "ERR_MESSAGE_RETRY");
+    hrnErrorThrowP();
 
     // No FUNCTION_HARNESS_RETURN_VOID() because the function does not return
 }
 
 #define TEST_PROTOCOL_COMMAND_ERROR                                 STRID5("error", 0x127ca450)
 
+static unsigned int testCommandErrorProtocolTotal = 0;
+
 __attribute__((__noreturn__)) static void
 testCommandErrorProtocol(PackRead *const param, ProtocolServer *const server)
 {
@@ -53,7 +52,8 @@ testCommandErrorProtocol(PackRead *const param, ProtocolServer *const server)
     ASSERT(param == NULL);
     ASSERT(server != NULL);
 
-    hrnErrorThrowP(.errorType = &FormatError);
+    testCommandErrorProtocolTotal++;
+    hrnErrorThrowP(.errorType = &FormatError, .message = testCommandErrorProtocolTotal <= 2 ? NULL : "ERR_MESSAGE_RETRY");
 
     // No FUNCTION_HARNESS_RETURN_VOID() because the function does not return
 }
@@ -180,10 +180,10 @@ static ProtocolParallelJob *testParallelJobCallback(void *data, unsigned int cli
         ProtocolParallelJob *job = *(ProtocolParallelJob **)lstGet(listData->jobList, listData->jobIdx);
         listData->jobIdx++;
 
-        FUNCTION_TEST_RETURN(protocolParallelJobMove(job, memContextCurrent()));
+        FUNCTION_TEST_RETURN(PROTOCOL_PARALLEL_JOB, protocolParallelJobMove(job, memContextCurrent()));
     }
 
-    FUNCTION_TEST_RETURN(NULL);
+    FUNCTION_TEST_RETURN(PROTOCOL_PARALLEL_JOB, NULL);
 }
 
 /***********************************************************************************************************************************
@@ -279,18 +279,21 @@ testRun(void)
         // Create bogus client and exec to generate errors
         ProtocolHelperClient protocolHelperClient = {0};
 
-        OBJ_NEW_BEGIN(ProtocolClient)
+        IoWrite *write = ioFdWriteNewOpen(STRDEF("invalid"), 0, 0);
+
+        OBJ_NEW_BEGIN(ProtocolClient, .childQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
         {
             protocolHelperClient.client = OBJ_NEW_ALLOC();
-            *protocolHelperClient.client = (ProtocolClient){.name = STRDEF("test"), .state = protocolClientStateIdle};
+            *protocolHelperClient.client = (ProtocolClient){
+                .name = strNewZ("test"), .state = protocolClientStateIdle, .write = write};
             memContextCallbackSet(memContextCurrent(), protocolClientFreeResource, protocolHelperClient.client);
         }
         OBJ_NEW_END();
 
-        OBJ_NEW_BEGIN(Exec)
+        OBJ_NEW_BEGIN(Exec, .childQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
         {
             protocolHelperClient.exec = OBJ_NEW_ALLOC();
-            *protocolHelperClient.exec = (Exec){.name = STRDEF("test"), .command = strNewZ("test"), .processId = INT_MAX};
+            *protocolHelperClient.exec = (Exec){.name = strNewZ("test"), .command = strNewZ("test"), .processId = INT_MAX};
             memContextCallbackSet(memContextCurrent(), execFreeResource, protocolHelperClient.exec);
         }
         OBJ_NEW_END();
@@ -298,7 +301,7 @@ testRun(void)
         TEST_RESULT_VOID(protocolHelperClientFree(&protocolHelperClient), "free");
 
         TEST_RESULT_LOG(
-            "P00   WARN: assertion 'write != NULL' failed\n"
+            "P00   WARN: unable to write to invalid: [9] Bad file descriptor\n"
             "P00   WARN: unable to wait on child process: [10] No child processes");
     }
 
@@ -487,6 +490,8 @@ testRun(void)
                 ioWriteFlush(HRN_FORK_CHILD_WRITE());
                 ioWriteStrLine(HRN_FORK_CHILD_WRITE(), STRDEF("{\"name\":null}"));
                 ioWriteFlush(HRN_FORK_CHILD_WRITE());
+                ioWriteStrLine(HRN_FORK_CHILD_WRITE(), STRDEF("{}"));
+                ioWriteFlush(HRN_FORK_CHILD_WRITE());
                 ioWriteStrLine(HRN_FORK_CHILD_WRITE(), STRDEF("{\"name\":\"bogus\"}"));
                 ioWriteFlush(HRN_FORK_CHILD_WRITE());
                 ioWriteStrLine(HRN_FORK_CHILD_WRITE(), STRDEF("{\"name\":\"pgBackRest\",\"service\":\"bogus\"}"));
@@ -515,14 +520,21 @@ testRun(void)
                 const ProtocolServerHandler commandHandler[] = {TEST_PROTOCOL_SERVER_HANDLER_LIST};
 
                 TEST_ERROR(
-                    protocolServerProcess(server, NULL, commandHandler, PROTOCOL_SERVER_HANDLER_LIST_SIZE(commandHandler)),
-                    ProtocolError, "invalid command 'BOGUS' (0x38eacd271)");
+                    protocolServerProcess(server, NULL, commandHandler, LENGTH_OF(commandHandler)), ProtocolError,
+                    "invalid command 'BOGUS' (0x38eacd271)");
+
+                // -----------------------------------------------------------------------------------------------------------------
+                TEST_TITLE("server restart and assert");
+
+                // This does not run in a TEST* macro because tests are run by the command handlers
+                TEST_ERROR(
+                    protocolServerProcess(server, NULL, commandHandler, LENGTH_OF(commandHandler)), AssertError, "ERR_MESSAGE");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("server restart");
 
                 // This does not run in a TEST* macro because tests are run by the command handlers
-                protocolServerProcess(server, NULL, commandHandler, PROTOCOL_SERVER_HANDLER_LIST_SIZE(commandHandler));
+                protocolServerProcess(server, NULL, commandHandler, LENGTH_OF(commandHandler));
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("server with retries");
@@ -536,7 +548,7 @@ testRun(void)
                     "new server");
 
                 // This does not run in a TEST* macro because tests are run by the command handlers
-                protocolServerProcess(server, retryList, commandHandler, PROTOCOL_SERVER_HANDLER_LIST_SIZE(commandHandler));
+                protocolServerProcess(server, retryList, commandHandler, LENGTH_OF(commandHandler));
             }
             HRN_FORK_CHILD_END();
 
@@ -547,7 +559,10 @@ testRun(void)
 
                 TEST_ERROR(
                     protocolClientNew(STRDEF("test client"), STRDEF("test"), HRN_FORK_PARENT_READ(0), HRN_FORK_PARENT_WRITE(0)),
-                    JsonFormatError, "expected '{' at 'bogus greeting'");
+                    JsonFormatError, "invalid type at: bogus greeting");
+                TEST_ERROR(
+                    protocolClientNew(STRDEF("test client"), STRDEF("test"), HRN_FORK_PARENT_READ(0), HRN_FORK_PARENT_WRITE(0)),
+                    ProtocolError, "greeting key 'name' must be string type");
                 TEST_ERROR(
                     protocolClientNew(STRDEF("test client"), STRDEF("test"), HRN_FORK_PARENT_READ(0), HRN_FORK_PARENT_WRITE(0)),
                     ProtocolError, "greeting key 'name' must be string type");
@@ -605,7 +620,7 @@ testRun(void)
                     protocolClientExecute(client, protocolCommandNew(TEST_PROTOCOL_COMMAND_ASSERT), false);
                     THROW(TestError, "error was expected");
                 }
-                CATCH_ANY()
+                CATCH_FATAL()
                 {
                     TEST_RESULT_PTR(errorType(), &AssertError, "check type");
                     TEST_RESULT_Z(errorFileName(), TEST_PGB_PATH "/src/protocol/client.c", "check file");
@@ -679,10 +694,10 @@ testRun(void)
                 TEST_TITLE("command throws assert with retry messages");
 
                 TEST_ERROR(
-                    protocolClientExecute(client, protocolCommandNew(TEST_PROTOCOL_COMMAND_ASSERT), false), AssertError,
+                    protocolClientExecute(client, protocolCommandNew(TEST_PROTOCOL_COMMAND_ERROR), false), FormatError,
                     "raised from test client: ERR_MESSAGE\n"
-                    "[AssertError] on retry after 0ms\n"
-                    "[AssertError] on retry after 500ms: ERR_MESSAGE_RETRY");
+                    "[FormatError] on retry after 0ms\n"
+                    "[FormatError] on retry after 500ms: ERR_MESSAGE_RETRY");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("free client");
@@ -717,7 +732,7 @@ testRun(void)
 
                 // Send ping
                 ProtocolClient *protocolClient = protocolClientNew(
-                    PROTOCOL_SERVICE_REMOTE_STR, PROTOCOL_SERVICE_REMOTE_STR, ioSessionIoRead(tlsSession),
+                    PROTOCOL_SERVICE_REMOTE_STR, PROTOCOL_SERVICE_REMOTE_STR, ioSessionIoReadP(tlsSession),
                     ioSessionIoWrite(tlsSession));
                 protocolClientNoExit(protocolClient);
                 protocolClientNoOp(protocolClient);
@@ -971,9 +986,9 @@ testRun(void)
                         protocolClientNew(
                             strNewFmt("local client %u", clientIdx), STRDEF("test"), HRN_FORK_PARENT_READ(clientIdx),
                             HRN_FORK_PARENT_WRITE(clientIdx)),
-                        strZ(strNewFmt("local client %u new", clientIdx)));
+                        zNewFmt("local client %u new", clientIdx));
                     TEST_RESULT_VOID(
-                        protocolParallelClientAdd(parallel, client[clientIdx]), strZ(strNewFmt("local client %u add", clientIdx)));
+                        protocolParallelClientAdd(parallel, client[clientIdx]), zNewFmt("local client %u add", clientIdx));
                 }
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -1089,7 +1104,7 @@ testRun(void)
                 TEST_TITLE("free clients");
 
                 for (unsigned int clientIdx = 0; clientIdx < HRN_FORK_PROCESS_TOTAL(); clientIdx++)
-                    TEST_RESULT_VOID(protocolClientFree(client[clientIdx]), strZ(strNewFmt("free client %u", clientIdx)));
+                    TEST_RESULT_VOID(protocolClientFree(client[clientIdx]), zNewFmt("free client %u", clientIdx));
             }
             HRN_FORK_PARENT_END();
         }
