@@ -32,6 +32,7 @@ typedef struct TestRequestParam
     VAR_PARAM_HEADER;
     const char *content;
     const char *blobType;
+    const char *range;
 } TestRequestParam;
 
 #define testRequestP(write, verb, path, ...)                                                                                       \
@@ -75,8 +76,7 @@ testRequest(IoWrite *write, const char *verb, const char *path, TestRequestParam
     if (param.content != NULL)
     {
         strCatFmt(
-            request, "content-md5:%s\r\n",
-            strZ(strNewEncode(encodeBase64, cryptoHashOne(HASH_TYPE_MD5_STR, BUFSTRZ(param.content)))));
+            request, "content-md5:%s\r\n", strZ(strNewEncode(encodeBase64, cryptoHashOne(hashTypeMd5, BUFSTRZ(param.content)))));
     }
 
     // Add date
@@ -86,6 +86,10 @@ testRequest(IoWrite *write, const char *verb, const char *path, TestRequestParam
     // Add host
     strCatFmt(request, "host:%s\r\n", strZ(hrnServerHost()));
 
+    // Add range
+    if (param.range != NULL)
+        strCatFmt(request, "range:bytes=%s\r\n", param.range);
+
     // Add blob type
     if (param.blobType != NULL)
         strCatFmt(request, "x-ms-blob-type:%s\r\n", param.blobType);
@@ -201,7 +205,6 @@ testRun(void)
         TEST_RESULT_STR_Z(((StorageAzure *)storageDriver(storage))->pathPrefix, "/" TEST_CONTAINER, "check path prefix");
         TEST_RESULT_UINT(((StorageAzure *)storageDriver(storage))->blockSize, STORAGE_AZURE_BLOCKSIZE_MIN, "check block size");
         TEST_RESULT_BOOL(storageFeature(storage, storageFeaturePath), false, "check path feature");
-        TEST_RESULT_BOOL(storageFeature(storage, storageFeatureCompress), false, "check compress feature");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("storage with host but force host-style uri");
@@ -357,6 +360,23 @@ testRun(void)
         TEST_RESULT_STR_Z(((StorageAzure *)storageDriver(storage))->host, "test-host", "check host");
         TEST_RESULT_STR_Z(
             ((StorageAzure *)storageDriver(storage))->pathPrefix, "/" TEST_ACCOUNT "/" TEST_CONTAINER, "check path prefix");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("invalid shared key base64");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgRawStrId(argList, cfgOptRepoType, STORAGE_AZURE_TYPE);
+        hrnCfgArgRawZ(argList, cfgOptRepoAzureContainer, TEST_CONTAINER);
+        hrnCfgEnvRawZ(cfgOptRepoAzureAccount, TEST_ACCOUNT);
+        hrnCfgEnvRawZ(cfgOptRepoAzureKey, BOGUS_STR);
+
+        HRN_CFG_LOAD(cfgCmdArchivePush, argList);
+
+        TEST_ERROR(
+            storageRepoGet(0, false), OptionInvalidValueError,
+            "invalid value for 'repo1-azure-key' option: base64 size 5 is not evenly divisible by 4\n"
+            "HINT: value must be valid base64 when 'repo1-azure-key-type = shared'.");
     }
 
     // *****************************************************************************************************************************
@@ -457,7 +477,7 @@ testRun(void)
 
                 driver = (StorageAzure *)storageDriver(storage);
                 TEST_RESULT_STR(driver->host, hrnServerHost(), "check host");
-                TEST_RESULT_STR_Z(driver->pathPrefix,  "/" TEST_ACCOUNT "/" TEST_CONTAINER, "check path prefix");
+                TEST_RESULT_STR_Z(driver->pathPrefix, "/" TEST_ACCOUNT "/" TEST_CONTAINER, "check path prefix");
                 TEST_RESULT_BOOL(driver->fileId == 0, false, "check file id");
 
                 // Tests need the block size to be 16
@@ -484,13 +504,14 @@ testRun(void)
                     "unable to open missing file '/file.txt' for read");
 
                 // -----------------------------------------------------------------------------------------------------------------
-                TEST_TITLE("get file");
+                TEST_TITLE("get file with offset and limit");
 
-                testRequestP(service, HTTP_VERB_GET, "/file.txt");
+                testRequestP(service, HTTP_VERB_GET, "/file.txt", .range = "1-21");
                 testResponseP(service, .content = "this is a sample file");
 
                 TEST_RESULT_STR_Z(
-                    strNewBuf(storageGetP(storageNewReadP(storage, STRDEF("file.txt")))), "this is a sample file", "get file");
+                    strNewBuf(storageGetP(storageNewReadP(storage, STRDEF("file.txt"), .offset = 1, .limit = VARUINT64(21)))),
+                    "this is a sample file", "get file");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("get zero-length file");
@@ -696,22 +717,15 @@ testRun(void)
                         "    <NextMarker/>"
                         "</EnumerationResults>");
 
-                HarnessStorageInfoListCallbackData callbackData =
-                {
-                    .content = strNew(),
-                };
-
                 TEST_ERROR(
-                    storageInfoListP(storage, STRDEF("/"), hrnStorageInfoListCallback, NULL, .errorOnMissing = true),
-                    AssertError, "assertion '!param.errorOnMissing || storageFeature(this, storageFeaturePath)' failed");
+                    storageNewItrP(storage, STRDEF("/"), .errorOnMissing = true), AssertError,
+                    "assertion '!param.errorOnMissing || storageFeature(this, storageFeaturePath)' failed");
 
-                TEST_RESULT_VOID(
-                    storageInfoListP(storage, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData), "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test_path {path}\n"
-                    "test_file {file, s=787, t=1255369830}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/path/to",
+                    "test_file {s=787, t=1255369830}\n"
+                    "test_path/\n",
+                    .level = storageInfoLevelBasic, .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list exists level");
@@ -734,17 +748,11 @@ testRun(void)
                         "    <NextMarker/>"
                         "</EnumerationResults>");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/"), hrnStorageInfoListCallback, &callbackData, .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "path1 {}\n"
-                    "test1.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/",
+                    "path1/\n"
+                    "test1.txt\n",
+                    .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list a file in root with expression");
@@ -764,17 +772,10 @@ testRun(void)
                         "    <NextMarker/>"
                         "</EnumerationResults>");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/"), hrnStorageInfoListCallback, &callbackData, .expression = STRDEF("^test.*$"),
-                        .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test1.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/",
+                    "test1.txt\n",
+                    .noRecurse = true, .expression = "^test.*$");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list files with continuation");
@@ -820,20 +821,14 @@ testRun(void)
                         "    <NextMarker/>"
                         "</EnumerationResults>");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData, .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "path1 {}\n"
-                    "test1.txt {}\n"
-                    "test2.txt {}\n"
-                    "path2 {}\n"
-                    "test3.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/path/to",
+                    "path1/\n"
+                    "path2/\n"
+                    "test1.txt\n"
+                    "test2.txt\n"
+                    "test3.txt\n",
+                    .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list files with expression");
@@ -867,19 +862,12 @@ testRun(void)
                         "    <NextMarker/>"
                         "</EnumerationResults>");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData, .expression = STRDEF("^test(1|3)"),
-                        .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test1.path {}\n"
-                    "test1.txt {}\n"
-                    "test3.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/path/to",
+                    "test1.path\n"
+                    "test1.txt\n"
+                    "test3.txt\n",
+                    .level = storageInfoLevelExists, .noRecurse = true, .expression = "^test(1|3)");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("switch to SAS auth");
