@@ -72,8 +72,9 @@ typedef struct TestRequestParam
     bool noAuth;
     const char *object;
     const char *query;
-    const char *range;
+    const char *contentRange;
     const char *content;
+    const char *range;
 } TestRequestParam;
 
 #define testRequestP(write, verb, ...)                                                                                             \
@@ -104,12 +105,16 @@ testRequest(IoWrite *write, const char *verb, TestRequestParam param)
     strCatFmt(request, "content-length:%zu\r\n", param.content == NULL ? 0 : strlen(param.content));
 
     // Add content-range
-    if (param.range != NULL)
-        strCatFmt(request, "content-range:bytes %s\r\n", param.range);
+    if (param.contentRange != NULL)
+        strCatFmt(request, "content-range:bytes %s\r\n", param.contentRange);
 
     // Add host
     strCatFmt(request, "host:%s\r\n", strZ(hrnServerHost()));
 
+    // Add range
+    if (param.range != NULL)
+        strCatFmt(request, "range:bytes=%s\r\n", param.range);
+
     // Complete headers
     strCatZ(request, "\r\n");
 
@@ -221,7 +226,6 @@ testRun(void)
         TEST_RESULT_UINT(((StorageGcs *)storageDriver(storage))->chunkSize, STORAGE_GCS_CHUNKSIZE_DEFAULT, "check chunk size");
         TEST_RESULT_STR(((StorageGcs *)storageDriver(storage))->token, TEST_TOKEN_STR, "check token");
         TEST_RESULT_BOOL(storageFeature(storage, storageFeaturePath), false, "check path feature");
-        TEST_RESULT_BOOL(storageFeature(storage, storageFeatureCompress), false, "check compress feature");
     }
 
     // *****************************************************************************************************************************
@@ -364,7 +368,11 @@ testRun(void)
 
                 storageGcsRequestP(
                     (StorageGcs *)storageDriver(storage), HTTP_VERB_POST_STR, .noBucket = true,
-                    .content = BUFSTR(jsonFromKv(kvPut(kvNew(), GCS_JSON_NAME_VAR, VARSTRDEF("bucket")))));
+                    .content = BUFSTR(
+                        jsonWriteResult(
+                            jsonWriteObjectEnd(
+                                jsonWriteStr(
+                                    jsonWriteKeyZ(jsonWriteObjectBegin(jsonWriteNewP()), GCS_JSON_NAME), STRDEF("bucket"))))));
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("auth error");
@@ -408,13 +416,14 @@ testRun(void)
                     "unable to open missing file '/file.txt' for read");
 
                 // -----------------------------------------------------------------------------------------------------------------
-                TEST_TITLE("get file");
+                TEST_TITLE("get file with offset and limit");
 
-                testRequestP(service, HTTP_VERB_GET, .object = "file.txt", .query = "alt=media");
+                testRequestP(service, HTTP_VERB_GET, .object = "file.txt", .query = "alt=media", .range = "1-21");
                 testResponseP(service, .content = "this is a sample file");
 
                 TEST_RESULT_STR_Z(
-                    strNewBuf(storageGetP(storageNewReadP(storage, STRDEF("file.txt")))), "this is a sample file", "get file");
+                    strNewBuf(storageGetP(storageNewReadP(storage, STRDEF("file.txt"), .offset = 1, .limit = VARUINT64(21)))),
+                    "this is a sample file", "get file");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("switch to auto auth");
@@ -577,13 +586,13 @@ testRun(void)
 
                 testRequestP(
                     service, HTTP_VERB_PUT, .upload = true, .noAuth = true,
-                    .query = "name=file.txt&uploadType=resumable&upload_id=ulid1", .range = "0-15/*",
+                    .query = "name=file.txt&uploadType=resumable&upload_id=ulid1", .contentRange = "0-15/*",
                     .content = "1234567890123456");
                 testResponseP(service, .code = 308);
 
                 testRequestP(
                     service, HTTP_VERB_PUT, .upload = true, .noAuth = true,
-                    .query = "fields=md5Hash%2Csize&name=file.txt&uploadType=resumable&upload_id=ulid1", .range = "16-31/32",
+                    .query = "fields=md5Hash%2Csize&name=file.txt&uploadType=resumable&upload_id=ulid1", .contentRange = "16-31/32",
                     .content = "7890123456789012");
                 testResponseP(service, .content = "{\"md5Hash\":\"dnF5x6K/8ZZRzpfSlMMM+w==\",\"size\":\"32\"}");
 
@@ -598,18 +607,18 @@ testRun(void)
 
                 testRequestP(
                     service, HTTP_VERB_PUT, .upload = true, .noAuth = true,
-                    .query = "name=file.txt&uploadType=resumable&upload_id=ulid2", .range = "0-15/*",
+                    .query = "name=file.txt&uploadType=resumable&upload_id=ulid2", .contentRange = "0-15/*",
                     .content = "1234567890123456");
                 testResponseP(service, .code = 503);
                 testRequestP(
                     service, HTTP_VERB_PUT, .upload = true, .noAuth = true,
-                    .query = "name=file.txt&uploadType=resumable&upload_id=ulid2", .range = "0-15/*",
+                    .query = "name=file.txt&uploadType=resumable&upload_id=ulid2", .contentRange = "0-15/*",
                     .content = "1234567890123456");
                 testResponseP(service, .code = 308);
 
                 testRequestP(
                     service, HTTP_VERB_PUT, .upload = true, .noAuth = true,
-                    .query = "fields=md5Hash%2Csize&name=file.txt&uploadType=resumable&upload_id=ulid2", .range = "16-19/20",
+                    .query = "fields=md5Hash%2Csize&name=file.txt&uploadType=resumable&upload_id=ulid2", .contentRange = "16-19/20",
                     .content = "7890");
                 testResponseP(service, .content = "{\"md5Hash\":\"/YXmLZvrRUKHcexohBiycQ==\",\"size\":\"20\"}");
 
@@ -624,7 +633,7 @@ testRun(void)
 
                 testRequestP(
                     service, HTTP_VERB_PUT, .upload = true, .noAuth = true,
-                    .query = "name=file.txt&uploadType=resumable&upload_id=ulid3", .range = "0-15/*",
+                    .query = "name=file.txt&uploadType=resumable&upload_id=ulid3", .contentRange = "0-15/*",
                     .content = "1234567890123456");
                 testResponseP(service, .code = 403);
 
@@ -704,22 +713,15 @@ testRun(void)
                         "  ]"
                         "}");
 
-                HarnessStorageInfoListCallbackData callbackData =
-                {
-                    .content = strNew(),
-                };
-
                 TEST_ERROR(
-                    storageInfoListP(storage, STRDEF("/"), hrnStorageInfoListCallback, NULL, .errorOnMissing = true),
-                    AssertError, "assertion '!param.errorOnMissing || storageFeature(this, storageFeaturePath)' failed");
+                    storageNewItrP(storage, STRDEF("/"), .errorOnMissing = true), AssertError,
+                    "assertion '!param.errorOnMissing || storageFeature(this, storageFeaturePath)' failed");
 
-                TEST_RESULT_VOID(
-                    storageInfoListP(storage, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData), "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test_path {path}\n"
-                    "test_file {file, s=787, t=1255369830}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/path/to",
+                    "test_file {s=787, t=1255369830}\n"
+                    "test_path/\n",
+                    .level = storageInfoLevelBasic, .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list exists level");
@@ -739,17 +741,11 @@ testRun(void)
                         "  ]"
                         "}");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/"), hrnStorageInfoListCallback, &callbackData, .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "path1 {}\n"
-                    "test1.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/",
+                    "path1/\n"
+                    "test1.txt\n",
+                    .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list a file in root with expression");
@@ -767,17 +763,10 @@ testRun(void)
                         "  ]"
                         "}");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/"), hrnStorageInfoListCallback, &callbackData, .expression = STRDEF("^test.*$"),
-                        .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test1.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/",
+                    "test1.txt\n",
+                    .noRecurse = true, .expression = "^test.*$");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list files with continuation");
@@ -820,20 +809,14 @@ testRun(void)
                         "  ]"
                         "}");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData, .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "path1 {}\n"
-                    "test1.txt {}\n"
-                    "test2.txt {}\n"
-                    "path2 {}\n"
-                    "test3.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/path/to",
+                    "path1/\n"
+                    "path2/\n"
+                    "test1.txt\n"
+                    "test2.txt\n"
+                    "test3.txt\n",
+                    .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list files with expression");
@@ -862,19 +845,12 @@ testRun(void)
                         "  ]"
                         "}");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData, .expression = STRDEF("^test(1|3)"),
-                        .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test1.path {}\n"
-                    "test1.txt {}\n"
-                    "test3.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/path/to",
+                    "test1.path\n"
+                    "test1.txt\n"
+                    "test3.txt\n",
+                    .level = storageInfoLevelExists, .noRecurse = true, .expression = "^test(1|3)");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("switch to token auth");
