@@ -19,7 +19,7 @@ storageTestPathExpression(const String *expression, const String *path)
     String *result = NULL;
 
     if (strcmp(strZ(expression), "<TEST>") == 0)
-        result = strNewFmt("test%s", path == NULL ? "" : strZ(strNewFmt("/%s", strZ(path))));
+        result = strNewFmt("test%s", path == NULL ? "" : zNewFmt("/%s", strZ(path)));
     else if (strcmp(strZ(expression), "<NULL>") != 0)
         THROW_FMT(AssertError, "invalid expression '%s'", strZ(expression));
 
@@ -109,7 +109,6 @@ testRun(void)
         TEST_RESULT_PTR(storageDriver(storageTest), storageTest->pub.driver, "check driver");
         TEST_RESULT_UINT(storageType(storageTest), storageTest->pub.type, "check type");
         TEST_RESULT_BOOL(storageFeature(storageTest, storageFeaturePath), true, "check path feature");
-        TEST_RESULT_BOOL(storageFeature(storageTest, storageFeatureCompress), true, "check compress feature");
     }
 
     // *****************************************************************************************************************************
@@ -341,7 +340,7 @@ testRun(void)
     }
 
     // *****************************************************************************************************************************
-    if (testBegin("storageInfoList()"))
+    if (testBegin("storageNewItrP()"))
     {
 #ifdef TEST_CONTAINER_REQUIRED
         TEST_CREATE_NOPERM();
@@ -351,49 +350,40 @@ testRun(void)
         TEST_TITLE("path missing");
 
         TEST_ERROR_FMT(
-            storageInfoListP(storageTest, STRDEF(BOGUS_STR), (StorageInfoListCallback)1, NULL, .errorOnMissing = true),
-            PathMissingError, STORAGE_ERROR_LIST_INFO_MISSING, TEST_PATH "/BOGUS");
+            storageNewItrP(storageTest, STRDEF(BOGUS_STR), .errorOnMissing = true), PathMissingError,
+            STORAGE_ERROR_LIST_INFO_MISSING, TEST_PATH "/BOGUS");
 
-        TEST_RESULT_BOOL(
-            storageInfoListP(storageTest, STRDEF(BOGUS_STR), (StorageInfoListCallback)1, NULL), false, "ignore missing dir");
+        TEST_RESULT_PTR(storageNewItrP(storageTest, STRDEF(BOGUS_STR), .nullOnMissing = true), NULL, "ignore missing dir");
 
 #ifdef TEST_CONTAINER_REQUIRED
         TEST_ERROR_FMT(
-            storageInfoListP(storageTest, pathNoPerm, (StorageInfoListCallback)1, NULL), PathOpenError,
-            STORAGE_ERROR_LIST_INFO ": [13] Permission denied", strZ(pathNoPerm));
+            storageNewItrP(storageTest, pathNoPerm), PathOpenError, STORAGE_ERROR_LIST_INFO ": [13] Permission denied",
+            strZ(pathNoPerm));
 
         // Should still error even when ignore missing
         TEST_ERROR_FMT(
-            storageInfoListP(storageTest, pathNoPerm, (StorageInfoListCallback)1, NULL), PathOpenError,
-            STORAGE_ERROR_LIST_INFO ": [13] Permission denied", strZ(pathNoPerm));
+            storageNewItrP(storageTest, pathNoPerm), PathOpenError, STORAGE_ERROR_LIST_INFO ": [13] Permission denied",
+            strZ(pathNoPerm));
 #endif // TEST_CONTAINER_REQUIRED
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("helper function - storagePosixInfoListEntry()");
-
-        HarnessStorageInfoListCallbackData callbackData =
-        {
-            .content = strNew(),
-        };
+        TEST_TITLE("helper function - storagePosixListEntry()");
 
         TEST_RESULT_VOID(
-            storagePosixInfoListEntry(
-                (StoragePosix *)storageDriver(storageTest), STRDEF("pg"), STRDEF("missing"), storageInfoLevelBasic,
-                hrnStorageInfoListCallback, &callbackData),
+            storagePosixListEntry(
+                (StoragePosix *)storageDriver(storageTest), storageLstNew(storageInfoLevelBasic), STRDEF("pg"), "missing",
+                storageInfoLevelBasic),
             "missing path");
-        TEST_RESULT_STR_Z(callbackData.content, "", "check content");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("path with only dot");
 
         storagePathCreateP(storageTest, STRDEF("pg"), .mode = 0766);
 
-        callbackData.content = strNew();
-
-        TEST_RESULT_VOID(
-            storageInfoListP(storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData),
-            "directory with one dot file sorted");
-        TEST_RESULT_STR_Z(callbackData.content, ". {path, m=0766, u=" TEST_USER ", g=" TEST_GROUP "}\n", "check content");
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "./ {u=" TEST_USER ", g=" TEST_GROUP ", m=0766}\n",
+            .level = storageInfoLevelDetail, .includeDot = true);
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("path with file, link, pipe");
@@ -403,114 +393,102 @@ testRun(void)
         HRN_SYSTEM("sudo chown 77777:77777 " TEST_PATH "/pg/.include");
 #endif // TEST_CONTAINER_REQUIRED
 
-        storagePutP(storageNewWriteP(storageTest, STRDEF("pg/file"), .modeFile = 0660), BUFSTRDEF("TESTDATA"));
+        storagePutP(
+            storageNewWriteP(storageTest, STRDEF("pg/file"), .modeFile = 0660, .timeModified = 1656433838), BUFSTRDEF("TESTDATA"));
 
         HRN_SYSTEM("ln -s ../file " TEST_PATH "/pg/link");
         HRN_SYSTEM("mkfifo -m 777 " TEST_PATH "/pg/pipe");
 
-        callbackData = (HarnessStorageInfoListCallbackData)
-        {
-            .content = strNew(),
-            .timestampOmit = true,
-            .modeOmit = true,
-            .modePath = 0766,
-            .modeFile = 0600,
-            .userOmit = true,
-            .groupOmit = true,
-        };
-
-        TEST_RESULT_VOID(
-            storageInfoListP(storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderAsc),
-            "directory with one dot file sorted");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            ". {path}\n"
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "./ {u=" TEST_USER ", g=" TEST_GROUP ", m=0766}\n"
 #ifdef TEST_CONTAINER_REQUIRED
-            ".include {path, m=0755, u=77777, g=77777}\n"
+            ".include/ {u=77777, g=77777, m=0755}\n"
 #endif // TEST_CONTAINER_REQUIRED
-            "file {file, s=8, m=0660}\n"
-            "link {link, d=../file}\n"
-            "pipe {special}\n",
-            "check content");
+            "file {s=8, t=1656433838, u=" TEST_USER ", g=" TEST_GROUP ", m=0660}\n"
+            "link> {d=../file, u=" TEST_USER ", g=" TEST_GROUP "}\n"
+            "pipe*\n",
+            .level = storageInfoLevelDetail, .includeDot = true);
 
 #ifdef TEST_CONTAINER_REQUIRED
         HRN_SYSTEM("sudo rmdir " TEST_PATH "/pg/.include");
 #endif // TEST_CONTAINER_REQUIRED
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("path - recurse");
+        TEST_TITLE("storageItrMore() twice in a row");
+
+        StorageIterator *storageItr = NULL;
+
+        TEST_ASSIGN(storageItr, storageNewItrP(storageTest, STRDEF("pg")), "new iterator");
+        TEST_RESULT_BOOL(storageItrMore(storageItr), true, "check more");
+        TEST_RESULT_BOOL(storageItrMore(storageItr), true, "check more again");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("path - recurse desc");
 
         storagePathCreateP(storageTest, STRDEF("pg/path"), .mode = 0700);
-        storagePutP(storageNewWriteP(storageTest, STRDEF("pg/path/file"), .modeFile = 0600), BUFSTRDEF("TESTDATA"));
+        storagePutP(
+            storageNewWriteP(storageTest, STRDEF("pg/path/file"), .modeFile = 0600, .timeModified = 1656434296),
+            BUFSTRDEF("TESTDATA"));
 
-        callbackData.content = strNew();
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "pipe*\n"
+            "path/file {s=8, t=1656434296}\n"
+            "path/\n"
+            "link> {d=../file}\n"
+            "file {s=8, t=1656433838}\n"
+            "./\n",
+            .level = storageInfoLevelBasic, .includeDot = true, .sortOrder = sortOrderDesc);
 
-        TEST_RESULT_VOID(
-            storageInfoListP(
-                storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderDesc, .recurse = true),
-            "recurse descending");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            "pipe {special}\n"
-            "path/file {file, s=8}\n"
-            "path {path, m=0700}\n"
-            "link {link, d=../file}\n"
-            "file {file, s=8, m=0660}\n"
-            ". {path}\n",
-            "check content");
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("path - recurse asc");
+
+
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "./\n"
+            "file {s=8, t=1656433838}\n"
+            "link> {d=../file}\n"
+            "path/\n"
+            "path/file {s=8, t=1656434296}\n"
+            "pipe*\n",
+            .level = storageInfoLevelBasic, .includeDot = true);
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("path basic info - recurse");
 
-        storagePathCreateP(storageTest, STRDEF("pg/path"), .mode = 0700);
-        storagePutP(storageNewWriteP(storageTest, STRDEF("pg/path/file"), .modeFile = 0600), BUFSTRDEF("TESTDATA"));
-
-        callbackData.content = strNew();
-
         storageTest->pub.interface.feature ^= 1 << storageFeatureInfoDetail;
 
-        TEST_RESULT_VOID(
-            storageInfoListP(
-                storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderDesc, .recurse = true),
-            "recurse descending");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            "pipe {special}\n"
-            "path/file {file, s=8}\n"
-            "path {path}\n"
-            "link {link}\n"
-            "file {file, s=8}\n"
-            ". {path}\n",
-            "check content");
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "pipe*\n"
+            "path/file {s=8, t=1656434296}\n"
+            "path/\n"
+            "link> {d=../file}\n"
+            "file {s=8, t=1656433838}\n"
+            "./\n",
+            .levelForce = true, .includeDot = true, .sortOrder = sortOrderDesc);
 
         storageTest->pub.interface.feature ^= 1 << storageFeatureInfoDetail;
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("path - filter");
+        TEST_TITLE("empty path - filter");
 
-        callbackData.content = strNew();
+        storagePathCreateP(storageTest, STRDEF("pg/empty"), .mode = 0700);
 
-        TEST_RESULT_VOID(
-            storageInfoListP(
-                storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderAsc,
-                .expression = STRDEF("^path")),
-            "filter");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            "path {path, m=0700}\n",
-            "check content");
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "empty/\n",
+            .level = storageInfoLevelType, .expression = "^empty");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("filter in subpath during recursion");
 
-        callbackData.content = strNew();
-
-        TEST_RESULT_VOID(
-            storageInfoListP(
-                storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderAsc, .recurse = true,
-                .expression = STRDEF("\\/file$")),
-            "filter");
-        TEST_RESULT_STR_Z(callbackData.content, "path/file {file, s=8}\n", "check content");
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "path/file {s=8, t=1656434296}\n",
+            .level = storageInfoLevelBasic, .expression = "\\/file$");
     }
 
     // *****************************************************************************************************************************
@@ -1032,13 +1010,34 @@ testRun(void)
         TEST_RESULT_BOOL(memcmp(bufPtrConst(buffer), "TESTFILE\n", bufSize(buffer)) == 0, true, "check content");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("read limited bytes");
+        TEST_TITLE("error on invalid read offset bytes");
 
-        ioBufferSizeSet(2);
+        TEST_ERROR(
+            storageGetP(storageNewReadP(storageTest, STRDEF(TEST_PATH "/test.txt"), .offset = UINT64_MAX)), FileOpenError,
+            "unable to seek to 18446744073709551615 in file '" TEST_PATH "/test.txt': [22] Invalid argument");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("read limited bytes");
 
         TEST_ASSIGN(buffer, storageGetP(storageNewReadP(storageTest, STRDEF(TEST_PATH "/test.txt"), .limit = VARUINT64(7))), "get");
         TEST_RESULT_UINT(bufSize(buffer), 7, "check size");
         TEST_RESULT_BOOL(memcmp(bufPtrConst(buffer), "TESTFIL", bufSize(buffer)) == 0, true, "check content");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("read offset bytes");
+
+        TEST_ASSIGN(buffer, storageGetP(storageNewReadP(storageTest, STRDEF(TEST_PATH "/test.txt"), .offset = 4)), "get");
+        TEST_RESULT_UINT(bufSize(buffer), 5, "check size");
+        TEST_RESULT_BOOL(memcmp(bufPtrConst(buffer), "FILE\n", bufSize(buffer)) == 0, true, "check content");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("read offset/limited bytes");
+
+        TEST_ASSIGN(
+            buffer, storageGetP(storageNewReadP(storageTest, STRDEF(TEST_PATH "/test.txt"), .offset = 4,
+            .limit = VARUINT64(4))), "get");
+        TEST_RESULT_UINT(bufSize(buffer), 4, "check size");
+        TEST_RESULT_BOOL(memcmp(bufPtrConst(buffer), "FILE", bufSize(buffer)) == 0, true, "check content");
     }
 
     // *****************************************************************************************************************************
@@ -1138,7 +1137,8 @@ testRun(void)
         MEM_CONTEXT_TEMP_BEGIN()
         {
             TEST_ASSIGN(
-                file, storageReadMove(storageNewReadP(storageTest, fileName, .limit = VARUINT64(44)), memContextPrior()),
+                file,
+                storageReadMove(storageNewReadP(storageTest, fileName, .limit = VARUINT64(44)), memContextPrior()),
                 "new read file");
         }
         MEM_CONTEXT_TEMP_END();
@@ -1146,6 +1146,7 @@ testRun(void)
         TEST_RESULT_BOOL(ioReadOpen(storageReadIo(file)), true, "open file");
         TEST_RESULT_STR(storageReadName(file), fileName, "check file name");
         TEST_RESULT_UINT(storageReadType(file), STORAGE_POSIX_TYPE, "check file type");
+        TEST_RESULT_UINT(storageReadOffset(file), 0, "check offset");
         TEST_RESULT_UINT(varUInt64(storageReadLimit(file)), 44, "check limit");
 
         TEST_RESULT_VOID(ioRead(storageReadIo(file), outBuffer), "load data");
@@ -1558,7 +1559,6 @@ testRun(void)
         TEST_ASSIGN(storage, storageRepoGet(0, true), "get cifs repo storage");
         TEST_RESULT_UINT(storageType(storage), STORAGE_CIFS_TYPE, "check storage type");
         TEST_RESULT_BOOL(storageFeature(storage, storageFeaturePath), true, "check path feature");
-        TEST_RESULT_BOOL(storageFeature(storage, storageFeatureCompress), true, "check compress feature");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("write object path sync false");
