@@ -28,7 +28,7 @@ testRun(void)
         PROTOCOL_SERVER_HANDLER_STORAGE_REMOTE_LIST
     };
 
-    hrnProtocolRemoteShimInstall(testRemoteHandlerList, PROTOCOL_SERVER_HANDLER_LIST_SIZE(testRemoteHandlerList));
+    hrnProtocolRemoteShimInstall(testRemoteHandlerList, LENGTH_OF(testRemoteHandlerList));
 
     // Test storage
     Storage *storageTest = storagePosixNewP(TEST_PATH_STR, .write = true);
@@ -77,7 +77,6 @@ testRun(void)
     {
         TEST_RESULT_UINT(storageInterface(storageRepoWrite).feature, storageInterface(storageTest).feature, "check features");
         TEST_RESULT_BOOL(storageFeature(storageRepoWrite, storageFeaturePath), true, "check path feature");
-        TEST_RESULT_BOOL(storageFeature(storageRepoWrite, storageFeatureCompress), true, "check compress feature");
         TEST_RESULT_STR_Z(storagePathP(storageRepo, NULL), TEST_PATH "/repo128", "check repo path");
         TEST_RESULT_STR_Z(storagePathP(storageRepoWrite, NULL), TEST_PATH "/repo128", "check repo write path");
         TEST_RESULT_STR_Z(storagePathP(storagePgWrite, NULL), TEST_PATH "/pg256", "check pg write path");
@@ -194,22 +193,14 @@ testRun(void)
     }
 
     // *****************************************************************************************************************************
-    if (testBegin("storageInfoList()"))
+    if (testBegin("storageNewItrP()"))
     {
         TEST_TITLE("path not found");
 
-        HarnessStorageInfoListCallbackData callbackData =
-        {
-            .content = strNew(),
-        };
-
-        TEST_RESULT_BOOL(
-            storageInfoListP(storageRepo, STRDEF(BOGUS_STR), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderAsc),
-            false, "info list");
-        TEST_RESULT_STR_Z(callbackData.content, "", "check content");
+        TEST_RESULT_PTR(storageNewItrP(storageRepo, STRDEF(BOGUS_STR), .nullOnMissing = true), NULL, "path missing");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("list path and file");
+        TEST_TITLE("list path and file (no user/group");
 
         storagePutP(storageNewWriteP(storagePgWrite, STRDEF("test"), .timeModified = 1555160001), BUFSTRDEF("TESTME"));
 
@@ -221,17 +212,32 @@ testRun(void)
         // Path timestamp must be set after file is created since file creation updates it
         HRN_STORAGE_TIME(storagePgWrite, NULL, 1555160000);
 
-        TEST_RESULT_BOOL(
-            storageInfoListP(storagePgWrite, NULL, hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderAsc), true,
-            "info list");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            ". {path, m=0750, u=" TEST_USER ", g=" TEST_GROUP "}\n"
+        TEST_STORAGE_LIST(
+            storagePgWrite, NULL,
+            "./ {u=" TEST_USER ", g=" TEST_GROUP ", m=0750}\n"
 #ifdef TEST_CONTAINER_REQUIRED
-            "noname {file, s=6, m=0640, t=1555160002, u=99999, g=99999}\n"
+            "noname {s=6, t=1555160002, u=99999, g=99999, m=0640}\n"
 #endif // TEST_CONTAINER_REQUIRED
-            "test {file, s=6, m=0640, t=1555160001, u=" TEST_USER ", g=" TEST_GROUP "}\n",
-            "check content");
+            "test {s=6, t=1555160001, u=" TEST_USER ", g=" TEST_GROUP ", m=0640}\n",
+            .level = storageInfoLevelDetail, .includeDot = true);
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("list path and file");
+
+#ifdef TEST_CONTAINER_REQUIRED
+        HRN_SYSTEM_FMT("sudo chown " TEST_USER ":" TEST_GROUP " %s", strZ(storagePathP(storagePgWrite, STRDEF("noname"))));
+#endif // TEST_CONTAINER_REQUIRED
+
+        // Path timestamp must be set after file is updated
+        HRN_STORAGE_TIME(storagePgWrite, NULL, 1555160000);
+
+        TEST_STORAGE_LIST(
+            storagePgWrite, NULL,
+#ifdef TEST_CONTAINER_REQUIRED
+            "noname {s=6, t=1555160002, u=" TEST_USER ", g=" TEST_GROUP ", m=0640}\n"
+#endif // TEST_CONTAINER_REQUIRED
+            "test {s=6, t=1555160001, u=" TEST_USER ", g=" TEST_GROUP ", m=0640}\n",
+            .level = storageInfoLevelDetail);
     }
 
     // *****************************************************************************************************************************
@@ -295,7 +301,7 @@ testRun(void)
 
         IoFilterGroup *filterGroup = ioReadFilterGroup(storageReadIo(fileRead));
         ioFilterGroupAdd(filterGroup, ioSizeNew());
-        ioFilterGroupAdd(filterGroup, cryptoHashNew(HASH_TYPE_SHA1_STR));
+        ioFilterGroupAdd(filterGroup, cryptoHashNew(hashTypeSha1));
         ioFilterGroupAdd(filterGroup, pageChecksumNew(0, PG_SEGMENT_PAGE_DEFAULT, 0));
         ioFilterGroupAdd(filterGroup, cipherBlockNew(cipherModeEncrypt, cipherTypeAes256Cbc, BUFSTRZ("x"), NULL));
         ioFilterGroupAdd(filterGroup, cipherBlockNew(cipherModeDecrypt, cipherTypeAes256Cbc, BUFSTRZ("x"), NULL));
@@ -322,7 +328,7 @@ testRun(void)
 
         filterGroup = ioReadFilterGroup(storageReadIo(fileRead));
         ioFilterGroupAdd(filterGroup, ioSizeNew());
-        ioFilterGroupAdd(filterGroup, cryptoHashNew(HASH_TYPE_SHA1_STR));
+        ioFilterGroupAdd(filterGroup, cryptoHashNew(hashTypeSha1));
         ioFilterGroupAdd(filterGroup, ioSinkNew());
 
         TEST_RESULT_STR_Z(strNewBuf(storageGetP(fileRead)), "", "no content");
@@ -444,18 +450,11 @@ testRun(void)
 
         TEST_RESULT_VOID(storagePathCreateP(storageRepoWrite, STRDEF("parent/testpath"), .mode = 0777), "path create");
 
-        HarnessStorageInfoListCallbackData callbackData = {.content = strNew(), .userOmit = true, .groupOmit = true};
-
-        TEST_RESULT_BOOL(
-            storageInfoListP(
-                storageRepo, STRDEF(TEST_PATH "/repo128/parent"), hrnStorageInfoListCallback, &callbackData,
-                .sortOrder = sortOrderAsc),
-            true, "info list");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            ". {path, m=0777}\n"
-            "testpath {path, m=0777}\n",
-            "check content");
+        TEST_STORAGE_LIST(
+            storageRepo, TEST_PATH "/repo128/parent",
+            "./ {u=" TEST_USER ", g=" TEST_GROUP ", m=0777}\n"
+            "testpath/ {u=" TEST_USER ", g=" TEST_GROUP ", m=0777}\n",
+            .level = storageInfoLevelDetail, .includeDot = true);
     }
 
     // *****************************************************************************************************************************
