@@ -70,7 +70,6 @@ test.pl [options]
    --clean              clean working and result paths for a completely fresh build
    --clean-only         execute --clean and exit
    --pg-version         version of postgres to test (all, defaults to minimal)
-   --log-force          force overwrite of current test log files
    --build-only         build the binary (and honor --build-package) but don't run tests
    --build-package      build the package
    --build-max          max processes to use for builds (default 4)
@@ -78,10 +77,9 @@ test.pl [options]
    --container-only     only run tests that must be run in a container
    --no-performance     do not run performance tests
    --gen-only           only run auto-generation
-   --no-gen             do not run code generation
+   --min-gen            only run required code generation
    --gen-check          check that auto-generated files are correct (used in CI to detect changes)
    --code-count         generate code counts
-   --expect             --vm=rh7 --pg-version=9.6 --log-force
    --no-valgrind        don't run valgrind on C unit tests (saves time)
    --no-coverage        don't run coverage on C unit tests (saves time)
    --no-coverage-report run coverage but don't generate coverage report (saves time)
@@ -143,7 +141,6 @@ my $bVersion = false;
 my $bHelp = false;
 my $bQuiet = false;
 my $strPgVersion = 'minimal';
-my $bLogForce = false;
 my $strVm;
 my $bVmBuild = false;
 my $bVmForce = false;
@@ -159,11 +156,10 @@ my $bContainerOnly = false;
 my $bNoPerformance = false;
 my $bGenOnly = false;
 my $bGenCheck = false;
-my $bNoGen = false;
+my $bMinGen = false;
 my $bCodeCount = false;
 my $bBackTrace = false;
 my $bProfile = false;
-my $bExpect = false;
 my $bNoValgrind = false;
 my $bNoOptimize = false;
 my $bNoDebug = false;
@@ -198,7 +194,6 @@ GetOptions ('q|quiet' => \$bQuiet,
             'dry-run' => \$bDryRun,
             'no-cleanup' => \$bNoCleanup,
             'pg-version=s' => \$strPgVersion,
-            'log-force' => \$bLogForce,
             'build-only' => \$bBuildOnly,
             'build-package' => \$bBuildPackage,
             'build-max=s' => \$iBuildMax,
@@ -211,11 +206,10 @@ GetOptions ('q|quiet' => \$bQuiet,
             'no-performance' => \$bNoPerformance,
             'gen-only' => \$bGenOnly,
             'gen-check' => \$bGenCheck,
-            'no-gen' => \$bNoGen,
+            'min-gen' => \$bMinGen,
             'code-count' => \$bCodeCount,
             'backtrace' => \$bBackTrace,
             'profile' => \$bProfile,
-            'expect' => \$bExpect,
             'no-valgrind' => \$bNoValgrind,
             'no-optimize' => \$bNoOptimize,
             'no-debug', => \$bNoDebug,
@@ -258,7 +252,7 @@ eval
     ################################################################################################################################
     if ($bDryRun)
     {
-        $bNoGen = true;
+        $bMinGen = true;
     }
 
     ################################################################################################################################
@@ -279,16 +273,6 @@ eval
         $bNoCoverage = true;
     }
 
-    ################################################################################################################################
-    # Update options for --expect
-    ################################################################################################################################
-    if ($bExpect)
-    {
-        $strVm = VM_EXPECT;
-        $strPgVersion = '9.6';
-        $bLogForce = true;
-    }
-
     ################################################################################################################################
     # Setup
     ################################################################################################################################
@@ -409,9 +393,9 @@ eval
         #---------------------------------------------------------------------------------------------------------------------------
         executeTest("make -C ${strBackRestBase}/src -f Makefile.in clean-all");
 
-        # Auto-generate configure files unless --no-gen specified
+        # Auto-generate configure files unless --min-gen specified
         #---------------------------------------------------------------------------------------------------------------------------
-        if (!$bNoGen)
+        if (!$bMinGen)
         {
             &log(INFO, "autogenerate configure");
 
@@ -507,38 +491,6 @@ eval
             &log(INFO, "    autogenerated configure script: " . (@stryBuilt ? join(', ', @stryBuilt) : 'no changes'));
         }
 
-        # Create the build path
-        #---------------------------------------------------------------------------------------------------------------------------
-        my $strBuildPath = "${strTestPath}/build";
-
-        # Determine if we need to start from scratch due to changes that make may not detect
-        if (!-e "${strBuildPath}/Makefile" ||
-            stat("${strBackRestBase}/src/Makefile.in")->mtime > stat("${strBuildPath}/Makefile")->mtime ||
-            stat("${strBackRestBase}/src/configure")->mtime > stat("${strBuildPath}/Makefile")->mtime ||
-            stat("${strBackRestBase}/src/build.auto.h.in")->mtime > stat("${strBuildPath}/Makefile")->mtime)
-        {
-            &log(INFO, "configure build");
-
-            $oStorageTest->pathCreate("${strBuildPath}/repo", {strMode => '0770', bIgnoreExists => true, bCreateParent => true});
-            executeTest("find ${strBuildPath} -mindepth 1 -print0 | xargs -0 rm -rf");
-            executeTest("cd ${strBuildPath} && ${strBackRestBase}/src/configure -q --enable-test");
-        }
-
-        # Auto-generate code files unless --no-gen specified
-        #---------------------------------------------------------------------------------------------------------------------------
-        if (!$bNoGen)
-        {
-            &log(INFO, "autogenerate code");
-
-            # Build code
-            executeTest("make -C ${strBuildPath} build-config build-error build-help");
-
-            if ($bGenOnly)
-            {
-                exit 0;
-            }
-        }
-
         # Make a copy of the repo to track which files have been changed
         #---------------------------------------------------------------------------------------------------------------------------
         my $strRepoCachePath = "${strTestPath}/repo";
@@ -554,6 +506,46 @@ eval
                 (trim(`uname`) ne 'Darwin' ? ' --ignore-missing-args' : '') .
                 " ${strBackRestBase}/ ${strRepoCachePath}");
 
+        # Auto-generate code files (if --min-gen specified then do minimum required)
+        #---------------------------------------------------------------------------------------------------------------------------
+        my $strBuildPath = "${strTestPath}/build";
+
+        &log(INFO, (!-e $strBuildPath ? 'clean ' : '') . 'autogenerate code');
+
+        # Auto-generate version for root meson.build script
+        my $strMesonBuildOld = ${$oStorageTest->get("${strBackRestBase}/meson.build")};
+        my $strMesonBuildNew;
+
+        foreach my $strLine (split("\n", $strMesonBuildOld))
+        {
+            if ($strLine =~ /^    version\: '/)
+            {
+                $strLine = "    version: '" . PROJECT_VERSION . "',";
+            }
+
+            $strMesonBuildNew .= "${strLine}\n";
+        }
+
+        buildPutDiffers($oStorageBackRest, "${strBackRestBase}/meson.build", $strMesonBuildNew);
+
+        # Setup build if it does not exist
+        if (!-e $strBuildPath)
+        {
+            executeTest("meson setup -Dwerror=true -Dfatal-errors=true -Dbuildtype=debug ${strBuildPath} ${strBackRestBase}");
+        }
+
+        # Build code
+        executeTest(
+            "ninja -C ${strBuildPath}" . ($bMinGen ? '' : ' src/build-config src/build-error') . ' src/build-postgres' .
+            ($bMinGen ? '' : " && ${strBuildPath}/src/build-config ${strBackRestBase}/src") .
+            ($bMinGen ? '' : " && ${strBuildPath}/src/build-error ${strBackRestBase}/src") .
+            " && cd $strRepoCachePath/src && ${strBuildPath}/src/build-postgres");
+
+        if ($bGenOnly)
+        {
+            exit 0;
+        }
+
         # Generate code counts
         #---------------------------------------------------------------------------------------------------------------------------
         if ($bCodeCount)
@@ -688,58 +680,71 @@ eval
                 foreach my $strBuildVM (@stryBuildVm)
                 {
                     my $strBuildPath = "${strBinPath}/${strBuildVM}";
-                    my $bRebuild = false;
-                    $rhBinBuild->{$strBuildVM} = false;
 
-                    # Build configure/compile options and see if they have changed from the previous build
-                    my $strCFlags =
-                        (vmWithBackTrace($strBuildVM) && $bBackTrace ? ' -DWITH_BACKTRACE' : '') .
-                        ($bDebugTestTrace ? ' -DDEBUG_TEST_TRACE' : '');
-                    my $strLdFlags = vmWithBackTrace($strBuildVM) && $bBackTrace  ? '-lbacktrace' : '';
-                    my $strConfigOptions = (vmDebugIntegration($strBuildVM) ? ' --enable-test' : '');
-                    my $strBuildFlags = "CFLAGS_EXTRA=${strCFlags}\nLDFLAGS_EXTRA=${strLdFlags}\nCONFIGURE=${strConfigOptions}";
-                    my $strBuildFlagFile = "${strBinPath}/${strBuildVM}/build.flags";
-
-                    my $bBuildOptionsDiffer = buildPutDiffers($oStorageBackRest, $strBuildFlagFile, $strBuildFlags);
-
-                    &log(INFO, "    build bin for ${strBuildVM} (${strBuildPath})");
-
-                    if ($strBuildVM ne VM_NONE)
+                    if ($strBuildVM eq VM_NONE)
                     {
+                        &log(INFO, "    " . (!-e $strBuildPath ? 'clean ' : '') . "bin build for ${strBuildVM} (${strBuildPath})");
+
+                        # Setup build if it does not exist
+                        if (!-e $strBuildPath)
+                        {
+                            executeTest(
+                                "meson setup -Dwerror=true -Dfatal-errors=true -Dbuildtype=debug ${strBuildPath}" .
+                                    " ${strBackRestBase}");
+                        }
+
+                        # Build code
+                        executeTest("ninja -C ${strBuildPath}");
+                    }
+                    else
+                    {
+                        &log(INFO, "    bin build for ${strBuildVM} (${strBuildPath})");
+
+                        my $bRebuild = false;
+                        $rhBinBuild->{$strBuildVM} = false;
+
+                        # Build configure/compile options and see if they have changed from the previous build
+                        my $strCFlags =
+                            (vmWithBackTrace($strBuildVM) && $bBackTrace ? ' -DWITH_BACKTRACE' : '') .
+                            ($bDebugTestTrace ? ' -DDEBUG_TEST_TRACE' : '');
+                        my $strLdFlags = vmWithBackTrace($strBuildVM) && $bBackTrace ? '-lbacktrace' : '';
+                        my $strConfigOptions = (vmDebugIntegration($strBuildVM) ? ' --enable-test' : '');
+                        my $strBuildFlags = "CFLAGS_EXTRA=${strCFlags}\nLDFLAGS_EXTRA=${strLdFlags}\nCONFIGURE=${strConfigOptions}";
+                        my $strBuildFlagFile = "${strBinPath}/${strBuildVM}/build.flags";
+
+                        my $bBuildOptionsDiffer = buildPutDiffers($oStorageBackRest, $strBuildFlagFile, $strBuildFlags);
+
                         executeTest(
                             "docker run -itd -h test-build --name=test-build" .
                                 " -v ${strBackRestBase}:${strBackRestBase} -v ${strTestPath}:${strTestPath} " .
                                 containerRepo() . ":${strBuildVM}-test",
                             {bSuppressStdErr => true});
-                    }
 
-                    if ($bBuildOptionsDiffer ||
-                        !-e "${strBuildPath}/Makefile" ||
-                        stat("${strBackRestBase}/src/Makefile.in")->mtime > stat("${strBuildPath}/Makefile")->mtime ||
-                        stat("${strBackRestBase}/src/configure")->mtime > stat("${strBuildPath}/Makefile")->mtime ||
-                        stat("${strBackRestBase}/src/build.auto.h.in")->mtime > stat("${strBuildPath}/Makefile")->mtime)
-                    {
-                        &log(INFO, '        bin dependencies have changed, rebuilding');
+                        if ($bBuildOptionsDiffer ||
+                            !-e "${strBuildPath}/Makefile" ||
+                            stat("${strBackRestBase}/src/Makefile.in")->mtime > stat("${strBuildPath}/Makefile")->mtime ||
+                            stat("${strBackRestBase}/src/configure")->mtime > stat("${strBuildPath}/Makefile")->mtime ||
+                            stat("${strBackRestBase}/src/build.auto.h.in")->mtime > stat("${strBuildPath}/Makefile")->mtime)
+                        {
+                            &log(INFO, '        bin dependencies have changed, rebuilding');
 
-                        # Remove old path if it exists and save the build flags
-                        executeTest("rm -rf ${strBuildPath}");
-                        buildPutDiffers($oStorageBackRest, $strBuildFlagFile, $strBuildFlags);
+                            # Remove old path if it exists and save the build flags
+                            executeTest("rm -rf ${strBuildPath}");
+                            buildPutDiffers($oStorageBackRest, $strBuildFlagFile, $strBuildFlags);
+
+                            executeTest(
+                                'docker exec -i -u ' . TEST_USER . ' test-build ' .
+                                "bash -c 'cd ${strBuildPath} && ${strBackRestBase}/src/configure -q${strConfigOptions}'",
+                                {bShowOutputAsync => $bLogDetail});
+                        }
 
                         executeTest(
-                            ($strBuildVM ne VM_NONE ? 'docker exec -i -u ' . TEST_USER . ' test-build ' : '') .
-                            "bash -c 'cd ${strBuildPath} && ${strBackRestBase}/src/configure -q${strConfigOptions}'",
+                            'docker exec -i -u ' . TEST_USER . ' test-build ' .
+                            "${strMakeCmd} -s -j ${iBuildMax}" . ($bLogDetail ? '' : ' --silent') .
+                                " --directory ${strBuildPath} CFLAGS_EXTRA='${strCFlags}'" .
+                                ($strLdFlags ne '' ? " LDFLAGS_EXTRA='${strLdFlags}'" : ''),
                             {bShowOutputAsync => $bLogDetail});
-                    }
 
-                    executeTest(
-                        ($strBuildVM ne VM_NONE ? 'docker exec -i -u ' . TEST_USER . ' test-build ' : '') .
-                        "${strMakeCmd} -s -j ${iBuildMax}" . ($bLogDetail ? '' : ' --silent') .
-                            " --directory ${strBuildPath} CFLAGS_EXTRA='${strCFlags}'" .
-                            ($strLdFlags ne '' ? " LDFLAGS_EXTRA='${strLdFlags}'" : ''),
-                        {bShowOutputAsync => $bLogDetail});
-
-                    if ($strBuildVM ne VM_NONE)
-                    {
                         executeTest("docker rm -f test-build");
                     }
                 }
@@ -789,7 +794,7 @@ eval
                         # Patch files in debian package builds
                         #
                         # Use these commands to create a new patch (may need to modify first line):
-                        # BRDIR=/backrest;BRVM=u20;BRPATCHFILE=${BRDIR?}/test/patch/debian-package.patch
+                        # BRDIR=/home/vagrant/pgbackrest;BRVM=u22;BRPATCHFILE=${BRDIR?}/test/patch/debian-package.patch
                         # DBDIR=${BRDIR?}/test/result/package/${BRVM}/debian
                         # diff -Naur ${DBDIR?}.old ${DBDIR}.new > ${BRPATCHFILE?}
                         my $strDebianPackagePatch = "${strBackRestBase}/test/patch/debian-package.patch";
@@ -901,7 +906,7 @@ eval
                         # Patch files in RHEL package builds
                         #
                         # Use these commands to create a new patch (may need to modify first line):
-                        # BRDIR=/backrest;BRVM=rh7;BRPATCHFILE=${BRDIR?}/test/patch/rhel-package.patch
+                        # BRDIR=/home/vagrant/pgbackrest;BRVM=rh7;BRPATCHFILE=${BRDIR?}/test/patch/rhel-package.patch
                         # PKDIR=${BRDIR?}/test/result/package/${BRVM}/SPECS
                         # diff -Naur ${PKDIR?}.old ${PKDIR}.new > ${BRPATCHFILE?}
                         my $strPackagePatch = "${strBackRestBase}/test/patch/rhel-package.patch";
@@ -1032,8 +1037,8 @@ eval
                     my $oJob = new pgBackRestTest::Common::JobTest(
                         $oStorageTest, $strBackRestBase, $strTestPath, $$oyTestRun[$iTestIdx], $bDryRun, $bVmOut, $iVmIdx, $iVmMax,
                         $strMakeCmd, $iTestIdx, $iTestMax, $strLogLevel, $strLogLevelTest, $strLogLevelTestFile, !$bNoLogTimestamp,
-                        $bLogForce, $bShowOutputAsync, $bNoCleanup, $iRetry, !$bNoValgrind, !$bNoCoverage, $bCoverageSummary,
-                        !$bNoOptimize, $bBackTrace, $bProfile, $iScale, $strTimeZone, !$bNoDebug, $bDebugTestTrace,
+                        $bShowOutputAsync, $bNoCleanup, $iRetry, !$bNoValgrind, !$bNoCoverage, $bCoverageSummary, !$bNoOptimize,
+                        $bBackTrace, $bProfile, $iScale, $strTimeZone, !$bNoDebug, $bDebugTestTrace,
                         $iBuildMax / $iVmMax < 1 ? 1 : int($iBuildMax / $iVmMax));
                     $iTestIdx++;
 
@@ -1092,11 +1097,11 @@ eval
         $strBackRestBase,                                           # Base backrest directory
         $strTestPath,                                               # Path where the tests will run
         dirname($strTestPath) . "/bin/${strVm}/" . PROJECT_EXE,     # Path to the pgbackrest binary
-        dirname($strTestPath) . "/bin/" . VM_NONE . '/' . PROJECT_EXE,  # Path to the backrest Perl storage helper
+        dirname($strTestPath) . "/bin/" . VM_NONE . '/src/' . PROJECT_EXE,  # Path to the pgbackrest storage helper
         $strPgVersion ne 'minimal' ? $strPgSqlBin: undef,           # Pg bin path
         $strPgVersion ne 'minimal' ? $strPgVersion: undef,          # Pg version
         $stryModule[0], $stryModuleTest[0], \@iyModuleTestRun,      # Module info
-        $bVmOut, $bDryRun, $bNoCleanup, $bLogForce,                 # Test options
+        $bVmOut, $bDryRun, $bNoCleanup,                             # Test options
         $strLogLevelTestFile,                                       # Log options
         TEST_USER, TEST_GROUP);                                     # User/group info
 
